@@ -2,7 +2,7 @@
  *  Implement command GETVCP
  */
 
-// Copyright (C) 2014-2022 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2023 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 #include "app_ddcutil/app_getvcp.h"
@@ -18,7 +18,7 @@
 #include "util/string_util.h"
 #include "util/report_util.h"
 
-#ifdef USE_USB
+#ifdef ENABLE_USB
 #include "usb_util/hiddev_reports.h"
 #include "usb_util/hiddev_util.h"
 #endif
@@ -128,7 +128,7 @@ app_show_single_vcp_value_by_feature_id(
                                        dh,
                                        force || feature_id >= 0xe0);  // with_default
    if (!dfm) {
-      printf("Unrecognized VCP feature code: 0x%02x\n", feature_id);
+      printf("Unrecognized VCP feature code: x%02X\n", feature_id);
       psc = DDCRC_UNKNOWN_FEATURE;
    }
    else {
@@ -167,7 +167,7 @@ app_show_vcp_subset_values_by_dh(
 
    if (features_seen)
       DBGTRC_RET_DDCRC(debug, TRACE_GROUP, psc, "features_seen=%s",
-                                           bs256_to_string(*features_seen, "x", ", ") );
+                                           bs256_to_string_t(*features_seen, "x", ", ") );
    else
       DBGTRC_RET_DDCRC(debug, TRACE_GROUP, psc, "");
    return psc;
@@ -188,12 +188,14 @@ app_show_feature_set_values_by_dh(
       Parsed_Cmd *         parsed_cmd)
 {
    bool debug = false;
-   if (debug || IS_TRACING()) {
-      DBGTRC_STARTING(debug, TRACE_GROUP, "dh: %s. fsref: %s, flags: %s",
-                             dh_repr(dh), fsref_repr_t(parsed_cmd->fref),
-                             feature_set_flag_names_t(parsed_cmd->flags));
+   DBGTRC_STARTING(debug, TRACE_GROUP, "dh: %s. fsref: %s, flags: %s",
+                                       dh_repr(dh), fsref_repr_t(parsed_cmd->fref),
+                                       feature_set_flag_names_t(parsed_cmd->flags));
+   if (debug || IS_TRACING())
       dbgrpt_feature_set_ref(parsed_cmd->fref,1);
-   }
+
+   if (parsed_cmd->flags & CMD_FLAG_EXPLICIT_I2C_SOURCE_ADDR)
+      alt_source_addr = parsed_cmd->explicit_i2c_source_addr;
 
    Feature_Set_Ref *    fsref = parsed_cmd->fref;
 
@@ -201,8 +203,8 @@ app_show_feature_set_values_by_dh(
    Feature_Set_Flags flags = 0x00;
    if (parsed_cmd->flags & CMD_FLAG_SHOW_UNSUPPORTED)
       flags |= FSF_SHOW_UNSUPPORTED;
-   if (parsed_cmd->flags & CMD_FLAG_FORCE)
-      flags |= FSF_FORCE;
+   // if (parsed_cmd->flags & CMD_FLAG_FORCE)
+   //    flags |= FSF_FORCE;                     // unused for getvcp, 11/18/2023
    if (parsed_cmd->flags & CMD_FLAG_NOTABLE)
       flags |= FSF_NOTABLE;
    if (parsed_cmd->flags & CMD_FLAG_RW_ONLY)
