@@ -3,7 +3,7 @@
  *  Implement the SETVCP command
  */
 
-// Copyright (C) 2014-2022 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2023 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 /** \cond */
@@ -26,6 +26,7 @@
 #include "cmdline/parsed_cmd.h"
 
 #include "ddc/ddc_vcp.h"
+#include "ddc/ddc_packet_io.h"      // for alt_source_addr
 
 #include "dynvcp/dyn_feature_codes.h"
 
@@ -57,7 +58,7 @@ parse_vcp_value(
    bool ok = str_to_int(canonical, parsed_value_loc, 0);
    free(canonical);
    if (!ok) {
-      f0printf(errf, "Not a number: \"%s\"", string_value);
+      f0printf(errf, "Not a number: \"%s\"\n", string_value);
       ok = false;
    }
    else if (*parsed_value_loc < 0 || *parsed_value_loc > 65535) {
@@ -101,20 +102,20 @@ app_set_vcp_value(
 
    dfm = dyn_get_feature_metadata_by_dh(feature_code,dh, (force || feature_code >= 0xe0) );
    if (!dfm) {
-      ddc_excp = errinfo_new(DDCRC_UNKNOWN_FEATURE, __func__,
+      ddc_excp = ERRINFO_NEW(DDCRC_UNKNOWN_FEATURE,
                               "Unrecognized VCP feature code: 0x%02x", feature_code);
       goto bye;
    }
 
    if (!(dfm->feature_flags & DDCA_WRITABLE)) {
-      ddc_excp = errinfo_new(DDCRC_INVALID_OPERATION, __func__,
+      ddc_excp = ERRINFO_NEW(DDCRC_INVALID_OPERATION,
                  "Feature 0x%02x (%s) is not writable", feature_code, dfm->feature_name);
       goto bye;
    }
 
    if (dfm->feature_flags & DDCA_TABLE) {
       if (value_type != VALUE_TYPE_ABSOLUTE) {
-         ddc_excp = errinfo_new(DDCRC_INVALID_OPERATION, __func__,
+         ddc_excp = ERRINFO_NEW(DDCRC_INVALID_OPERATION,
                                  "Relative VCP values valid only for Continuous VCP features");
          goto bye;
       }
@@ -122,7 +123,7 @@ app_set_vcp_value(
       Byte * value_bytes;
       int bytect = hhs_to_byte_array(new_value, &value_bytes);
       if (bytect < 0) {    // bad hex string
-         ddc_excp = errinfo_new(DDCRC_ARG, __func__, "Invalid hex value");
+         ddc_excp = ERRINFO_NEW(DDCRC_ARG, "Invalid hex value");
          goto bye;
       }
 
@@ -136,13 +137,13 @@ app_set_vcp_value(
       good_value = parse_vcp_value(new_value, &itemp);
       if (!good_value) {
          // what is better status code?
-         ddc_excp = errinfo_new(DDCRC_ARG, __func__,  "Invalid VCP value: %s", new_value);
+         ddc_excp = ERRINFO_NEW(DDCRC_ARG, "Invalid VCP value: %s", new_value);
          goto bye;
       }
 
       if (value_type != VALUE_TYPE_ABSOLUTE) {
          if ( !(dfm->feature_flags & DDCA_CONT) ) {
-            ddc_excp = errinfo_new(DDCRC_INVALID_OPERATION, __func__,
+            ddc_excp = ERRINFO_NEW(DDCRC_INVALID_OPERATION,
                            "Relative VCP values valid only for Continuous VCP features");
             goto bye;
          }
@@ -191,7 +192,7 @@ app_set_vcp_value(
                        "Verification failed for feature %02x", feature_code);
       else
          ddc_excp = errinfo_new_with_cause(ddcrc, ddc_excp, __func__,
-                       "Setting value failed for feature %02x, rc=%s",
+                       "Setting value failed for feature x%02X, rc=%s",
                        feature_code, psc_desc(ddcrc));
    }
 
@@ -219,18 +220,20 @@ app_setvcp(Parsed_Cmd * parsed_cmd, Display_Handle * dh)
    for (int ndx = 0; ndx < parsed_cmd->setvcp_values->len; ndx++) {
       Parsed_Setvcp_Args * cur =
             &g_array_index(parsed_cmd->setvcp_values, Parsed_Setvcp_Args, ndx);
+      if (parsed_cmd->flags & CMD_FLAG_EXPLICIT_I2C_SOURCE_ADDR)
+         alt_source_addr = parsed_cmd->explicit_i2c_source_addr;
       ddc_excp = app_set_vcp_value(
             dh,
             cur->feature_code,
             cur->feature_value_type,
             cur->feature_value,
-            parsed_cmd->flags & CMD_FLAG_FORCE);
+            parsed_cmd->flags & CMD_FLAG_FORCE_UNRECOGNIZED_VCP_CODE);
       if (ddc_excp) {
          f0printf(ferr(), "%s\n", ddc_excp->detail);
          if (ddc_excp->status_code == DDCRC_RETRIES)
             f0printf(ferr(), "    Try errors: %s\n", errinfo_causes_string(ddc_excp));
          ddcrc = ERRINFO_STATUS(ddc_excp);
-         ERRINFO_FREE_WITH_REPORT(ddc_excp, debug || IS_TRACING() || report_freed_exceptions);
+         BASE_ERRINFO_FREE_WITH_REPORT(ddc_excp, IS_DBGTRC(debug,TRACE_GROUP));
          break;
       }
    }
