@@ -3,7 +3,7 @@
  *  Implement VCPINFO and (deprecated) LISTVCP commands
  */
 
-// Copyright (C) 2020-2022 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2020-2023 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 #include "config.h"
@@ -17,6 +17,8 @@
 #include "util/report_util.h"
 
 #include "base/core.h"
+#include "base/rtti.h"
+#include "base/vcp_version.h"
 
 #include "vcp/vcp_feature_set.h"
 #include "vcp/vcp_feature_codes.h"
@@ -30,8 +32,7 @@
  *  @param  flags   version specific feature flags
  *  @param  buf     pointer to buffer
  *  @param  buflen  buffer size
- *
- * Returns:   buf
+ *  @return buf
  */
 static char *
 vcp_interpret_version_feature_flags(
@@ -48,7 +49,7 @@ vcp_interpret_version_feature_flags(
    else if (flags & DDCA_RW)
       rwmsg = "ReadWrite";
 
-   char * typemsg = "";
+   char * typemsg = NULL;
    // NEED TO ALSO HANDLE TABLE TYPE
    if (flags & DDCA_CONT)
       typemsg = "Continuous";
@@ -123,46 +124,44 @@ valid_versions(
    Byte result = 0x00;
 
    if (pentry->v20_flags)
-      result |= DDCA_MCCS_V20;
+      result |= MCCS_SPEC_V20;
    if (pentry->v21_flags) {
       if ( !(pentry->v21_flags & DDCA_DEPRECATED) )
-         result |= DDCA_MCCS_V21;
+         result |= MCCS_SPEC_V21;
    }
    else {
-      if (result & DDCA_MCCS_V20)
-         result |= DDCA_MCCS_V21;
+      if (result & MCCS_SPEC_V20)
+         result |= MCCS_SPEC_V21;
    }
    if (pentry->v30_flags) {
       if ( !(pentry->v30_flags & DDCA_DEPRECATED) )
-         result |= DDCA_MCCS_V30;
+         result |= MCCS_SPEC_V30;
    }
    else {
-      if (result & DDCA_MCCS_V21)
-         result |= DDCA_MCCS_V30;
+      if (result & MCCS_SPEC_V21)
+         result |= MCCS_SPEC_V30;
    }
    if (pentry->v22_flags) {
       if ( !(pentry->v22_flags & DDCA_DEPRECATED) )
-         result |= DDCA_MCCS_V22;
+         result |= MCCS_SPEC_V22;
    }
    else {
-      if (result & DDCA_MCCS_V21)
-         result |= DDCA_MCCS_V22;
+      if (result & MCCS_SPEC_V21)
+         result |= MCCS_SPEC_V22;
    }
    return result;
 }
 
 
-/* Given a byte of flags indicating MCCS versions, return a string containing a
- * comma delimited list of MCCS version names.
+/** Given a byte of flags indicating MCCS versions, return a string containing a
+ *  comma delimited list of MCCS version names.
  *
- * Arguments:
- *   valid_version_flags
- *   version_name_buf      buffer in which to return string
- *   bufsz                 buffer size
+ *  @param  valid_version_flags
+ *  @param  version_name_buf      buffer in which to return string
+ *  @param  bufsz                 buffer size
+ *  @return version_name_buf
  *
- * Returns:                version_name_buf
- *
- * Note: MCCS 1.0 is not reported
+ *  Note: MCCS 1.0 is not reported
  */
 static char *
 valid_version_names_r(
@@ -173,19 +172,19 @@ valid_version_names_r(
    assert(bufsz >= (4*5));        // max 4 version names, 5 chars/name
    *version_name_buf = '\0';
 
-   if (valid_version_flags & DDCA_MCCS_V20)
+   if (valid_version_flags & MCCS_SPEC_V20)
       strcpy(version_name_buf, "2.0");
-   if (valid_version_flags & DDCA_MCCS_V21) {
+   if (valid_version_flags & MCCS_SPEC_V21) {
       if (strlen(version_name_buf) > 0)
          strcat(version_name_buf, ", ");
       strcat(version_name_buf, "2.1");
    }
-   if (valid_version_flags & DDCA_MCCS_V30) {
+   if (valid_version_flags & MCCS_SPEC_V30) {
       if (strlen(version_name_buf) > 0)
          strcat(version_name_buf, ", ");
       strcat(version_name_buf, "3.0");
    }
-   if (valid_version_flags & DDCA_MCCS_V22) {
+   if (valid_version_flags & MCCS_SPEC_V22) {
       if (strlen(version_name_buf) > 0)
          strcat(version_name_buf, ", ");
       strcat(version_name_buf, "2.2");
@@ -349,11 +348,6 @@ report_vcp_feature_table_entry(
          rpt_vstring(d1, "Simple NC values:");
          report_sl_values(pentry->default_sl_values, d1+1);
       }
-
-      if (pentry->default_sl_values && output_level >= DDCA_OL_VERBOSE) {
-         rpt_vstring(d1, "Simple NC values:");
-         report_sl_values(pentry->default_sl_values, d1+1);
-      }
    }
 }
 
@@ -407,3 +401,7 @@ app_vcpinfo(Parsed_Cmd * parsed_cmd)
    DBGTRC_RET_BOOL(debug, DDCA_TRC_VCP|DDCA_TRC_TOP, vcpinfo_ok, "");
    return vcpinfo_ok;
 }
+
+void init_app_vcpinfo() {
+   RTTI_ADD_FUNC(app_vcpinfo);
+}
