@@ -3,7 +3,7 @@
  *  ddcutil standalone application mainline
  */
 
-// Copyright (C) 2014-2023 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2024 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 /** \cond */
@@ -18,12 +18,11 @@
 #include <sys/stat.h>
 #include <stdbool.h>
 #include <stdio.h>
+#include <unistd.h>
 
 #include <stdlib.h>
 #include <string.h>
-#ifdef ENABLE_SYSLOG
 #include <syslog.h>
-#endif
 #include <unistd.h>
 
 #include "util/data_structures.h"
@@ -33,6 +32,9 @@
 #include "util/file_util.h"
 #include "util/glib_string_util.h"
 #include "util/i2c_util.h"
+#ifdef USE_LIBDRM
+#include "util/libdrm_util.h"
+#endif
 #include "util/linux_util.h"
 #include "util/report_util.h"
 #include "util/simple_ini_file.h"
@@ -50,16 +52,15 @@
 #include "base/core.h"
 #include "base/ddc_errno.h"
 #include "base/ddc_packets.h"
+#include "base/display_retry_data.h"
 #include "base/displays.h"
-#include "base/dynamic_sleep.h"
+#include "base/dsa2.h"
 #include "base/linux_errno.h"
 #include "base/monitor_model_key.h"
 #include "base/parms.h"
 #include "base/rtti.h"
 #include "base/sleep.h"
 #include "base/status_code_mgt.h"
-#include "base/thread_retry_data.h"
-#include "base/thread_sleep_data.h"
 #include "base/tuned_sleep.h"
 
 #include "i2c/i2c_sysfs.h"
@@ -72,10 +73,11 @@
 #include "dynvcp/dyn_parsed_capabilities.h"
 
 #include "i2c/i2c_bus_core.h"
+#include "i2c/i2c_dpms.h"
 #include "i2c/i2c_strategy_dispatcher.h"
 #include "i2c/i2c_sysfs.h"
 
-#ifdef USE_USB
+#ifdef ENABLE_USB
 #include "usb/usb_displays.h"
 #endif
 
@@ -87,10 +89,12 @@
 #include "ddc/ddc_output.h"
 #include "ddc/ddc_packet_io.h"
 #include "ddc/ddc_read_capabilities.h"
+#include "ddc/ddc_serialize.h"
 #include "ddc/ddc_services.h"
-#include "ddc/ddc_try_stats.h"
+#include "ddc/ddc_try_data.h"
 #include "ddc/ddc_vcp_version.h"
 #include "ddc/ddc_vcp.h"
+#include "ddc/ddc_watch_displays.h"
 
 #include "cmdline/cmd_parser_aux.h"    // for parse_feature_id_or_subset(), should it be elsewhere?
 #include "cmdline/cmd_parser.h"
@@ -105,7 +109,7 @@
 #include "app_ddcutil/app_interrogate.h"
 #include "app_ddcutil/app_probe.h"
 #include "app_ddcutil/app_getvcp.h"
-#include "app_ddcutil/app_services.h"
+#include <app_ddcutil/app_ddcutil_services.h>
 #include "app_ddcutil/app_setvcp.h"
 #include "app_ddcutil/app_vcpinfo.h"
 #include "app_ddcutil/app_watch.h"
@@ -113,16 +117,19 @@
 #include "app_ddcutil/app_testcases.h"
 #endif
 
+#ifdef ENABLE_ENVCMDS
+#include "app_sysenv/app_sysenv_services.h"
 #include "app_sysenv/query_sysenv.h"
-#ifdef USE_USB
+#ifdef ENABLE_USB
 #include "app_sysenv/query_sysenv_usb.h"
 #endif
+#endif
 
 
 // Default trace class for this file
 static DDCA_Trace_Group TRACE_GROUP = DDCA_TRC_TOP;
 
-static void add_rtti_functions();
+static void add_local_rtti_functions();
 
 
 //
@@ -134,14 +141,11 @@ report_performance_options(int depth)
 {
       int d1 = depth+1;
       rpt_label(depth, "Performance and Retry Options:");
-      rpt_vstring(d1, "Deferred sleep enabled:                      %s", sbool( is_deferred_sleep_enabled() ) );
-#ifdef OLD   // SLEEP_SUPPRESSION
-      rpt_vstring(d1, "Sleep suppression (reduced sleeps) enabled:  %s", sbool( is_sleep_suppression_enabled() ) );
-#endif
-      bool dsa_enabled =  tsd_get_dsa_enabled_default();
-      rpt_vstring(d1, "Dynamic sleep adjustment enabled:            %s", sbool(dsa_enabled) );
-      if ( dsa_enabled )
-        rpt_vstring(d1, "Sleep multiplier factor:                %5.2f", tsd_get_sleep_multiplier_factor() );
+      rpt_vstring(d1, "Deferred sleep enabled:                 %s", sbool( is_deferred_sleep_enabled() ) );
+      rpt_vstring(d1, "Dynamic sleep algorithm enabled:        %s", sbool(dsa2_is_enabled()));
+      if (dsa2_is_enabled())
+      rpt_vstring(d1, "Minimum dynamic sleep multiplier:    %7.2f", dsa2_get_minimum_multiplier());
+      rpt_vstring(d1, "Default sleep multiplier factor:     %7.2f", pdd_get_default_sleep_multiplier_factor() );
       rpt_nl();
 }
 
@@ -153,6 +157,8 @@ report_optional_features(Parsed_Cmd * parsed_cmd, int depth) {
    rpt_vstring( depth, "%.*s%-*s%s", 0, "", 28, "User defined features:",
                        (enable_dynamic_features) ? "enabled" : "disabled" );
                        // "Enable user defined features" is too long a title
+   rpt_vstring( depth, "%.*s%-*s%s", 0, "", 28, "Mock feature values:",
+                       (enable_mock_data) ? "enabled" : "disabled" );
    rpt_nl();
 }
 
@@ -169,14 +175,14 @@ report_all_options(Parsed_Cmd * parsed_cmd, char * config_fn, char * default_opt
     if (config_fn)
        rpt_vstring(depth, "%.*s%-*s%s", 0, "", 28, "Configuration file options:", default_options);
 
-
-    // if (parsed_cmd->output_level >= DDCA_OL_VV)
-       report_build_options(depth);
+    // report_build_options(depth);
     show_reporting();  // uses fout()
     report_optional_features(parsed_cmd, depth);
     report_tracing(depth);
+    rpt_nl();
     report_performance_options(depth);
     report_experimental_options(parsed_cmd, depth);
+    report_build_options(depth);
 
     DBGMSF(debug, "Done");
 }
@@ -276,7 +282,8 @@ validate_environment()
 }
 
 
-int verify_i2c_access() {   // non-static to enable tracing
+STATIC int
+verify_i2c_access() {
    bool debug = false;
    DBGTRC_STARTING(debug, TRACE_GROUP, "");
 
@@ -289,7 +296,7 @@ int verify_i2c_access() {   // non-static to enable tracing
    Bit_Set_256 buses = get_possible_ddc_ci_bus_numbers();  //sysfs bus numbers, not dev-i2c
    buses_ct = bs256_count(buses);
    DBGTRC(debug, TRACE_GROUP, "/sys/bus/i2c/devices to check: %s",
-                              bs256_to_string_decimal(buses, "i2c-", ", "));
+                              bs256_to_string_decimal_t(buses, "i2c-", ", "));
    if (buses_ct == 0) {
       fprintf(stderr, "No /sys/bus/i2c buses that might be used for DDC/CI communication found.\n");
       fprintf(stderr, "No display adapters with i2c buses appear to exist.\n");
@@ -321,13 +328,13 @@ int verify_i2c_access() {   // non-static to enable tracing
 
       if (buses_without_devices_ct > 0) {
          fprintf(stderr, "/sys/bus/i2c buses without /dev/i2c-N devices: %s\n",
-                bs256_to_string_decimal(buses_without_devices, "/sys/bus/i2c/devices/i2c-", " ") );
+                bs256_to_string_decimal_t(buses_without_devices, "/sys/bus/i2c/devices/i2c-", " ") );
          fprintf(stderr, "Driver i2c_dev must be loaded or builtin\n");
          fprintf(stderr, "See https://www.ddcutil.com/kernel_module\n");
       }
       if (inaccessible_devices_ct > 0) {
          fprintf(stderr, "Devices possibly used for DDC/CI communication cannot be opened: %s\n",
-                bs256_to_string_decimal(inaccessible_devices, "/dev/i2c-", " "));
+                bs256_to_string_decimal_t(inaccessible_devices, "/dev/i2c-", " "));
          fprintf(stderr, "See https://www.ddcutil.com/i2c_permissions\n");
       }
    }
@@ -345,12 +352,13 @@ int verify_i2c_access() {   // non-static to enable tracing
  *   \param  parsed_cmd  parsed command line
  *   \return ok if successful, false if error
  */
-static bool
+STATIC bool
 master_initializer(Parsed_Cmd * parsed_cmd) {
    bool debug = false;
    DBGMSF(debug, "Starting ...");
    bool ok = false;
-   submaster_initializer(parsed_cmd);   // shared with libddcutil
+   if (!submaster_initializer(parsed_cmd))    // shared with libddcutil
+      goto bye;
 
 #ifdef ENABLE_ENVCMDS
    if (parsed_cmd->cmd_id != CMDID_ENVIRONMENT) {
@@ -358,8 +366,6 @@ master_initializer(Parsed_Cmd * parsed_cmd) {
       if (!validate_environment())
          goto bye;
    }
-
-   init_sysenv();
 #else
    if (!validate_environment())
       goto bye;
@@ -375,7 +381,7 @@ bye:
 }
 
 
-static void
+STATIC void
 ensure_vcp_version_set(Display_Handle * dh)
 {
    bool debug = false;
@@ -383,7 +389,7 @@ ensure_vcp_version_set(Display_Handle * dh)
    DDCA_MCCS_Version_Spec vspec = get_vcp_version_by_dh(dh);
    if (vspec.major < 2 && get_output_level() >= DDCA_OL_NORMAL) {
       f0printf(stdout, "VCP (aka MCCS) version for display is undetected or less than 2.0. "
-            "Output may not be accurate.\n");
+            "Interpretation may not be accurate.\n");
    }
    DBGMSF(debug, "Done");
 }
@@ -425,15 +431,12 @@ find_dref(
       Display_Ref ** dref_loc)
 {
    bool debug = false;
-   DBGTRC_STARTING(debug, TRACE_GROUP, "did: %s, set_default_display: %s",
+   DBGTRC_STARTING(debug, TRACE_GROUP, "did: %s, displayid_required: %s",
                                     did_repr(parsed_cmd->pdid),
                                     displayid_requirement_name(displayid_required));
    FILE * outf = fout();
    Status_Errno_DDC final_result = DDCRC_OK;
    Display_Ref * dref = NULL;
-   Call_Options callopts = CALLOPT_ERR_MSG;        // emit error messages
-   if (parsed_cmd->flags & CMD_FLAG_FORCE)
-      callopts |= CALLOPT_FORCE;
 
    Display_Identifier * did_work = parsed_cmd->pdid;
    if (did_work && did_work->id_type == DISP_ID_BUSNO) {
@@ -445,13 +448,14 @@ find_dref(
          if (businfo->flags & I2C_BUS_ADDR_0X50)  {
             dref = create_bus_display_ref(busno);
             dref->dispno = DISPNO_INVALID;      // or should it be DISPNO_NOT_SET?
-            dref->pedid = businfo->edid;    // needed?
+            dref->pedid = copy_parsed_edid(businfo->edid);
             dref->mmid  = monitor_model_key_new(
                              dref->pedid->mfg_id,
                              dref->pedid->model_name,
                              dref->pedid->product_code);
             // dref->driver_name = get_i2c_device_sysfs_driver(busno);
             // DBGMSG("driver_name = %p -> %s", dref->driver_name, dref->driver_name);
+            dref->drm_connector = g_strdup(businfo->drm_connector_name);
 
             // dref->pedid = i2c_get_parsed_edid_by_busno(did_work->busno);
             dref->detail = businfo;
@@ -469,6 +473,10 @@ find_dref(
                DBGTRC_NOPREFIX(debug, TRACE_GROUP, "Synthetic Display_Ref");
                final_result = DDCRC_OK;
             }
+            if (dref && (dref->flags&DREF_DPMS_SUSPEND_STANDBY_OFF)) {
+               // should go nowhere, but just in case:
+               f0printf(outf, "Monitor on bus /dev/i2c-%d is in a DPMS sleep mode. Expect DDC errors.", busno);
+            }
          }  // has edid
          else {   // no EDID found
             f0printf(fout(), "No monitor detected on bus /dev/i2c-%d\n", busno);
@@ -488,9 +496,9 @@ find_dref(
          final_result = DDCRC_OK;
       }
       else {
-         DBGTRC_NOPREFIX(debug, DDCA_TRC_NONE, "No monitor specified, treat as  --display 1");
          bool temporary_did_work = false;
          if (!did_work) {
+            DBGTRC_NOPREFIX(debug, DDCA_TRC_NONE, "No monitor specified, treat as  --display 1");
             did_work = create_dispno_display_identifier(1);   // default monitor
             temporary_did_work = true;
          }
@@ -498,9 +506,11 @@ find_dref(
          DBGTRC_NOPREFIX(debug, TRACE_GROUP, "Detecting displays...");
          ddc_ensure_displays_detected();
          DBGTRC_NOPREFIX(debug, TRACE_GROUP, "display detection complete");
-         dref = get_display_ref_for_display_identifier(did_work, callopts);
+         dref = get_display_ref_for_display_identifier(did_work, CALLOPT_NONE);
          if (temporary_did_work)
             free_display_identifier(did_work);
+         if (!dref)
+            f0printf(ferr(), "Display not found\n");
          final_result = (dref) ? DDCRC_OK : DDCRC_INVALID_DISPLAY;
       }
    }  // !DISP_ID_BUSNO
@@ -546,10 +556,6 @@ execute_cmd_with_optional_display_handle(
 
    case CMDID_LOADVCP:
       {
-         // check_dynamic_features();
-         // ensure_vcp_version_set();
-
-         tsd_dsa_enable(parsed_cmd->flags & CMD_FLAG_DSA);
          // loadvcp will search monitors to find the one matching the
          // identifiers in the record
          ddc_ensure_displays_detected();
@@ -661,6 +667,33 @@ execute_cmd_with_optional_display_handle(
 }
 
 
+DDCA_Syslog_Level preparse_syslog_level(int argc, char** argv) {
+   bool debug = false;
+   DDCA_Syslog_Level result = DDCA_SYSLOG_NOT_SET;
+   char * syslog_arg = NULL;
+   int syslog_pos = ntsa_find(argv, "--syslog");
+   if (syslog_pos >= 0 && syslog_pos < (argc-1))
+      syslog_arg = argv[syslog_pos+1];
+   else {
+      syslog_pos = ntsa_findx(argv, "--syslog=", str_starts_with);
+      if (syslog_pos >= 0)
+         syslog_arg = argv[syslog_pos] + strlen("--syslog");
+   }
+   if (syslog_arg) {
+      DBGMSF(debug, "Parsing initial log level: |%s|", syslog_arg);
+      GPtrArray *sink = g_ptr_array_new();
+      g_ptr_array_set_free_func(sink, g_free);
+      DDCA_Syslog_Level parsed_level;
+      bool ok_level = parse_syslog_level(syslog_arg, &parsed_level, sink);
+      g_ptr_array_free(sink, true);
+      if (ok_level)
+         result = parsed_level;
+   }
+   DBGMSF(debug, "Returning: %s", syslog_level_name(result));
+   return result;
+}
+
+
 //
 // Mainline
 //
@@ -683,50 +716,99 @@ main(int argc, char *argv[]) {
    int main_rc = EXIT_FAILURE;
    bool start_time_reported = false;
 
+   bool explicit_syslog_level = false;
+   syslog_level = DEFAULT_DDCUTIL_SYSLOG_LEVEL;
+   bool syslog_opened = false;
+   bool preparse_verbose = false;
+   bool skip_config = false;
+   Parsed_Cmd * parsed_cmd = NULL;
+
    time_t program_start_time = time(NULL);
    char * program_start_time_s = asctime(localtime(&program_start_time));
    if (program_start_time_s[strlen(program_start_time_s)-1] == 0x0a)
         program_start_time_s[strlen(program_start_time_s)-1] = 0;
 
-   Parsed_Cmd * parsed_cmd = NULL;
-   add_rtti_functions();      // add entries for this file
-   init_base_services();      // so tracing related modules are initialized
+   add_local_rtti_functions();      // add entries for this file
+   init_base_services();            // so tracing related modules are initialized
+   init_ddc_services();             // initializes i2c, usb, ddc, vcp, dynvcp
+   init_app_ddcutil_services();
+#ifdef ENABLE_ENVCMDS
+   init_app_sysenv_services();
+#endif
    DBGMSF(main_debug, "init_base_services() complete, ol = %s",
                       output_level_name(get_output_level()) );
+   // dbgrpt_rtti_func_name_table(3);
 
-   GPtrArray * config_file_errs = g_ptr_array_new_with_free_func(g_free);
    char ** new_argv = NULL;
    int     new_argc = 9;
    char *  untokenized_cmd_prefix = NULL;
    char *  configure_fn = NULL;
 
-   int apply_config_rc = apply_config_file(
-                    "ddcutil",     // use this section of config file
-                    argc,
-                    argv,
-                    &new_argc,
-                    &new_argv,
-                    &untokenized_cmd_prefix,
-                    &configure_fn,
-                    config_file_errs);
-#ifdef LATER
-   if (untokenized_cmd_prefix && strlen(untokenized_cmd_prefix) > 0)
-      fprintf(fout(), "Applying ddcutil options from %s: %s\n", configure_fn,
-            untokenized_cmd_prefix);
-#endif
-   DBGMSF(main_debug, "apply_config_file() returned %s", psc_desc(apply_config_rc));
-   if (config_file_errs->len > 0) {
-      f0printf(ferr(), "Error(s) reading ddcutil configuration from file %s:\n", configure_fn);
-      for (int ndx = 0; ndx < config_file_errs->len; ndx++) {
-         char * s = g_strdup_printf("   %s\n", (char *) g_ptr_array_index(config_file_errs, ndx));
-         f0printf(ferr(), s);
-         free(s);
+   DDCA_Syslog_Level preparsed_level = preparse_syslog_level(argc, argv);
+   if (preparsed_level != DDCA_SYSLOG_NOT_SET) {
+      DBGMSF(main_debug, "Setting syslog_level = %s", syslog_level_name(preparsed_level));
+      syslog_level = preparsed_level;
+      explicit_syslog_level = true;
+   }
+   DBGMSF(main_debug, "syslog_level=%s, explicit_syslog_level=%s",
+                      syslog_level_name(syslog_level),  sbool(explicit_syslog_level));
+
+   preparse_verbose = ntsa_find(argv, "--verbose") >= 0 || ntsa_find(argv, "-v") >= 0;
+
+   skip_config = (ntsa_find(argv, "--noconfig") >= 0);
+   if (skip_config) {
+      // DBGMSG("Skipping config file");
+      new_argv = ntsa_copy(argv, true);
+      new_argc = argc;
+   }
+   else {
+      GPtrArray * config_file_errs = g_ptr_array_new_with_free_func(g_free);
+      int apply_config_rc = apply_config_file(
+                       "ddcutil",     // use this section of config file
+                       argc,
+                       argv,
+                       &new_argc,
+                       &new_argv,
+                       &untokenized_cmd_prefix,
+                       &configure_fn,
+                       config_file_errs);
+      DBGMSF(main_debug, "apply_config_file() returned %s", psc_desc(apply_config_rc));
+      DBGMSF(main_debug, "syslog_level=%s, explicit_syslog_level=%s",
+                         syslog_level_name(syslog_level), SBOOL(explicit_syslog_level));
+      if (config_file_errs->len > 0) {
+         if (syslog_level > DDCA_SYSLOG_NEVER) {
+            openlog("ddcutil",    // prepended to every log message
+                     LOG_CONS |   // write to system console if error sending to system logger
+                     LOG_PID,     // include caller's process id
+                     LOG_USER);   // generic user program, syslogger can use to determine how to handle
+            syslog_opened = true;
+            DBGMSF(main_debug, "openlog() executed");
+         }
+         f0printf(ferr(), "Error(s) reading ddcutil configuration from file %s:\n", configure_fn);
+         if (syslog_opened)
+            syslog(LOG_ERR, "Error(s) reading ddcutil configuration from file %s:\n", configure_fn);
+         for (int ndx = 0; ndx < config_file_errs->len; ndx++) {
+            char * s = g_strdup_printf("   %s\n", (char *) g_ptr_array_index(config_file_errs, ndx));
+            f0printf(ferr(), s);
+            if (syslog_opened)
+               syslog(LOG_ERR, "%s", s);
+            free(s);
+         }
+      }
+
+      g_ptr_array_free(config_file_errs, true);
+
+      if (apply_config_rc < 0)
+         goto bye;
+
+      if (preparse_verbose) {
+         if (untokenized_cmd_prefix && strlen(untokenized_cmd_prefix) > 0) {
+            fprintf(fout(), "Applying ddcutil options from %s: %s\n", configure_fn, untokenized_cmd_prefix);
+            if (syslog_opened)
+               syslog(LOG_INFO, "Applying ddcutil options from %s: %s\n", configure_fn, untokenized_cmd_prefix);
+         }
       }
    }
-   g_ptr_array_free(config_file_errs, true);
-
-   if (apply_config_rc < 0)
-      goto bye;
 
    assert(new_argc == ntsa_length(new_argv));
 
@@ -735,59 +817,88 @@ main(int argc, char *argv[]) {
       rpt_ntsa(new_argv, 1);
    }
 
-   parsed_cmd = parse_command(new_argc, new_argv, MODE_DDCUTIL);
+   parsed_cmd = parse_command(new_argc, new_argv, MODE_DDCUTIL, NULL);
    DBGMSF(main_debug, "parse_command() returned %p", parsed_cmd);
    ntsa_free(new_argv, true);
-
-   if (!parsed_cmd) {
+   if (!parsed_cmd)
       goto bye;      // main_rc == EXIT_FAILURE
+
+   if (parsed_cmd->cmd_id == CMDID_CHKUSBMON) {
+      // prevent io
+      parsed_cmd->flags &= ~(CMD_FLAG_DSA2 | CMD_FLAG_ENABLE_CACHED_CAPABILITIES | CMD_FLAG_ENABLE_CACHED_DISPLAYS);
    }
 
-   init_tracing(parsed_cmd);
+   Error_Info * errs = init_tracing(parsed_cmd);
+   if (errs) {
+      for (int ndx = 0; ndx < errs->cause_ct; ndx++) {
+         Error_Info * cur = errs->causes[ndx];
+         fprintf(stderr, "%s\n", cur->detail);
+         if (syslog_opened)
+            syslog(LOG_ERR, "%s\n", cur->detail);
+      }
+      goto bye;
+   }
+   if (preparse_verbose)
+      parsed_cmd->output_level = DDCA_OL_VERBOSE;
+
+   if (explicit_syslog_level)
+      parsed_cmd->syslog_level = explicit_syslog_level;
+
+   if (parsed_cmd->syslog_level > DDCA_SYSLOG_NEVER && !syslog_opened) {
+      if (parsed_cmd->syslog_level > DDCA_SYSLOG_NEVER ) {   // global
+         openlog("ddcutil",          // prepended to every log message
+                 LOG_CONS |          // write to system console if error sending to system logger
+                 LOG_PID,            // include caller's process id
+                 LOG_USER);          // generic user program, syslogger can use to determine how to handle
+         syslog_opened = true;
+         DBGMSF(main_debug, "openlog() executed");
+      }
+   }
+   else if (parsed_cmd->syslog_level == DDCA_SYSLOG_NEVER && syslog_opened) {
+      // oops
+      DBGMSF(main_debug, "parsed_cmd=>syslog_level == DDCA_SYSLOG_NEVER, calling closelog()");
+      closelog();
+      syslog_opened = false;
+   }
 
    // tracing is sufficiently initialized, can report start time
    start_time_reported = parsed_cmd->traced_groups    ||
-                         parsed_cmd->traced_functions ||
                          parsed_cmd->traced_files     ||
                          IS_TRACING()                 ||
                          main_debug;
-   if (main_debug)
-      printf("(%s) start_time_reported = %s\n", __func__, SBOOL(start_time_reported));
+   DBGMSF(main_debug, "start_time_reported = %s", SBOOL(start_time_reported));
    DBGMSF(start_time_reported, "Starting %s execution, %s",
                parser_mode_name(parsed_cmd->parser_mode),
                program_start_time_s);
-#ifdef ENABLE_SYSLOG
-   if (trace_to_syslog) {
-      openlog("ddcutil",          // prepended to every log message
-              LOG_CONS |          // write to system console if error sending to system logger
-              LOG_PID,            // include caller's process id
-              LOG_USER);          // generic user program, syslogger can use to determine how to handle
-      syslog(LOG_INFO, "Starting.  ddcutil version %s", get_full_ddcutil_version());
+
+   SYSLOG2(DDCA_SYSLOG_NOTICE, "Starting.  ddcutil version %s", get_full_ddcutil_version());
+
+   if (preparse_verbose) {
+      if (untokenized_cmd_prefix && strlen(untokenized_cmd_prefix) > 0) {
+         SYSLOG2(DDCA_SYSLOG_NOTICE,"Applying ddcutil options from %s: %s",   configure_fn, untokenized_cmd_prefix);
+      }
+   }
+
+#ifdef UNUSED
+#ifdef USE_X11
+   if (!(parsed_cmd->flags&CMD_FLAG_F12)) {
+      dpms_check_x11_asleep();
+      if (dpms_state & DPMS_STATE_X11_ASLEEP) {
+         // DBGMSF(true, "DPMS sleep mode is active. Terminating execution.");
+         MSG_W_SYSLOG(DDCA_SYSLOG_NOTICE, "DPMS sleep mode is active.  Terminating execution");
+        goto bye;
+      }
    }
+#endif
 #endif
 
-   bool ok = master_initializer(parsed_cmd);
-   if (!ok)
+   if (!master_initializer(parsed_cmd))
       goto bye;
    if (parsed_cmd->flags&CMD_FLAG_SHOW_SETTINGS)
       report_all_options(parsed_cmd, configure_fn, untokenized_cmd_prefix, 0);
 
    // xdg_tests(); // for development
 
-   // Initialization complete, rtti now contains entries for all traced functions
-   // Check that any functions specified on --trcfunc are actually traced.
-   // dbgrpt_rtti_func_name_table(0);
-   if (parsed_cmd->traced_functions) {
-      for (int ndx = 0; ndx < ntsa_length(parsed_cmd->traced_functions); ndx++) {
-         char * func_name = parsed_cmd->traced_functions[ndx];
-         // DBGMSG("Verifying: %s", func_name);
-         if (!rtti_get_func_addr_by_name(func_name)) {
-            rpt_vstring(0, "Traced function not found: %s", func_name);
-            goto bye;
-         }
-      }
-   }
-
    if (parsed_cmd->flags & CMD_FLAG_F2) {
       consolidated_i2c_sysfs_report(0);
       // rpt_label(0, "*** Tests Done ***");
@@ -796,11 +907,12 @@ main(int argc, char *argv[]) {
 
    Call_Options callopts = CALLOPT_NONE;
    i2c_forceable_slave_addr_flag = parsed_cmd->flags & CMD_FLAG_FORCE_SLAVE_ADDR;
-   if (parsed_cmd->flags & CMD_FLAG_FORCE)
-      callopts |= CALLOPT_FORCE;
 
-   // affects all current threads and new threads
-   tsd_dsa_enable_globally(parsed_cmd->flags & CMD_FLAG_DSA);
+#ifdef ENABLE_USB
+   usb_ignore_hiddevs(parsed_cmd->ignored_hiddevs);
+   Vid_Pid_Value * values = (parsed_cmd->ignored_usb_vid_pid_ct == 0) ? NULL : parsed_cmd->ignored_usb_vid_pids;
+   usb_ignore_vid_pid_values(parsed_cmd->ignored_usb_vid_pid_ct, values);
+#endif
 
    main_rc = EXIT_SUCCESS;     // from now on assume success;
    DBGTRC_NOPREFIX(main_debug, TRACE_GROUP, "Initialization complete, process commands");
@@ -815,6 +927,53 @@ main(int argc, char *argv[]) {
       main_rc = (vcpinfo_ok) ? EXIT_SUCCESS : EXIT_FAILURE;
    }
 
+   else if (parsed_cmd->cmd_id == CMDID_LIST_RTTI) {
+      report_rtti_func_name_table(0, "Functions traceable by name:");
+      main_rc = EXIT_SUCCESS;
+   }
+
+   // else if (parsed_cmd->cmd_id == CMDID_DISCARD_CACHE) {
+   //    i2c_discard_caches(parsed_cmd->discarded_cache_types);
+   // }
+
+   else if (parsed_cmd->cmd_id == CMDID_C1) {
+      DBGMSG("Executing temporarily defined command C1");
+      if (!drm_enabled) {
+         DBGMSG("Requires DRM capable video drivers.");
+         main_rc = EXIT_FAILURE;
+      }
+      else {
+         ddc_ensure_displays_detected();
+         DDCA_Display_Event_Class event_classes = DDCA_EVENT_CLASS_ALL;
+         if (parsed_cmd->flags&CMD_FLAG_F13)
+            event_classes = DDCA_EVENT_CLASS_DISPLAY_CONNECTION;
+         if (parsed_cmd->flags&CMD_FLAG_F14)
+            event_classes = DDCA_EVENT_CLASS_DPMS;
+         Error_Info * erec = ddc_start_watch_displays(event_classes);
+         if (erec) {
+            DBGMSG(erec->detail);
+            ERRINFO_FREE_WITH_REPORT(erec, true);
+            main_rc = EXIT_FAILURE;
+         }
+         else {
+            DBGMSG("Sleeping for 60 minutes");
+            sleep(60*60);
+            main_rc = EXIT_SUCCESS;
+         }
+      }
+   }
+
+   else if (parsed_cmd->cmd_id == CMDID_C2) {
+      DBGMSG("Executing temporarily defined command C2: noop");
+      main_rc = EXIT_SUCCESS;
+   }
+
+   else if (parsed_cmd->cmd_id == CMDID_C3 || parsed_cmd->cmd_id == CMDID_C4) {
+      Cmd_Desc * desc = get_command(parsed_cmd->cmd_id);
+      DBGMSG("Unrecognized command: %s", desc->cmd_name);
+      main_rc = EXIT_FAILURE;
+   }
+
 #ifdef INCLUDE_TESTCASES
    else if (parsed_cmd->cmd_id == CMDID_LISTTESTS) {
       show_test_cases();
@@ -851,12 +1010,12 @@ main(int argc, char *argv[]) {
    else if (parsed_cmd->cmd_id == CMDID_ENVIRONMENT) {
       DBGTRC_NOPREFIX(main_debug, TRACE_GROUP, "Processing command ENVIRONMENT...");
       dup2(1,2);   // redirect stderr to stdout
-      query_sysenv();
+      query_sysenv(parsed_cmd->flags & CMD_FLAG_QUICK);
       main_rc = EXIT_SUCCESS;
    }
 
    else if (parsed_cmd->cmd_id == CMDID_USBENV) {
-#ifdef USE_USB
+#ifdef ENABLE_USB
       DBGTRC_NOPREFIX(main_debug, TRACE_GROUP, "Processing command USBENV...");
       dup2(1,2);   // redirect stderr to stdout
       query_usbenv();
@@ -869,13 +1028,12 @@ main(int argc, char *argv[]) {
 #endif
 
    else if (parsed_cmd->cmd_id == CMDID_CHKUSBMON) {
-#ifdef USE_USB
+#ifdef ENABLE_USB
       // DBGMSG("Processing command chkusbmon...\n");
       DBGTRC_NOPREFIX(main_debug, TRACE_GROUP, "Processing command CHKUSBMON...");
       bool is_monitor = check_usb_monitor( parsed_cmd->args[0] );
       main_rc = (is_monitor) ? EXIT_SUCCESS : EXIT_FAILURE;
 #else
-      PROGRAM_LOGIC_ERROR("ddcutil not built with USB support");
       main_rc = EXIT_FAILURE;
 #endif
    }
@@ -903,10 +1061,11 @@ main(int argc, char *argv[]) {
          if (dref) {
             DBGMSF(main_debug,
                    "mainline - display detection complete, about to call ddc_open_display() for dref" );
-            Status_Errno_DDC ddcrc = ddc_open_display(dref, callopts |CALLOPT_ERR_MSG, &dh);
-            ASSERT_IFF( (ddcrc==0), dh);
+            Error_Info* err = ddc_open_display(dref, callopts, &dh);
+            ASSERT_IFF( !err, dh);
             if (!dh) {
-               f0printf(ferr(), "Error %s opening display ref %s", psc_desc(ddcrc), dref_repr_t(dref));
+               f0printf(ferr(), "Error opening %s: %s\n", dref_repr_t(dref), psc_name(err->status_code));
+               errinfo_free(err);
                main_rc = EXIT_FAILURE;
             }
          }  // dref
@@ -915,27 +1074,51 @@ main(int argc, char *argv[]) {
             main_rc = execute_cmd_with_optional_display_handle(parsed_cmd, dh);
          }
 
-         if (dh)
-               ddc_close_display(dh);
+         if (dh) {
+            Error_Info * err = ddc_close_display(dh);
+            if (err) {
+               MSG_W_SYSLOG(DDCA_SYSLOG_ERROR, "%s: %s", err->detail, psc_desc(err->status_code));
+               errinfo_free(err);
+            }
+         }
          if (dref && (dref->flags & DREF_TRANSIENT))
             free_display_ref(dref);
       }
    }
+   DBGTRC(main_debug, DDCA_TRC_TOP, "After command processing");
 
    if (parsed_cmd->stats_types != DDCA_STATS_NONE
+         && ( ddc_displays_already_detected() ||
+              (parsed_cmd->pdid && parsed_cmd->pdid->id_type == DISP_ID_BUSNO)
+            )
 #ifdef ENABLE_ENVCMDS
          && parsed_cmd->cmd_id != CMDID_INTERROGATE
 #endif
       )
    {
-      ddc_report_stats_main(parsed_cmd->stats_types, parsed_cmd->flags & CMD_FLAG_PER_THREAD_STATS, 0);
+      ddc_report_stats_main(
+            parsed_cmd->stats_types,
+            parsed_cmd->flags & CMD_FLAG_VERBOSE_STATS,
+            parsed_cmd->flags & CMD_FLAG_INTERNAL_STATS,
+            parsed_cmd->flags & CMD_FLAG_STATS_TO_SYSLOG,
+            0);
       // report_timestamp_history();  // debugging function
    }
 
 bye:
+   DBGTRC(main_debug, DDCA_TRC_TOP, "at label bye");
    free(untokenized_cmd_prefix);
    free(configure_fn);
    free_regex_hash_table();
+   if (parsed_cmd && parsed_cmd->cmd_id != CMDID_CHKUSBMON && parsed_cmd->cmd_id != CMDID_DISCARD_CACHE) {
+      if (dsa2_is_enabled())
+         dsa2_save_persistent_stats();
+      if (display_caching_enabled)
+         ddc_store_displays_cache();
+   }
+   if (parsed_cmd)
+      free_parsed_cmd(parsed_cmd);
+
    DBGTRC_DONE(main_debug, TRACE_GROUP, "main_rc=%d", main_rc);
 
    time_t end_time = time(NULL);
@@ -944,28 +1127,27 @@ bye:
       end_time_s[strlen(end_time_s)-1] = 0;
    DBGMSF(start_time_reported, "ddcutil execution complete, %s", end_time_s);
 
-   if (parsed_cmd)
-      free_parsed_cmd(parsed_cmd);
-   release_base_services();
-#ifdef ENABLE_SYSLOG
-   if (trace_to_syslog) {
-      syslog(LOG_INFO, "Terminating. Returning %d", main_rc);
+   DBGMSF(main_debug, "syslog_opened=%s", sbool(syslog_opened));
+   if (syslog_opened) {
+      SYSLOG2(DDCA_SYSLOG_NOTICE, "Terminating. Returning %d", main_rc);
       closelog();
    }
-#endif
+
+   terminate_ddc_services();
+   terminate_base_services();
+
    return main_rc;
 }
 
 
-static void add_rtti_functions() {
+static void add_local_rtti_functions() {
    RTTI_ADD_FUNC(main);
    RTTI_ADD_FUNC(execute_cmd_with_optional_display_handle);
    RTTI_ADD_FUNC(find_dref);
-      RTTI_ADD_FUNC(verify_i2c_access);
+   RTTI_ADD_FUNC(verify_i2c_access);
 #ifdef UNUSED
 #ifdef TARGET_LINUX
    RTTI_ADD_FUNC(validate_environment_using_libkmod);
 #endif
 #endif
-   init_app_services();
 }
