@@ -43,6 +43,7 @@
 
 #include "base/build_info.h"
 #include "base/core.h"
+#include "base/dsa2.h"
 #include "base/linux_errno.h"
 #include "base/parms.h"
 #include "base/rtti.h"
@@ -52,8 +53,11 @@
 
 #include "ddc/ddc_displays.h"     // for ddc_ensure_displays_detected()
 #include "ddc/ddc_display_ref_reports.h"
+#include "ddc/ddc_serialize.h"
 #include "ddc/ddc_watch_displays.h"
 
+#include "vcp/persistent_capabilities.h"
+
 #include "query_sysenv_access.h"
 #include "query_sysenv_base.h"
 #include "query_sysenv_dmidecode.h"
@@ -415,7 +419,7 @@ static void probe_i2c_devices_using_udev() {
    char * subsys_name = "i2c-dev";
    rpt_vstring(0,"*** Probe I2C devices using udev, susbsystem %s ***", subsys_name);
    sysenv_rpt_current_time(NULL, 1);
-   // probe_udev_subsystem() is in udev_util.c, which is only linked in if USE_USB
+   // probe_udev_subsystem() is in udev_util.c, which is only linked in if ENABLE_USB
 
    // Detailed scan of I2C device information.   TMI
    // probe_udev_subsystem(subsys_name, /*show_usb_parent=*/ false, 1);
@@ -454,7 +458,7 @@ static void probe_i2c_devices_using_udev() {
 
 
 void rpt_module_status(int depth, const char * module_name) {
-   int module_status =  module_status_using_libkmod(module_name);
+   int module_status = module_status_by_modules_builtin_or_existence(module_name);
    switch( module_status) {
    case KERNEL_MODULE_NOT_FOUND:    // 0
       rpt_vstring(depth,"Kernel module %-16s not found", module_name);
@@ -463,20 +467,20 @@ void rpt_module_status(int depth, const char * module_name) {
       rpt_vstring(depth,"Kernel module %-16s is builtin", module_name);
       break;
    case KERNEL_MODULE_LOADABLE_FILE: // 2
-      if (is_module_loaded_using_libkmod(module_name))
+      if (is_module_loaded_using_sysfs(module_name))
          rpt_vstring(depth, "Kernel module %-16s is loaded", module_name);
       else
          rpt_vstring(depth, "Kernel module %-16s found but not loaded", module_name);
       break;
    default:
-      rpt_vstring(depth, "Error %s from module_status_using_libkmod() for %s",
+      rpt_vstring(depth, "Error %s from module_status_by_modules_builtin_or_existence() for %s",
                   psc_desc(module_status), module_name);
       break;
    }
 }
 
 
-void query_loaded_modules_using_libkmod() {
+void query_loaded_modules() {
    rpt_vstring(0,"*** Checking if modules are loaded or builtin... ***");
 
    char ** pmodule_names = get_known_video_driver_module_names();
@@ -546,6 +550,7 @@ static void query_xdg_files(int depth) {
    rpt_nl();
 
    rpt_label(depth, "*** ddcutil Configuration, Cache, and Data files ***");
+
    char * config_fn = find_xdg_config_file("ddcutil", "ddcutilrc");
    rpt_nl();
    if (config_fn) {
@@ -557,6 +562,9 @@ static void query_xdg_files(int depth) {
       rpt_label(d1, "Configuration file ddcutilrc not found");
    rpt_nl();
 
+
+   probe_cache_files(depth);
+#ifdef OLD
    char * cache_fn = find_xdg_cache_file("ddcutil", "capabilities");
    if (cache_fn) {
       rpt_vstring(d1, "Found capabilities cache file: %s", cache_fn);
@@ -566,6 +574,7 @@ static void query_xdg_files(int depth) {
    else
       rpt_label(d1, "Capabilities cache file not found");
    rpt_nl();
+#endif
 
    rpt_label(d1, "Files on data path:");
    char * data_path = xdg_data_path();
@@ -734,19 +743,19 @@ void final_analysis(Env_Accumulator * accum, int depth) {
  *
  * Returns:      nothing
  */
-void query_sysenv() {
+void query_sysenv(bool quick_env) {
    bool debug = false;
    DBGTRC_STARTING(debug, TRACE_GROUP, "");
    rpt_label(0,
        "The following tests probe the runtime environment using multiple overlapping methods.");
 
    char * s = getenv("SYSENV_QUICK_TEST");
-   if (s && strlen(s) > 0) {
+   if ((s && strlen(s) > 0) || quick_env) {
       sysfs_quick_test = true;
-      rpt_label(0, "Environment variable SYSENV_QUICK_TEST is set.  Skipping some tests.");
+      rpt_label(0, "Environment variable SYSENV_QUICK_TEST or option --quickenv is set.  Skipping some tests.");
    }
    else if (get_output_level() >= DDCA_OL_VERBOSE) {
-      rpt_label(0, "Set environment variable SYSENV_QUICK_TEST to skip some long-running tests.");
+      rpt_label(0, "Set environment variable SYSENV_QUICK_TEST or option --quickenv to skip some long-running tests.");
    }
 
    i2c_forceable_slave_addr_flag = true;    // be a bully
@@ -822,119 +831,166 @@ void query_sysenv() {
 
    final_analysis(accumulator, 0);
 
+   // A quick hack to reduce the amount of output when testing
+   typedef enum {
+      Probe_Class_None        = 0,
+      Probe_Class_Detect      = 1,
+      Probe_Class_Drivers     = 2,
+      Probe_Class_Most        = 4,
+      Probe_Class_Logs        = 8,
+      Probe_Class_Sysfs       = 16,
+      Probe_Class_Libdrm      = 32,
+      Probe_Class_I2cdetect   = 64,
+      Probe_Class_Local_Files = 128,
+      Probe_Class_All         = 255
+   } Probe_Class;
+
+   Probe_Class probe_what = Probe_Class_All;
+
    if (output_level >= DDCA_OL_VERBOSE) {
       rpt_nl();
       rpt_nl();
       rpt_label(0, "*** Additional checks for remote diagnosis ***");
       rpt_nl();
-
-      rpt_vstring(0, "*** Displays as reported by DETECT Command ***");
-      /* int display_ct =  */ ddc_report_displays(     // function used by DETECT command
-                                 true,   // include_invalid_displays
-                                 1);     // logical depth
-      // printf("Detected: %d displays\n", display_ct);   // not needed
-
-      query_loaded_modules_using_libkmod();
+      rpt_label(0, "Disabling capabilities cache ...");
+      enable_capabilities_cache(false);
+#ifdef DISPLAYS_CACHE
+      rpt_label(0, "Disabling displays cache ...");
+      ddc_enable_displays_cache(false);
+#endif
+      rpt_label(0, "Disabling dynamic sleep ...");
+      dsa2_enable(false);
       rpt_nl();
 
-      // printf("Gathering card and driver information...\n");
-      query_proc_modules_for_video();
-      if (!accumulator->is_arm) {
-         // rpt_nl();
-         // query_card_and_driver_using_lspci();
-         //rpt_nl();
-         //query_card_and_driver_using_lspci_alt();
+
+      if (probe_what & Probe_Class_Detect) {
+         rpt_vstring(0, "*** Displays as reported by DETECT Command ***");
+         /* int display_ct =  */ ddc_report_displays(     // function used by DETECT command
+                                    true,   // include_invalid_displays
+                                    1);     // logical depth
+         // printf("Detected: %d displays\n", display_ct);   // not needed
       }
-      rpt_nl();
 
-      if ( driver_name_list_find_exact(accumulator->driver_list, "nvidia")) {
-         query_proc_driver_nvidia();
+      if (probe_what & Probe_Class_Drivers) {
+         query_loaded_modules();
          rpt_nl();
+
+         // printf("Gathering card and driver information...\n");
+         query_proc_modules_for_video();
+         if (!accumulator->is_arm) {
+            // rpt_nl();
+            // query_card_and_driver_using_lspci();
+            //rpt_nl();
+            //query_card_and_driver_using_lspci_alt();
+         }
+         rpt_nl();
+
+         if ( driver_name_list_find_exact(accumulator->driver_list, "nvidia")) {
+            query_proc_driver_nvidia();
+            rpt_nl();
+         }
+         if (driver_name_list_find_exact(accumulator->driver_list, "amdgpu")) {
+            rpt_vstring(0, "amdgpu configuration parameters:");
+            query_sys_amdgpu_parameters(1);
+            rpt_nl();
+         }
       }
-      if (driver_name_list_find_exact(accumulator->driver_list, "amdgpu")) {
-         rpt_vstring(0, "amdgpu configuration parameters:");
-         query_sys_amdgpu_parameters(1);
+
+      if (probe_what & Probe_Class_Most) {
+
+         rpt_vstring(0, "Checking display manager environment variables...");
+         char * s = getenv("DISPLAY");
+         rpt_vstring(1, "DISPLAY=%s", (s) ? s : "(not set)");
+         s = getenv("WAYLAND_DISPLAY");
+         rpt_vstring(1, "WAYLAND_DISPLAY=%s", (s) ? s : "(not set)");
+         s = getenv("XDG_SESSION_TYPE");
+         rpt_vstring(1, "XDG_SESSION_TYPE=%s", (s) ? s : "(not set)");
+         rpt_nl();
+
+         query_i2c_buses();
+         rpt_nl();
+
+         rpt_vstring(0,"xrandr connection report:");
+         execute_shell_cmd_rpt("xrandr|grep connected", 1 /* depth */);
+         rpt_nl();
+
+         rpt_vstring(0,"Checking for possibly conflicting programs...");
+         execute_shell_cmd_rpt("ps aux | grep ddccontrol | grep -v grep", 1);
+         rpt_nl();
+         execute_shell_cmd_rpt("lsmod | grep ddcci | grep -v grep", 1);
          rpt_nl();
       }
 
-      rpt_vstring(0, "Checking display manager environment variables...");
-      char * s = getenv("DISPLAY");
-      rpt_vstring(1, "DISPLAY=%s", (s) ? s : "(not set)");
-      s = getenv("WAYLAND_DISPLAY");
-      rpt_vstring(1, "WAYLAND_DISPLAY=%s", (s) ? s : "(not set)");
-      s = getenv("XDG_SESSION_TYPE");
-      rpt_vstring(1, "XDG_SESSION_TYPE=%s", (s) ? s : "(not set)");
-      rpt_nl();
+      if (probe_what & Probe_Class_I2cdetect) {
 
-      query_i2c_buses();
-      rpt_nl();
-
-      rpt_vstring(0,"xrandr connection report:");
-      execute_shell_cmd_rpt("xrandr|grep connected", 1 /* depth */);
-      rpt_nl();
-
-      rpt_vstring(0,"Checking for possibly conflicting programs...");
-      execute_shell_cmd_rpt("ps aux | grep ddccontrol | grep -v grep", 1);
-      rpt_nl();
-      execute_shell_cmd_rpt("lsmod | grep ddcci | grep -v grep", 1);
-      rpt_nl();
-
-      if (sysfs_quick_test)
-         DBGMSG("!!! Skipping i2cdetect and get-edid|parse-edid to speed up testing !!!");
-      else {
-         query_using_shell_command(accumulator->dev_i2c_device_numbers,
-                                   "i2cdetect -y %d",   // command to issue
-                                   "i2cdetect");        // command name for error message
-         rpt_nl();
-         query_using_shell_command(accumulator->dev_i2c_device_numbers,
-                                   "get-edid -b %d -i | parse-edid",   // command to issue
-                                   "get-edid | parse-edid");        // command name for error message
+         if (sysfs_quick_test)
+            DBGMSG("!!! Skipping i2cdetect and get-edid|parse-edid to speed up testing !!!");
+         else {
+            query_using_shell_command(accumulator->dev_i2c_device_numbers,
+                                      "i2cdetect -y %d",   // command to issue
+                                      "i2cdetect");        // command name for error message
+            rpt_nl();
+            query_using_shell_command(accumulator->dev_i2c_device_numbers,
+                                      "get-edid -b %d -i | parse-edid",   // command to issue
+                                      "get-edid | parse-edid");        // command name for error message
+         }
+      }
 
+      if (probe_what & Probe_Class_Most) {
          if (get_output_level() >= DDCA_OL_VV) {
             DBGTRC_NOPREFIX(debug, TRACE_GROUP, "--VV only output: test_read_variants()");
             test_edid_read_variants(accumulator);
          }
-      }
-      raw_scan_i2c_devices(accumulator);
+
+         raw_scan_i2c_devices(accumulator);
 
 #ifdef USE_X11
-      query_x11();
+         query_x11();
 #endif
 
 #ifdef ENABLE_UDEV
-      probe_i2c_devices_using_udev();
+         probe_i2c_devices_using_udev();
 #endif
 
-      // temp
-      // get_i2c_smbus_devices_using_udev();
+         // temp
+         // get_i2c_smbus_devices_using_udev();
 
-      if (sysfs_quick_test)
-         DBGMSG("!!! Skipping config file and log checking to speed up testing !!!");
-      else {
          probe_config_files(accumulator);
-         probe_logs(accumulator);
+
       }
 
+      if (probe_what & Probe_Class_Logs) {
+         if (sysfs_quick_test)
+            DBGMSG("!!! Skipping log checking to speed up testing !!!");
+         else {
+            probe_logs(accumulator);
+         }
+      }
+
+      if (probe_what & Probe_Class_Libdrm) {
 #ifdef USE_LIBDRM
-      probe_using_libdrm();
+         probe_using_libdrm();
 #else
-      rpt_vstring(0, "Not built with libdrm support.  Skipping DRM related checks");
+         rpt_vstring(0, "Not built with libdrm support.  Skipping DRM related checks");
 #endif
+      }
 
-      query_drm_using_sysfs();
-      rpt_nl();
+      if (probe_what & Probe_Class_Sysfs) {
+         query_drm_using_sysfs();
+         rpt_nl();
 
-      rpt_title("Query file system for i2c nodes under /sys/class/drm/card*...", 0);
-      execute_shell_cmd_rpt("ls -ld /sys/class/drm/card*/card*/i2c*", 1);
-      rpt_title("Query file system for i2c nodes under /sys/class/drm/card*/ddc/i2c-dev/...", 0);
-      execute_shell_cmd_rpt("ls -ld /sys/class/drm/card*/card*/ddc/i2c-dev/i2c*", 1);
+         rpt_title("Query file system for i2c nodes under /sys/class/drm/card*...", 0);
+         execute_shell_cmd_rpt("ls -ld /sys/class/drm/card*/card*/i2c*", 1);
+         rpt_title("Query file system for i2c nodes under /sys/class/drm/card*/ddc/i2c-dev/...", 0);
+         execute_shell_cmd_rpt("ls -ld /sys/class/drm/card*/card*/ddc/i2c-dev/i2c*", 1);
 
-      device_xref_report(0);
+         device_xref_report(0);
 
-      probe_modules_d(0);
+         probe_modules_d(0);
 
-      dump_sysfs_i2c();
-      rpt_nl();
+         dump_sysfs_i2c(accumulator);
+         rpt_nl();
+      }
 
 #ifdef OLD
       if (get_output_level() >= DDCA_OL_VV) {
@@ -959,21 +1015,20 @@ void query_sysenv() {
 #endif
 #endif
 
-      query_xdg_files(0);
-
+      if (probe_what & Probe_Class_Most) {
+         query_xdg_files(0);
+      }
       rpt_label(0, "*** environment command complete ***");
    }
-
    env_accumulator_free(accumulator);     // make Coverity happy
    DBGTRC_DONE(debug, TRACE_GROUP, "");
 }
 
 
-void init_sysenv() {
+void init_query_sysenv() {
    RTTI_ADD_FUNC(query_sysenv);
 #ifdef ENABLE_UDEV
    RTTI_ADD_FUNC(probe_i2c_devices_using_udev);
 #endif
-   init_query_sysfs();
 }
 
