@@ -3,7 +3,7 @@
  *  Module checks
  */
 
-// Copyright (C) 2014-2021 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2023 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 /** \cond */
@@ -46,23 +46,16 @@ void check_i2c_dev_module(Env_Accumulator * accum, int depth) {
    rpt_vstring(d0,"Checking for driver i2c_dev...");
    DDCA_Output_Level output_level = get_output_level();
 
-   // Eventually use only one test
    bool is_loaded = is_module_loaded_using_sysfs("i2c-dev");
    rpt_vstring(d1, "sysfs reports module i2c_dev is%s loaded.", (is_loaded) ? "" : " NOT");
 
-   bool is_loaded2 = is_module_loaded_using_libkmod("i2c-dev");
-   if (is_loaded2 != is_loaded) {
-      rpt_vstring(d1, "BUT libkmod reports module i2c_dev is%s loaded. !!!", (is_loaded) ? "" : " NOT");
-      rpt_vstring(d1, "REGARDING sysfs AS CORRECT !!!");
-   }
-
-
    bool is_builtin = false;
    bool loadable = false;
-   int module_status = module_status_using_libkmod("i2c-dev");
+   int module_status = module_status_by_modules_builtin_or_existence("i2c-dev");
    if (module_status < 0) {
       rpt_vstring(d1, "Unable to determine i2c-dev status.");
-      rpt_vstring(d1, "module_status_using_libkmod() returned %s", psc_desc(module_status));
+      rpt_vstring(d1, "module_status_by_modules_builtin_or_existence() returned %s",
+                      psc_desc(module_status));
       rpt_vstring(d1, "Treating i2c-dev as not builtin and not loadable!!!");
    }
    else if (module_status == 0) {
