@@ -427,6 +427,7 @@ void each_video_pci_device(
    free(device_class);
 
 bye:
+   DBGMSF(debug, "Done");
    return;
 }
 
@@ -489,6 +490,7 @@ void query_card_and_driver_using_sysfs(Env_Accumulator * accum) {
       // each entry in /sys/bus/pci/devices is a symbolic link
       dir_foreach(pci_devices_dir_name, /*fn_filter*/ NULL, each_video_pci_device, accum, 0);
    }
+   DBGMSF(debug, "Done");
 }
 
 // end query_card_and_driver_using_sysfs() section
@@ -783,7 +785,8 @@ void show_top_level_sys_entries(int depth) {
              "/proc/devices",
              names,
              false,     // ignore_case,
-             0);        // no limit
+             0,         // no limit
+             false);    // free strings
 
    for (int ndx = 0; ndx < filtered_proc_devices->len; ndx++) {
      rpt_label(depth+1, g_ptr_array_index(filtered_proc_devices, ndx));
@@ -839,7 +842,7 @@ void show_top_level_sys_entries(int depth) {
 
 /** Master function for dumping /sys directory
  */
-void dump_sysfs_i2c() {
+void dump_sysfs_i2c(Env_Accumulator * accum) {
    bool debug = false;
    DBGTRC_STARTING(debug, TRACE_GROUP, "");
    rpt_nl();
@@ -861,11 +864,25 @@ void dump_sysfs_i2c() {
 
    consolidated_i2c_sysfs_report(0);
 
+   if (accum->is_arm) {
+      rpt_label(0,"*** Extended dump of sysfs video devices for ARM architecture ***");
+      rpt_nl();
+      GPtrArray *  video_devices =   execute_shell_cmd_collect(
+            "find /sys/devices -name class | xargs grep -il 0x03 | xargs dirname | xargs ls -lR");
+      rpt_nl();
+
+      rpt_vstring(0, "Display devices: (class 0x03nnnn)");
+      for (int ndx = 0; ndx < video_devices->len; ndx++) {
+         char * dirname = g_ptr_array_index(video_devices, ndx);
+         rpt_vstring(2, "%s", dirname);
+      }
+   }
+
    DBGTRC_DONE(debug, TRACE_GROUP, "");
 }
 
 
-void init_query_sysfs() {
+void init_query_sysenv_sysfs() {
    RTTI_ADD_FUNC(query_drm_using_sysfs);
    RTTI_ADD_FUNC(query_sys_bus_i2c);
    RTTI_ADD_FUNC(each_i2c_device);
