@@ -10,7 +10,7 @@
  *  - abnormal termination
  */
 
-// Copyright (C) 2014-2022 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2024 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 #ifndef BASE_CORE_H_
@@ -27,19 +27,20 @@
 #include <stdbool.h>
 #include <stdio.h>
 #include <stdlib.h>
-#ifdef ENABLE_SYSLOG
 #include <syslog.h>
-#endif
 /** \endcond */
 
 #include "public/ddcutil_types.h"
 
 #include "util/coredefs.h"
 #include "util/error_info.h"
+#include "util/failsim.h"
 
 #include "base/parms.h"      // ensure available to any file that includes core.h
 #include "base/core_per_thread_settings.h"
 #include "base/linux_errno.h"
+#include "base/status_code_mgt.h"
+#include "base/trace_control.h"  // so don't need to repeatedly include trace_control.h
 
 //
 // Common macros
@@ -48,6 +49,16 @@
 #define ASSERT_MARKER(_struct_ptr, _marker_value) \
    assert(_struct_ptr && memcmp(_struct_ptr->marker, _marker_value, 4) == 0)
 
+// Remove static function qualifier to make it visible to asan, valgrind, backtrace
+#ifdef STATIC_FUNCTIONS_VISIBLE
+#define STATIC
+#else
+#define STATIC static
+#endif
+
+// Indicates that all tracing facilities have been configured
+extern bool tracing_initialized;
+
 
 //
 // Standard function call arguments and return values
@@ -83,52 +94,25 @@ typedef struct {
 // Trace message control
 //
 
-#ifdef ENABLE_SYSLOG
-#define SYSLOG(priority, format, ...) syslog(priority, format, ##__VA_ARGS__)
-#else
-#define SYSLOG(priority, format, ...) do {} while (0)
-#endif
-
+extern __thread  int  trace_api_call_depth;
+extern __thread  unsigned int  trace_callstack_call_depth;
+// extern __thread  char * trace_callstack[100];
 
 extern bool dbgtrc_show_time;       // prefix debug/trace messages with elapsed time
 extern bool dbgtrc_show_wall_time;  // prefix debug/trace messages with wall time
 extern bool dbgtrc_show_thread_id;  // prefix debug/trace messages with thread id
+extern bool dbgtrc_show_process_id; // prefix debug/trace messsages with process id
+extern bool dbgtrc_trace_to_syslog_only;
 
-void set_libddcutil_output_destination(const char * filename, const char * traced_unit);
-void add_traced_function(const char * funcname);
-bool is_traced_function( const char * funcname);
-
-void add_traced_file(const char * filename);
-bool is_traced_file( const char * filename);
-
-DDCA_Trace_Group trace_class_name_to_value(char * name);
-void set_trace_groups(DDCA_Trace_Group trace_flags);
-void add_trace_groups(DDCA_Trace_Group trace_flags);
-// char * get_active_trace_group_names();  // unimplemented
-
-void report_tracing(int depth);
-
-bool is_tracing(DDCA_Trace_Group trace_group, const char * filename, const char * funcname);
-
-/** Checks if tracking is currently active for the globally defined TRACE_GROUP value,
- *  current file and function.
- *
- *  Wrappers call to **is_tracing()**, using the current **TRACE_GROUP** value,
- *  filename, and function as implicit arguments.
- */
-#define IS_TRACING() is_tracing(TRACE_GROUP, __FILE__, __func__)
-
-#define IS_TRACING_GROUP(grp) is_tracing((grp), __FILE__, __func__)
-
-#define IS_TRACING_BY_FUNC_OR_FILE() is_tracing(DDCA_TRC_NONE, __FILE__, __func__)
-
-#define IS_DBGTRC(debug_flag, group) \
-    ( (debug_flag)  || is_tracing((group), __FILE__, __func__) )
+// void set_libddcutil_output_destination(const char * filename, const char * traced_unit);
 
 typedef uint16_t Dbgtrc_Options;
-#define DBGTRC_OPTIONS_NONE   0
-#define DBGTRC_OPTIONS_SYSLOG 1
-#define DBGTRC_OPTIONS_SEVERE 2
+#define DBGTRC_OPTIONS_NONE      0x00
+#define DBGTRC_OPTIONS_SYSLOG    0x01
+#define DBGTRC_OPTIONS_SEVERE    0x02
+#define DBGTRC_OPTIONS_API_CALL  0x04   // used for tracing API
+#define DBGTRC_OPTIONS_STARTING  0x08
+#define DBGTRC_OPTIONS_DONE      0x10
 
 
 //
@@ -174,7 +158,6 @@ bool ddcmsg(
    ddcmsg(( (debug_flag) ) ? 0xff : (trace_group), \
             __func__, __LINE__, __FILE__, format, ##__VA_ARGS__)
 
-
 /** Macro that wrappers function ddcmsg(), passing the current TRACE_GROUP,
  *  file name, line number, and function name as arguments.
  *
@@ -186,6 +169,16 @@ bool ddcmsg(
    ddcmsg(( (debug_flag) ) ? 0xff : (TRACE_GROUP), \
             __func__, __LINE__, __FILE__, format, ##__VA_ARGS__)
 
+bool logable_msg(DDCA_Syslog_Level log_level,
+            const char * funcname,
+            const int    lineno,
+            const char * filename,
+            char *       format,
+            ...);
+
+#define LOGABLE_MSG(importance, format, ...) \
+   logable_msg(importance, __func__, __LINE__, __FILE__, format, ##__VA_ARGS__)
+
 
 // Show report levels for all types
 void show_reporting();
@@ -198,19 +191,6 @@ void show_ddcutil_version();
 // Issue messages of various types
 //
 
-#ifdef OLD
-void severemsg(
-        const char * funcname,
-        const int    lineno,
-        const char * fn,
-        char *       format,
-        ...);
-#endif
-
-#ifdef ENABLE_SYSLOG
-extern bool trace_to_syslog;
-#endif
-
 bool dbgtrc(
         DDCA_Trace_Group trace_group,
         Dbgtrc_Options   options,
@@ -230,6 +210,19 @@ bool dbgtrc_ret_ddcrc(
         char *           format,
         ...);
 
+#ifdef UNNECESSARY
+// use dbgtrc_returning_expression()
+bool dbgtrc_ret_bool(
+        DDCA_Trace_Group trace_group,
+        Dbgtrc_Options   options,
+        const char *     funcname,
+        const int        lineno,
+        const char *     fn,
+        bool             result,
+        char *           format,
+        ...);
+#endif
+
 bool dbgtrc_returning_errinfo(
         DDCA_Trace_Group trace_group,
         Dbgtrc_Options   options,
@@ -250,7 +243,6 @@ bool dbgtrc_returning_expression(
         char *           format,
         ...);
 
-
 /* __assert_fail() is not part of the C spec, it is part of the Linux
  * implementation of assert(), etc., which are macros.
  * It is reported to not exist on Termux.
@@ -273,7 +265,7 @@ bool dbgtrc_returning_expression(
          dbgtrc(DDCA_TRC_ALL, DBGTRC_OPTIONS_NONE, __func__, __LINE__, __FILE__,   \
                       "Assertion failed: \"%s\" in file %s at line %d",  \
                       #_assertion, __FILE__,  __LINE__);   \
-         SYSLOG(LOG_ERR, "Assertion failed: \"%s\" in file %s at line %d",  \
+         SYSLOG2(DDCA_SYSLOG_ERROR, "Assertion failed: \"%s\" in file %s at line %d",  \
                          #_assertion, __FILE__,  __LINE__);   \
          /* assert(#_assertion); */ \
          /*  __assert_fail(#_assertion, __FILE__, line, __func__); */  \
@@ -283,7 +275,6 @@ bool dbgtrc_returning_expression(
    } while (0)
 #endif
 
-
 #ifndef TRACED_ASSERT_IFF
 #define TRACED_ASSERT_IFF(_cond1, _cond2) \
    TRACED_ASSERT( ( (_cond1) && (_cond2) ) || ( !(_cond1) && !(_cond2) ) )
@@ -302,7 +293,6 @@ bool dbgtrc_returning_expression(
    } while (0)
 #endif
 
-
 #define SEVEREMSG(format, ...) \
    dbgtrc(DDCA_TRC_ALL, DBGTRC_OPTIONS_SEVERE, \
           __func__, __LINE__, __FILE__, format, ##__VA_ARGS__)
@@ -328,80 +318,184 @@ bool dbgtrc_returning_expression(
 #endif
 
 #define DBGTRC_STARTING(debug_flag, trace_group, format, ...) \
-    dbgtrc( (debug_flag) ? DDCA_TRC_ALL : (trace_group), DBGTRC_OPTIONS_NONE, \
+    dbgtrc( (debug_flag || trace_callstack_call_depth > 0 || is_traced_callstack_call(__func__) ) ? DDCA_TRC_ALL : (trace_group), DBGTRC_OPTIONS_STARTING, \
             __func__, __LINE__, __FILE__, "Starting  "format, ##__VA_ARGS__)
 
-#define DBGTRC_DONE(debug_flag, trace_group, format, ...) \
-    dbgtrc( (debug_flag) ? DDCA_TRC_ALL : (trace_group), DBGTRC_OPTIONS_NONE, \
+#define DBGTRC_DONE(debug_flag , trace_group, format, ...) \
+    dbgtrc( (debug_flag) || trace_callstack_call_depth > 0  ? DDCA_TRC_ALL : (trace_group), DBGTRC_OPTIONS_DONE, \
             __func__, __LINE__, __FILE__, "Done      "format, ##__VA_ARGS__)
 
+#define DBGTRC_EXECUTED(debug_flag, trace_group, format, ...) \
+    dbgtrc( (debug_flag) || trace_callstack_call_depth > 0  ? DDCA_TRC_ALL : (trace_group), DBGTRC_OPTIONS_STARTING | DBGTRC_OPTIONS_DONE, \
+            __func__, __LINE__, __FILE__, "Executed  "format, ##__VA_ARGS__)
+
 #define DBGTRC_NOPREFIX(debug_flag, trace_group, format, ...) \
-    dbgtrc( (debug_flag) ? DDCA_TRC_ALL : (trace_group), DBGTRC_OPTIONS_NONE, \
+    dbgtrc( (debug_flag) || trace_callstack_call_depth > 0  ? DDCA_TRC_ALL : (trace_group), DBGTRC_OPTIONS_NONE, \
             __func__, __LINE__, __FILE__, "          "format, ##__VA_ARGS__)
 
+#define DBGTRC_RETURNING(debug_flag, trace_group, _result, format, ...) \
+    dbgtrc_returning_expression( \
+          (debug_flag) || trace_callstack_call_depth > 0  ? DDCA_TRC_ALL : (trace_group), \
+          DBGTRC_OPTIONS_DONE, \
+          __func__, __LINE__, __FILE__, _result, format, ##__VA_ARGS__)
+
+/* Notes on macros that have ENABLE_FAILSIM variants.
+ *
+ * Care must be taken with the DBGTRC_ macros that have ENABLE_FAILSIM variants.
+ *
+ * - The macros are passed the name of a variable that may be modified by a
+ *   failure simulation function. Therefore, what is specified in the return
+ *   value field of these macros must be a simple variable that can be the
+ *   lvalue of an assignment, not an expression or a constant.
+ *
+ * - DBGTRC__2() variants of the macros specify an lvalue expression that is to be
+ *   set to NULL if an error is injected.  The supports the common case
+ *   where the return code is 0 or NULL iff the lvalue is non-null.
+ */
+#ifdef ENABLE_FAILSIM
 #define DBGTRC_RET_DDCRC(debug_flag, trace_group, rc, format, ...) \
-    dbgtrc_ret_ddcrc( \
-          (debug_flag) ? DDCA_TRC_ALL : (trace_group), DBGTRC_OPTIONS_NONE, \
-          __func__, __LINE__, __FILE__, rc, format, ##__VA_ARGS__)
+   do { \
+      if (failsim_enabled && rc != 0) { \
+         int injected = fsim_int_injector(rc, __FILE__, __func__); \
+         if (injected) { \
+            rc = injected; \
+            printf("(%s) failsim: injected error %s\n", __func__, psc_desc(rc)); \
+         } \
+      } \
+      dbgtrc_ret_ddcrc( \
+         (debug_flag) || trace_callstack_call_depth > 0  ? DDCA_TRC_ALL : (trace_group), DBGTRC_OPTIONS_DONE, \
+         __func__, __LINE__, __FILE__, rc, format, ##__VA_ARGS__); \
+  } while (0)
+#define DBGTRC_RET_DDCRC2(debug_flag, trace_group, _rc, _data_pointer, format, ...) \
+   do { \
+      if (failsim_enabled && _rc == 0) { \
+         int injected = fsim_int_injector(_rc, __FILE__, __func__); \
+         if (injected) { \
+            _rc = injected; \
+            printf("(%s) failsim: injected error %s, setting %s = NULL\n", __func__, psc_desc(_rc), #_data_pointer); \
+            if (_data_pointer) { \
+               free(_data_pointer); \
+               _data_pointer = NULL; \
+            } \
+         } \
+      } \
+      dbgtrc_ret_ddcrc( \
+         (debug_flag) || trace_callstack_call_depth > 0  ? DDCA_TRC_ALL : (trace_group), DBGTRC_OPTIONS_DONE, \
+         __func__, __LINE__, __FILE__, _rc, format, ##__VA_ARGS__); \
+  } while (0)
+#else
+#define DBGTRC_RET_DDCRC(debug_flag, trace_group, rc, format, ...) \
+   dbgtrc_ret_ddcrc( \
+      (debug_flag) || trace_callstack_call_depth > 0  ? DDCA_TRC_ALL : (trace_group), DBGTRC_OPTIONS_DONE, \
+      __func__, __LINE__, __FILE__, rc, format, ##__VA_ARGS__)
+#define DBGTRC_RET_DDCRC2(debug_flag, trace_group, rc, data_pointer, format, ...) \
+   dbgtrc_ret_ddcrc( \
+      (debug_flag) || trace_callstack_call_depth > 0  ? DDCA_TRC_ALL : (trace_group), DBGTRC_OPTIONS_DONE, \
+      __func__, __LINE__, __FILE__, rc, format, ##__VA_ARGS__)
+#endif
+
+#ifdef ENABLE_FAILSIM
+#define DBGTRC_RET_ERRINFO(debug_flag, trace_group, errinfo_result, format, ...) \
+   do { \
+      if (failsim_enabled && !errinfo_result) { \
+         Error_Info * injected = fsim_errinfo_injector(errinfo_result, __FILE__, __func__); \
+         if (injected) { \
+            errinfo_result = injected; \
+            printf("(%s) Injected error %s\n", __func__, errinfo_summary(injected)); \
+         } \
+      } \
+      dbgtrc_returning_errinfo( \
+          (debug_flag) || trace_callstack_call_depth > 0  ? DDCA_TRC_ALL : (trace_group), DBGTRC_OPTIONS_DONE, \
+          __func__, __LINE__, __FILE__, errinfo_result, format, ##__VA_ARGS__); \
+   } while(0)
+#define DBGTRC_RET_ERRINFO2(debug_flag, trace_group, errinfo_result, data_pointer, format, ...) \
+   do { \
+      if (failsim_enabled && !errinfo_result) { \
+         Error_Info * injected = fsim_errinfo_injector(errinfo_result, __FILE__, __func__); \
+         if (injected) { \
+            errinfo_result = injected; \
+            if (data_pointer) { \
+               free(data_pointer); \
+               data_pointer = NULL; \
+            } \
+            printf("(%s) Injected error %s, setting %s = NULL\n", __func__, errinfo_summary(injected), #data_pointer); \
+         } \
+      } \
+      dbgtrc_returning_errinfo( \
+          (debug_flag) || trace_callstack_call_depth > 0  ? DDCA_TRC_ALL : (trace_group), DBGTRC_OPTIONS_DONE, \
+          __func__, __LINE__, __FILE__, errinfo_result, format, ##__VA_ARGS__); \
+   } while(0)
+#else
+#define DBGTRC_RET_ERRINFO(debug_flag, trace_group, errinfo_result, format, ...) \
+       dbgtrc_returning_errinfo( \
+             (debug_flag) || trace_callstack_call_depth > 0  ? DDCA_TRC_ALL : (trace_group), DBGTRC_OPTIONS_DONE, \
+             __func__, __LINE__, __FILE__, errinfo_result, format, ##__VA_ARGS__)
+#define DBGTRC_RET_ERRINFO2(debug_flag, trace_group, errinfo_result, pointer, format, ...) \
+       dbgtrc_returning_errinfo( \
+             (debug_flag) || trace_callstack_call_depth > 0  ? DDCA_TRC_ALL : (trace_group), DBGTRC_OPTIONS_DONE, \
+             __func__, __LINE__, __FILE__, errinfo_result, format, ##__VA_ARGS__)
+#endif
+
 
 #define DBGTRC_RET_BOOL(debug_flag, trace_group, bool_result, format, ...) \
     dbgtrc_returning_expression( \
-          (debug_flag) ? DDCA_TRC_ALL : (trace_group), \
-          DBGTRC_OPTIONS_NONE, \
+          (debug_flag) || trace_callstack_call_depth > 0  ? DDCA_TRC_ALL : (trace_group), \
+          DBGTRC_OPTIONS_DONE, \
           __func__, __LINE__, __FILE__, SBOOL(bool_result), format, ##__VA_ARGS__)
 
-#define DBGTRC_RET_ERRINFO(debug_flag, trace_group, errinfo_result, format, ...) \
-    dbgtrc_returning_errinfo( \
-          (debug_flag) ? DDCA_TRC_ALL : (trace_group), DBGTRC_OPTIONS_NONE, \
-          __func__, __LINE__, __FILE__, errinfo_result, format, ##__VA_ARGS__)
-
 // typedef (*dbg_struct_func)(void * structptr, int depth);
 #define DBGMSF_RET_STRUCT(_flag, _structname, _dbgfunc, _structptr) \
-if (_flag) { \
-   dbgtrc(DDCA_TRC_ALL, DBGTRC_OPTIONS_NONE, \
-         __func__, __LINE__, __FILE__, "Returning %s at %p", #_structname, _structptr); \
-   if (_structptr) { \
-      _dbgfunc(_structptr, 1); \
+do { \
+   if ((_flag) || trace_callstack_call_depth > 0)  { \
+      dbgtrc(DDCA_TRC_ALL, DBGTRC_OPTIONS_DONE, \
+            __func__, __LINE__, __FILE__, "Returning %s at %p", #_structname, _structptr); \
+      if (_structptr) { \
+         _dbgfunc(_structptr, 1); \
+      } \
    } \
-}
+} while (0)
 
 #define DBGTRC_RET_STRUCT(_flag, _trace_group, _structname, _dbgfunc, _structptr) \
-if ( (_flag) || is_tracing(_trace_group, __FILE__, __func__) )  { \
-   dbgtrc(DDCA_TRC_ALL, DBGTRC_OPTIONS_NONE, \
-          __func__, __LINE__, __FILE__, \
-          "Returning %s at %p", #_structname, _structptr); \
-   if (_structptr) { \
-      _dbgfunc(_structptr, 1); \
+do { \
+   if ( (_flag)  || trace_callstack_call_depth > 0 || is_tracing(_trace_group, __FILE__, __func__) )  { \
+      dbgtrc(DDCA_TRC_ALL, DBGTRC_OPTIONS_DONE, \
+             __func__, __LINE__, __FILE__, \
+             "Returning %s at %p", #_structname, _structptr); \
+      if (_structptr) { \
+         _dbgfunc(_structptr, 1); \
+      } \
    } \
-}
+} while(0)
+
+#define DBGTRC_RET_STRUCT_VALUE(_flag, _trace_group, _structname, _dbgfunc, _structval) \
+do { \
+   if ( (_flag)  || trace_callstack_call_depth > 0 || is_tracing(_trace_group, __FILE__, __func__) )  { \
+      dbgtrc(DDCA_TRC_ALL, DBGTRC_OPTIONS_DONE, \
+             __func__, __LINE__, __FILE__, \
+             "Returning %s value:", #_structname); \
+      _dbgfunc(_structval, 2); \
+   } \
+} while(0)
+
 
 #define DBGTRC_RET_ERRINFO_STRUCT(_debug_flag, _trace_group, _errinfo_result, \
                                   _structptr_loc, _dbgfunc)                   \
-   if ( (_debug_flag) || is_tracing(_trace_group, __FILE__, __func__) )  {    \
-      dbgtrc_returning_errinfo(DDCA_TRC_ALL, DBGTRC_OPTIONS_NONE,             \
+do { \
+   if ( (_debug_flag || trace_callstack_call_depth > 0 ) || is_tracing(_trace_group, __FILE__, __func__) )  {    \
+      dbgtrc_returning_errinfo(DDCA_TRC_ALL, DBGTRC_OPTIONS_DONE,             \
               __func__, __LINE__, __FILE__,                                   \
               _errinfo_result, "*%s = %p", #_structptr_loc, *_structptr_loc); \
       if (*_structptr_loc) {                                                  \
          _dbgfunc(*_structptr_loc, 1);                                        \
       }                                                                       \
-   }
+   } \
+} while(0)
+
 
 //
 // Error handling
 //
 
-#ifdef OLD
-void report_ioctl_error(
-      const char * ioctl_name,
-      int          errnum,
-      const char * funcname,
-      const char * filename,
-      int          lineno);
-
-#define REPORT_IOCTL_ERROR(_ioctl_name, _errnum) \
-   report_ioctl_error(_ioctl_name, _errnum, __func__, __FILE__, __LINE__);
-#endif
-
 #define REPORT_IOCTL_ERROR(_ioctl_name, _errnum) \
       dbgtrc(DDCA_TRC_ALL, DBGTRC_OPTIONS_SEVERE, __func__, __LINE__, __FILE__, \
              "Error in ioctl(%s), %s", _ioctl_name, linux_errno_desc(_errnum));
@@ -424,6 +518,101 @@ void program_logic_error(
    program_logic_error(__func__, __LINE__, __FILE__, format, ##__VA_ARGS__)
 
 void set_default_thread_output_settings(FILE * fout, FILE * ferr);
+
+#ifdef UNUSED
+void core_errmsg_emitter(
+      GPtrArray*   errmsgs,
+      GPtrArray *  errinfo_accum,
+      bool         verbose,
+      int          rc,
+      const char * func,
+      const char * msg, ...);
+#endif
+
+
+//
+// Use of system log
+//
+
+extern bool enable_syslog;
+extern DDCA_Syslog_Level syslog_level;
+
+DDCA_Syslog_Level   syslog_level_name_to_value(const char * name);
+const char *        syslog_level_name(DDCA_Syslog_Level level);
+bool                test_emit_syslog(DDCA_Syslog_Level msg_level);
+int                 syslog_importance_from_ddcutil_syslog_level(DDCA_Syslog_Level level);
+extern const char * valid_syslog_levels_string;
+
+
+/** The specified ddcutil severity level converted to a syslog priority and
+ *  written to the system log.
+ *
+ *  @param _ddcutil_severity   e.g. DDCA_SYSLOG_ERROR
+ *  @param  fmt                message format
+ *  @param  ...                message arguments
+ *
+ *  Messages are written to the system log with the syslog priority
+ *  corresponding to the ddcutil severity.
+ */
+#define SYSLOG2(_ddcutil_severity, format, ...) \
+do { \
+   if (test_emit_syslog(_ddcutil_severity)) { \
+      int syslog_priority = syslog_importance_from_ddcutil_syslog_level(_ddcutil_severity);  \
+      if (syslog_priority >= 0) { \
+         syslog(syslog_priority, format, ##__VA_ARGS__); \
+      } \
+   } \
+} while(0)
+
+
+/** Writes a message to the current ferr() or fout() device and, depending on
+ *  the specified ddcutil severity and current syslog level, to the system log.
+ *
+ *  @param _ddcutil_severity   e.g. DDCA_SYSLOG_ERROR
+ *  @param  fmt                message format
+ *  @param  ...                message arguments
+ *
+ *  Messages with ddcutil severity DDCA_SYSLOG_WARNING or more severe are
+ *  written to the ferr() device.  Others are written to the fout() device.
+ *
+ *  Messages are written to the system log with the syslog priority
+ *  corresponding to the ddcutil severity.
+ */
+#define MSG_W_SYSLOG(_ddcutil_severity, format, ...) \
+do { \
+   FILE * f = (_ddcutil_severity <= DDCA_SYSLOG_WARNING) ? ferr() : fout(); \
+   fprintf(f, format, ##__VA_ARGS__); \
+   fprintf(f, "\n"); \
+   if (test_emit_syslog(_ddcutil_severity)) { \
+      int syslog_priority = syslog_importance_from_ddcutil_syslog_level(_ddcutil_severity);  \
+      if (syslog_priority >= 0) { \
+         syslog(syslog_priority, format, ##__VA_ARGS__); \
+      } \
+   } \
+} while(0)
+
+
+void base_errinfo_free_with_report(
+      Error_Info *  erec,
+      bool          report,
+      const char *  func);
+
+#define BASE_ERRINFO_FREE_WITH_REPORT(_erec, _report) \
+   base_errinfo_free_with_report(_erec, (_report), __func__)
+
+
+//
+// Output capture
+//
+
+void   start_capture(DDCA_Capture_Option_Flags flags);
+char * end_capture(void);
+
+
+//
+// Initialization
+//
+
 void init_core();
 
 #endif /* BASE_CORE_H_ */
