@@ -1,8 +1,9 @@
-/** \file ddc_packets.c
+/** @file ddc_packets.c
+ *
  * Functions for creating DDC packets and interpreting DDC response packets.
  */
 
-// Copyright (C) 2014-2022 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2023 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 /** \cond */
@@ -257,7 +258,7 @@ void free_ddc_packet(DDC_Packet * packet) {
 DDC_Packet *
 create_empty_ddc_packet(int max_size, const char * tag) {
    bool debug = false;
-   DBGMSF(debug, "Starting. max_size=%d, tag=%s", max_size, (tag) ? tag : "(nil)");
+   DBGTRC_STARTING(debug, TRACE_GROUP, "max_size=%d, tag=%s", max_size, (tag) ? tag : "(nil)");
 
    DDC_Packet * packet = malloc(sizeof(DDC_Packet));
    packet->raw_bytes = buffer_new(max_size, "empty DDC packet");
@@ -271,10 +272,7 @@ create_empty_ddc_packet(int max_size, const char * tag) {
    packet->type = DDC_PACKET_TYPE_NONE;
    packet->parsed.raw_parsed = NULL;
 
-   DBGMSF(debug, "Done. Returning %p, packet->tag=%p", packet, packet->tag);
-   if (debug)
-      dbgrpt_packet(packet, 2);
-
+   DBGTRC_RET_STRUCT(debug, TRACE_GROUP, DDC_Packet, dbgrpt_packet, packet);
    return packet;
 }
 
@@ -292,18 +290,20 @@ create_empty_ddc_packet(int max_size, const char * tag) {
  */
 DDC_Packet *
 create_ddc_base_request_packet(
+      Byte         source_addr,
       Byte *       data_bytes,
       int          data_bytect,
       const char*  tag)
 {
    bool debug = false;
-   DBGMSF(debug, "Starting.  bytes=%s, tag=%s", hexstring_t(data_bytes,data_bytect), tag);
+   DBGTRC_STARTING(debug, TRACE_GROUP, "source_addr=0x%02x, data_bytes=%s, tag=%s",
+                                       source_addr, hexstring_t(data_bytes,data_bytect), tag);
 
    assert( data_bytect <= 32 );
 
    DDC_Packet * packet = create_empty_ddc_packet(3+data_bytect+1, tag);
    buffer_set_byte( packet->raw_bytes, 0, 0x6e);   // x37<<1 + 0   destination address, write
-   buffer_set_byte( packet->raw_bytes, 1, 0x51);   // x28<<1 + 1   source address
+   buffer_set_byte( packet->raw_bytes, 1, source_addr);   // x28<<1 + 1   source address
    buffer_set_byte( packet->raw_bytes, 2, data_bytect | 0x80);
    buffer_set_bytes(packet->raw_bytes, 3, data_bytes, data_bytect);
    int packet_size_wo_checksum = 3 + data_bytect;
@@ -316,7 +316,7 @@ create_ddc_base_request_packet(
       packet->type = 0x00;
    // dump_buffer(packet->buf);
 
-   DBGMSF(debug, "Done. packet=%p", packet);
+   DBGTRC_RET_STRUCT(debug, TRACE_GROUP, "DDC_Packet", dbgrpt_packet, packet);
    return packet;
 }
 
@@ -337,10 +337,14 @@ create_ddc_multi_part_read_request_packet(
       int          offset,
       const char*  tag)
 {
+   bool debug = false;
+   DBGTRC_STARTING(debug, TRACE_GROUP,
+         "request_type=0x%02x, request_subtype=0x%02x, offset=%d, tag=%s",
+         request_type, request_subtype, offset, tag);
    assert (request_type == DDC_PACKET_TYPE_CAPABILITIES_REQUEST ||
            request_type == DDC_PACKET_TYPE_TABLE_READ_REQUEST );
-   DDC_Packet * packet_ptr = NULL;
 
+   DDC_Packet * packet_ptr = NULL;
    Byte ofs_hi_byte = (offset >> 16) & 0xff;
    Byte ofs_lo_byte = offset & 0xff;
 
@@ -349,7 +353,7 @@ create_ddc_multi_part_read_request_packet(
                               ofs_hi_byte,
                               ofs_lo_byte
                             };
-        packet_ptr = create_ddc_base_request_packet(data_bytes, 3, tag);
+        packet_ptr = create_ddc_base_request_packet(0x51, data_bytes, 3, tag);
    }
    else {
       Byte data_bytes[] = { DDC_PACKET_TYPE_TABLE_READ_REQUEST,
@@ -357,11 +361,12 @@ create_ddc_multi_part_read_request_packet(
                             ofs_hi_byte,
                             ofs_lo_byte
                           };
-      packet_ptr = create_ddc_base_request_packet(data_bytes, 4, tag);
+      packet_ptr = create_ddc_base_request_packet(0x51, data_bytes, 4, tag);
    }
 
    // DBGMSG("Done. packet_ptr=%p", packet_ptr);
    // dump_packet(packet_ptr);
+   DBGTRC_RET_STRUCT(debug, TRACE_GROUP, "DDC_Packet", dbgrpt_packet, packet_ptr);
    return packet_ptr;
 }
 
@@ -376,6 +381,8 @@ update_ddc_multi_part_read_request_packet_offset(
       DDC_Packet * packet,
       int          new_offset)
 {
+   bool debug = false;
+   DBGTRC_STARTING(debug, TRACE_GROUP, "packet=%p, new_offset=%d", packet, new_offset);
    assert (packet->type == DDC_PACKET_TYPE_CAPABILITIES_REQUEST ||
            packet->type == DDC_PACKET_TYPE_TABLE_READ_REQUEST );
 
@@ -402,6 +409,10 @@ update_ddc_multi_part_read_request_packet_offset(
 
    // DBGMSG("Done.");
    // dump_packet(packet);
+   if (IS_DBGTRC(debug, TRACE_GROUP)) {
+      DBGTRC_DONE(true, TRACE_GROUP, "");
+      dbgrpt_packet(packet,  2);
+   }
 }
 
 
@@ -424,28 +435,35 @@ create_ddc_multi_part_write_request_packet(
                 int    bytect,
                 const char * tag)
 {
+   bool debug = false;
+   DBGTRC_STARTING(debug, TRACE_GROUP,
+         "request_type=0x%02x, request_subtype=0x%02x, offset=%d, bytect=%d, bytes_to_write=%p->%s",
+         request_type, request_subtype, offset, bytect, bytes_to_write,
+         hexstring_t(bytes_to_write, bytect));
    assert (request_type ==  DDC_PACKET_TYPE_TABLE_WRITE_REQUEST );
    assert (bytect + 4 <= 35);    // is this the right limit?, spec unclear
-   DDC_Packet * packet_ptr = NULL;
 
+   DDC_Packet * packet_ptr = NULL;
    Byte ofs_hi_byte = (offset >> 16) & 0xff;
    Byte ofs_lo_byte = offset & 0xff;
-
-
    Byte data_bytes[40] = { DDC_PACKET_TYPE_TABLE_WRITE_REQUEST,
                            request_subtype,    // VCP code
                            ofs_hi_byte,
                            ofs_lo_byte
                          };
    memcpy(data_bytes+4, bytes_to_write, bytect);
-   packet_ptr = create_ddc_base_request_packet(data_bytes, 4+bytect, tag);
+   packet_ptr = create_ddc_base_request_packet(0x51, data_bytes, 4+bytect, tag);
 
    // DBGMSG("Done. packet_ptr=%p", packet_ptr);
    // dump_packet(packet_ptr);
+   DBGTRC_RET_STRUCT(debug, TRACE_GROUP, "DDC_Packet", dbgrpt_packet, packet_ptr);
    return packet_ptr;
 }
 
 
+Byte alt_source_addr = 0x00;
+
+
 /** Creates a Get VCP request packet
  *
  *  \param  vcp_code  VCP feature code
@@ -455,12 +473,17 @@ create_ddc_multi_part_write_request_packet(
 DDC_Packet *
 create_ddc_getvcp_request_packet(Byte vcp_code, const char * tag)
 {
-   Byte data_bytes[] = { DDC_PACKET_TYPE_QUERY_VCP_REQUEST,  // 0x01
+   bool debug = false;
+   DBGTRC_STARTING(debug, TRACE_GROUP, "vcp_code = 0x%02x, tag = %s", vcp_code, tag);
+   Byte cmd_code = DDC_PACKET_TYPE_QUERY_VCP_REQUEST;   // 0x01
+   // if (alt_cmd_code)
+   //   cmd_code = alt_cmd_code;
+   Byte data_bytes[] = { cmd_code,
                          vcp_code  // VCP opcode
                        };
-   DDC_Packet * pkt = create_ddc_base_request_packet(data_bytes, 2, tag);
+   DDC_Packet * pkt = create_ddc_base_request_packet(0x51, data_bytes, 2, tag);
 
-   // DBGMSG("Done. rc=%d, packet_ptr=%p, *packet_ptr=%p", rc, packet_ptr, *packet_ptr);
+   DBGTRC_RET_STRUCT(debug, TRACE_GROUP, "DDC_Packet",dbgrpt_packet,pkt);
    return pkt;
 }
 
@@ -475,14 +498,21 @@ create_ddc_getvcp_request_packet(Byte vcp_code, const char * tag)
 DDC_Packet *
 create_ddc_setvcp_request_packet(Byte vcp_code, int new_value, const char * tag)
 {
-   Byte data_bytes[] = { DDC_PACKET_TYPE_SET_VCP_REQUEST,   // 0x03
+   bool debug = false;
+   DBGTRC_STARTING(debug, TRACE_GROUP, "vcp_code=0x%02x, new_value=%d, tag=%s",
+                          vcp_code, new_value, tag);
+   Byte cmd_code = DDC_PACKET_TYPE_SET_VCP_REQUEST;   // 0x03
+   Byte source_addr = 0x51;
+   if (alt_source_addr)
+     source_addr = alt_source_addr;
+   Byte data_bytes[] = { cmd_code,
                          vcp_code,  // VCP opcode
                          (new_value >> 8) & 0xff,
                          new_value & 0xff
                        };
-   DDC_Packet * pkt = create_ddc_base_request_packet(data_bytes, 4, tag);
+   DDC_Packet * pkt = create_ddc_base_request_packet(source_addr, data_bytes, 4, tag);
 
-   // DBGMSG("Done. rc=%d, packet_ptr%p, *packet_ptr=%p", rc, packet_ptr, *packet_ptr);
+   DBGTRC_RET_STRUCT(debug, TRACE_GROUP, "DDC_Packet",dbgrpt_packet,pkt);
    return pkt;
 }
 
@@ -497,7 +527,7 @@ create_ddc_save_settings_request_packet(const char * tag)
 {
    Byte data_bytes[] = { DDC_PACKET_TYPE_SAVE_CURRENT_SETTINGS   // 0x0C
                        };
-   DDC_Packet * pkt = create_ddc_base_request_packet(data_bytes, 1, tag);
+   DDC_Packet * pkt = create_ddc_base_request_packet(0x51, data_bytes, 1, tag);
 
    // DBGMSG("Done. rc=%d, packet_ptr=%p, *packet_ptr=%p", rc, packet_ptr, *packet_ptr);
    return pkt;
@@ -534,10 +564,10 @@ create_ddc_base_response_packet(
    DDC_Packet ** packet_ptr_loc)
 {
    bool debug = false;
-   DBGTRC_STARTING(debug, TRACE_GROUP, "i2c_response_bytes=%p, response_bytes_buffer_size=%d",
-                              i2c_response_bytes, response_bytes_buffer_size);
-   DBGTRC_NOPREFIX(debug, TRACE_GROUP, "i2c_response_bytes -> |%s|",
-                              hexstring_t(i2c_response_bytes, response_bytes_buffer_size) );
+   DBGTRC_STARTING(debug, TRACE_GROUP,
+         "response_bytes_buffer_size=%d, i2c_response_bytes=%p->|%s|",
+         response_bytes_buffer_size, i2c_response_bytes,
+         hexstring_t(i2c_response_bytes, response_bytes_buffer_size));
 
    int result = DDCRC_OK;
    DDC_Packet * packet = NULL;
@@ -547,7 +577,7 @@ create_ddc_base_response_packet(
    }
    else {
       int data_ct = i2c_response_bytes[1] & 0x7f;
-      // DBGMSG("data_ct=%d", data_ct);
+      DBGTRC_NOPREFIX(debug, DDCA_TRC_NONE, "data_ct=%d", data_ct);
       if (data_ct > MAX_DDC_DATA_SIZE) {
          if ( is_double_byte(&i2c_response_bytes[1])) {
             result = DDCRC_DDC_DATA;    // was DDCRC_DOUBLE_BYTE
@@ -566,6 +596,8 @@ create_ddc_base_response_packet(
          Byte * packet_bytes = packet->raw_bytes->bytes;
          buffer_set_byte(  packet->raw_bytes, 0, 0x6f);     // implicit, would be 0x50 on access bus
          buffer_set_byte(  packet->raw_bytes, 1, 0x6e);     // i2c_response_bytes[0]
+         // DBGMSG("packet->raw_bytes+2=%p, i2c_response_bytes+1=%p, 1+data_ct+1=%d",
+         //       packet->raw_bytes+2, i2c_response_bytes+1, 1+data_ct+1);
          buffer_set_bytes( packet->raw_bytes, 2, i2c_response_bytes+1, 1 + data_ct + 1);
          buffer_set_length(packet->raw_bytes, 3 + data_ct + 1);
          Byte calculated_checksum = ddc_checksum(packet_bytes, 3 + data_ct, true);   // replacing right byte?
@@ -589,8 +621,10 @@ create_ddc_base_response_packet(
    else
       *packet_ptr_loc = NULL;
 
-   DBGTRC_RET_DDCRC(debug, TRACE_GROUP, result, "*packet_ptr_loc=%p", *packet_ptr_loc);
    ASSERT_IFF(result==DDCRC_OK, *packet_ptr_loc);
+   DBGTRC_RET_DDCRC2(debug, TRACE_GROUP, result, *packet_ptr_loc, "*packet_ptr_loc=%p", *packet_ptr_loc);
+   if (*packet_ptr_loc && IS_DBGTRC(debug,TRACE_GROUP))
+      dbgrpt_packet(*packet_ptr_loc, 2);
    return result;
 }
 
@@ -617,12 +651,13 @@ create_ddc_response_packet(
        int             response_bytes_buffer_size,
        DDC_Packet_Type expected_type,
        const char *    tag,
-       DDC_Packet **   packet_ptr_addr)
+       DDC_Packet **   packet_ptr_loc)
 {
    bool debug = false;
    DBGTRC_STARTING(debug, TRACE_GROUP,
-          "response_bytes_buffer_size=%d, i2c_response_bytes=|%s|",
-          response_bytes_buffer_size, hexstring_t(i2c_response_bytes, response_bytes_buffer_size));
+          "response_bytes_buffer_size=%d, i2c_response_bytes=%p->|%s|",
+          response_bytes_buffer_size, i2c_response_bytes,
+          hexstring_t(i2c_response_bytes, response_bytes_buffer_size));
 
    if (response_bytes_buffer_size > 2 && i2c_response_bytes[0] == 0x6e
                                       && i2c_response_bytes[1] == 0x6e) {
@@ -635,31 +670,33 @@ create_ddc_response_packet(
                           i2c_response_bytes,
                           response_bytes_buffer_size,
                           tag,
-                          packet_ptr_addr);
-   DBGMSF(debug, "create_ddc_base_response_packet() returned %d, *packet_ptr_addr=%p",
-                 result, *packet_ptr_addr);
+                          packet_ptr_loc);
+   DBGMSF(debug, "create_ddc_base_response_packet() returned %d, *packet_ptr_loc=%p",
+                 result, *packet_ptr_loc);
    if (result == 0) {
-      if (isNullPacket(*packet_ptr_addr)) {
+      if (isNullPacket(*packet_ptr_loc)) {
          result = DDCRC_NULL_RESPONSE;
       }
-      else if ( get_data_start(*packet_ptr_addr)[0] != expected_type) {
+      else if ( get_data_start(*packet_ptr_loc)[0] != expected_type) {
          result = DDCRC_DDC_DATA;      // was: DDCRC_RESPONSE_TYPE
       }
    }
 
-   if (result != DDCRC_OK && *packet_ptr_addr) {
-      DBGTRC_NOPREFIX(debug, TRACE_GROUP, "failure, freeing response packet at %p", *packet_ptr_addr);
+   if (result != DDCRC_OK && *packet_ptr_loc) {
+      DBGTRC_NOPREFIX(debug, TRACE_GROUP, "failure, freeing response packet at %p", *packet_ptr_loc);
       // does this cause the free(readbuf) failure in try_read?
-      free_ddc_packet(*packet_ptr_addr);
-      *packet_ptr_addr = 0;
+      free_ddc_packet(*packet_ptr_loc);
+      *packet_ptr_loc = NULL;
    }
 
    if (result < 0) {
       log_status_code(result, __func__);
    }
 
-   DBGTRC_RET_DDCRC(debug, TRACE_GROUP, result, "*packet_ptr_addr=%p", *packet_ptr_addr);
-   assert( (result==DDCRC_OK && *packet_ptr_addr) || (result != DDCRC_OK && !*packet_ptr_addr));
+   ASSERT_IFF( result==DDCRC_OK, *packet_ptr_loc);
+   DBGTRC_RET_DDCRC2(debug, TRACE_GROUP, result, *packet_ptr_loc, "*packet_ptr_loc=%p", *packet_ptr_loc);
+   if (*packet_ptr_loc && IS_DBGTRC(debug,TRACE_GROUP))
+      dbgrpt_packet(*packet_ptr_loc, 2);
    return result;
 }
 
@@ -708,7 +745,7 @@ interpret_multi_part_read_response(
       aux_data->fragment_type = response_type;    // set in caller?  would make response_type parm unnecessary
       aux_data->fragment_offset = offset_hi_byte << 8 | offset_lo_byte;
       aux_data->fragment_length = read_data_length;      // changed
-      assert(read_data_length <= MAX_DDC_CAPABILITIES_FRAGMENT_SIZE);   // ???
+      assert(read_data_length <= MAX_DDC_MULTI_PART_FRAGMENT_SIZE);   // ???
       memcpy(aux_data->bytes, read_data_start, read_data_length);    // CHANGED
       // aux_data->text[text_length] = '\0';     // CHANGED
    }
@@ -929,9 +966,13 @@ create_ddc_typed_response_packet(
       DDC_Packet**    packet_ptr_loc)
 {
    bool debug = false;
-   DBGTRC_STARTING(debug, TRACE_GROUP, "response_bytes_buffer_size=%d, response_bytes=|%s|",
-                              response_bytes_buffer_size, hexstring_t(i2c_response_bytes, response_bytes_buffer_size) );
+   assert(i2c_response_bytes);
+   DBGTRC_STARTING(debug, TRACE_GROUP,
+         "response_bytes_buffer_size=%d, i2c_response_bytes=%p -> |%s|",
+         response_bytes_buffer_size, i2c_response_bytes,
+         hexstring_t(i2c_response_bytes, response_bytes_buffer_size) );
 
+   *packet_ptr_loc = NULL;
    // DBGMSG("before create_ddc_response_packet(), *packet_ptr_addr=%p", *packet_ptr_loc);
    // n. may return DDC_NULL_RESPONSE??   (old note)
    Status_DDC rc = create_ddc_response_packet(
@@ -944,6 +985,7 @@ create_ddc_typed_response_packet(
                                psc_desc(rc), *packet_ptr_loc);
    if (rc == 0) {
       DDC_Packet * packet = *packet_ptr_loc;
+      assert(packet);
       switch (expected_type) {
 
       case DDC_PACKET_TYPE_CAPABILITIES_RESPONSE:
@@ -951,6 +993,7 @@ create_ddc_typed_response_packet(
          {
             Interpreted_Multi_Part_Read_Fragment * aux_data
                   = calloc(1, sizeof(Interpreted_Multi_Part_Read_Fragment));
+            assert(packet);
             packet->parsed.multi_part_read_fragment = aux_data;
             rc = interpret_multi_part_read_response(
                    expected_type,
@@ -964,6 +1007,7 @@ create_ddc_typed_response_packet(
          {
             Parsed_Nontable_Vcp_Response * aux_data
                   = calloc(1, sizeof(Parsed_Nontable_Vcp_Response));
+            assert(packet);
             packet->parsed.nontable_response = aux_data;
             rc = interpret_vcp_feature_response_std(
                     get_data_start(packet),
@@ -1202,10 +1246,14 @@ Status_DDC get_vcp_cur_value(DDC_Packet * packet, int * value_ptr) {
 
 
 void init_ddc_packets() {
+   RTTI_ADD_FUNC(create_empty_ddc_packet);
+   RTTI_ADD_FUNC(create_ddc_base_request_packet);
    RTTI_ADD_FUNC(create_ddc_base_response_packet);
+   RTTI_ADD_FUNC(create_ddc_getvcp_request_packet);
    RTTI_ADD_FUNC(create_ddc_getvcp_response_packet);
 // RTTI_ADD_FUNC(create_ddc_multi_part_read_response_packet);  // unused
    RTTI_ADD_FUNC(create_ddc_response_packet);
+   RTTI_ADD_FUNC(create_ddc_setvcp_request_packet);
    RTTI_ADD_FUNC(create_ddc_typed_response_packet);
    RTTI_ADD_FUNC(interpret_vcp_feature_response_std);
 }
