@@ -1,8 +1,9 @@
 /** @file displays.c
+ *
  * Monitor identifier, reference, handle
  */
 
-// Copyright (C) 2014-2022 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2024 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 #include <config.h>
@@ -30,37 +31,15 @@
 #include "public/ddcutil_status_codes.h"
 
 #include "core.h"
+#include "i2c_bus_base.h"
 #include "monitor_model_key.h"
+#include "per_display_data.h"
 #include "rtti.h"
 #include "vcp_version.h"
 
 #include "displays.h"
 
 
-#ifdef NOT_NEEDED
-typedef struct {
-   char * s_did;
-} Thread_Displays_Data;
-
-static Thread_Displays_Data *  get_thread_displays_data() {
-   static GPrivate per_thread_data_key = G_PRIVATE_INIT(g_free);
-
-   Thread_Displays_Data *thread_data = g_private_get(&per_thread_data_key);
-
-   // GThread * this_thread = g_thread_self();
-   // printf("(%s) this_thread=%p, settings=%p\n", __func__, this_thread, settings);
-
-   if (!thread_data) {
-      thread_data = g_new0(Thread_Displays_Data, 1);
-      g_private_set(&per_thread_data_key, thread_data);
-   }
-
-   // printf("(%s) Returning: %p\n", __func__, thread_data);
-   return thread_data;
-}
-#endif
-
-
 // *** DDCA_IO_Path ***
 
 /** Tests 2 #DDCA_IO_Path instances for equality
@@ -76,10 +55,6 @@ bool dpath_eq(DDCA_IO_Path p1, DDCA_IO_Path p2) {
       case DDCA_IO_I2C:
          result = (p1.path.i2c_busno == p2.path.i2c_busno);
          break;
-      case DDCA_IO_ADL:
-         result = (p1.path.adlno.iAdapterIndex == p2.path.adlno.iAdapterIndex) &&
-                  (p1.path.adlno.iDisplayIndex == p2.path.adlno.iDisplayIndex);
-         break;
       case DDCA_IO_USB:
          result = p1.path.hiddev_devno == p2.path.hiddev_devno;
       }
@@ -88,131 +63,14 @@ bool dpath_eq(DDCA_IO_Path p1, DDCA_IO_Path p2) {
 }
 
 
-// *** Display_Async_Rec ***
-
-// At least temporarily for development, base all async operations for a display
-// on this struct.
-
-
-static GMutex displays_master_list_mutex;
-static GPtrArray * displays_master_list = NULL;  // only handful of displays, simple data structure suffices
-
-
-void dbgrpt_displays_master_list(GPtrArray* displays_master_list, int depth) {
-   int d1 = depth+1;
-
-   rpt_structure_loc("displays_master_list", displays_master_list, depth);
-   if (displays_master_list) {
-      for (int ndx = 0; ndx < displays_master_list->len; ndx++) {
-          Display_Async_Rec * cur = g_ptr_array_index(displays_master_list, ndx);
-          // DBGMSG("%p", cur);
-          rpt_vstring(d1, "%p - %s", cur, dpath_repr_t(&cur->dpath));
-      }
-   }
-}
-
-
-Display_Async_Rec * display_async_rec_new(DDCA_IO_Path dpath) {
-   Display_Async_Rec * newrec = calloc(1, sizeof(Display_Async_Rec));
-   memcpy(newrec->marker, DISPLAY_ASYNC_REC_MARKER, 4);
-   newrec->dpath = dpath;
-   // g_mutex_init(&newrec->thread_lock);
-   // newrec->owning_thread = NULL;
-
-   newrec->request_queue = g_queue_new();
-   g_mutex_init(&newrec->request_queue_lock);
-#ifdef FUTURE
-   newrec->request_execution_thread =
-         g_thread_new(
-               strdup(dpath_repr_t(dref)),       // thread name
-               NULL,                             // GThreadFunc    *** TEMP ***
-               dref->request_queue);             // or just dref?, how to pass dh?
-#endif
-
-   return newrec;
-}
-
-
-Display_Async_Rec * find_display_async_rec(DDCA_IO_Path dpath) {
-   bool debug = false;
-   assert(displays_master_list);
-   Display_Async_Rec * result = NULL;
-
-   for (int ndx = 0; ndx < displays_master_list->len; ndx++) {
-      Display_Async_Rec * cur = g_ptr_array_index(displays_master_list, ndx);
-      if ( dpath_eq(cur->dpath, dpath) ) {
-         result = cur;
-         break;
-      }
-   }
-
-   DBGMSF(debug, "Returning %p", result);
-   return result;
-}
-
-
-/** Obtains a reference to the #Display_Async_Rec for a display.
+/** Creates a unique integer number from a #DDCA_IO_Path, suitable
+ *  for use as a hash key.
  *
+ *  \param  path   io path
+ *  \return integer value
  */
-Display_Async_Rec * get_display_async_rec(DDCA_IO_Path dpath) {
-   bool debug = false;
-   assert(displays_master_list);
-   DBGMSF(debug, "dpath=%s", dpath_repr_t(&dpath));
-   if (debug)
-      dbgrpt_displays_master_list(displays_master_list, 1);
-
-   // This is a simple critical section.  Always wait.
-   // G_LOCK(global_locks_mutex);
-   g_mutex_lock(&displays_master_list_mutex);
-
-   Display_Async_Rec * result = find_display_async_rec(dpath);
-   if (!result) {
-      result = display_async_rec_new(dpath);
-      // DBGMSG("Adding %p", gdl);
-      g_ptr_array_add(displays_master_list, result);
-   }
-   // G_UNLOCK(global_locks_mutex);
-   g_mutex_unlock(&displays_master_list_mutex);
-   DBGMSF(debug, "Returning %p", result);
-   return result;
-}
-
-
-// GLOCK... macros confuse Eclipse
-// GLOCK_DEFINE_STATIC(global_locks_mutex);
-
-
-
-
-/** Acquired at display open time.
- *  Only 1 thread can open
- */
-bool lock_display_lock(Display_Async_Rec * async_rec, bool wait) {
-   assert(async_rec && memcmp(async_rec->marker, DISPLAY_ASYNC_REC_MARKER, 4) == 0);
-
-   bool lock_acquired = false;
-
-   if (wait) {
-      g_mutex_lock(&async_rec->display_lock);
-      lock_acquired = true;
-   }
-   else {
-      lock_acquired = g_mutex_trylock(&async_rec->display_lock);
-   }
-   if (lock_acquired)
-      async_rec->thread_owning_display_lock = g_thread_self();
-
-   return lock_acquired;
-}
-
-
-void unlock_display_lock(Display_Async_Rec * async_rec) {
-   assert(async_rec && memcmp(async_rec->marker, DISPLAY_ASYNC_REC_MARKER, 4) == 0);
-
-   if (async_rec->thread_owning_display_lock == g_thread_self()) {
-      async_rec->thread_owning_display_lock = NULL;
-      g_mutex_unlock(&async_rec->display_lock);
-   }
+int dpath_hash(DDCA_IO_Path path) {
+   return path.io_mode * 100 + path.path.i2c_busno;
 }
 
 
@@ -402,13 +260,6 @@ void dbgrpt_display_identifier(Display_Identifier * pdid, int depth) {
    char * edidstr = hexstring(pdid->edidbytes, 128);
    rpt_str( "edid",          NULL, edidstr,             d1);
    free(edidstr);
-
-#ifdef ALTERNATIVE
-   // avoids a malloc and free, but less clear
-   char edidbuf[257];
-   char * edidstr2 = hexstring2(pdid->edidbytes, 128, NULL, true, edidbuf, 257);
-   rpt_str( "edid",          NULL, edidstr2, d1);
-#endif
 }
 
 
@@ -509,8 +360,7 @@ void dsel_free(Display_Selector * dsel) {
 // *** DDCA_IO_Mode and DDCA_IO_Path ***
 
 static char * IO_Mode_Names[] = {
-      "DDCA_IO_DEVI2C",
-      "DDCA_IO_ADL",
+      "DDCA_IO_I2C",
       "DDCA_IO_USB"
 };
 
@@ -521,15 +371,34 @@ static char * IO_Mode_Names[] = {
  * \return symbolic name, e.g. "DDCA_IO_DEVI2C"
  */
 char * io_mode_name(DDCA_IO_Mode val) {
-   return (val >= 0 && val < 3)            // protect against bad arg
+   return (val >= 0 && val < 2)            // protect against bad arg
          ? IO_Mode_Names[val]
          : NULL;
 }
 
 
+
+/** A simple function allowing for the assignment of a value to a
+ *  #DDCA_IO_Path instance in a single line of code.
+ *
+ *  @parm   busno    I2C bus number
+ *  @return DDCA_IO_Path value
+ */
+DDCA_IO_Path i2c_io_path(int busno) {
+   DDCA_IO_Path path;
+   path.io_mode = DDCA_IO_I2C;
+   path.path.i2c_busno = busno;
+   return path;
+}
+
+
 /** Thread safe function that returns a brief string representation of a #DDCA_IO_Path.
  *  The returned value is valid until the next call to this function on the current thread.
  *
+ *  \remark
+ *  A bus number of 255 represents a value that has not been set.  The string
+ *  "NOT SET" is returned.
+ *
  *  \param  dpath  pointer to ##DDCA_IO_Path
  *  \return string representation of #DDCA_IO_Path
  */
@@ -539,13 +408,13 @@ char * dpath_short_name_t(DDCA_IO_Path * dpath) {
    char * buf = get_thread_fixed_buffer(&dpath_short_name_key, 100);
    switch(dpath->io_mode) {
    case DDCA_IO_I2C:
-      snprintf(buf, 100, "bus /dev/i2c-%d", dpath->path.i2c_busno);
-      break;
-   case DDCA_IO_ADL:
-      snprintf(buf, 100, "adlno (%d.%d)", dpath->path.adlno.iAdapterIndex, dpath->path.adlno.iDisplayIndex);
+      if (dpath->path.i2c_busno == 255)
+         g_strlcpy(buf, "NOT SET", 100);
+      else
+         g_snprintf(buf, 100, "bus /dev/i2c-%d", dpath->path.i2c_busno);
       break;
    case DDCA_IO_USB:
-      snprintf(buf, 100, "usb /dev/usb/hiddev%d", dpath->path.hiddev_devno);
+      g_snprintf(buf, 100, "usb /dev/usb/hiddev%d", dpath->path.hiddev_devno);
    }
    return buf;
 }
@@ -564,10 +433,10 @@ char * dpath_repr_t(DDCA_IO_Path * dpath) {
    char * buf = get_thread_fixed_buffer(&dpath_repr_key, 100);
    switch(dpath->io_mode) {
    case DDCA_IO_I2C:
-      snprintf(buf, 100, "Display_Path[/dev/i2c-%d]", dpath->path.i2c_busno);
-      break;
-   case DDCA_IO_ADL:
-      snprintf(buf, 100, "Display_Path[adl=(%d.%d)]", dpath->path.adlno.iAdapterIndex, dpath->path.adlno.iDisplayIndex);
+      if (dpath->path.i2c_busno == BUSNO_NOT_SET)
+         snprintf(buf, 100, "Display Path not set");
+      else
+         snprintf(buf, 100, "Display_Path[/dev/i2c-%d]", dpath->path.i2c_busno);
       break;
    case DDCA_IO_USB:
       snprintf(buf, 100, "Display_Path[/dev/usb/hiddev%d]", dpath->path.hiddev_devno);
@@ -578,15 +447,18 @@ char * dpath_repr_t(DDCA_IO_Path * dpath) {
 
 // *** Display_Ref ***
 
-static Display_Ref * create_base_display_ref(DDCA_IO_Path io_path) {
+Display_Ref * create_base_display_ref(DDCA_IO_Path io_path) {
+   bool debug = false;
+   DBGTRC_STARTING(debug, DDCA_TRC_BASE, "io_path=%s", dpath_repr_t(&io_path));
    Display_Ref * dref = calloc(1, sizeof(Display_Ref));
    memcpy(dref->marker, DISPLAY_REF_MARKER, 4);
    dref->io_path = io_path;
    dref->vcp_version_xdf = DDCA_VSPEC_UNQUERIED;
    dref->vcp_version_cmdline = DDCA_VSPEC_UNQUERIED;
-
-   dref->async_rec  = get_display_async_rec(io_path);    // keep?
-
+   // Per_Display_Data * pdd = pdd_get_per_display_data(io_path, true);
+   // dref->pdd = pdd;
+   // DBGTRC_RET_STRUCT(debug, DDCA_TRC_BASE, "Display_Ref", dbgrpt_display_ref, dref);
+   DBGTRC_DONE(debug, DDCA_TRC_BASE, "Returning %p", dref);
    return dref;
 }
 
@@ -599,6 +471,7 @@ static Display_Ref * create_base_display_ref(DDCA_IO_Path io_path) {
  */
 Display_Ref * create_bus_display_ref(int busno) {
    bool debug = false;
+   DBGTRC_STARTING(debug, DDCA_TRC_BASE, "busno=%d", busno);
    DDCA_IO_Path io_path;
    io_path.io_mode   = DDCA_IO_I2C;
    io_path.path.i2c_busno = busno;
@@ -609,11 +482,13 @@ Display_Ref * create_bus_display_ref(int busno) {
       DBGMSG("Done.  Constructed bus display ref %s:", dref_repr_t(dref));
       dbgrpt_display_ref(dref,0);
    }
+
+   DBGTRC_RET_STRUCT(debug, DDCA_TRC_BASE, "Display_Ref", dbgrpt_display_ref, dref);
    return dref;
 }
 
 
-#ifdef USE_USB
+#ifdef ENABLE_USB
 /** Creates a #Display_Ref for IO mode #DDCA_IO_USB
  *
  * @param  usb_bus USB bus number
@@ -624,6 +499,8 @@ Display_Ref * create_bus_display_ref(int busno) {
 Display_Ref * create_usb_display_ref(int usb_bus, int usb_device, char * hiddev_devname) {
    assert(hiddev_devname);
    bool debug = false;
+   DBGTRC_STARTING(debug, DDCA_TRC_BASE, "usb_bus=%d, usb_device=%d, hiddev_devname=%s",
+         usb_bus, usb_device, hiddev_devname);
    DDCA_IO_Path io_path;
    io_path.io_mode      = DDCA_IO_USB;
    io_path.path.hiddev_devno = hiddev_name_to_number(hiddev_devname);
@@ -631,81 +508,106 @@ Display_Ref * create_usb_display_ref(int usb_bus, int usb_device, char * hiddev_
 
    dref->usb_bus     = usb_bus;
    dref->usb_device  = usb_device;
-   dref->usb_hiddev_name = strdup(hiddev_devname);
+   dref->usb_hiddev_name = g_strdup(hiddev_devname);
 
+#ifdef OLD
    if (debug) {
       DBGMSG("Done.  Constructed USB display ref:");
       dbgrpt_display_ref(dref,0);
    }
+#endif
+   DBGTRC_RET_STRUCT(debug, DDCA_TRC_BASE, "Display_Ref", dbgrpt_display_ref, dref);
+   // DBGTRC_DONE(debug, DDCA_TRC_BASE, "Returning %p", dref);
    return dref;
 }
 #endif
 
 
-#ifdef THANKFULLY_UNNEEDED
-// Issue: what to do with referenced data structures
-Display_Ref * clone_display_ref(Display_Ref * old) {
-   assert(old);
-   Display_Ref * dref = calloc(1, sizeof(Display_Ref));
-   // dref->ddc_io_mode = old->ddc_io_mode;
-   // dref->busno         = old->busno;
-   // dref->iAdapterIndex = old->iAdapterIndex;
-   // dref->iDisplayIndex = old->iDisplayIndex;
-   // DBGMSG("dref=%p, old=%p, len=%d  ", dref, old, (int) sizeof(BasicDisplayRef) );
-   memcpy(dref, old, sizeof(Display_Ref));
-   if (old->usb_hiddev_name) {
-      dref->usb_hiddev_name = strcpy(dref->usb_hiddev_name, old->usb_hiddev_name);
+Display_Ref * copy_display_ref(Display_Ref * dref) {
+   bool debug = false;
+   DBGTRC_STARTING(debug, DDCA_TRC_BASE, "dref=%p, iopath=%s", dref, (dref) ? dpath_repr_t(&dref->io_path) : NULL);
+   Display_Ref * copy = NULL;
+   if (dref) {
+      DDCA_IO_Path iopath = dref->io_path;
+      copy = create_base_display_ref(iopath);
+      copy->usb_bus = dref->usb_bus;
+      copy->usb_device = dref->usb_device;
+      copy->usb_hiddev_name = g_strdup(dref->usb_hiddev_name);
+      copy->vcp_version_xdf = dref->vcp_version_xdf;
+      copy->vcp_version_cmdline = dref->vcp_version_cmdline;
+      copy->flags = dref->flags & ~DREF_DYNAMIC_FEATURES_CHECKED;
+      copy->capabilities_string = g_strdup(dref->capabilities_string);
+      if (dref->pedid) {
+         copy->pedid = copy_parsed_edid(dref->pedid);
+      }
+      if (dref->mmid) {
+         copy->mmid = calloc(1, sizeof(Monitor_Model_Key));
+         memcpy(copy->mmid, dref->mmid, sizeof(Monitor_Model_Key));
+      }
+      copy->dispno = dref->dispno;
+      // do not set detail
+      // do not set dfr
+      // do not set actual_display
+      copy->actual_display_path = dref->actual_display_path;
+      copy->driver_name = g_strdup(dref->driver_name);
+      // dont set pdd
+      copy->drm_connector = g_strdup(dref->drm_connector);
    }
-   return dref;
+   // DBGTRC_RET_STRUCT(debug, DDCA_TRC_BASE, "Display_Ref", dbgrpt_display_ref, copy);
+   DBGTRC_DONE(debug, DDCA_TRC_BASE, "Returning %p", copy);
+   return copy;
 }
-#endif
+
+//#define CK_INVALIDATED_MARKER(_marker, _marker_name)  ( (memcmp(_marker, _marker_name, 3) == 0) && (_marker[3] = 'x')))
+//    assert( !(INVALIDATED_MARKER(dref->marker, DISPLAY_REF_MARKER));
 
 
 /** Frees a display reference.
  *
  *  \param  dref  ptr to display reference to free, if NULL no operation is performed
  *  \retval DDCRC_OK       success
- *  \retval DDCRC_ARG      invalid display reference
  *  \retval DDCRC_LOCKED   display reference not marked as transient
  */
 DDCA_Status free_display_ref(Display_Ref * dref) {
    bool debug = false;
-   DBGTRC_STARTING(debug, DDCA_TRC_BASE, "dref=%p -> %s", dref, dref_repr_t(dref));
+   DBGTRC_STARTING(debug, DDCA_TRC_BASE, "dref=%p", dref);
    DDCA_Status ddcrc = 0;
    if (dref) {
-      if ( memcmp(dref->marker, DISPLAY_REF_MARKER, 4) != 0) {
-         ddcrc = DDCRC_ARG;
-         DBGMSG("Invalid dref.");
-         rpt_hex_dump((Byte*) dref->marker, 4, 2);
-         goto bye;
-      }
+      assert ( memcmp(dref->marker, DISPLAY_REF_MARKER, 4) == 0);
+      DBGTRC_NOPREFIX(debug, DDCA_TRC_BASE, "dref=%s, DREF_TRANSIENT=%s, DREF_OPEN=%s",
+            dref_repr_t(dref), SBOOL(dref->flags & DREF_TRANSIENT), SBOOL(dref->flags&DREF_OPEN));
       if (dref->flags & DREF_TRANSIENT)  {
          if (dref->flags & DREF_OPEN) {
             ddcrc = DDCRC_LOCKED;
          }
          else {
-            if (dref->usb_hiddev_name)       // always set using strdup()
-               free(dref->usb_hiddev_name);
-            if (dref->capabilities_string)   // always a private copy
-               free(dref->capabilities_string);
-            if (dref->mmid)                  // always a private copy
-               free(dref->mmid);
-            // 9/2017: what about pedid, detail2?
-            // what to do with gdl, request_queue?
-            if (dref->dfr)
-               dfr_free(dref->dfr);
-            if (dref->driver_name)
-               free(dref->driver_name);
+            free(dref->usb_hiddev_name);        // private copy
+            free(dref->capabilities_string);    // private copy
+            free(dref->mmid);                   // private copy
+            if (dref->pedid)  {
+               DBGTRC(debug, DDCA_TRC_NONE, "Freeing dref->pedid = %p", dref->pedid);
+               free_parsed_edid(dref->pedid);  // private copy
+            }
+            dfr_free(dref->dfr);
+            free(dref->driver_name);
+            free(dref->drm_connector);
+            free(dref->communication_error_summary);
             dref->marker[3] = 'x';
             free(dref);
          }
       }
    }
-bye:
    DBGTRC_RET_DDCRC(debug, DDCA_TRC_BASE, ddcrc, "");
    return ddcrc;
 }
 
+#ifdef UNNEEDED
+// wraps free_display_ref() as GDestroyNotify()
+void gdestroy_display_ref(void * data) {
+   free_display_ref((Display_Ref*) data);
+}
+#endif
+
 
 /** Tests if 2 #Display_Ref instances specify the same path to the
  *  display.
@@ -723,6 +625,26 @@ bool dref_eq(Display_Ref* this, Display_Ref* that) {
 }
 
 
+#ifdef UNUSED
+bool dref_set_alive(Display_Ref * dref, bool alive) {
+   assert(dref);
+   bool debug = true;
+   bool old = dref->flags & DREF_ALIVE;
+   if (old != alive)
+      DBGTRC_EXECUTED(debug, DDCA_TRC_BASE, "dref=%s, alive changed: %s -> %s",
+                             dref_repr_t(dref), SBOOL(old), SBOOL(alive));
+   SETCLR_BIT(dref->flags, DREF_ALIVE, alive);
+   return old;
+}
+
+
+bool dref_get_alive(Display_Ref * dref) {
+   assert(dref);
+   return dref->flags & DREF_ALIVE;;
+}
+#endif
+
+
 /** Reports the contents of a #Display_Ref in a format useful for debugging.
  *
  *  \param  dref  pointer to #Display_Ref instance
@@ -730,8 +652,9 @@ bool dref_eq(Display_Ref* this, Display_Ref* that) {
  */
 void dbgrpt_display_ref(Display_Ref * dref, int depth) {
    bool debug = false;
-   DBGMSF(debug, "Starting. dref=%s", dref_repr_t(dref));
+   DBGTRC_STARTING(debug, DDCA_TRC_NONE, "dref=%s", dref_repr_t(dref));
    int d1 = depth+1;
+   int d2 = depth+2;
 
    rpt_structure_loc("Display_Ref", dref, depth);
    rpt_vstring(d1, "io_path:          %s", dpath_repr_t(&(dref->io_path)));
@@ -741,13 +664,29 @@ void dbgrpt_display_ref(Display_Ref * dref, int depth) {
       rpt_str("usb_hiddev_name", NULL, dref->usb_hiddev_name, d1);
    }
 
-   rpt_vstring(d1, "vcp_version_xdf:  %s", format_vspec(dref->vcp_version_xdf) );
-   rpt_vstring(d1, "flags:            %s", interpret_dref_flags_t(dref->flags) );
-   rpt_vstring(d1, "mmid:             %s", (dref->mmid) ? mmk_repr(*dref->mmid) : "NULL");
+   rpt_vstring(d1, "vcp_version_xdf:     %s", format_vspec(dref->vcp_version_xdf) );
+   rpt_vstring(d1, "vcp_version_cmdline: %s", format_vspec(dref->vcp_version_cmdline) );
+   rpt_vstring(d1, "flags:               %s", interpret_dref_flags_t(dref->flags) );
+   rpt_vstring(d1, "capabilities_string: %s", dref->capabilities_string);
+   rpt_vstring(d1, "mmid:                %s", (dref->mmid) ? mmk_repr(*dref->mmid) : "NULL");
+   rpt_vstring(d1, "dispno:              %d", dref->dispno);
+   rpt_vstring(d1, "pedid:               %p", dref->pedid);
+   report_parsed_edid(dref->pedid, /*verbose*/ false, depth+1);
 
    rpt_vstring(d1, "driver:           %s", dref->driver_name);
+   rpt_vstring(d1, "actual_display:   %p", dref->actual_display);
+   rpt_vstring(d1, "actual_display_path: %s",
+         (dref->actual_display_path) ? dpath_repr_t(dref->actual_display_path) : "NULL");
+   rpt_vstring(d1, "detail:         %p", dref->detail);
+   if (dref->io_path.io_mode == DDCA_IO_I2C) {
+      I2C_Bus_Info * businfo = dref->detail;
+      if (businfo) {
+         i2c_dbgrpt_bus_info(businfo, d2);
+      }
+   }
+   rpt_vstring(d1, "drm_connector:   %s", dref->drm_connector);
 
-   DBGMSF(debug, "Done");
+   DBGTRC_DONE(debug, DDCA_TRC_NONE, "");
 }
 
 
@@ -775,7 +714,11 @@ char * dref_repr_t(Display_Ref * dref) {
 
    char * buf = get_thread_fixed_buffer(&dref_repr_key, 100);
    if (dref)
+#ifdef WITH_ADDR
       g_snprintf(buf, 100, "Display_Ref[%s @%p]", dpath_short_name_t(&dref->io_path), (void*)dref);
+#else
+   g_snprintf(buf, 100, "Display_Ref[%s]", dpath_short_name_t(&dref->io_path));
+#endif
    else
       strcpy(buf, "Display_Ref[NULL]");
    return buf;
@@ -801,10 +744,15 @@ Display_Handle * create_base_display_handle(int fd, Display_Ref * dref) {
    dh->dref = dref;
    if (dref->io_path.io_mode == DDCA_IO_I2C) {
       dh->repr = g_strdup_printf(
+#ifdef WITH_ADDR
                      "Display_Handle[i2c-%d: fd=%d @%p]",
                      dh->dref->io_path.path.i2c_busno, dh->fd, (void*)dh);
+#else
+      "Display_Handle[i2c-%d: fd=%d]",
+      dh->dref->io_path.path.i2c_busno, dh->fd);
+#endif
    }
-#ifdef USE_USB
+#ifdef ENABLE_USB
    else if (dref->io_path.io_mode == DDCA_IO_USB) {
       dh->repr = g_strdup_printf(
                 "Display_Handle[usb: %d:%d, %s/hiddev%d @%p]",
@@ -815,7 +763,7 @@ Display_Handle * create_base_display_handle(int fd, Display_Ref * dref) {
    }
 #endif
    else {
-      // DDCA_IO_ADL, DDCA_IO_USB if !USE_USB
+      // DDCA_IO_USB if !ENABLE_USB
       PROGRAM_LOGIC_ERROR("Unimplemented io_mode = %d", dref->io_path.io_mode);
       dh->repr = NULL;
    }
@@ -824,7 +772,6 @@ Display_Handle * create_base_display_handle(int fd, Display_Ref * dref) {
 }
 
 
-
 /** Reports the contents of a #Display_Handle in a format useful for debugging.
  *
  *  \param  dh       display handle
@@ -850,11 +797,6 @@ void dbgrpt_display_handle(Display_Handle * dh, const char * msg, int depth) {
             rpt_vstring(d1, "fd:                  %d", dh->fd);
             rpt_vstring(d1, "busno:               %d", dh->dref->io_path.path.i2c_busno);
             break;
-         case (DDCA_IO_ADL):
-            // rpt_vstring(d1, "ddc_io_mode = DDC_IO_ADL");
-            rpt_vstring(d1, "iAdapterIndex:       %d", dh->dref->io_path.path.adlno.iAdapterIndex);
-            rpt_vstring(d1, "iDisplayIndex:       %d", dh->dref->io_path.path.adlno.iDisplayIndex);
-            break;
          case (DDCA_IO_USB):
             // rpt_vstring(d1, "ddc_io_mode = USB_IO");
             rpt_vstring(d1, "fd:                  %d", dh->fd);
@@ -863,6 +805,8 @@ void dbgrpt_display_handle(Display_Handle * dh, const char * msg, int depth) {
             rpt_vstring(d1, "hiddev_device_name:  %s", dh->dref->usb_hiddev_name);
             break;
          }
+         rpt_vstring(d1, "testing_unsupported_feature_active: %s",
+                          sbool(dh->testing_unsupported_feature_active));
       }
       // rpt_vstring(d1, "vcp_version:         %d.%d", dh->vcp_version.major, dh->vcp_version.minor);
    }
@@ -902,7 +846,7 @@ void   free_display_handle(Display_Handle * dh) {
 
 // *** Miscellaneous ***
 
-#ifdef USE_USB
+#ifdef ENABLE_USB
 /** Given a hiddev device name, e.g. /dev/usb/hiddev3,
  *  extract its number, e.g. 3.
  *
@@ -947,20 +891,29 @@ char * hiddev_number_to_name(int hiddev_number) {
 #endif
 
 
+// globals
+bool ddc_never_uses_null_response_for_unsupported = false;
+// bool ddc_always_uses_null_response_for_unsupported = false;
+
 Value_Name_Table dref_flags_table = {
       VN(DREF_DDC_COMMUNICATION_CHECKED),
- //   VN(DREF_DDC_COMMUNICATION_WORKING),
+      VN(DREF_DDC_COMMUNICATION_WORKING),
       VN(DREF_DDC_IS_MONITOR_CHECKED),
       VN(DREF_DDC_IS_MONITOR),
-      VN(DREF_TRANSIENT),
-      VN(DREF_DYNAMIC_FEATURES_CHECKED),
-      VN(DREF_OPEN),
+
+      VN(DREF_UNSUPPORTED_CHECKED),
       VN(DREF_DDC_USES_NULL_RESPONSE_FOR_UNSUPPORTED),
       VN(DREF_DDC_USES_MH_ML_SH_SL_ZERO_FOR_UNSUPPORTED),
       VN(DREF_DDC_USES_DDC_FLAG_FOR_UNSUPPORTED),
       VN(DREF_DDC_DOES_NOT_INDICATE_UNSUPPORTED),
+
+      VN(DREF_TRANSIENT),
+      VN(DREF_DYNAMIC_FEATURES_CHECKED),
+      VN(DREF_OPEN),
       VN(DREF_DDC_BUSY),
-      VN(CALLOPT_NONE),                // special entry
+      VN(DREF_REMOVED),
+      VN(DREF_DPMS_SUSPEND_STANDBY_OFF),
+//    VN(CALLOPT_NONE),                // special entry
       VN_END
 };
 
@@ -975,10 +928,10 @@ Value_Name_Table dref_flags_table = {
  */
 char * interpret_dref_flags_t(Dref_Flags flags) {
    static GPrivate  buf_key = G_PRIVATE_INIT(g_free);
-   char * buf = get_thread_fixed_buffer(&buf_key, 200);
+   char * buf = get_thread_fixed_buffer(&buf_key, 300);
 
    char * buftemp = vnt_interpret_flags(flags, dref_flags_table, false, ", ");
-   g_strlcpy(buf, buftemp, 200);    // n. this is a debug msg, truncation benign
+   g_strlcpy(buf, buftemp, 300);    // n. this is a debug msg, truncation benign
    free(buftemp);
 
    return buf;
@@ -986,9 +939,15 @@ char * interpret_dref_flags_t(Dref_Flags flags) {
 
 
 void init_displays() {
+   RTTI_ADD_FUNC(copy_display_ref);
+   RTTI_ADD_FUNC(create_base_display_handle);
+   RTTI_ADD_FUNC(create_base_display_ref);
+   RTTI_ADD_FUNC(create_bus_display_ref);
+#ifdef ENABLE_USB
+   RTTI_ADD_FUNC(create_usb_display_ref);
+#endif
+   RTTI_ADD_FUNC(dbgrpt_display_ref);
    RTTI_ADD_FUNC(free_display_handle);
    RTTI_ADD_FUNC(free_display_ref);
-
-   displays_master_list = g_ptr_array_new();
 }
 
