@@ -9,6 +9,7 @@
 /** \cond */
 #include <assert.h>
 #include <ctype.h>
+#include <glib-2.0/glib.h>
 #include <stdio.h>
 #include <string.h>
 /** \endcond */
@@ -92,10 +93,10 @@ dbgrpt_dynamic_features_rec(
    if (dfr->features) {
       rpt_vstring(d1, "features count: %d", g_hash_table_size(dfr->features));
       for (int ndx = 1; ndx < 256; ndx++) {
-         DDCA_Feature_Metadata * cur_feature = g_hash_table_lookup(dfr->features,
+         Dyn_Feature_Metadata * cur_feature = g_hash_table_lookup(dfr->features,
                GINT_TO_POINTER(ndx));
          if (cur_feature)
-            dbgrpt_ddca_feature_metadata(cur_feature, d1);
+            dbgrpt_dyn_feature_metadata(cur_feature, d1);
       }
    }
 }
@@ -122,14 +123,14 @@ dfr_repr_t(Dynamic_Features_Rec * dfr) {
 }
 
 
-/** Gets a #DDCA_Feature_Metadata record from the features hash table
+/** Gets a #Dyn_Feature_Metadata record from the features hash table
  *  of a #Dynamic_Features_Rec.
  *
  *  @param  dfr          pointer to dynamic feature record
  *  @param  feature_code feature code
  *  @return pointer to feature metadata, NULL if not found
  */
-DDCA_Feature_Metadata *
+Dyn_Feature_Metadata *
 get_dynamic_feature_metadata(
       Dynamic_Features_Rec * dfr,
       uint8_t                feature_code)
@@ -137,7 +138,7 @@ get_dynamic_feature_metadata(
    bool debug = false;
    DBGMSF(debug, "dfr=%s, feature_code=0x%02x", dfr_repr_t(dfr), feature_code);
 
-   DDCA_Feature_Metadata * result = NULL;
+   Dyn_Feature_Metadata * result = NULL;
    if (dfr && dfr->features)
       result = g_hash_table_lookup(dfr->features, GINT_TO_POINTER(feature_code));
 
@@ -146,7 +147,7 @@ get_dynamic_feature_metadata(
 }
 
 
-/** Free a #DDCA_Feature_Metadata record.
+/** Free a #Dyn_Feature_Metadata record.
  *
  *  @param data pointer to record
  *
@@ -154,17 +155,17 @@ get_dynamic_feature_metadata(
  */
 void
 free_feature_metadata(
-      gpointer data)    // i.e. DDCA_Feature_Metadata *
+      gpointer data)    // i.e. Dyn_Feature_Metadata *
 {
    bool debug = false;
-   DBGTRC_STARTING(debug, TRACE_GROUP, "Starting. DDCA_Feature_Metadata * data = %p", data);
+   DBGTRC_STARTING(debug, TRACE_GROUP, "Starting. Dyn_Feature_Metadata * data = %p", data);
 
-   DDCA_Feature_Metadata * info = (DDCA_Feature_Metadata*) data;
+   Dyn_Feature_Metadata * info = (Dyn_Feature_Metadata*) data;
    assert(info && memcmp(info->marker, DDCA_FEATURE_METADATA_MARKER, 4) == 0);
    // compare vs ddca_free_metadata_contents()
 
    if (debug)
-      dbgrpt_ddca_feature_metadata(info, 2);
+      dbgrpt_dyn_feature_metadata(info, 2);
 
    free(info->feature_desc);
    free(info->feature_name);
@@ -205,12 +206,12 @@ dfr_new(
    Dynamic_Features_Rec * frec = calloc(1, sizeof(Dynamic_Features_Rec));
    memcpy(frec->marker, DYNAMIC_FEATURES_REC_MARKER, 4);
 
-   frec->mfg_id       = strdup(mfg_id);
-   frec->model_name   = strdup(model_name);
+   frec->mfg_id       = g_strdup(mfg_id);
+   frec->model_name   = g_strdup(model_name);
    frec->product_code = product_code;
    frec->vspec        = DDCA_VSPEC_UNKNOWN;   // redundant, since set by calloc(), but be explicit
    if (filename)
-      frec->filename  = strdup(filename);
+      frec->filename  = g_strdup(filename);
 
    DBGTRC_DONE(debug, TRACE_GROUP, "Returning %p", frec);
    return frec;
@@ -266,6 +267,18 @@ dfr_gdestroy(gpointer p) {
 // Functions private to create_monitor_dynamic_features()
 //
 
+
+/** Creates a #Error_Info using the error location (line number) and
+ *  detail, and appends it to array **errors**..
+ *
+ *  @param errors   GPtrArray of #Error_Info
+ *  @param filename name of file whose lines are being processed
+ *  @param linectr  line number of error
+ *  @param caller   function creating the error message
+ *  @param fmt      message format string
+ *  @param ..       substitution values
+ *  @return newly allocated #Error_Info struct, with status code DDCRC_BAD_DATA
+ */
 static void
 add_error(
       GPtrArray *  errors,      // array of Error_Info
@@ -303,7 +316,7 @@ add_error(
 
 static bool
 attr_keyword(
-      DDCA_Feature_Metadata * cur_feature_metadata,
+      Dyn_Feature_Metadata * cur_feature_metadata,
       char *                  keyword)
 {
    bool debug = false;
@@ -348,7 +361,7 @@ switch_bits(
 static void
 finalize_feature(
       Dynamic_Features_Rec *  frec,
-      DDCA_Feature_Metadata * cur_feature_metadata,
+      Dyn_Feature_Metadata * cur_feature_metadata,
       GArray *                cur_feature_values,
       const char *            filename,
       GPtrArray *             errors)
@@ -403,9 +416,10 @@ finalize_feature(
  *  @param  dynamic_features_loc where to return pointer to newly allocated #Dynamic_Features_Rec,
  *                               NULL if an #Error_Info struct is returned
  *  @return pointer to #Error_Info, NULL if no error
+ *          The #Error_Info, and all of its causes, have status code DDCRC_BAD_DATA
  */
 Error_Info *
-create_monitor_dynamic_features(
+create_dynamic_features_rec(
       const char * mfg_id,
       const char * model_name,
       uint16_t     product_code,
@@ -428,7 +442,7 @@ create_monitor_dynamic_features(
                               NULL,                     // key_destroy_func
                               free_feature_metadata);   // value_destroy_func
 
-   DDCA_Feature_Metadata * cur_feature_metadata = NULL;
+   Dyn_Feature_Metadata * cur_feature_metadata = NULL;
    GArray * cur_feature_values = NULL;
 
    int     linectr = 0;
@@ -508,7 +522,7 @@ create_monitor_dynamic_features(
                   }
                }
 
-               cur_feature_metadata = calloc(1, sizeof(DDCA_Feature_Metadata));
+               cur_feature_metadata = calloc(1, sizeof(Dyn_Feature_Metadata));
                memcpy(cur_feature_metadata->marker, DDCA_FEATURE_METADATA_MARKER, 4);
                cur_feature_metadata->feature_flags = DDCA_USER_DEFINED | DDCA_PERSISTENT_METADATA;
 
@@ -532,7 +546,7 @@ create_monitor_dynamic_features(
                   }
                   else {     // valid opcode
                      cur_feature_metadata->feature_code = feature_id;
-                     cur_feature_metadata->feature_name = strdup(feature_name);
+                     cur_feature_metadata->feature_name = g_strdup(feature_name);
                      cur_feature_metadata->feature_desc = NULL;   // ignore for now
                   }
                }
@@ -555,7 +569,7 @@ create_monitor_dynamic_features(
                         cur_feature_values = g_array_new(false, false, sizeof(DDCA_Feature_Value_Entry));
                      DDCA_Feature_Value_Entry entry;
                      entry.value_code = feature_value;
-                     entry.value_name = strdup(t2.rest);
+                     entry.value_name = g_strdup(t2.rest);
                      g_array_append_val(cur_feature_values, entry);
                   }
                }
@@ -629,7 +643,7 @@ create_monitor_dynamic_features(
 
 
 void init_base_dynamic_features() {
-   RTTI_ADD_FUNC(create_monitor_dynamic_features);
+   RTTI_ADD_FUNC(create_dynamic_features_rec);
    RTTI_ADD_FUNC(free_feature_metadata);
    RTTI_ADD_FUNC(dfr_new);
    RTTI_ADD_FUNC(dfr_free);
