@@ -3,10 +3,9 @@
  * Dynamic Feature Record definition, creation, destruction, and conversion
  */
 
-// Copyright (C) 2022 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2022-2023 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
-
 #ifndef BASE_DYNAMIC_FEATURES_H_
 #define BASE_DYNAMIC_FEATURES_H_
 
@@ -25,6 +24,24 @@ typedef enum {
    DFR_FLAGS_NOT_FOUND = 1
 } DFR_Flags;
 
+// Replaces use of DDCA_Feature_Metadata for representing dynamic spec read from file
+#define DYN_FEATURE_METADATA_MARKER  "DMET"
+/** Describes a VCP feature code, as read from a dynamic feature record.
+ */
+typedef
+struct {
+   char                                  marker[4];      /**< always "DMET" */
+   DDCA_Vcp_Feature_Code                 feature_code;   /**< VCP feature code */
+   DDCA_MCCS_Version_Spec                vcp_version;    /**< MCCS version    */
+   DDCA_Feature_Flags                    feature_flags;  /**< feature type description */
+   DDCA_Feature_Value_Entry *            sl_values;      /**< valid when DDCA_SIMPLE_NC set */
+   void *                                unused;         /** no longer used, was latest_sl_values */
+   char *                                feature_name;   /**< feature name */
+   char *                                feature_desc;   /**< feature description */
+   // possibly add pointers to formatting functions
+} Dyn_Feature_Metadata;
+
+
 #define DYNAMIC_FEATURES_REC_MARKER "DFRC"
 typedef struct {
    char                       marker[4];
@@ -34,7 +51,7 @@ typedef struct {
    char *                     filename;     // source filename, if applicable
    DDCA_MCCS_Version_Spec     vspec;
    DFR_Flags                  flags;
-   GHashTable *               features;     // hash table of DDCA_Feature_Metadata
+   GHashTable *               features;     // hash table of Dyn_Feature_Metadata
 } Dynamic_Features_Rec;
 
 // value valid until next call:
@@ -58,7 +75,7 @@ dfr_gdestroy(
       gpointer p);
 
 Error_Info *
-create_monitor_dynamic_features(
+create_dynamic_features_rec(
       const char *            mfg_id,
       const char *            model_name,
       uint16_t                product_code,
@@ -66,7 +83,7 @@ create_monitor_dynamic_features(
       const char *            filename,     // may be NULL
       Dynamic_Features_Rec ** dynamic_features_loc);
 
-DDCA_Feature_Metadata *
+Dyn_Feature_Metadata *
 get_dynamic_feature_metadata(
       Dynamic_Features_Rec *  dfr,
       uint8_t                 feature_code);
@@ -74,7 +91,7 @@ get_dynamic_feature_metadata(
 // satisfies glib signature
 void
 free_feature_metadata(
-      gpointer data);    // i.e. DDCA_Feature_Metadata *
+      gpointer data);    // i.e. Dyn_Feature_Metadata *
 
 void dbgrpt_dynamic_features_rec(
       Dynamic_Features_Rec*   dfr,
