@@ -4,13 +4,14 @@
  * display-specific feature metadata.
  */
 
-// Copyright (C) 2018-2022 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2018-2023 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 #ifndef FEATURE_METADATA_H_
 #define FEATURE_METADATA_H_
 
 /** \cond */
+#include <glib-2.0/glib.h>
 #include <stdbool.h>
 /** \endcond */
 
@@ -18,13 +19,15 @@
 
 #include "util/data_structures.h"
 
+#include "base/dynamic_features.h"
+
 
 /** Simple stripped-down version of Parsed_Nontable_Vcp_Response */
 typedef
 struct {
    DDCA_Vcp_Feature_Code   vcp_code;
-   ushort max_value;
-   ushort cur_value;
+   gushort max_value;
+   gushort cur_value;
    // for new way
    Byte   mh;
    Byte   ml;
@@ -102,16 +105,22 @@ interpret_feature_flags_t(DDCA_Version_Feature_Flags flags);
 void
 dbgrpt_ddca_feature_metadata(DDCA_Feature_Metadata * md, int depth);
 
+
+void
+dbgrpt_dyn_feature_metadata(Dyn_Feature_Metadata * md, int depth);
+
 void
 free_ddca_feature_metadata(DDCA_Feature_Metadata * metadata);
 
 
 // Display_Feature_Metadata
 
-// merges DDCA_Version_Feature_Info, DDCA_Feature_Metadata, Internal_Feature_Metadata
-
 #define DISPLAY_FEATURE_METADATA_MARKER "DFMD"
-/** Internal version of display specific feature metadata, includes formatting functions */
+/** Internal version of display specific feature metadata, includes formatting functions
+ *
+ *  Represents merged internal metadata from vcp_code_tables.c, synthetic metadata,
+ *  and user defined features, for a specific VCP version.
+ * */
 typedef
 struct {
    char                                    marker[4];
@@ -121,7 +130,6 @@ struct {
    char *                                  feature_name;
    char *                                  feature_desc;
    DDCA_Feature_Value_Entry *              sl_values;     /**< valid when DDCA_SIMPLE_NC set */
-   // DDCA_Feature_Value_Entry *           latest_sl_values;
    DDCA_Feature_Flags                      feature_flags;
    Format_Normal_Feature_Detail_Function   nontable_formatter;
    Format_Normal_Feature_Detail_Function2  nontable_formatter_sl;
@@ -149,6 +157,8 @@ DDCA_Feature_Metadata *
 dfm_to_ddca_feature_metadata(Display_Feature_Metadata * dfm);
 
 Display_Feature_Metadata *
-dfm_from_ddca_feature_metadata(DDCA_Feature_Metadata * meta);
+dfm_from_dyn_feature_metadata(Dyn_Feature_Metadata * meta);
+
+void init_feature_metadata();
 
 #endif /* FEATURE_METADATA_H_ */
