@@ -6,27 +6,33 @@
 
 #include "config.h"
 
+#define _GNU_SOURCE
 #include <assert.h>
+#include <dlfcn.h>
 #include <glib-2.0/glib.h>
 #include <string.h>
 #include <sys/types.h>
 
 #include "util/debug_util.h"
 #include "util/glib_util.h"
+#include "util/linux_util.h"
 #include "util/report_util.h"
 #include "util/string_util.h"
+#include "util/timestamp.h"
 
 #include "base/core.h"
+#include "base/core_per_thread_settings.h"
 #include "base/displays.h"
 #include "base/parms.h"
+#include "base/rtti.h"
 #include "base/sleep.h"
-#include "base/thread_retry_data.h"    // temp circular
-#include "base/thread_sleep_data.h"
 
 #include "base/per_thread_data.h"
 
+void ptd_profile_function_report(Per_Thread_Data * ptd, gpointer depth);
+
 // Master table of sleep data for all threads
-GHashTable *    per_thread_data_hash = NULL;
+GHashTable *    per_thread_data_hash = NULL;   // key is thread id
 static GPrivate this_thread_has_lock;
 static GPrivate lock_depth; // GINT_TO_POINTER(0);
 static bool     debug_mutex = false;
@@ -138,23 +144,22 @@ void ptd_cross_thread_operation_block() {
 }
 
 
-// GDestroyNotify: void (*GDestroyNotify) (gpointer data);
-
+// satisfies type GDestroyNotify: void (*GDestroyNotify) (gpointer data);
 void per_thread_data_destroy(void * data) {
    if (data) {
       Per_Thread_Data * ptd = data;
+#ifdef REMOVED_20
       free(ptd->description);
+#endif
+      free(ptd->cur_func);
+      if (ptd->function_stats)
+         g_hash_table_destroy(ptd->function_stats);
       free(ptd);
    }
 }
 
-/** Initialize per_thread_data.c at program startup */
-void init_thread_data_module() {
-   per_thread_data_hash = g_hash_table_new_full(g_direct_hash, NULL, NULL, per_thread_data_destroy);
-   // DBGMSG("per_thead_data_hash = %p", per_thread_data_hash);
-}
 
-void release_thread_data_module() {
+void terminate_per_thread_data() {
    if (per_thread_data_hash) {
       g_hash_table_destroy(per_thread_data_hash);
    }
@@ -164,10 +169,10 @@ void release_thread_data_module() {
 // Locking
 //
 
+#ifdef PTD
 static void ptd_init(Per_Thread_Data * ptd) {
-   tsd_init_thread_sleep_data(ptd);
-   trd_init(ptd);
 }
+#endif
 
 
 /** Gets the #Per_Thread_Data struct for the current thread, using the
@@ -178,7 +183,7 @@ static void ptd_init(Per_Thread_Data * ptd) {
  *
  *  @remark
  *  The structs are maintained centrally rather than using a thread-local pointer
- *  to a block on the heap because the of a problems when the thread is closed.
+ *  to a block on the heap because the of a problem when the thread is closed.
  *  Valgrind complains of access errors for closed threads, even though the
  *  struct is on the heap and still readable.
  */
@@ -199,25 +204,29 @@ Per_Thread_Data * ptd_get_per_thread_data() {
       DBGMSF(debug, "==> Per_Thread_Data not found for thread %d", cur_thread_id);
       data = g_new0(Per_Thread_Data, 1);
       data->thread_id = cur_thread_id;
+      // data->sleep_multiplier = -1.0f;
       g_private_set(&lock_depth, GINT_TO_POINTER(0));
+#ifdef PTD
       ptd_init(data);
       DBGMSF(debug, "Initialized: %s. thread_sleep_data_defined: %s. thread_retry_data_defined; %s",
            sbool(data->initialized),
            sbool( data->thread_sleep_data_defined), sbool( data->thread_retry_data_defined));
+#endif
 
       g_hash_table_insert(per_thread_data_hash,
                           GINT_TO_POINTER(cur_thread_id),
                           data);
       DBGMSF(debug, "Created Per_Thread_Data struct for thread id = %d", data->thread_id);
       DBGMSF(debug, "per_thread_data_hash size=%d", g_hash_table_size(per_thread_data_hash));
-      // if (debug)
-      //   dbgrpt_per_thread_data(data, 1);
+      if (debug)
+        dbgrpt_per_thread_data(data, 1);
    }
    // ptd_unlock_if_needed(this_function_owns_lock);
    return data;
 }
 
 
+#ifdef REMOVED_20
 // Thread description operations always operate on the Per_Thread_Data
 // instance for the currently executing thread.
 
@@ -227,7 +236,7 @@ void ptd_set_thread_description(const char * description) {
    // DBGMSG("thread: %d, description: %s", ptd->thread_id, description);
    if (ptd->description)
       free(ptd->description);
-   ptd->description = strdup(description);
+   ptd->description = g_strdup(description);
 }
 
 
@@ -236,14 +245,12 @@ void ptd_append_thread_description(const char * addl_description) {
    Per_Thread_Data *  ptd = ptd_get_per_thread_data();
    // DBGMSG("ptd->description = %s, addl_descripton = %s", ptd->description, addl_description);
    if (!ptd->description)
-      ptd->description = strdup(addl_description);
+      ptd->description = g_strdup(addl_description);
    else if (str_contains(ptd->description, addl_description) < 0) {
       char * s = ptd->description;
       ptd->description = g_strdup_printf("%s; %s", ptd->description, addl_description);
       free(s);
    }
-
-   // DBGMSG("Final ptd->description = %s", ptd->description);
 }
 
 
@@ -260,19 +267,7 @@ const char * ptd_get_thread_description_t() {
    }
    return buf;
 }
-
-
-char * int_array_to_string(uint16_t * start, int ct) {
-   int bufsz = ct*10;
-   char * buf = calloc(1, bufsz);
-   int next = 0;
-   for (int ctr =0; ctr < ct && next < bufsz; ctr++) {
-      g_snprintf(buf+next, bufsz-next,"%s%d", (next > 0) ? ", " : "", *(start+ctr) );
-      next = strlen(buf);
-   }
-   // DBGMSG("start=%p, ct=%d, returning %s", start, ct, buf);
-   return buf;
-}
+#endif
 
 
 /** Output a debug report of a #Per_Thread_Data struct.
@@ -285,58 +280,16 @@ char * int_array_to_string(uint16_t * start, int ct) {
 void dbgrpt_per_thread_data(Per_Thread_Data * data, int depth) {
    int d1 = depth+1;
    rpt_structure_loc("Per_Thread_Data", data, depth);
- //rpt_int( "sizeof(Per_Thread_Data)",  NULL, sizeof(Per_Thread_Data),   d1);
-   rpt_bool("initialized",                NULL, data->initialized,           d1);
-   rpt_int( "thread_id",                  NULL, data->thread_id,             d1);
+   rpt_vstring(d1,"initialized                %s", sbool(data->initialized));
+   rpt_vstring(d1,"thread_id                  %d", data->thread_id);
+#ifdef REMOVED_20
    rpt_vstring(d1,"description                %s", data->description);
-   rpt_bool("sleep data initialized" ,    NULL, data->thread_sleep_data_defined, d1);
-
-   // Sleep multiplier adjustment:
-   rpt_vstring(d1, "sleep-multiplier value:           %15.2f", data->sleep_multiplier_factor);
-   rpt_int("sleep_multiplier_ct",         NULL, data->sleep_multiplier_ct,        d1);
-   rpt_vstring(d1, "sleep_multiplier_changer_ct:      %15d",   data->sleep_multipler_changer_ct);
-   rpt_vstring(d1, "highest_sleep_multiplier_ct:      %15d",   data->highest_sleep_multiplier_ct);
-
-   rpt_vstring(d1,"cur_dh:                 %s", dh_repr(data->cur_dh) );
-
-   // Dynamic sleep adjustment:
-   rpt_bool("dynamic_sleep_enabled",      NULL, data->dynamic_sleep_enabled,     d1);
-   rpt_int("cur_ok_status_count",         NULL, data->cur_ok_status_count,       d1);
-   rpt_int("cur_error_status_count",      NULL, data->cur_error_status_count,    d1);
-   rpt_int("total_ok_status_count",       NULL, data->total_ok_status_count,     d1);
-   rpt_int("total_error",                 NULL, data->total_error_status_count,  d1);
-   rpt_int("other_status_ct",             NULL, data->total_other_status_ct,     d1);
-
-   rpt_int("calls_since_last_check",      NULL, data->calls_since_last_check,    d1);
-   rpt_int("total_adjustment_checks",     NULL, data->total_adjustment_checks,   d1);
-   rpt_int("adjustment_ct",               NULL, data->total_adjustment_ct,       d1);
-// rpt_int("max_adjustment_ct",           NULL, data->total_max_adjustment_ct,   d1);
-// rpt_int("non_adjustment_ct",           NULL, data->total_non_adjustment_ct,   d1);
-
-   rpt_vstring(d1, "cur_sleep_adjustmet_factor     %15.2f", data->cur_sleep_adjustment_factor);
-// rpt_vstring(d1, "thread_adjustment_increment        %15.2f", data->thread_adjustment_increment);
-
-   // Maxtries history
-   rpt_bool("retry data initialized"    , NULL, data->thread_retry_data_defined, d1);
-
-   rpt_vstring(d1, "Highest maxtries:                  %d,%d,%d,%d",
-                    data->highest_maxtries[0], data->highest_maxtries[1],
-                    data->highest_maxtries[2], data->highest_maxtries[3]);
-   rpt_vstring(d1, "Current maxtries:                  %d,%d,%d,%d",
-                    data->current_maxtries[0], data->current_maxtries[1],
-                    data->current_maxtries[2], data->current_maxtries[3]);
-   rpt_vstring(d1, "Lowest maxtries:                   %d,%d,%d,%d",
-                    data->lowest_maxtries[0], data->lowest_maxtries[1],
-                    data->lowest_maxtries[2], data->lowest_maxtries[3]);
-
-   for (int retry_type = 0; retry_type < 4; retry_type++) {
-      int upper_bound = data->highest_maxtries[retry_type] + 1;
-      assert(upper_bound <= MAX_MAX_TRIES + 1);
-      char * buf = int_array_to_string( data->try_stats[retry_type].counters, upper_bound);
-      rpt_vstring(d1, "try_stats[%d=%-27s].counters = %s",
-                      retry_type, retry_type_name(retry_type), buf);
-      free(buf);
-   }
+#endif
+   rpt_vstring(d1,"cur_dh:                    %s", dh_repr(data->cur_dh) );
+   rpt_vstring(d1,"cur_func                   %s", data->cur_func);
+   rpt_vstring(d1,"cur_start                  %"PRIu64, data->cur_start);
+   rpt_vstring(d1,"function profile stats: ");
+   ptd_profile_function_report(data, GINT_TO_POINTER(depth+1));
 }
 
 
@@ -425,6 +378,8 @@ void ptd_thread_summary(Per_Thread_Data * ptd, void * arg) {
    char header[100];
    g_snprintf(header, 100, "Thread %d: ", ptd->thread_id);
 
+   rpt_vstring(d1, "%s", header);
+#ifdef REMOVED_20
    int hdrlen = strlen(header);
    if (!ptd->description)
       rpt_vstring(d1, "%s", header);
@@ -445,9 +400,9 @@ void ptd_thread_summary(Per_Thread_Data * ptd, void * arg) {
       }
       ntsa_free(pieces, true);
    }
+#endif
 }
 
-
 /** Emits a brief summary (thread id and description) for each
  * #Per_Thread_Data instance.
  *
@@ -463,7 +418,7 @@ void ptd_list_threads(int depth) {
    rpt_nl();
 }
 
-
+#ifdef PTD
 void report_all_thread_status_counts(int depth) {
    bool debug = false;
    DBGMSF(debug, "Starting");
@@ -471,4 +426,328 @@ void report_all_thread_status_counts(int depth) {
    rpt_nl();
    DBGMSF(debug, "Done");
 }
+#endif
 
+
+#ifdef TIMING_TEST
+// test timing for getting thread id
+// conclusion:
+//   at   10 calls using cached value is   2 times as fast
+//   at  100 calls using cached value is  11 times as fast
+//   at 1000 calls using cached value is  24 times as fast
+void test_get_thread_id() {
+   // int loopct = 10 * 1000 * 1000; // 10 million
+   int loopct = 1000;
+   intmax_t threadid;
+   intmax_t start = cur_realtime_nanosec();
+   for (int ndx = 0; ndx < loopct; ndx++) {
+      threadid = get_thread_id();
+   }
+   intmax_t end = cur_realtime_nanosec();
+   intmax_t elapsed = (end-start)/1000;
+   printf("threadid = %ld,  direct:         %10"PRIu64"\n", threadid, elapsed);
+
+   start = cur_realtime_nanosec();
+   Thread_Output_Settings* ts = get_thread_settings();
+   ts->tid = get_thread_id();
+   for (int ndx = 0; ndx < loopct; ndx++) {
+      Thread_Output_Settings* ts = get_thread_settings();
+      threadid = ts->tid;
+   }
+   end = cur_realtime_nanosec();
+   intmax_t elapsed2 = (end-start)/1000;
+   printf("threadid = %ld,  thread_setings: %10"PRIu64"\n", threadid, elapsed2);
+   int ratio = elapsed/elapsed2;
+   printf("ratio: %d\n", ratio);
+}
+#endif
+
+
+//
+// Collect Performance Stats
+//
+
+bool ptd_api_profiling_enabled = false;
+
+
+void ptd_profile_function_stats_key_destroy(void * data) {   // GDestroyNotify
+   // key is function name, type char*
+   g_free(data);
+}
+
+
+void ptd_profile_function_stats_value_destroy(void * data) {  // GDestroyNofify
+   // value is Per_Thread_Function_Stats*
+   Per_Thread_Function_Stats * stats = (Per_Thread_Function_Stats *) data;
+   free(stats->function);
+   free(stats);
+}
+
+
+void ptd_profile_reset_thread_stats(Per_Thread_Data * ptd, void * data) {
+   bool debug = false;
+   DBGMSF(debug, "Starting. ptd=%p, data=%p", ptd, data);
+   ptd_cross_thread_operation_block();    // wait for any cross-thread operation to complete
+   if (ptd->function_stats)
+      g_hash_table_remove_all(ptd->function_stats);
+   DBGMSF(debug, "Done");
+}
+
+
+void ptd_profile_reset_all_stats()  {
+   ptd_cross_thread_operation_start();
+   ptd_apply_all(ptd_profile_reset_thread_stats, NULL);
+   ptd_cross_thread_operation_end();
+}
+
+
+// gets the Function_Stats_Hash table for the current thread
+static inline Function_Stats_Hash * ptd_profile_get_stats() {
+   // does this function need to block?
+   Per_Thread_Data *  ptd = ptd_get_per_thread_data();
+   if (!ptd->function_stats) {
+      ptd->function_stats = g_hash_table_new_full(g_str_hash, g_str_equal,
+            ptd_profile_function_stats_key_destroy, ptd_profile_function_stats_value_destroy);
+   }
+   return ptd->function_stats;
+}
+
+
+void ptd_profile_function_start(const char * func) {
+   // bool debug = false;
+   Per_Thread_Data *  ptd = ptd_get_per_thread_data();
+   if (!ptd->cur_func) {
+      ptd->cur_func = strdup(func);
+      ptd->cur_start = cur_realtime_nanosec();
+   }
+}
+
+
+void ptd_profile_function_end(const char * func) {
+   bool debug = false;
+   Per_Thread_Data *  ptd = ptd_get_per_thread_data();
+   DBGF(debug, "Starting. func=%s, cur_func=%s", func, ptd->cur_func);
+   if (streq(ptd->cur_func, func)) {
+      Function_Stats_Hash * stats_table = ptd_profile_get_stats();
+      Per_Thread_Function_Stats * function_stats = g_hash_table_lookup(stats_table, func);
+      DBGF(debug, "       stats_table=%p, function_stats=%p", stats_table, function_stats);
+      if (!function_stats) {
+         function_stats = calloc(1, sizeof(Per_Thread_Function_Stats));
+         function_stats->function = strdup(func);
+         g_hash_table_insert(stats_table, strdup(func), function_stats);
+      }
+      function_stats->total_calls++;
+      function_stats->total_nanosec = (cur_realtime_nanosec() - ptd->cur_start);
+      free(ptd->cur_func);
+      ptd->cur_func = NULL;
+   }
+}
+
+
+//
+// Stats Utility Functions
+//
+#ifdef NOT_NEEDED
+const char * func_addr_to_name(gpointer addr) {
+#define ALT
+#ifdef ALT
+   const char * name = "Unknown";
+   Dl_info info;
+   int rc = dladdr(addr, &info);
+   if (rc) {
+      name = info.dli_sname;
+   }
+#else
+   char * name =  rtti_get_func_name_by_addr(addr);
+   if (!name)
+      name = "Unknown";
+#endif
+   return name;
+}
+#endif
+
+
+//
+// Create summary table
+//
+
+/** Adds the stats for one function on a thread to the summary record for all
+ *  threads for that function.
+ *
+ *  Satisfies type GHFunc
+ *
+ *  @oaram  key   function name (char*)
+ *  @param  value pointer to Per_Thread_Function_Stats struct for the function on a thread
+ *  @data   pointer to summary hash table of function names -> Per_Thread_Function_Stats struct
+ */
+void add_one_func_to_summary(gpointer key, gpointer cur_func_stats, gpointer data) {
+   bool debug = false;
+   DBGMSF(debug, "Starting. key=%p -> %s, cur_func_stats=%p, data=%p", key, key, cur_func_stats, data);
+   Per_Thread_Function_Stats * cfs = (Per_Thread_Function_Stats*) cur_func_stats;
+   assert(streq(key,cfs->function));
+   Function_Stats_Hash * summary_table = (GHashTable *) data;
+   Per_Thread_Function_Stats * cur_summary_entry =
+         g_hash_table_lookup(summary_table, cfs->function);
+   if (!cur_summary_entry) {
+      DBGMSF(debug, "      Per_Thread_Function_Stats not found for %s", cfs->function);
+      cur_summary_entry = calloc(1, sizeof(Per_Thread_Function_Stats));
+      cur_summary_entry->function = strdup(cfs->function);
+      g_hash_table_insert(summary_table, strdup(cfs->function), cur_summary_entry);
+   }
+   cur_summary_entry->total_calls += cfs->total_calls;
+   cur_summary_entry->total_nanosec += cfs->total_nanosec;
+   DBGMSF(debug, "Done.   cur_summary_entry=%p, total_calls=%d, total_nanosec=%d, function=%p->%s",
+         cur_summary_entry, cur_summary_entry->total_calls, cur_summary_entry->total_nanosec,
+         cur_summary_entry->function, cur_summary_entry->function);
+}
+
+
+/** Adds the stats for all functions on single thread to the summary record
+ *  for all functions on all threads
+ *
+ *  @param ptd  pointer to Per_Thread_Data struct for a thread
+ *  @param data pointer to summary
+ */
+void ptd_add_stats(Per_Thread_Data * ptd, void * data) {
+   bool debug = false;
+   DBGMSF(debug, "Starting. ptd=%p, data=%p", ptd, data);
+   g_hash_table_foreach(ptd->function_stats, add_one_func_to_summary, data);
+   DBGMSF(debug, "Done");
+}
+
+
+/** Creates a hash table with the total stats for each function across all threads
+ *
+ *  @return hash table with key = function name, value = Per_Thread_Function_Stats*
+ */
+Function_Stats_Hash * summarize_per_thread_stats() {
+   Function_Stats_Hash * summary = g_hash_table_new(g_str_hash, g_str_equal);
+   ptd_apply_all(ptd_add_stats, summary);   // ptd_apply_all manages locking
+   return summary;
+}
+
+
+/** Reports stats for one function on one thread, or reports the
+ *  summary record for one function on all threads
+ *
+ *  Satisfies typedef GHFunc
+ *
+ *  @oaram  key   function name (char*)
+ *  @param  value pointer to Per_Thread_Function_Stats struct for the function
+ *  @data   depth logical indentation depth
+ */
+static void ptd_report_one_func0(Per_Thread_Function_Stats * pts, void * arg) {
+   // bool debug = false;
+   int depth = GPOINTER_TO_INT(arg);
+   // DBGMSF(debug, "pts=%p, pts->total_calls=%d", pts, pts->total_calls);
+   // DBGMSF(debug, "pts=%p, pts->total_millisec=%d", pts, pts->total_millisec);
+   // DBGMSF(debug, "pts=%p, pts->function=%p", pts, pts->function);
+   rpt_vstring(depth, "%5d  %8"PRIu64"  %s",
+         pts->total_calls, (pts->total_nanosec+500)/1000, pts->function);
+}
+
+static void ptd_report_one_func(gpointer key, gpointer value, gpointer user_data) {
+   bool debug = false;
+   DBGMSF(debug, "gpointer=%p->%s, value=%p, user_data=%p", key, key, value, user_data);
+   Per_Thread_Function_Stats * pts = (Per_Thread_Function_Stats *) value;
+   //const char * func_name = func_addr_to_name(pts->function);
+   // DBGMSF(debug, "pts=%p, pts->total_calls=%d", pts, pts->total_calls);
+   // DBGMSF(debug, "pts=%p, pts->total_millisec=%d", pts, pts->total_millisec);
+   // DBGMSF(debug, "pts=%p, pts->function=%p", pts, pts->function);
+   ptd_report_one_func0(pts, user_data);
+}
+
+
+// Reports function stats for a single thread
+void ptd_profile_function_report(Per_Thread_Data * ptd, gpointer depth) {
+   int d0 = GPOINTER_TO_INT(depth);
+   // int d1 = d0 + 1;
+   rpt_vstring(d0, "Per-Thread Function Profile Report for thread %d:", ptd->thread_id);
+   if (ptd->function_stats) {
+      rpt_label(d0, "Count  Microsec  Function Name");
+      g_hash_table_foreach(ptd->function_stats, ptd_report_one_func, depth);
+   }
+   else
+      rpt_label(d0, "No function stats");
+   rpt_nl();
+}
+
+
+// Apply a function to all Per_Thread_Data records
+// typedef void (*Ptd_Func)(Per_Thread_Data * data, void * arg);   // Template for function to apply
+
+void ptd_profile_report_all_threads(int depth) {
+   bool debug = false;
+   DBGMSF(debug, "Starting");
+   ptd_apply_all_sorted(ptd_profile_function_report, GINT_TO_POINTER(depth));
+   // rpt_nl();
+   DBGMSF(debug, "Done");
+}
+
+
+gint
+gaux_scomp(
+
+      gconstpointer a,
+      gconstpointer b)
+{
+   // DBGMSG("a=%p -> %s", a,a);
+   // DBGMSG("b=%p -> %s", b,b);
+   return g_ascii_strcasecmp(a,b);
+}
+
+
+
+typedef void (*Ptd_Stats_Func)(Per_Thread_Function_Stats * data, void * arg);   // Template for function to apply
+
+/** Apply a given function to all #Per_Thread_Data structs, ordered by thread id.
+ *  Note that this report includes structs for threads that have been closed.
+ *
+ *  @param func function to apply
+ *  @param arg pointer or integer value
+ *
+ *  This is a multi-instance operation.
+ */
+void ptd_profile_apply_all_sorted(Function_Stats_Hash * function_stats_hash,
+                            Ptd_Stats_Func func, void * arg) {
+   bool debug = false;
+   DBGMSF(debug, "Starting");
+   assert(function_stats_hash);
+
+   DBGMSF(debug, "hash table size = %d", g_hash_table_size(function_stats_hash));
+   GList * keys = g_hash_table_get_keys (function_stats_hash);
+   GList * new_head = g_list_sort(keys, gaux_scomp);
+   GList * l;
+   for (l = new_head; l != NULL; l = l->next) {
+      char * key = (char*) l->data;
+      DBGMSF(debug, "Key: %s", key);
+      Per_Thread_Function_Stats * data = g_hash_table_lookup(function_stats_hash, l->data);
+      assert(data);
+      func(data, arg);
+   }
+   g_list_free(new_head);   // would keys also work?
+
+   DBGMSF(debug, "Done");
+}
+
+
+void ptd_profile_report_stats_summary(int depth) {
+   bool debug = false;
+   DBGMSF(debug, "Starting");
+   // int d1 = depth+1;
+   rpt_label(depth, "Summary Function Profile Report for all Threads");
+   rpt_label(depth, "Count  Microsec  Function Name");
+   Function_Stats_Hash * summary_stats = summarize_per_thread_stats();
+   DBGMSF(debug, "    summary_stats=%p", summary_stats);
+   // g_hash_table_foreach(summary_stats, ptd_report_one_func, GINT_TO_POINTER(depth));
+   ptd_profile_apply_all_sorted(summary_stats, ptd_report_one_func0, GINT_TO_POINTER(depth));
+   DBGMSF(debug, "Done");
+}
+
+
+/** Initialize per_thread_data.c at program startup */
+void init_per_thread_data() {
+   per_thread_data_hash = g_hash_table_new_full(g_direct_hash, NULL, NULL, per_thread_data_destroy);
+   // DBGMSG("per_thead_data_hash = %p", per_thread_data_hash);
+   // test_get_thread_id();
+}
