@@ -45,14 +45,19 @@ static Cmd_Desc cmdinfo[] = {
    {CMDID_USBENV,       "usbenvironment", 6,  0,       0,                  Option_None},
 #endif
    {CMDID_VCPINFO,      "vcpinfo",        5,  0,       MAX_GETVCP_VALUES,  Option_None},
-#ifdef WATCH_COMMAND
+// #ifdef WATCH_COMMAND
    {CMDID_READCHANGES,  "watch",          3,  0,       0,                  Option_Explicit_Display},
-#endif
-#ifdef USE_USB
+//#endif
    {CMDID_CHKUSBMON,    "chkusbmon",      3,  1,       1,                  Option_None},
-#endif
    {CMDID_PROBE,        "probe",          5,  0,       0,                  Option_Explicit_Display},
    {CMDID_SAVE_SETTINGS,"scs",            3,  0,       0,                  Option_Explicit_Display},
+   {CMDID_DISCARD_CACHE,"discard",        4,  1,       2,                  Option_None},
+   {CMDID_LIST_RTTI,    "traceable-functions",
+                                          2,  0,       0,                  Option_None},
+   {CMDID_C1,           "c1",             2,  0,       9,                  Option_None},
+   {CMDID_C2,           "c2",             2,  0,       9,                  Option_None},
+   {CMDID_C3,           "c3",             2,  0,       9,                  Option_None},
+   {CMDID_C4,           "c4",             2,  0,       9,                  Option_None},
 };
 static int cmdct = sizeof(cmdinfo)/sizeof(Cmd_Desc);
 
@@ -155,7 +160,22 @@ bool parse_dot_separated_arg(const char * val, int * piAdapterIndex, int * piDis
 }
 
 
-bool parse_colon_separated_arg(const char * val, int * pv1, int * pv2) {
+bool parse_colon_separated_vid_pid(const char * val, uint16_t * pv1, uint16_t * pv2) {
+   Null_Terminated_String_Array parts = strsplit(val, ":");
+   bool ok = false;
+   if (ntsa_length(parts) == 2 && strlen(parts[0]) == 4 && strlen(parts[1]) == 4) {
+      ok = true;
+      // DBGMSG("parts[0] = |%s|", parts[0]);
+      // DBGMSG("parts[1] = |%s|",  parts[1]);
+      ok &= hhs4_to_uint16(parts[0], pv1);
+      ok &= hhs4_to_uint16(parts[1], pv2);
+   }
+   ntsa_free(parts,  true);
+   // DBGMSG("Returning %s *pv1=0x%04x, *pv2=0x%04x", sbool(ok), *pv1, *pv2);
+   return ok;
+}
+
+bool parse_colon_separated_arg(const char * val, int* pv1, int* pv2) {
    int rc = sscanf(val, "%d:%d", pv1, pv2);
    // DBGMSG("val=|%s| sscanf() returned %d  ", val, rc );
    bool ok = (rc == 2);
@@ -434,7 +454,7 @@ char * commands_list_help =
 #endif
 #ifdef ENABLE_ENVCMDS
        "   environment                             Probe execution environment\n"
-#ifdef USE_USB
+#ifdef ENABLE_USB
        "   usbenv                                  Probe for USB connected monitors\n"
 #endif
 #endif
@@ -442,13 +462,14 @@ char * commands_list_help =
 #ifdef ENABLE_ENVCMDS
        "   interrogate                             Report everything possible\n"
 #endif
-#ifdef USE_USB
+#ifdef ENABLE_USB
        "   chkusbmon                               Check if USB device is monitor (for UDEV)\n"
 #endif
 #ifdef DEPRECATED
        "   watch                                   Watch display for reported changes (under development)\n"
 #endif
-      "\n";
+       "   discard (all|capabilities|dsa) cache(s) Delete cache files\n"
+       "   traceable-functions                     List traceable functions\n";
 
 #ifdef OLD
 char * command_argument_help =
@@ -495,9 +516,10 @@ char * monitor_selection_option_help =
        "  The monitor to be communicated with can be specified using the following options:\n"
        "  --display <display_number>, where <display_number> ranges from 1 to the number of\n"
        "    displays detected\n"
-       "  --bus <bus number>, for /dev/"I2C"-<bus number>\n"
-#ifdef USE_USB
+       "  --bus <i2c bus number>, /dev/"I2C" bus number\n"
+#ifdef ENABLE_USB
        "  --usb <usb bus number>.<usb device number>, for monitors communicating via USB\n"
+       "  --hiddev <hiddev device number>, for monitors communicationg via USB\n"
 #endif
        "  --edid <hex string>, where <hex string> is a 256 hex character representation of the\n"
        "          128 byte first block of the EDID\n"
@@ -547,7 +569,7 @@ char * stats_multiple_call_option_help =
        "Stats:\n"
        "  The argument to --stats is a statistics class.  Specify the --stats option multiple\n"
        "  times to activate multiple statistics classes, e.g. \"--stats calls --stats errors\"\n"
-       "  Valid statistics classes are:  TRY, TRIES, ERRS, ERRORS, CALLS, ALL.\n"
+       "  Valid statistics classes are:  TRY, TRIES, ERRS, ERRORS, CALLS, ELAPSED, ALL.\n"
        "  Statistics class names are not case sensitive and can abbreviated to 3 characters.\n"
        "  If no argument is specified, or ALL is specified, then all statistics classes are\n"
        "  output.\n"
