@@ -1,9 +1,10 @@
 /** @file ddc_display_ref_reports.c
+ *
  *  Report functions factored out of ddc_displays.c due to size of that file.
  *  ddc_display_ref_reports.c and ddc_displays.c cross-reference each other.
  */
 
-// Copyright (C) 2014-2022 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2023 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 #include "config.h"
@@ -17,18 +18,21 @@
 #include "util/data_structures.h"
 #include "util/report_util.h"
 #include "util/string_util.h"
+#include "util/sysfs_util.h"
 
 #include "public/ddcutil_types.h"
 
 #include "base/core.h"
+#include "base/dsa2.h"
 #include "base/monitor_model_key.h"
 #include "base/monitor_quirks.h"
+#include "base/per_display_data.h"
 #include "base/rtti.h"
 
 #include "i2c/i2c_bus_core.h"
 #include "i2c/i2c_sysfs.h"
 
-#ifdef USE_USB
+#ifdef ENABLE_USB
 #include "usb/usb_displays.h"
 #endif
 
@@ -43,6 +47,9 @@
 // Default trace class for this file
 static const DDCA_Trace_Group TRACE_GROUP = DDCA_TRC_DDC;
 
+
+struct Results_Table;
+
 //
 // Display_Ref reports
 //
@@ -150,6 +157,30 @@ get_controller_mfg_string_t(Display_Handle * dh) {
 }
 
 
+static void report_drm_dpms_status(int depth, const char * connector_name) {
+   char * drm_dpms = NULL;
+   RPT_ATTR_TEXT(-1, &drm_dpms, "/sys/class/drm", connector_name, "dpms");
+   if (drm_dpms && !streq(drm_dpms,"On")) {
+      rpt_vstring(1, "DRM reports the monitor is in a DPMS sleep state (%s).", drm_dpms);
+      free(drm_dpms);
+   }
+
+   char * drm_enabled = NULL;
+   RPT_ATTR_TEXT(-1, &drm_enabled, "/sys/class/drm", connector_name, "enabled");
+   if (drm_enabled && !streq(drm_enabled, "enabled")) {
+      rpt_vstring(1, "DRM reports the monitor is %s.", drm_enabled);
+      free(drm_enabled);
+   }
+
+   char * drm_status = NULL;
+   RPT_ATTR_TEXT(-1, &drm_status, "/sys/class/drm", connector_name, "status");
+   if (drm_status && !streq(drm_status, "connected")) {
+      rpt_vstring(1, "DRM reports the monitor status is %s.", drm_status);
+      free(drm_status);
+   }
+}
+
+
 /** Shows information about a display, specified by a #Display_Ref
  *
  *  This function is used by the DISPLAY command.
@@ -173,6 +204,9 @@ ddc_report_display_by_dref(Display_Ref * dref, int depth) {
    TRACED_ASSERT(dref && memcmp(dref->marker, DISPLAY_REF_MARKER, 4) == 0);
    int d1 = depth+1;
 
+   I2C_Bus_Info * businfo = (dref->io_path.io_mode == DDCA_IO_I2C) ? dref->detail : NULL;
+   TRACED_ASSERT(businfo && memcmp(businfo, I2C_BUS_INFO_MARKER, 4) == 0);
+
    switch(dref->dispno) {
    case DISPNO_BUSY:       // -4
       rpt_vstring(depth, "Busy display");
@@ -182,6 +216,8 @@ ddc_report_display_by_dref(Display_Ref * dref, int depth) {
       break;
    case DISPNO_PHANTOM:    // -2
       rpt_vstring(depth, "Phantom display");
+      rpt_vstring(d1, "Associated non-phantom display: %s",
+            dref_short_name_t(dref->actual_display));
       break;
    case DISPNO_INVALID:   // -1
       rpt_vstring(depth, "Invalid display");
@@ -196,17 +232,11 @@ ddc_report_display_by_dref(Display_Ref * dref, int depth) {
    switch(dref->io_path.io_mode) {
    case DDCA_IO_I2C:
       {
-         I2C_Bus_Info * curinfo = dref->detail;
-         TRACED_ASSERT(curinfo && memcmp(curinfo, I2C_BUS_INFO_MARKER, 4) == 0);
-
-         i2c_report_active_display(curinfo, d1);
+         i2c_report_active_bus(businfo, d1);
       }
       break;
-   case DDCA_IO_ADL:
-      PROGRAM_LOGIC_ERROR("ADL implementation removed");
-      break;
    case DDCA_IO_USB:
-#ifdef USE_USB
+#ifdef ENABLE_USB
       usb_show_active_display_by_dref(dref, d1);
 #else
       PROGRAM_LOGIC_ERROR("ddcutil not built with USB support");
@@ -214,13 +244,34 @@ ddc_report_display_by_dref(Display_Ref * dref, int depth) {
       break;
    }
 
-   TRACED_ASSERT(dref->flags & DREF_DDC_COMMUNICATION_CHECKED);
+   TRACED_ASSERT(dref->flags & (DREF_DDC_COMMUNICATION_CHECKED|DREF_DPMS_SUSPEND_STANDBY_OFF));
 
    DDCA_Output_Level output_level = get_output_level();
 
    if (output_level >= DDCA_OL_NORMAL) {
+
       if (!(dref->flags & DREF_DDC_COMMUNICATION_WORKING) ) {
-         rpt_vstring(d1, "DDC communication failed");
+         char * drm_status  = NULL;
+         char * drm_dpms    = NULL;
+         char * drm_enabled = NULL;
+         char * drm_connector_name = i2c_get_drm_connector_name(businfo);
+         if (drm_connector_name) { // would be null for a non drm driver
+            RPT_ATTR_TEXT(-1, &drm_dpms,    "/sys/class/drm", drm_connector_name, "dpms");
+            RPT_ATTR_TEXT(-1, &drm_status,  "/sys/class/drm", drm_connector_name, "status");  // connected, disconnected
+            RPT_ATTR_TEXT(-1, &drm_enabled, "/sys/class/drm", drm_connector_name, "enabled");  //enabled, disabled
+         }
+
+         I2C_Bus_Info * bus_info = dref->detail;
+         if (!(bus_info->flags & I2C_BUS_LVDS_OR_EDP)) {
+            char * s = NULL;
+            if (dref->communication_error_summary) {
+               s = g_strdup_printf("(getvcp of feature x10 returned %s)", dref->communication_error_summary);
+               rpt_vstring(d1, "DDC communication failed. %s", s);
+            }
+            else
+               rpt_vstring(d1, "DDC communication failed");
+            free(s);
+         }
          char msgbuf[100] = {0};
          char * msg = NULL;
          if (dref->dispno == DISPNO_PHANTOM) {
@@ -234,16 +285,32 @@ ddc_report_display_by_dref(Display_Ref * dref, int depth) {
                msg = "Use non-phantom device";
             }
          }
-         else {
+         else { // non-phantom
             if (dref->io_path.io_mode == DDCA_IO_I2C)
             {
-                I2C_Bus_Info * curinfo = dref->detail;
-                if (curinfo->flags & I2C_BUS_EDP)
+#ifdef OLD
+                if (businfo->flags & I2C_BUS_EDP)
                     msg = "This is an eDP laptop display. Laptop displays do not support DDC/CI.";
-                else if (curinfo->flags & I2C_BUS_LVDS)
+                else if (businfo->flags & I2C_BUS_LVDS)
                      msg = "This is a LVDS laptop display. Laptop displays do not support DDC/CI.";
-                else if ( is_embedded_parsed_edid(dref->pedid) )
+                else if ( is_laptop_parsed_edid(dref->pedid) )
                     msg = "This appears to be a laptop display. Laptop displays do not support DDC/CI.";
+#endif
+                if (businfo->flags & I2C_BUS_LVDS_OR_EDP)
+                   msg = "This is a laptop display.  Laptop displays do not support DDC/CI";
+                else if (businfo->flags & I2C_BUS_APPARENT_LAPTOP)
+                   msg = "This appears to be a laptop display.  Laptop displays do not support DDC/CI";
+                else if (drm_dpms || drm_status || drm_enabled) {
+                   if (drm_dpms && !streq(drm_dpms,"On")) {
+                      rpt_vstring(d1, "DRM reports the monitor is in a DPMS sleep state (%s).", drm_dpms);
+                   }
+                   if (drm_enabled && !streq(drm_enabled,"enabled")) {
+                      rpt_vstring(d1, "DRM reports the monitor is %s.", drm_enabled);
+                   }
+                   if (drm_status && !streq(drm_status, "connected")) {
+                      rpt_vstring(d1, "DRM reports the monitor status is %s.", drm_status);
+                   }
+                }
                 // else if ( curinfo->flags & I2C_BUS_BUSY) {
                 else if ( dref->dispno == DISPNO_BUSY) {
                    rpt_label(d1, "I2C device is busy");
@@ -270,23 +337,29 @@ ddc_report_display_by_dref(Display_Ref * dref, int depth) {
 // #endif
                 }
             }
-#ifdef TMI
-            if (output_level >= DDCA_OL_VERBOSE) {
-               if (!msg) {
-                  msg = "Is DDC/CI enabled in the monitor's on-screen display?";
-               }
-            }
-#endif
          }
          if (msg) {
             rpt_vstring(d1, msg);
+            if (dref->dispno > 0 && (dref->flags & DREF_DPMS_SUSPEND_STANDBY_OFF)) {
+               report_drm_dpms_status(d1, businfo->drm_connector_name);
+            }
          }
-      }
+      }         // communication not working
+
       else {    // communication working
+         // if (dref->dispno == DISPNO_PHANTOM)
+         //    rpt_vstring(d1, "Associated non-phantom display: %s", dref_repr_t(dref->actual_display));
+         if (dref->flags & DREF_DPMS_SUSPEND_STANDBY_OFF) {
+            report_drm_dpms_status(1, businfo->drm_connector_name);
+            rpt_label(1, "DDC communication appears to work, but output is likely invalid.");
+         }
+         bool comm_error_occurred = false;
          DDCA_MCCS_Version_Spec vspec = get_vcp_version_by_dref(dref);
          // DBGMSG("vspec = %d.%d", vspec.major, vspec.minor);
-         if ( vspec.major   == 0)
+         if ( vspec.major   == 0) {
             rpt_vstring(d1, "VCP version:         Detection failed");
+            comm_error_occurred = true;
+         }
          else
             rpt_vstring(d1, "VCP version:         %d.%d", vspec.major, vspec.minor);
 
@@ -294,27 +367,51 @@ ddc_report_display_by_dref(Display_Ref * dref, int depth) {
             // n. requires write access since may call get_vcp_value(), which does a write
             Display_Handle * dh = NULL;
             DBGMSF(debug, "Calling ddc_open_display() ...");
-            Public_Status_Code psc = ddc_open_display(dref, CALLOPT_ERR_MSG, &dh);
-            if (psc != 0) {
-               rpt_vstring(d1, "Error opening display %s, error = %s",
-                                  dref_short_name_t(dref), psc_desc(psc));
+            Error_Info * err = ddc_open_display(dref, CALLOPT_NONE, &dh);
+            if (err) {
+               rpt_vstring(d1, "Error opening display %s: %s",
+                                  dref_short_name_t(dref), psc_desc(err->status_code));
+               comm_error_occurred = true;
+               errinfo_free(err);
+               err = NULL;
             }
             else {
                // display controller mfg, firmware version
                rpt_vstring(d1, "Controller mfg:      %s", get_controller_mfg_string_t(dh) );
                rpt_vstring(d1, "Firmware version:    %s", get_firmware_version_string_t(dh));
                DBGMSF(debug, "Calling ddc_close_display()...");
-               ddc_close_display(dh);
+               ddc_close_display_wo_return(dh);
             }
 
             if (dref->io_path.io_mode != DDCA_IO_USB) {
-               rpt_vstring(d1, "Monitor returns DDC Null Response for unsupported features: %s",
-                                  sbool(dref->flags & DREF_DDC_USES_NULL_RESPONSE_FOR_UNSUPPORTED));
-               // rpt_vstring(d1, "Monitor returns success with mh=ml=sh=sl=0 for unsupported features: %s",
-               //                    sbool(dref->flags & DREF_DDC_USES_MH_ML_SH_SL_ZERO_FOR_UNSUPPORTED));
+               if (dref->flags & DREF_DDC_DOES_NOT_INDICATE_UNSUPPORTED)
+                  rpt_vstring(d1, "Unable to determine how monitor reports unsupported features");
+               else {
+                  char * how = "unknown";
+                  // DBGMSG("flags: %s", interpret_dref_flags_t(dref->flags));
+                  if (dref->flags & DREF_DDC_USES_DDC_FLAG_FOR_UNSUPPORTED)
+                     how  = "invalid feature flag in DDC reply packet";
+                  else if (dref->flags & DREF_DDC_USES_NULL_RESPONSE_FOR_UNSUPPORTED)
+                     how  = "DDC Null Message";
+                  else if (dref->flags & DREF_DDC_USES_MH_ML_SH_SL_ZERO_FOR_UNSUPPORTED)
+                     how = "all data bytes 0 in DDC reply packet";
+                  rpt_vstring(d1, "Monitor uses %s to indicate unsupported feature.", how);
+               }
+            }
+
+            if (dref->pdd->dsa2_enabled) {
+               struct Results_Table * rt = (void *)dref->pdd->dsa2_data;
+               DDCA_Sleep_Multiplier cur_multiplier = dsa2_get_adjusted_sleep_mult(rt);
+               rpt_vstring(1, "Current dynamic sleep adjustment multiplier: %5.2f", cur_multiplier);
             }
          }
-         DDCA_Monitor_Model_Key mmk = monitor_model_key_value_from_edid(dref->pedid);
+         if (comm_error_occurred) {
+            // if (dref->flags & DREF_DPMS_SUSPEND_STANDBY_OFF) {
+            //    rpt_vstring(d1, "Display is asleep");
+            // }
+         }
+
+         Monitor_Model_Key mmk = monitor_model_key_value_from_edid(dref->pedid);
          // DBGMSG("mmk = %s", mmk_repr(mmk) );
          Monitor_Quirk_Data * quirk = get_monitor_quirks(&mmk);
          if (quirk) {
@@ -443,7 +540,7 @@ record_i2c_edid_use(GPtrArray * edid_use_records, Display_Ref * dref) {
          EDID_Use_Record * cur = get_edid_use_record(edid_use_records, binfo->edid->bytes);
          cur->bus_numbers = bs256_insert(cur->bus_numbers, binfo->busno);
          DBGTRC_DONE(debug, DDCA_TRC_NONE, "Updated bus list %s for edid %s",
-                         bs256_to_string_decimal(cur->bus_numbers, NULL, ", "),
+                         bs256_to_string_decimal_t(cur->bus_numbers, NULL, ", "),
                          hexstring_t(binfo->edid->bytes+122,6));
       }
    }
@@ -464,7 +561,7 @@ report_ambiguous_connector_for_edid(GPtrArray * edid_use_records, int depth) {
       EDID_Use_Record * cur_use_record = g_ptr_array_index(edid_use_records, ndx);
       if (bs256_count(cur_use_record->bus_numbers) > 1) {
          rpt_vstring(depth,"Displays with I2C bus numbers %s have identical EDIDs.",
-                             bs256_to_string_decimal(cur_use_record->bus_numbers, NULL, ", "));
+                             bs256_to_string_decimal_t(cur_use_record->bus_numbers, NULL, ", "));
          rpt_label(depth, "DRM connector names may not be accurate.");
       }
    }
@@ -490,7 +587,7 @@ ddc_report_displays(bool include_invalid_displays, int depth) {
    ddc_ensure_displays_detected();
 
    int display_ct = 0;
-   GPtrArray * all_displays = ddc_get_all_displays();
+   GPtrArray * all_displays = ddc_get_all_display_refs();
    GPtrArray * edid_use_records = create_edid_use_table();
    // DBGTRC_NOPREFIX(debug, DDCA_TRC_NONE, "Created edid_use_records = %p", edid_use_records);
    for (int ndx=0; ndx<all_displays->len; ndx++) {
@@ -548,11 +645,8 @@ ddc_dbgrpt_display_ref(Display_Ref * dref, int depth) {
          TRACED_ASSERT( memcmp(businfo->marker, I2C_BUS_INFO_MARKER, 4) == 0);
          i2c_dbgrpt_bus_info(businfo, d2);
          break;
-   case(DDCA_IO_ADL):
-         PROGRAM_LOGIC_ERROR("ADL implementation removed");
-         break;
    case(DDCA_IO_USB):
-#ifdef USE_USB
+#ifdef ENABLE_USB
          rpt_vstring(d1, "USB device information: ");
          Usb_Monitor_Info * moninfo = dref->detail;
          TRACED_ASSERT(memcmp(moninfo->marker, USB_MONITOR_INFO_MARKER, 4) == 0);
@@ -610,7 +704,6 @@ void dbgrpt_valid_display_refs(int depth) {
 
 
 void init_ddc_display_ref_reports() {
-
    RTTI_ADD_FUNC(ddc_report_display_by_dref);
    RTTI_ADD_FUNC(ddc_report_displays);
    RTTI_ADD_FUNC(get_controller_mfg_string_t);
