@@ -4,7 +4,7 @@
  * Capabilities.
  */
 
-// Copyright (C) 2014-2022 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2023 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 /** \cond */
@@ -29,45 +29,42 @@
 #include "base/tuned_sleep.h"
 
 #include "ddc/ddc_packet_io.h"
-#include "ddc/ddc_try_stats.h"
-
 #include "ddc/ddc_multi_part_io.h"
+#include "ddc/ddc_try_data.h"
 
 
+// temp
+int multi_part_null_adjustment_millis = 0;
+
 // Trace management
 static DDCA_Trace_Group TRACE_GROUP = DDCA_TRC_DDC;
 
 
 /** Makes one attempt to read the entire capabilities string or table feature value
 *
-* @param  dh             display handle for open i2c or adl device
-* @param  request_type   DDC_PACKET_TYPE_CAPABILITIES_REQUEST or DDC_PACKET_TYPE_TABLE_REQD_REQUEST
+* @param  dh               display handle for open i2c device
+* @param  request_type     DDC_PACKET_TYPE_CAPABILITIES_REQUEST or DDC_PACKET_TYPE_TABLE_REQD_REQUEST
 * @param  request_subtype  VCP feature code for table read, ignore for capabilities
-* @param  all_zero_response_ok  if true, an all zero response is not regarded
-*         as an error
-* @param  accumulator    buffer in which to return result (already allocated)
-*
-* @return @Error_Info struct with error detail, NULL if no error
+* @param  write_read_flags if flag all_zero_response_ok is set, an all zero response is not regarded
+*                          as an error
+* @param  accumulator      buffer in which to return result (already allocated)
+* @return #Error_Info struct with error detail, NULL if no error
 */
 static Error_Info *
 try_multi_part_read(
-      Display_Handle * dh,
-      Byte             request_type,
-      Byte             request_subtype,
-      bool             all_zero_response_ok,
-      Buffer *         accumulator)
+      Display_Handle *     dh,
+      Byte                 request_type,
+      Byte                 request_subtype,
+      DDC_Write_Read_Flags write_read_flags,
+      Buffer *             accumulator)
 {
    bool debug = false;
    DBGTRC_STARTING(debug, TRACE_GROUP,
           "request_type=0x%02x, request_subtype=x%02x, all_zero_response_ok=%s, accumulator=%p",
-          request_type, request_subtype, sbool(all_zero_response_ok), accumulator);
+          request_type, request_subtype,
+          sbool(write_read_flags & Write_Read_Flag_All_Zero_Response_Ok), accumulator);
 
-   const int MAX_FRAGMENT_SIZE = 32;
-   const int readbuf_size = 6 + MAX_FRAGMENT_SIZE + 1;
-
-   Public_Status_Code psc = 0;
    Error_Info * excp = NULL;
-
    DDC_Packet * request_packet_ptr  = NULL;
    DDC_Packet * response_packet_ptr = NULL;
    request_packet_ptr = create_ddc_multi_part_read_request_packet(
@@ -87,34 +84,34 @@ try_multi_part_read(
       Byte expected_response_type = (request_type == DDC_PACKET_TYPE_CAPABILITIES_REQUEST)
                                        ? DDC_PACKET_TYPE_CAPABILITIES_RESPONSE
                                        : DDC_PACKET_TYPE_TABLE_READ_RESPONSE;
-      Byte expected_subtype = request_subtype;     // 0x00 for capabilities, VCP feature code for table read
+      Byte expected_subtype = request_subtype;  // 0x00 for capabilities, VCP feature code for table read
       excp = ddc_write_read_with_retry(
            dh,
            request_packet_ptr,
-           readbuf_size,
+           MAX_DDC_PACKET_SIZE,
            expected_response_type,
            expected_subtype,
-           all_zero_response_ok,
+           write_read_flags,
            &response_packet_ptr
           );
-      psc = (excp) ? excp->status_code : 0;
       DBGTRC_NOPREFIX(debug, DDCA_TRC_NONE,
              "ddc_write_read_with_retry() request_type=0x%02x, request_subtype=0x%02x, returned %s",
              request_type, request_subtype, errinfo_summary(excp));
-      TUNED_SLEEP_WITH_TRACE(dh, SE_POST_CAP_TABLE_SEGMENT, NULL);
+      char * s = g_strdup_printf("Called from %s, request_subtype=0x%02x", __func__, request_subtype);
+      TUNED_SLEEP_WITH_TRACE(dh, SE_POST_CAP_TABLE_SEGMENT, s);
+      free(s);
 
       if (excp) {
-      // if (psc != 0) {
          if (response_packet_ptr)
             free_ddc_packet(response_packet_ptr);
          continue;
       }
       assert(response_packet_ptr);
-      assert(!excp && psc == 0);
 
       if ( IS_TRACING_BY_FUNC_OR_FILE() || debug ) {
          DBGMSG("After try_write_read():");
-         dbgrpt_interpreted_multi_read_fragment(response_packet_ptr->parsed.multi_part_read_fragment, 0);
+         dbgrpt_interpreted_multi_read_fragment(
+               response_packet_ptr->parsed.multi_part_read_fragment, 0);
       }
 
       Interpreted_Multi_Part_Read_Fragment * aux_data_ptr =
@@ -124,12 +121,12 @@ try_multi_part_read(
       if (display_current_offset != cur_offset) {
          DBGTRC_NOPREFIX(debug, DDCA_TRC_NONE,
                 "display_current_offset %d != cur_offset %d", display_current_offset, cur_offset);
-         psc = DDCRC_MULTI_PART_READ_FRAGMENT;
-         excp = errinfo_new(psc, __func__, NULL);
-         COUNT_STATUS_CODE(psc);
+         excp = ERRINFO_NEW(DDCRC_MULTI_PART_READ_FRAGMENT, NULL);
+         COUNT_STATUS_CODE(DDCRC_MULTI_PART_READ_FRAGMENT);
       }
       else {
-         DBGTRC_NOPREFIX(debug, DDCA_TRC_NONE, "display_current_offset = %d matches cur_offset", display_current_offset);
+         DBGTRC_NOPREFIX(debug, DDCA_TRC_NONE,
+               "display_current_offset = %d matches cur_offset", display_current_offset);
 
          fragment_size = aux_data_ptr->fragment_length;         // ***
          DBGTRC_NOPREFIX(debug, DDCA_TRC_NONE, "fragment_size = %d", fragment_size);
@@ -143,7 +140,7 @@ try_multi_part_read(
                DBGMSG("Currently assembled fragment: |%.*s|", accumulator->len, accumulator->bytes);
                DBGMSG("cur_offset = %d", cur_offset);
             }
-            all_zero_response_ok = false;           // accept all zero response only on first fragment
+            write_read_flags = write_read_flags & ~Write_Read_Flag_All_Zero_Response_Ok;
          }
       }
       free_ddc_packet(response_packet_ptr);
@@ -163,7 +160,7 @@ try_multi_part_read(
 *  @param  dh                    handle of open display
 *  @param  request_type
 *  @param  request_subtype       VCP function code for table read, ignore for capabilities
-*  @param  all_zero_response_ok  if true, zero response is not an error
+*  @param  write_read_flags
 *  @param  buffer_loc            address at which to return newly allocated #Buffer in which
 *                                result is returned
 *  @retval  NULL    success
@@ -176,8 +173,8 @@ Error_Info *
 multi_part_read_with_retry(
       Display_Handle * dh,
       Byte             request_type,
-      Byte             request_subtype,   // VCP feature code for table read, ignore for capabilities
-      bool             all_zero_response_ok,
+      Byte             request_subtype, // VCP feature code for table read, ignore for capabilities
+      DDC_Write_Read_Flags write_read_flags,
       Buffer**         buffer_loc)
 {
    bool debug = false;
@@ -185,7 +182,10 @@ multi_part_read_with_retry(
    DBGTRC_STARTING(debug, TRACE_GROUP,
           "request_type=0x%02x, request_subtype=0x%02x, all_zero_response_ok=%s"
           ", max_multi_part_read_tries=%d",
-          request_type, request_subtype, sbool(all_zero_response_ok), max_multi_part_read_tries);
+          request_type, request_subtype,
+          sbool(write_read_flags & Write_Read_Flag_All_Zero_Response_Ok),
+          max_multi_part_read_tries);
+   assert(write_read_flags & (Write_Read_Flag_Capabilities|Write_Read_Flag_Table_Read));
 
    Public_Status_Code rc = -1;   // dummy value for first call of while loop
    Error_Info * ddc_excp = NULL;
@@ -195,6 +195,9 @@ multi_part_read_with_retry(
    bool can_retry = true;
    Buffer * accumulator = buffer_new(2048, "multi part read buffer");
 
+   // not here
+   // if (write_read_flags & Write_Read_Flag_Table_Read)
+   //    write_read_flags |= Write_Read_Flag_All_Zero_Response_Ok;  // valid on first call, can indicate unsupported on Table
    while (tryctr < max_multi_part_read_tries && rc < 0 && can_retry) {
       DBGTRC_NOPREFIX(debug, DDCA_TRC_NONE,
              "Start of while loop. try_ctr=%d, max_multi_part_read_tries=%d",
@@ -204,7 +207,7 @@ multi_part_read_with_retry(
               dh,
               request_type,
               request_subtype,
-              all_zero_response_ok,
+              write_read_flags,
               accumulator);
       try_errors[tryctr] = ddc_excp;
       rc = (ddc_excp) ? ddc_excp->status_code : 0;
@@ -213,34 +216,37 @@ multi_part_read_with_retry(
          // generally means this, but could conceivably indicate a protocol error.
          // try multiple times to ensure it's really unsupported?
 
-         // just pass DDCRC_NULL_RESPONSE up the chain
-         // rc = DDCRC_DETERMINED_UNSUPPORTED;
-         // COUNT_STATUS_CODE(rc);   // double counting?
-
-         can_retry = false;
+         // Null Msg always indicates error for Capabilities command
+         if ( write_read_flags & Write_Read_Flag_Table_Read ) {
+            DBGTRC_NOPREFIX(debug, TRACE_GROUP, "Terminating loop for %s", psc_name(rc));
+            can_retry = false;
+         }
+         else {
+            int adjustment = multi_part_null_adjustment_millis;
+            if (adjustment > 0) {
+               SPECIAL_TUNED_SLEEP_WITH_TRACE(dh, adjustment, "special adjustment to recover from DDC_NULL_MSG");
+               DBGTRC_NOPREFIX(true, TRACE_GROUP, "Ad Hoc %d milliscecond sleep", adjustment);
+            }
+         }
       }
       else if (rc == DDCRC_READ_ALL_ZERO) {
+         DBGMSG("Accepting DDCRC_READ_ALL_ZERO");
+         // if (write_read_flags & Write_Read_Flag_All_Zero_Response_Ok)
+         //    rc = DDCRC_OK;
          can_retry = false;
-
-         // just pass DDCRC_READ_ALL_ZERO up the chain:
-         // rc = DDCRC_DETERMINED_UNSUPPORTED;    // ??
-         // COUNT_STATUS_CODE(rc);   // double counting?
       }
       else if (rc == DDCRC_ALL_TRIES_ZERO) {
          can_retry = false;
-
-         // just pass it up
-         // rc = DDCRC_DETERMINED_UNSUPPORTED;    // ??
-         // COUNT_STATUS_CODE(rc);   // double counting?
       }
       else if (rc == -EBADF) {
-         // DBGMSG("EBADF");
          can_retry = false;
       }
 
+      // WRONG LOCATION! This is not a fragment loop
+      // write_read_flags = write_read_flags & ~Write_Read_Flag_All_Zero_Response_Ok;           // accept all zero response only on first fragment
       tryctr++;
    }
-   assert( (rc<0 && ddc_excp) || (rc==0 && !ddc_excp) );
+   ASSERT_IFF( rc==0, !ddc_excp);
    DBGTRC_NOPREFIX(debug, DDCA_TRC_NONE, "After try loop. tryctr=%d, rc=%d. ddc_excp=%s",
                             tryctr, rc, errinfo_summary(ddc_excp));
 
@@ -268,15 +274,16 @@ multi_part_read_with_retry(
    }
 
    // if counts for DDCRC_ALL_TRIES_ZERO?
-   try_data_record_tries2(MULTI_PART_READ_OP, rc, tryctr);
+   try_data_record_tries2(dh, MULTI_PART_READ_OP, rc, tryctr);
 
    *buffer_loc = accumulator;
    ASSERT_IFF(ddc_excp, !*buffer_loc);
    DBGTRC_RET_ERRINFO_STRUCT(debug, TRACE_GROUP, ddc_excp, buffer_loc, buffer_rpt);
 #ifdef OLD
    DBGTRC_RET_ERRINFO(debug, TRACE_GROUP, ddc_excp, "*buffer_loc=%p", *buffer_loc);
-   if (IS_DBGTRC(debug, TRACE_GROUP) && *buffer_loc)
+   if (IS_DBGTRC(debug, TRACE_GROUP) && *buffer_loc) {
      buffer_rpt(*buffer_loc, 2);
+   }
 #endif
    return ddc_excp;
 }
@@ -303,7 +310,6 @@ try_multi_part_write(
           "request_type=0x%02x, request_subtype=x%02x, accumulator=%p",
           request_type, request_subtype, value_to_set);
 
-   Public_Status_Code psc = 0;
    Error_Info * ddc_excp = NULL;
    int MAX_FRAGMENT_SIZE = 32;
    int max_fragment_size = MAX_FRAGMENT_SIZE - 4;    // hack
@@ -312,7 +318,7 @@ try_multi_part_write(
    DDC_Packet * request_packet_ptr  = NULL;
    int bytes_remaining = value_to_set->len;
    int offset = 0;
-   while (bytes_remaining >= 0 && psc == 0) {
+   while (bytes_remaining >= 0 && !ddc_excp) {
       int bytect_to_write = (bytes_remaining <= max_fragment_size)
                                     ? bytes_remaining
                                     : max_fragment_size;
@@ -324,9 +330,7 @@ try_multi_part_write(
                    bytect_to_write,
                    __func__);
       ddc_excp = ddc_write_only_with_retry(dh, request_packet_ptr);
-      psc = (ddc_excp) ? ddc_excp->status_code : 0;
       free_ddc_packet(request_packet_ptr);
-      assert( (!ddc_excp && psc == 0) || (ddc_excp && psc!=0) );
 
       if (!ddc_excp) {
          if (bytect_to_write == 0)   // if just wrote final empty segment to indicate done
@@ -337,7 +341,6 @@ try_multi_part_write(
    }
 
    DBGTRC_RET_ERRINFO(debug, TRACE_GROUP, ddc_excp, "");
-   assert( (ddc_excp && psc<0) || (!ddc_excp && psc==0) );
    return ddc_excp;
 }
 
