@@ -1,10 +1,9 @@
-/** \file ddc_packet_io.h
+/** @file ddc_packet_io.h
  *
- *  Functions for performing DDC packet IO, using either the I2C bus API
- *  or the ADL API, as appropriate.  Handles I2C bus retry.
+ *  Functions for performing DDC packet IO,  Handles I2C bus retry.
  */
 
-// Copyright (C) 2014-2022 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2023 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 #ifndef DDC_PACKET_IO_H_
@@ -20,18 +19,29 @@
 
 
 extern bool DDC_Read_Bytewise;
+extern bool simulate_null_msg_means_unsupported;
 
-DDCA_Status ddc_open_display(
+typedef enum {
+   Write_Read_Flags_None = 0,
+   Write_Read_Flag_All_Zero_Response_Ok = 1,
+   Write_Read_Flag_Capabilities = 2,
+   Write_Read_Flag_Table_Read = 4
+} DDC_Write_Read_Flags ;
+
+Error_Info * ddc_open_display(
       Display_Ref *    dref,
       Call_Options     callopts,
       Display_Handle** dh_loc);
 
-Status_Errno ddc_close_display(
+Error_Info * ddc_close_display(
       Display_Handle * dh);
 
-void  ddc_close_all_displays();
+void ddc_close_display_wo_return(
+      Display_Handle * dh);
 
-bool ddc_is_valid_display_handle(Display_Handle * dh);
+void ddc_close_all_displays();
+
+DDCA_Status ddc_validate_display_handle(Display_Handle * dh);
 
 void ddc_dbgrpt_valid_display_handles(int depth);
 
@@ -59,12 +69,13 @@ Error_Info * ddc_write_read_with_retry(
       int              max_read_bytes,
       Byte             expected_response_type,
       Byte             expected_subtype,
-      bool             all_zero_response_ok,
-  //  bool             retry_null_response,
+      DDC_Write_Read_Flags flags,
       DDC_Packet **    response_packet_ptr_loc
      );
 
 void init_ddc_packet_io();
 
+void terminate_ddc_packet_io();
+
 #endif /* DDC_PACKET_IO_H_ */
 
