@@ -3,11 +3,9 @@
  *  Basic functions to get and set single values and save current settings.
  */
 
-// Copyright (C) 2014-2022 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2024 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
-#include <config.h>
-
 /** \cond */
 #include <assert.h>
 #include <errno.h>
@@ -17,6 +15,8 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include <config.h>
+
 #include "util/error_info.h"
 #include "util/report_util.h"
 #include "util/string_util.h"
@@ -31,7 +31,7 @@
 
 #include "i2c/i2c_bus_core.h"
 
-#ifdef USE_USB
+#ifdef ENABLE_USB
 #include "usb/usb_displays.h"
 #include "usb/usb_vcp.h"
 #endif
@@ -49,6 +49,8 @@
 // Trace class for this file
 static DDCA_Trace_Group TRACE_GROUP = DDCA_TRC_DDC;
 
+bool enable_mock_data = false;
+
 typedef struct {
    bool   verify_setvcp;
 } Thread_Vcp_Settings;
@@ -94,7 +96,7 @@ ddc_save_current_settings(
    if (dh->dref->io_path.io_mode == DDCA_IO_USB) {
       // command line parser should block this case
       PROGRAM_LOGIC_ERROR("MCCS over USB does not have Save Current Settings command");
-      ddc_excp = errinfo_new(DDCRC_UNIMPLEMENTED, __func__,
+      ddc_excp = ERRINFO_NEW(DDCRC_UNIMPLEMENTED,
                               "MCCS over USB does not have Save Current Settings command" );
    }
    else {
@@ -105,8 +107,7 @@ ddc_save_current_settings(
 
       ddc_excp = ddc_write_only_with_retry(dh, request_packet_ptr);
 
-      if (request_packet_ptr)
-         free_ddc_packet(request_packet_ptr);
+      free_ddc_packet(request_packet_ptr);
    }
 
    DBGTRC_RET_ERRINFO(debug, TRACE_GROUP, ddc_excp, "");
@@ -140,7 +141,7 @@ ddc_set_nontable_vcp_value(
    Public_Status_Code psc = 0;
    Error_Info * ddc_excp = NULL;
    if (dh->dref->io_path.io_mode == DDCA_IO_USB) {
-#ifdef USE_USB
+#ifdef ENABLE_USB
       psc = usb_set_nontable_vcp_value(dh, feature_code, new_value);
 #else
       PROGRAM_LOGIC_ERROR("ddcutil not built with USB support");
@@ -155,8 +156,7 @@ ddc_set_nontable_vcp_value(
       ddc_excp = ddc_write_only_with_retry(dh, request_packet_ptr);
       psc = (ddc_excp) ? ddc_excp->status_code : 0;
 
-      if (request_packet_ptr)
-         free_ddc_packet(request_packet_ptr);
+      free_ddc_packet(request_packet_ptr);
    }
 
    if ( psc==DDCRC_RETRIES )
@@ -190,13 +190,13 @@ set_table_vcp_value(
    Public_Status_Code psc = 0;
    Error_Info * ddc_excp = NULL;
    if (dh->dref->io_path.io_mode == DDCA_IO_USB) {
-#ifdef USE_USB
+#ifdef ENABLE_USB
       psc = DDCRC_UNIMPLEMENTED;
 #else
       PROGRAM_LOGIC_ERROR("ddcutil not built with USB support");
       psc = DDCRC_INTERNAL_ERROR;
 #endif
-      ddc_excp = errinfo_new(psc, __func__, "Error setting feature value");
+      ddc_excp = ERRINFO_NEW(psc, "Error setting feature value");
    }
    else {
       // TODO: clean up function signatures
@@ -248,13 +248,13 @@ ddc_get_verify_setvcp() {
 }
 
 
-static bool
+STATIC bool
 is_rereadable_feature(
       Display_Handle * dh,
       DDCA_Vcp_Feature_Code opcode)
 {
    bool debug = false;
-   DBGMSF(debug, "Starting opcode = 0x%02x", opcode);
+   DBGTRC_STARTING(debug, TRACE_GROUP, "opcode = 0x%02x", opcode);
 
    // readable features that should not be read after write
    DDCA_Vcp_Feature_Code unrereadable_features[] = {
@@ -283,12 +283,12 @@ is_rereadable_feature(
       }
    }
 
-   DBGMSF(debug, "Returning: %s", sbool(result));
+   DBGTRC_RET_BOOL(debug, TRACE_GROUP, result, "");
    return result;
 }
 
 
-static bool
+STATIC bool
 is_unreadable_sl_value(
       DDCA_Vcp_Feature_Code opcode,
       Byte                  sl_value)
@@ -311,7 +311,7 @@ is_unreadable_sl_value(
 }
 
 
-static bool
+STATIC bool
 single_vcp_value_equal(
       DDCA_Any_Vcp_Value * vrec1,
       DDCA_Any_Vcp_Value * vrec2)
@@ -389,7 +389,8 @@ ddc_set_vcp_value(
            )
          )
       {
-         f0printf(verbose_msg_dest, "Verifying that value of feature 0x%02x successfully set...\n", vrec->opcode);
+         f0printf(verbose_msg_dest,
+               "Verifying that value of feature 0x%02x successfully set...\n", vrec->opcode);
          DDCA_Any_Vcp_Value * newval = NULL;
          ddc_excp = ddc_get_vcp_value(
              dh,
@@ -398,10 +399,12 @@ ddc_set_vcp_value(
              &newval);
          psc = (ddc_excp) ? ddc_excp->status_code : 0;
          if (ddc_excp) {
-            f0printf(verbose_msg_dest, "(%s) Read after write failed. get_vcp_value() returned: %s\n",
-                           __func__, psc_desc(psc));
+            f0printf(verbose_msg_dest,
+                  "(%s) Read after write failed. get_vcp_value() returned: %s\n",
+                  __func__, psc_desc(psc));
             if (psc == DDCRC_RETRIES)
-               f0printf(verbose_msg_dest, "(%s)    Try errors: %s\n", __func__, errinfo_causes_string(ddc_excp));
+               f0printf(verbose_msg_dest,
+                     "(%s)    Try errors: %s\n", __func__, errinfo_causes_string(ddc_excp));
             // psc = DDCRC_VERIFY;
          }
          else {
@@ -410,7 +413,7 @@ ddc_set_vcp_value(
             // dbgrpt_ddca_single_vcp_value(newval, 3);
 
             if (! single_vcp_value_equal(vrec,newval)) {
-               ddc_excp = errinfo_new(DDCRC_VERIFY, __func__, "Current value does not match value set");
+               ddc_excp = ERRINFO_NEW(DDCRC_VERIFY, "Current value does not match value set");
                f0printf(verbose_msg_dest, "Current value does not match value set.\n");
             }
             else {
@@ -424,11 +427,13 @@ ddc_set_vcp_value(
       }
       else {
          if (!is_rereadable_feature(dh, vrec->opcode) )
-            f0printf(verbose_msg_dest, "Feature 0x%02x does not support verification\n", vrec->opcode);
+            f0printf(verbose_msg_dest, ""
+                  "Feature 0x%02x does not support verification\n", vrec->opcode);
          else
-            f0printf(verbose_msg_dest, "Feature 0x%02x, value 0x%02x does not support verification\n",
-                                       vrec->opcode,
-                                       vrec->val.c_nc.sl);
+            f0printf(verbose_msg_dest,
+                  "Feature 0x%02x, value 0x%02x does not support verification\n",
+                  vrec->opcode,
+                  vrec->val.c_nc.sl);
       }
    }
 
@@ -437,6 +442,20 @@ ddc_set_vcp_value(
 }
 
 
+static Parsed_Nontable_Vcp_Response * create_all_zero_response(Byte feature_code) {
+   Parsed_Nontable_Vcp_Response * resp = calloc(1, sizeof(Parsed_Nontable_Vcp_Response));
+   resp->vcp_code = feature_code;
+   resp->valid_response = true;
+   resp->supported_opcode = true;
+   resp->mh = 0;
+   resp->ml = 0;
+   resp->sh = 0;
+   resp->sl = 0;
+   return resp;
+}
+
+
+
 /** Possibly returns a mock value for a non-table feature
  *
  *  @param  feature_code  VCP Feature Code
@@ -449,7 +468,7 @@ ddc_set_vcp_value(
  */
 Error_Info *
 mock_get_nontable_vcp_value(
-      DDCA_Vcp_Feature_Code   feature_code,
+      DDCA_Vcp_Feature_Code          feature_code,
       Parsed_Nontable_Vcp_Response** resp_loc)
 {
    bool debug = false;
@@ -458,36 +477,54 @@ mock_get_nontable_vcp_value(
    Error_Info * pseudo_errinfo = NULL;
    *resp_loc = NULL;
 
-#ifdef TEST_X72
-   if (feature_code == 0x72) {
-      Parsed_Nontable_Vcp_Response * resp = calloc(1, sizeof(Parsed_Nontable_Vcp_Response));
-      resp->vcp_code = feature_code;
-      resp->valid_response = true;
-      resp->supported_opcode = true;
-      resp->mh = 0;
-      resp->ml = 0;
-      resp->sh = 0x78;
-      resp->sl = 0x00;
-      resp->max_value = resp->mh << 8 | resp->ml;
-      resp->cur_value = resp->sh << 8 | resp->sl;
+   if (enable_mock_data) {
 
-      *resp_loc = resp;
-   }
-#endif
+   #ifdef TEST_X72
+      if (feature_code == 0x72) {
+         Parsed_Nontable_Vcp_Response * resp = calloc(1, sizeof(Parsed_Nontable_Vcp_Response));
+         resp->vcp_code = feature_code;
+         resp->valid_response = true;
+         resp->supported_opcode = true;
+         resp->mh = 0;
+         resp->ml = 0;
+         resp->sh = 0x78;
+         resp->sl = 0x00;
+         resp->max_value = resp->mh << 8 | resp->ml;
+         resp->cur_value = resp->sh << 8 | resp->sl;
 
-#ifdef TEST_EIO
-   if (feature_code == 0x45) {
-      pseudo_errinfo = errinfo_new(-EIO, __func__, "Pseudo EIO error");
-   }
-#endif
+         *resp_loc = resp;
+      }
+   #endif
 
-   if (debug) {
-      DBGMSG("Feature 0x%02x, *resp_loc = %p, returning: %s",
-          feature_code,
-          *resp_loc,
-          errinfo_summary(pseudo_errinfo) );
-      if (*resp_loc)
-         dbgrpt_interpreted_nontable_vcp_response(*resp_loc, 2);
+   #ifdef TEST_EIO
+      if (feature_code == 0x45) {
+         pseudo_errinfo = ERRINFO_NEW(-EIO, "Pseudo EIO error");
+      }
+   #endif
+
+      if (feature_code == 0x00) {
+         // pseudo_errinfo = ERRINFO_NEW(DDCRC_NULL_RESPONSE, "Pseudo Null Response for feature 0x00");
+         Parsed_Nontable_Vcp_Response * resp = create_all_zero_response(feature_code);
+         *resp_loc = resp;
+      }
+
+      if (feature_code == 0x10) {
+         pseudo_errinfo = ERRINFO_NEW(DDCRC_INTERNAL_ERROR, "Pseudo Null Response for feature 0x10");
+      }
+
+      if (feature_code == 0x41) {
+         // *resp_loc = create_all_zero_response(feature_code);
+         pseudo_errinfo = ERRINFO_NEW(DDCRC_INTERNAL_ERROR, "Pseudo Null Response for feature 0x41");
+      }
+
+      if (debug) {
+         DBGMSG("Feature 0x%02x, *resp_loc = %p, returning: %s",
+             feature_code,
+             *resp_loc,
+             errinfo_summary(pseudo_errinfo) );
+         if (*resp_loc)
+            dbgrpt_interpreted_nontable_vcp_response(*resp_loc, 2);
+      }
    }
    return pseudo_errinfo;
 }
@@ -516,45 +553,45 @@ ddc_get_nontable_vcp_value(
 {
    bool debug = false;
    DBGTRC_STARTING(debug, TRACE_GROUP, "dh=%s, Reading feature 0x%02x", dh_repr(dh), feature_code);
+   // DBGTRC_NOPREFIX(debug, TRACE_GROUP,
+   //                 "communication flags: %s", interpret_dref_flags_t(dh->dref->flags));
 
    Error_Info * excp = NULL;
    Parsed_Nontable_Vcp_Response * parsed_response = NULL;
    *parsed_response_loc = NULL;
 
-   Error_Info * mock_errinfo = mock_get_nontable_vcp_value(feature_code, parsed_response_loc);
-   if (mock_errinfo || *parsed_response_loc) {
-      DBGMSF(debug, "Returning mock response for feature 0x%02x", feature_code);
-      return mock_errinfo;
+   if (enable_mock_data) {
+      Error_Info * mock_errinfo = mock_get_nontable_vcp_value(feature_code, parsed_response_loc);
+      if (mock_errinfo || *parsed_response_loc) {
+         DBGMSF(debug, "Returning mock response for feature 0x%02x", feature_code);
+         return mock_errinfo;
+      }
    }
 
-   DDC_Packet * request_packet_ptr  = NULL;
    DDC_Packet * response_packet_ptr = NULL;
-   request_packet_ptr = create_ddc_getvcp_request_packet(
-                           feature_code, "ddc_get_nontable_vcp_value:request packet");
+   DDC_Packet * request_packet_ptr = create_ddc_getvcp_request_packet(
+                                  feature_code, "ddc_get_nontable_vcp_value:request packet");
    // dump_packet(request_packet_ptr);
 
    Byte expected_response_type = DDC_PACKET_TYPE_QUERY_VCP_RESPONSE;
    Byte expected_subtype = feature_code;
-   // int max_read_bytes  = 20;    // actually 3 + 8 + 1, or is it 2 + 8 + 1?
-   // expected response size:
-   //  (src addr == x6e) (length) (response contents) (checkbyte)
-   //  1               + 1      + 8                 + 1           == 11
-   //  alternative is DDC Null Response, which is shorter
-   //  N. response does not include initial destination address byte of DDC/CI spec
-   int max_read_bytes = 11;
 
+   int max_read_bytes = MAX_DDC_PACKET_SIZE;
+
+   // DBGTRC_NOPREFIX(debug, TRACE_GROUP,
+   //                 "before ddc_write_read_with_retry(): communication flags: %s",
+   //                 interpret_dref_flags_t(dh->dref->flags));
    excp = ddc_write_read_with_retry(
            dh,
            request_packet_ptr,
            max_read_bytes,
            expected_response_type,
            expected_subtype,
-           false,                       // all_zero_response_ok
+           Write_Read_Flags_None,
            &response_packet_ptr
         );
    ASSERT_IFF(excp, !response_packet_ptr);
-//   assert( (!excp && response_packet_ptr) || (excp && !response_packet_ptr));
-   if (debug || IS_TRACING() ) {
+   if ( IS_DBGTRC(debug, TRACE_GROUP)) {
       if (excp)
          DBGTRC_NOPREFIX(debug, TRACE_GROUP,
                 "ddc_write_read_with_retry() returned %s, response_packet_ptr=%p",
@@ -564,7 +601,8 @@ ddc_get_nontable_vcp_value(
    if (!excp) {
       assert(response_packet_ptr);
       // dump_packet(response_packet_ptr);
-      Public_Status_Code psc = get_interpreted_vcp_code(response_packet_ptr, true /* make_copy */, &parsed_response);   // ???
+      Public_Status_Code psc =
+            get_interpreted_vcp_code(response_packet_ptr, true /* make_copy */, &parsed_response);   // ???
       if (psc == 0) {
 #ifdef NO_LONGER_NEEDED
          if (parsed_response->vcp_code != feature_code) {
@@ -576,10 +614,10 @@ ddc_get_nontable_vcp_value(
 #endif
 
          if (!parsed_response->valid_response)  {
-            excp = errinfo_new(DDCRC_DDC_DATA, __func__, "Invalid getvcp response");  // was DDCRC_INVALID_DATA
+            excp = ERRINFO_NEW(DDCRC_DDC_DATA, "Invalid getvcp response");  // was DDCRC_INVALID_DATA
          }
          else if (!parsed_response->supported_opcode) {
-            excp = errinfo_new(DDCRC_REPORTED_UNSUPPORTED, __func__, "Unsupported feature");
+            excp = ERRINFO_NEW(DDCRC_REPORTED_UNSUPPORTED, "Unsupported feature");
             if (!value_bytes_zero(parsed_response)) {
                // for exploring
                DBGMSG("supported_opcode == false, but not all value bytes 0");
@@ -588,10 +626,10 @@ ddc_get_nontable_vcp_value(
          else if (value_bytes_zero(parsed_response) &&
                (dh->dref->flags & DREF_DDC_USES_MH_ML_SH_SL_ZERO_FOR_UNSUPPORTED) )
          {
-            // just a messages for now
+            // just a message for now
             DBGMSG("all value bytes 0, supported_opcode == true,"
                    " setting DDCRC_DETERMINED_UNSUPPORTED)");
-            excp = errinfo_new(DDCRC_DETERMINED_UNSUPPORTED, __func__, "MH=ML=SH=SL=0");
+            excp = ERRINFO_NEW(DDCRC_DETERMINED_UNSUPPORTED, "MH=ML=SH=SL=0");
          }
 
          if (excp) {
@@ -600,12 +638,11 @@ ddc_get_nontable_vcp_value(
          }
       }
       else {
-         excp = errinfo_new(psc, __func__, NULL);
+         excp = ERRINFO_NEW(psc, NULL);
       }
    }
 
-   if (request_packet_ptr)
-      free_ddc_packet(request_packet_ptr);
+   free_ddc_packet(request_packet_ptr);
    if (response_packet_ptr)
       free_ddc_packet(response_packet_ptr);
 
@@ -622,7 +659,8 @@ ddc_get_nontable_vcp_value(
    }
    *parsed_response_loc = parsed_response;
 
-   DBGTRC_RET_ERRINFO(debug, TRACE_GROUP, excp, "");
+   DBGTRC_RET_ERRINFO2(debug, TRACE_GROUP, excp, *parsed_response_loc, "");
+   ASSERT_IFF(!excp, *parsed_response_loc);
    return excp;
 }
 
@@ -652,7 +690,7 @@ ddc_get_table_vcp_value(
             dh,
             DDC_PACKET_TYPE_TABLE_READ_REQUEST,
             feature_code,
-            true,                      // all_zero_response_ok
+            Write_Read_Flag_All_Zero_Response_Ok | Write_Read_Flag_Table_Read,
             &paccumulator);
    if (debug || ddc_excp) {
       DBGTRC_NOPREFIX(debug, TRACE_GROUP,
@@ -703,6 +741,7 @@ ddc_get_vcp_value(
    bool debug = false;
    DBGTRC_STARTING(debug, TRACE_GROUP, "Reading feature 0x%02x, dh=%s, dh->fd=%d",
             feature_code, dh_repr(dh), dh->fd);
+   // DBGTRC_NOPREFIX(debug, TRACE_GROUP, "communication flags: %s", interpret_dref_flags_t(dh->dref->flags));
 
    Error_Info * ddc_excp = NULL;
    Buffer * buffer = NULL;
@@ -711,7 +750,7 @@ ddc_get_vcp_value(
 
    // why are we coming here for USB?
    if (dh->dref->io_path.io_mode == DDCA_IO_USB) {
-#ifdef USE_USB
+#ifdef ENABLE_USB
       DBGMSF(debug, "USB case");
       Public_Status_Code psc = 0;
 
@@ -732,11 +771,11 @@ ddc_get_vcp_value(
                    free(parsed_nontable_response);
                 }
                 else
-                   ddc_excp = errinfo_new(psc, __func__, NULL);
+                   ddc_excp = ERRINFO_NEW(psc, NULL);
                 break;
 
           case (DDCA_TABLE_VCP_VALUE):
-                ddc_excp = errinfo_new(DDCRC_UNIMPLEMENTED, __func__,
+                ddc_excp = ERRINFO_NEW(DDCRC_UNIMPLEMENTED,
                                         "Table features not supported for USB connection");
                 break;
           }
@@ -786,12 +825,13 @@ ddc_get_vcp_value(
 
 
 void init_ddc_vcp() {
-   RTTI_ADD_FUNC(ddc_save_current_settings);
-   RTTI_ADD_FUNC(ddc_set_nontable_vcp_value);
-   RTTI_ADD_FUNC(set_table_vcp_value);
-   RTTI_ADD_FUNC(ddc_set_vcp_value);
    RTTI_ADD_FUNC(ddc_get_nontable_vcp_value);
    RTTI_ADD_FUNC(ddc_get_table_vcp_value);
    RTTI_ADD_FUNC(ddc_get_vcp_value);
+   RTTI_ADD_FUNC(ddc_save_current_settings);
+   RTTI_ADD_FUNC(ddc_set_nontable_vcp_value);
+   RTTI_ADD_FUNC(ddc_set_vcp_value);
+   RTTI_ADD_FUNC(is_rereadable_feature);
+   RTTI_ADD_FUNC(set_table_vcp_value);
 }
 
