@@ -49,11 +49,11 @@ char * dyn_feature_set_repr_t(Dyn_Feature_Set * fset) {
 
 
 static Display_Feature_Metadata *
-dyn_create_dynamic_feature_from_dfr_metadata_dfm(DDCA_Feature_Metadata * dfr_metadata)
+dyn_create_dynamic_feature_from_dfr_metadata(Dyn_Feature_Metadata * dfr_metadata)
 {
    bool debug = false;
    DBGMSF(debug, "Starting. id=0x%02x", dfr_metadata->feature_code);
-   Display_Feature_Metadata * dfm = dfm_from_ddca_feature_metadata(dfr_metadata);
+   Display_Feature_Metadata * dfm = dfm_from_dyn_feature_metadata(dfr_metadata);
 
    if (dfr_metadata->feature_flags & DDCA_SIMPLE_NC) {
       if (dfr_metadata->sl_values)
@@ -79,7 +79,7 @@ dyn_create_dynamic_feature_from_dfr_metadata_dfm(DDCA_Feature_Metadata * dfr_met
 }
 
 #ifdef UNUSED
-static DDCA_Feature_Metadata *
+static Dyn_Feature_Metadata *
 dyn_create_feature_metadata_from_vcp_feature_table_entry(
       VCP_Feature_Table_Entry * pentry,
       DDCA_MCCS_Version_Spec    vspec)
@@ -90,7 +90,7 @@ dyn_create_feature_metadata_from_vcp_feature_table_entry(
    if (pentry->vcp_global_flags & DDCA_SYNTHETIC_VCP_FEATURE_TABLE_ENTRY)
       free_synthetic_vcp_entry(pentry);
 
-   DDCA_Feature_Metadata * meta = dfm_to_ddca_feature_metadata(dfm);
+   Dyn_Feature_Metadata * meta = dfm_to_ddca_feature_metadata(dfm);
    dfm_free(dfm);
    return meta;
 }
@@ -106,8 +106,8 @@ dyn_create_dynamic_feature_from_vcp_feature_table_entry_dfm(
    bool debug = false;
    DBGMSF(debug, "Starting. id=0x%02x", vfte->code);
    // Internal_Feature_Metadata * ifm = calloc(1, sizeof(Internal_Feature_Metadata));
-   DDCA_Feature_Metadata * meta = dyn_create_feature_metadata_from_vcp_feature_table_entry(vfte, vspec);
-   Display_Feature_Metadata * dfm = dfm_from_ddca_feature_metadata(meta);
+   Dyn_Feature_Metadata * meta = dyn_create_feature_metadata_from_vcp_feature_table_entry(vfte, vspec);
+   Display_Feature_Metadata * dfm = dfm_from_dyn_feature_metadata(meta);
    free_ddca_feature_metadata(meta);
    free(meta);
 
@@ -187,11 +187,11 @@ dyn_create_feature_set(
           g_hash_table_iter_init(&iter, dref->dfr->features);
           bool found = g_hash_table_iter_next(&iter, &hash_key, &hash_value);
           while (found) {
-             DDCA_Feature_Metadata * feature_metadata = hash_value;
+             Dyn_Feature_Metadata * feature_metadata = hash_value;
              assert( memcmp(feature_metadata, DDCA_FEATURE_METADATA_MARKER, 4) == 0 );
 
-             // Test Feature_Set_Flags other than FSF_SHOW_UNSUPPORTED, FSF_FORCE,
-             // which do not apply in this context
+             // Test Feature_Set_Flags other than FSF_SHOW_UNSUPPORTED,
+             // which does not apply in this context
              bool include = true;
              //     Feature_Set_Flags                     //DDCA_Feature_Flags
              if ( ((feature_set_flags & FSF_NOTABLE) &&  (feature_metadata->feature_flags & DDCA_TABLE)) ||
@@ -202,7 +202,7 @@ dyn_create_feature_set(
                 include = false;
              if (include) {
                 Display_Feature_Metadata * dfm =
-                   dyn_create_dynamic_feature_from_dfr_metadata_dfm(feature_metadata);
+                   dyn_create_dynamic_feature_from_dfr_metadata(feature_metadata);
                 g_ptr_array_add(members_dfm, dfm);
              }
 
@@ -304,7 +304,7 @@ dyn_create_single_feature_set_by_hexid2(
       DDCA_Featurfree_dfm_funce_Metadata * feature_metadata  =
          get_dynamic_feature_metadata(dref->dfr, feature_code);
       if (feature_metadata) {
-         dfm = dyn_create_dynamic_feature_from_dfr_metadata_dfm(feature_metadata);
+         dfm = dyn_create_dynamic_feature_from_dfr_metadata(feature_metadata);
       }
    }
 
@@ -394,11 +394,13 @@ dyn_create_feature_set_from_feature_set_ref2(
 #endif
 
 
+#ifdef UNUSED
 // wrap dfm_free() in signature of GDestroyNotify()
 static void
 free_dfm_func(gpointer data) {
    dfm_free((Display_Feature_Metadata *) data);
 }
+#endif
 
 
 void dyn_free_feature_set(
@@ -407,7 +409,7 @@ void dyn_free_feature_set(
    bool debug = false;
    DBGMSF(debug, "Starting. feature_set=%s", dyn_feature_set_repr_t(feature_set));
    if (feature_set->members_dfm) {
-      g_ptr_array_set_free_func(feature_set->members_dfm, free_dfm_func);
+      g_ptr_array_set_free_func(feature_set->members_dfm, (GDestroyNotify) dfm_free);
       g_ptr_array_free(feature_set->members_dfm,true);
    }
    free(feature_set);
