@@ -32,7 +32,6 @@
 #include "base/core.h"
 #include "base/ddc_errno.h"
 #include "base/execution_stats.h"
-#include "base/last_io_event.h"
 #include "base/linux_errno.h"
 #include "base/parms.h"
 #include "base/rtti.h"
@@ -80,7 +79,7 @@ i2c_set_addr0(int fd, uint16_t op, int addr) {
       errno=EBUSY;
    }
    else {
-      RECORD_IO_EVENT( IE_OTHER, ( ioctl_rc = ioctl(fd, op, addr) ) );
+      RECORD_IO_EVENT(-1, IE_OTHER, ( ioctl_rc = ioctl(fd, op, addr) ) );
    }
 
    if (ioctl_rc < 0) {
@@ -147,7 +146,7 @@ i2c_set_addr(int fd, int addr) {
                              (op == I2C_SLAVE) ? "I2C_SLAVE" : "I2C_SLAVE_FORCE",
                              addr);
       DBGTRC_NOPREFIX(debug || get_output_level() >= DDCA_OL_VERBOSE, TRACE_GROUP, "%s", msgbuf);
-      SYSLOG(LOG_ERR, "%s", msgbuf);
+      SYSLOG2(DDCA_SYSLOG_ERROR, "%s", msgbuf);
 
    }
    else if (result == 0 && op == I2C_SLAVE_FORCE) {
@@ -156,7 +155,7 @@ i2c_set_addr(int fd, int addr) {
             filename_for_fd_t(fd),
             addr);
       DBGTRC_NOPREFIX(debug || get_output_level() >= DDCA_OL_VERBOSE, TRACE_GROUP, "%s", msgbuf);
-      SYSLOG(LOG_INFO, "%s", msgbuf);
+      SYSLOG2(DDCA_SYSLOG_ERROR, "%s", msgbuf);
    }
 
    assert(result <= 0);
@@ -216,7 +215,7 @@ i2c_fileio_writer(
 
         int pollrc;
         int timeout_msec = 100;
-        RECORD_IO_EVENTX(
+        RECORD_IO_EVENT(
               fd,
               IE_OTHER,
               ( pollrc = poll(pfds, 1, timeout_msec) )
@@ -242,7 +241,7 @@ i2c_fileio_writer(
      }
      // #endif
 
-     RECORD_IO_EVENTX(
+     RECORD_IO_EVENT(
            fd,
            IE_FILEIO_WRITE,
            ( rc = write(fd, pbytes, bytect) )
@@ -305,7 +304,7 @@ i2c_fileio_reader(
       // for Acer and P2411h, reads bytes 1,3,5,7 ..
       for (int ndx=0; ndx < bytect && rc == 0; ndx++) {
          // DBGMSF(debug, "Calling read() for 1 byte, ndx=%d", ndx);
-         RECORD_IO_EVENTX(
+         RECORD_IO_EVENT(
             fd,
             IE_FILEIO_READ,
             ( rc = read(fd, readbuf+ndx, 1) )
@@ -334,7 +333,7 @@ i2c_fileio_reader(
 
          int pollrc;
          int timeout_msec = 100;
-         RECORD_IO_EVENTX(
+         RECORD_IO_EVENT(
                fd,
                IE_OTHER,
                ( pollrc = poll(pfds, 1, timeout_msec) )
@@ -360,7 +359,7 @@ i2c_fileio_reader(
       }
 // #endif
 
-      RECORD_IO_EVENTX(
+      RECORD_IO_EVENT(
          fd,
          IE_FILEIO_READ,
          ( rc = read(fd, readbuf, bytect) )
@@ -387,26 +386,29 @@ bye:
 }
 
 
-void
+STATIC void
 dbgrpt_i2c_msg(int depth, struct i2c_msg message) {
-    rpt_vstring(depth, "addr:     0x%04x", message.addr);   //      __u16 addr;
-    rpt_vstring(depth, "flags:    0x%04x", message.flags);
+   rpt_vstring(depth, "addr:    0x%04x", message.addr);   //      __u16 addr;
+   rpt_vstring(depth, "flags:   0x%04x", message.flags);
    // #define I2C_M_RD     0x0001   /* guaranteed to be 0x0001! */
-   // #define I2C_M_TEN    0x0010   /* use only if I2C_FUNC_10BIT_ADDR */
+   // #define I2C_M_TEN          0x0010   /* use only if I2C_FUNC_10BIT_ADDR */
    // #define I2C_M_DMA_SAFE     0x0200   /* use only in kernel space */
    // #define I2C_M_RECV_LEN     0x0400   /* use only if I2C_FUNC_SMBUS_READ_BLOCK_DATA */
    // #define I2C_M_NO_RD_ACK    0x0800   /* use only if I2C_FUNC_PROTOCOL_MANGLING */
    // #define I2C_M_IGNORE_NAK   0x1000   /* use only if I2C_FUNC_PROTOCOL_MANGLING */
    // #define I2C_M_REV_DIR_ADDR 0x2000   /* use only if I2C_FUNC_PROTOCOL_MANGLING */
    // #define I2C_M_NOSTART      0x4000   /* use only if I2C_FUNC_NOSTART */
-   // #define I2C_M_STOP      0x8000   /* use only if I2C_FUNC_PROTOCOL_MANGLING */
-     rpt_vstring(depth, "len:    0x%04x (%d)", message.len, message.len);  // __u16
-     rpt_vstring(depth, "buf:    %p ->  %s", message.buf, hexstring_t(message.buf, message.len)); // __u8 *buf;
+   // #define I2C_M_STOP         0x8000   /* use only if I2C_FUNC_PROTOCOL_MANGLING */
+   rpt_vstring(depth, "len:     0x%04x (%d)", message.len, message.len);  // __u16
+   // rpt_vstring(depth, "buf:     %p ->  %s", message.buf, hexstring_t(message.buf, message.len)); // __u8 *buf;
+   rpt_vstring(depth, "buf:     %p", message.buf); // __u8 *buf;
 }
 
 
-void
+STATIC void
 dbgrpt_i2c_rdwr_ioctl_data(int depth, struct i2c_rdwr_ioctl_data * data) {
+   bool debug = false;
+   DBGMSF(debug, "data=%p", data);
    rpt_structure_loc("i2c_rdwr_ioctl_data", data, depth);
    int d1 = depth+1;
    int d2 = depth+2;
@@ -462,7 +464,7 @@ i2c_ioctl_writer(
    msgset.msgs  = messages;
    msgset.nmsgs = 1;
 
-  if (debug)
+   if (IS_TRACING())
       dbgrpt_i2c_rdwr_ioctl_data(1, &msgset);
 
    // per ioctl() man page:
@@ -472,7 +474,7 @@ i2c_ioctl_writer(
    // if error:
    //    -1, errno is set
    // 11/15: as seen: always returns 1 for success
-   RECORD_IO_EVENTX(
+   RECORD_IO_EVENT(
          fd,
          IE_IOCTL_WRITE,
          ( rc = ioctl(fd, I2C_RDWR, &msgset) )
@@ -509,8 +511,7 @@ i2c_ioctl_writer(
  * @retval 0    success
  * @retval <0   negative Linux errno value
  */
-// static  // disable to allow name in back trace
-Status_Errno_DDC
+STATIC Status_Errno_DDC
 i2c_ioctl_reader1(
       int    fd,
       Byte   slave_addr,
@@ -520,6 +521,11 @@ i2c_ioctl_reader1(
    DBGTRC_STARTING(debug, TRACE_GROUP, "fd=%d, fn=%s, slave_addr=0x%02x, bytect=%d, readbuf=%p",
                  fd, filename_for_fd_t(fd), slave_addr, bytect, readbuf);
 
+   // If read fails, readbuf will not be set.
+   // Initialize it here, otherwise valgrind complains about uninitialized variable
+   // in hexstring_t()
+   memset(readbuf, 0x00, bytect);
+
    int rc = 0;
    // messages needs to be allocated, cannot be on stack:
    struct i2c_msg * messages = calloc(1, sizeof(struct i2c_msg));
@@ -534,10 +540,10 @@ i2c_ioctl_reader1(
    msgset.msgs  = messages;
    msgset.nmsgs = 1;
 
-   if (debug)
+   if (IS_TRACING())
       dbgrpt_i2c_rdwr_ioctl_data(1, &msgset);
 
-   RECORD_IO_EVENTX(
+   RECORD_IO_EVENT(
       fd,
       IE_IOCTL_READ,
       ( rc = ioctl(fd, I2C_RDWR, &msgset))
@@ -560,6 +566,7 @@ i2c_ioctl_reader1(
       rc = -errsv;
 
    free(messages);
+   // DBGMSG("readbuf=%p, bytect=%d", readbuf, bytect);
    DBGTRC_RET_DDCRC(debug, TRACE_GROUP, rc, "readbuf: %s", hexstring_t(readbuf, bytect));
    return rc;
 }
@@ -585,8 +592,9 @@ i2c_ioctl_reader(
       Byte * readbuf)
 {
    bool debug = false;
-   DBGTRC_STARTING(debug, TRACE_GROUP, "fd=%d, fn=%s, slave_addr=0x%02x, bytect=%d, readbuf=%p",
-                 fd, filename_for_fd_t(fd), slave_addr, bytect, readbuf);
+   DBGTRC_STARTING(debug, TRACE_GROUP,
+         "fd=%d, fn=%s, slave_addr=0x%02x, read_bytewise=%s, bytect=%d, readbuf=%p",
+         fd, filename_for_fd_t(fd), slave_addr, SBOOL(read_bytewise), bytect, readbuf);
    int rc = 0;
 
    if (read_bytewise) {
