@@ -3,12 +3,13 @@
  *  Capabilities related functions of the API
  */
 
-// Copyright (C) 2015-2021 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2015-2023 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 #include "config.h"
 
 #include <assert.h>
+#include <glib-2.0/glib.h>
 #include <stdbool.h>
 #include <stddef.h>
 #include <string.h>
@@ -54,12 +55,14 @@ ddca_get_capabilities_string(
       char**               pcaps_loc)
 {
    bool debug = false;
-   DBGTRC_STARTING(debug, DDCA_TRC_API, "ddca_dh=%s", dh_repr((Display_Handle *) ddca_dh ) );
    free_thread_error_detail();
-   API_PRECOND(pcaps_loc);
+   API_PROLOGX(debug, "ddca_dh=%s", dh_repr((Display_Handle *) ddca_dh ) );
+   API_PRECOND_W_EPILOG(pcaps_loc);
    *pcaps_loc = NULL;
    Error_Info * ddc_excp = NULL;
-   WITH_VALIDATED_DH2(ddca_dh,
+   DDCA_Status psc = 0;
+
+   WITH_VALIDATED_DH3(ddca_dh, psc,
       {
          char * p_cap_string = NULL;
          ddc_excp = ddc_get_capabilities_string(dh, &p_cap_string);
@@ -69,15 +72,15 @@ ddca_get_capabilities_string(
          if (psc == 0) {
             // make copy to prevent caller from mucking around in ddcutil's
             // internal data structures
-            *pcaps_loc = strdup(p_cap_string);
+            *pcaps_loc = g_strdup(p_cap_string);
          }
          ASSERT_IFF(psc==0, *pcaps_loc);
-         DBGTRC_RET_DDCRC(debug, DDCA_TRC_API, psc,
-               "ddca_dh=%s, *pcaps_loc=%p -> |%s|",
-               dh_repr((Display_Handle *) ddca_dh),
-               *pcaps_loc, *pcaps_loc);
       }
    );
+
+   API_EPILOG(debug, psc, "ddca_dh=%s, *pcaps_loc=%p -> |%s|",
+                     dh_repr((Display_Handle *) ddca_dh),
+                     *pcaps_loc, *pcaps_loc );
 }
 
 
@@ -129,13 +132,11 @@ ddca_parse_capabilities_string(
       DDCA_Capabilities **     parsed_capabilities_loc)
 {
    bool debug = false;
-   DBGTRC_STARTING(debug, DDCA_TRC_API, "parsed_capabilities_loc=%p, capabilities_string: |%s|",
-                                       parsed_capabilities_loc, capabilities_string);
-   // assert(parsed_capabilities_loc);
    free_thread_error_detail();
-   API_PRECOND(parsed_capabilities_loc);
+   API_PROLOGX(debug, "parsed_capabilities_loc=%p, capabilities_string: |%s|",
+                     parsed_capabilities_loc, capabilities_string);
+   API_PRECOND_W_EPILOG(parsed_capabilities_loc);
    DDCA_Status ddcrc = DDCRC_BAD_DATA;
-   DBGMSF(debug, "ddcrc initialized to %s", psc_desc(ddcrc));
    DDCA_Capabilities * result = NULL;
 
    // need to control messages?
@@ -148,7 +149,7 @@ ddca_parse_capabilities_string(
       }
       result = calloc(1, sizeof(DDCA_Capabilities));
       memcpy(result->marker, DDCA_CAPABILITIES_MARKER, 4);
-      result->unparsed_string = strdup(capabilities_string);     // needed?
+      result->unparsed_string = g_strdup(capabilities_string);     // needed?
       result->version_spec = pcaps->parsed_mccs_version;
       DBGMSF(debug, "version: %d.%d", result->version_spec.major,  result->version_spec.minor);
       Byte_Value_Array bva = pcaps->commands;
@@ -175,7 +176,7 @@ ddca_parse_capabilities_string(
             cur_cap_vcp->feature_code = cur_cfr->feature_id;
             DBGMSF(debug, "cur_cfr = %p, feature_code - 0x%02x", cur_cfr, cur_cfr->feature_id);
 
-            // cur_cap_vcp->raw_values = strdup(cur_cfr->value_string);
+            // cur_cap_vcp->raw_values = g_strdup(cur_cfr->value_string);
             // TODO: get values from Byte_Bit_Flags cur_cfr->bbflags
 #ifdef CFR_BVA
             Byte_Value_Array bva = cur_cfr->values;
@@ -208,14 +209,11 @@ ddca_parse_capabilities_string(
       ddcrc = 0;
       free_parsed_capabilities(pcaps);
    }
-
    *parsed_capabilities_loc = result;
-
-   DBGTRC_RET_DDCRC(debug, DDCA_TRC_API, ddcrc, "*parsed_capabilities_loc=%p", *parsed_capabilities_loc);
+   API_EPILOG_WO_RETURN(debug, ddcrc, "*parsed_capabilities_loc=%p", *parsed_capabilities_loc);
+   ASSERT_IFF(ddcrc==0, *parsed_capabilities_loc);
    if ( IS_DBGTRC(debug, DDCA_TRC_API) && *parsed_capabilities_loc)
       dbgrpt_ddca_capabilities(*parsed_capabilities_loc, 2);
-
-   ASSERT_IFF(ddcrc==0, *parsed_capabilities_loc);
    return ddcrc;
 }
 
@@ -253,19 +251,17 @@ ddca_report_parsed_capabilities_by_dref(
       int                      depth)
 {
    bool debug = false;
-   DBGMSF(debug, "Starting. p_caps=%p, ddca_dref=%s", p_caps, dref_repr_t((Display_Ref*) ddca_dref));
-
-   free_thread_error_detail();
    DDCA_Status ddcrc = 0;
-
-   API_PRECOND(p_caps);   // no need to check marker, DDCA_CAPABILITIES not opaque
+   free_thread_error_detail();
+   API_PROLOGX(debug, "Starting. p_caps=%p, ddca_dref=%s",
+                      p_caps, dref_repr_t((Display_Ref*) ddca_dref));
+   API_PRECOND_W_EPILOG(p_caps);   // no need to check marker, DDCA_CAPABILITIES not opaque
 
    Display_Ref * dref = NULL;
    // dref may be NULL, but if not it must be valid
    if (ddca_dref) {
-      dref = validated_ddca_display_ref(ddca_dref);
-      if (!dref) {
-         ddcrc = DDCRC_ARG;
+      ddcrc = validate_ddca_display_ref(ddca_dref, /*basic_only*/ true, /*require_not_alseep*/ false, &dref);
+      if (ddcrc != 0) {
          goto bye;
       }
    }
@@ -346,7 +342,7 @@ ddca_report_parsed_capabilities_by_dref(
    }
 
 bye:
-   return ddcrc;
+   API_EPILOG(debug, ddcrc, "");
 }
 
 
@@ -366,10 +362,11 @@ ddca_report_parsed_capabilities_by_dh(
       int                      depth)
 {
    bool debug = false;
-   DBGMSF(debug, "Starting. p_caps=%p, ddca_dh=%s, depth=%d", p_caps, ddca_dh_repr(ddca_dh), depth);
-   DDCA_Status ddcrc = 0;
    free_thread_error_detail();
-   assert(library_initialized);
+   API_PROLOGX(debug, "p_caps=%p, ddca_dh=%s, depth=%d",
+                      p_caps, ddca_dh_repr(ddca_dh), depth);
+   DDCA_Status ddcrc = 0;
+
 
    Display_Handle * dh = (Display_Handle *) ddca_dh;
    if (dh == NULL || memcmp(dh->marker, DISPLAY_HANDLE_MARKER, 4) != 0 ) {
@@ -386,11 +383,38 @@ ddca_report_parsed_capabilities_by_dh(
    ddca_report_parsed_capabilities_by_dref(p_caps, dh->dref, depth);
 
 bye:
-   DBGMSF(debug, "Done.     Returning %s", ddca_rc_desc(ddcrc));
-   return ddcrc;
+   API_EPILOG(debug, ddcrc, "");
 }
 
 
+#ifdef UNPUBLISHED
+// UNPUBLISHED
+/** Parses a capabilities string, and reports the parsed string
+ *  using the code of command "ddcutil capabilities".
+ *
+ *  The report is written to the current FOUT location.
+ *
+ *  The detail level written is sensitive to the current output level.
+ *
+ *  @param[in]  capabilities_string  capabilities string
+ *  @param[in]  dref                 display reference
+ *  @param[in]  depth  logical       indentation depth
+ *
+ *  @remark
+ *  This function exists as a development aide.  Internally, ddcutil uses
+ *  a different data structure than DDCA_Parsed_Capabilities.  That
+ *  data structure uses internal collections that are not exposed at the
+ *  API level.
+ *  @remark
+ *  Signature changed in 0.9.3
+ *  @since 0.9.0
+ */
+void ddca_parse_and_report_capabilities(
+      char *                    capabilities_string,
+      DDCA_Display_Ref          dref,
+      int                       depth);
+
+
 // UNPUBLISHED
 void
 ddca_parse_and_report_capabilities(
@@ -402,6 +426,7 @@ ddca_parse_and_report_capabilities(
       dyn_report_parsed_capabilities(pcaps, NULL, dref, 0);
       free_parsed_capabilities(pcaps);
 }
+#endif
 
 
 DDCA_Feature_List
@@ -416,9 +441,10 @@ ddca_feature_list_from_capabilities(
    return result;
 }
 
-#ifdef NOT_NEEDED
 void init_api_capabilities() {
-   printf("(%s) Executing\n", __func__);
+   // printf("(%s) Executing\n", __func__);
    RTTI_ADD_FUNC(ddca_get_capabilities_string);
+   RTTI_ADD_FUNC(ddca_parse_capabilities_string);
+   RTTI_ADD_FUNC(ddca_report_parsed_capabilities_by_dref);
+   RTTI_ADD_FUNC(ddca_report_parsed_capabilities_by_dh);
 }
-#endif
