@@ -1,6 +1,6 @@
 /** @file api_displays.c */
 
-// Copyright (C) 2018-2022 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2018-2024 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 #include "config.h"
@@ -17,18 +17,25 @@
 #include "util/linux_util.h"
 #include "util/report_util.h"
 #include "util/string_util.h"
+#include "util/sysfs_util.h"
 
 #include "base/core.h"
+#include "base/dsa2.h"
 #include "base/displays.h"
 #include "base/monitor_model_key.h"
+#include "base/per_display_data.h"
+#include "base/rtti.h"
 
 #include "i2c/i2c_sysfs.h"
+#include "i2c/i2c_dpms.h"
 
 #include "ddc/ddc_displays.h"
 #include "ddc/ddc_display_ref_reports.h"
 #include "ddc/ddc_display_selection.h"
+#include "ddc/ddc_status_events.h"
 #include "ddc/ddc_packet_io.h"
 #include "ddc/ddc_vcp_version.h"
+#include "ddc/ddc_watch_displays.h"
 
 #include "libmain/api_base_internal.h"
 #include "libmain/api_error_info_internal.h"
@@ -39,22 +46,29 @@ static inline bool valid_display_handle(Display_Handle * dh) {
    return (dh && memcmp(dh->marker, DISPLAY_HANDLE_MARKER, 4) == 0);
 }
 
+#ifdef UNUSED
 static inline bool valid_display_ref(Display_Ref * dref) {
    return (dref && memcmp(dref->marker, DISPLAY_REF_MARKER, 4) == 0);
 }
+#endif
 
 
-Display_Ref * validated_ddca_display_ref(DDCA_Display_Ref ddca_dref) {
+DDCA_Status validate_ddca_display_ref(
+      DDCA_Display_Ref ddca_dref,
+      bool             basic_only,
+      bool             require_not_asleep,
+      Display_Ref**    dref_loc) {
+   if (dref_loc)
+      *dref_loc = NULL;
    Display_Ref * dref = (Display_Ref *) ddca_dref;
-   if (dref) {
-      if (memcmp(dref->marker, DISPLAY_REF_MARKER, 4) != 0  ||
-          !ddc_is_valid_display_ref(dref) )
-         dref=NULL;
-   }
-   return dref;
+   DDCA_Status result = ddc_validate_display_ref(dref, basic_only, require_not_asleep);
+   if (result == DDCRC_OK && dref_loc)
+      *dref_loc = dref;
+   return result;
 }
 
 
+#ifdef UNUSED
 Display_Handle * validated_ddca_display_handle(DDCA_Display_Handle ddca_dh) {
    Display_Handle * dh = (Display_Handle *) ddca_dh;
    if (dh) {
@@ -64,13 +78,29 @@ Display_Handle * validated_ddca_display_handle(DDCA_Display_Handle ddca_dh) {
    }
    return dh;
 }
+#endif
+
+
+DDCA_Status validate_ddca_display_handle(DDCA_Display_Handle ddca_dh, Display_Handle** dh_loc) {
+   if (dh_loc)
+      *dh_loc = NULL;
+   Display_Handle * dh = (Display_Handle *) ddca_dh;
+   DDCA_Status result = DDCRC_ARG;
+   if (dh && memcmp(dh->marker, DISPLAY_HANDLE_MARKER,4) == 0) {
+      result = ddc_validate_display_handle(dh);
+   }
+   if (result == DDCRC_OK && dh_loc)
+      *dh_loc = dh;
+   return result;
+}
+
 
 
 // forward declarations
 void dbgrpt_display_info(DDCA_Display_Info * dinfo, int depth);
 void dbgrpt_display_info_list(DDCA_Display_Info_List * dlist, int depth);
 
-
+#ifdef REMOVED
 DDCA_Status
 ddca_enable_usb_display_detection(bool onoff) {
    return ddc_enable_usb_display_detection(onoff);
@@ -80,6 +110,7 @@ bool
 ddca_ddca_is_usb_display_detection_enabled() {
    return ddc_is_usb_display_detection_enabled();
 }
+#endif
 
 
 //
@@ -249,21 +280,19 @@ ddca_get_display_ref(
 {
    free_thread_error_detail();
    bool debug = false;
-   DBGTRC_STARTING(debug, DDCA_TRC_API, "did=%p, dref_loc=%p", did, dref_loc);
+   API_PROLOGX(debug, "did=%p, dref_loc=%p", did, dref_loc);
    assert(library_initialized);
-   API_PRECOND(dref_loc);
-
-   DDCA_Status rc = 0;
+   API_PRECOND_W_EPILOG(dref_loc);
    *dref_loc = NULL;
-
+   DDCA_Status rc = 0;
    ddc_ensure_displays_detected();
 
    Display_Identifier * pdid = (Display_Identifier *) did;
    if (!pdid || memcmp(pdid->marker, DISPLAY_IDENTIFIER_MARKER, 4) != 0 )  {
-     rc = DDCRC_ARG;
+      rc = DDCRC_ARG;
    }
    else {
-      Display_Ref* dref = get_display_ref_for_display_identifier(pdid, CALLOPT_ERR_MSG);
+      Display_Ref* dref = get_display_ref_for_display_identifier(pdid, CALLOPT_NONE);
       if (debug)
          DBGMSG("get_display_ref_for_display_identifier() returned %p", dref);
       if (dref)
@@ -272,11 +301,14 @@ ddca_get_display_ref(
          rc = DDCRC_INVALID_DISPLAY;
    }
 
+   API_EPILOG_WO_RETURN(debug, rc, "*dref_loc=%p", psc_name_code(rc), *dref_loc);
    TRACED_ASSERT( (rc==0 && *dref_loc) || (rc!=0 && !*dref_loc) );
-   DBGTRC_DONE(debug, DDCA_TRC_API, "Returning: %s, *dref_loc=%p", psc_name_code(rc), *dref_loc);
    return rc;
 }
 
+
+
+
 // deprecated
 DDCA_Status
 ddca_create_display_ref(
@@ -286,12 +318,35 @@ ddca_create_display_ref(
    return ddca_get_display_ref(did, dref_loc);
 }
 
+#ifdef REMOVED
+/** @deprecated All display references are persistent
+ *
+ *  Frees a display reference.
+ *
+ *  Use this function to safely release a #DDCA_Display_Ref.
+ *  If the display reference was dynamically created, it is freed.
+ *  If the display reference was permanently allocated (normal case), does nothing.
+ *
+ *  @param[in] dref  display reference to free
+ *  @retval DDCRC_OK     success, or dref == NULL
+ *  @retval DDCRC_ARG    dref does not point to a valid display reference
+ *  @retval DDCRC_LOCKED dref is to a transient instance, and it is referenced
+ *                       by an open display handle
+ *
+ *  @ingroup api_display_spec
+ */
+// __attribute__ ((deprecated ("DDCA_Display_Refs are always persistent")))
+DDCA_Status
+ddca_free_display_ref(
+      DDCA_Display_Ref dref);
+#endif
 
+#ifdef REMOVED
 // deprecated, not needed, in library there are no transient display refs
 DDCA_Status
 ddca_free_display_ref(DDCA_Display_Ref ddca_dref) {
    bool debug = false;
-   DBGTRC_STARTING(debug, DDCA_TRC_API, "ddca_dref=%p", ddca_dref);
+   API_PROLOG(debug, "ddca_dref=%p", ddca_dref);
    DDCA_Status psc = 0;
    free_thread_error_detail();
    if (ddca_dref) {
@@ -302,45 +357,29 @@ ddca_free_display_ref(DDCA_Display_Ref ddca_dref) {
          }
       );
    }
-   DBGTRC_RET_DDCRC(debug, DDCA_TRC_API, psc, "");
+   API_EPILOG_WO_RETURN(debug, psc, "");
    return psc;
 }
+#endif
 
 
 DDCA_Status
 ddca_redetect_displays() {
    bool debug = false;
-   DBGTRC_STARTING(debug, DDCA_TRC_API, "");
+   API_PROLOGX(debug, "");
    ddc_redetect_displays();
-   DBGTRC_DONE(debug, DDCA_TRC_API, "Returning 0");
-   return 0;
+   API_EPILOG(debug, 0, "");
 }
 
 
-// static char dref_work_buf[100];
-
 const char *
 ddca_dref_repr(DDCA_Display_Ref ddca_dref) {
    bool debug = false;
    DBGMSF(debug, "Starting.  ddca_dref = %p", ddca_dref);
    char * result = NULL;
-   Display_Ref * dref = validated_ddca_display_ref(ddca_dref);
+   Display_Ref * dref = NULL;
+   validate_ddca_display_ref(ddca_dref, /*basic_only*/ true, /* require_not_asleep */ false, &dref);
    if (dref) {
-#ifdef TOO_MUCH_WORK
-      char * dref_type_name = io_mode_name(dref->ddc_io_mode);
-      switch (dref->ddc_io_mode) {
-      case(DISP_ID_BUSNO):
-         snprintf(dref_work_buf, 100,
-                  "Display Ref Type: %s, bus=/dev/i2c-%d", dref_type_name, dref->io_path.i2c_busno);
-         break;
-      case(DISP_ID_ADL):
-         snprintf(dref_work_buf, 100,
-                  "Display Ref Type: %s, adlno=%d.%d", dref_type_name, dref->io_path.adlno.iAdapterIndex, dref->io_path.adlno.iDisplayIndex);
-         break;
-      }
-      *repr = did_work_buf;
-#endif
-      // result = dref_short_name(dref);
       result = dref_repr_t(dref);
    }
    DBGMSF(debug, "Done.     Returning: %s", result);
@@ -355,7 +394,8 @@ ddca_dbgrpt_display_ref(
 {
    bool debug = false;
    DBGMSF(debug, "Starting.  ddca_dref = %p, depth=%d", ddca_dref, depth);
-   Display_Ref * dref = validated_ddca_display_ref(ddca_dref);
+   Display_Ref * dref = NULL;
+   validate_ddca_display_ref(ddca_dref, /* basic_only*/ true, /* require_not_asleep */ false, &dref);
    rpt_vstring(depth, "DDCA_Display_Ref at %p:", dref);
    if (dref)
       dbgrpt_display_ref(dref, depth+1);
@@ -367,18 +407,61 @@ ddca_report_display_by_dref(
       DDCA_Display_Ref ddca_dref,
       int              depth)
 {
-   DBGTRC_STARTING(false, DDCA_TRC_API, "ddca_dref=%p", ddca_dref);
+   bool debug = false;
    free_thread_error_detail();
-   DDCA_Status rc = 0;
-
+   API_PROLOGX(debug, "ddca_dref=%p", ddca_dref);
    assert(library_initialized);
 
    Display_Ref * dref = NULL;
-   VALIDATE_DDCA_DREF(ddca_dref, dref, /*debug=*/false);
+   DDCA_Status rc = validate_ddca_display_ref(ddca_dref,  /* basic_only*/ true, /*require_not_asleep*/ false,  &dref);
+   if (rc == 0)
+      ddc_report_display_by_dref(dref, depth);
 
-   ddc_report_display_by_dref(dref, depth);
+   API_EPILOG_WO_RETURN(debug, rc, "");
+   return rc;
+}
 
-   DBGTRC_DONE(false, DDCA_TRC_API, "Returning %s", psc_name_code(rc));
+
+DDCA_Status
+ddca_validate_display_ref(DDCA_Display_Ref ddca_dref, bool require_not_asleep)
+{
+   bool debug = false;
+   free_thread_error_detail();
+   API_PROLOGX(debug, "ddca_dref = %p", ddca_dref);
+   assert(library_initialized);
+
+   Display_Ref * dref = NULL;
+   DDCA_Status rc = DDCRC_ARG;
+   if (ddca_dref)
+      rc = validate_ddca_display_ref(ddca_dref, /* basic_only*/ false,  require_not_asleep, &dref);
+#ifdef REDUNDANT
+   Error_Info * errinfo = NULL;
+   if (rc != 0)
+      errinfo = ERRINFO_NEW(rc, "");
+   else {
+      assert(sys_drm_connectors);
+      if (!dref->drm_connector) {
+         errinfo = ERRINFO_NEW(DDCRC_INTERNAL_ERROR, "dref->drm_connector == NULL");
+      }
+      else {
+         int d = (debug) ? 1 : -1;
+        bool edid_exists = RPT_ATTR_EDID(d, NULL, "/sys/class/drm", dref->drm_connector, "edid");
+        if (!edid_exists) {
+           errinfo = ERRINFO_NEW(DDCRC_DISCONNECTED, "/dev/i2c-%d", dref->io_path.path.i2c_busno);
+        }
+        else {
+           if (dpms_check_drm_asleep_by_dref(dref))
+              errinfo = ERRINFO_NEW(DDCRC_DPMS_ASLEEP, "/dev/i2c-%d", dref->io_path.path.i2c_busno);
+        }
+      }
+   }
+   DDCA_Status ddcrc = ERRINFO_STATUS(errinfo);
+   DDCA_Error_Detail * public_error_detail = error_info_to_ddca_detail(errinfo);
+   errinfo_free_with_report(errinfo, debug, __func__);
+   save_thread_error_detail(public_error_detail);
+#endif
+
+   API_EPILOG_WO_RETURN(debug, rc, "");
    return rc;
 }
 
@@ -387,6 +470,23 @@ ddca_report_display_by_dref(
 // Open and close display
 //
 
+#ifdef DEPRECATED
+
+/** \deprecated Use #ddca_open_display2()
+ * Open a display
+ * @param[in]  ddca_dref    display reference for display to open
+ * @param[out] ddca_dh_loc  where to return display handle
+ * @return     status code
+ *
+ * Fails if display is already opened by another thread.
+ * \ingroup api_display_spec
+ */
+// __attribute__ ((deprecated ("use ddca_open_display2()")))
+DDCA_Status
+ddca_open_display(
+      DDCA_Display_Ref      ddca_dref,
+      DDCA_Display_Handle * ddca_dh_loc);
+
 DDCA_Status
 ddca_open_display(
       DDCA_Display_Ref      ddca_dref,
@@ -394,6 +494,7 @@ ddca_open_display(
 {
    return ddca_open_display2(ddca_dref, false, dh_loc);
 }
+#endif
 
 
 // unpublished extension
@@ -409,7 +510,7 @@ typedef enum {
 } DDCA_Open_Options;
 
 
-static
+STATIC
 DDCA_Status
 ddca_open_display3(
       DDCA_Display_Ref      ddca_dref,
@@ -417,42 +518,41 @@ ddca_open_display3(
       DDCA_Display_Handle * dh_loc)
 {
    bool debug = false;
-   DBGTRC_STARTING(debug, DDCA_TRC_API,
+   free_thread_error_detail();
+   API_PROLOGX(debug,
           "ddca_dref=%p, options=0x%02x, dh_loc=%p, on thread %d",
           ddca_dref, options, dh_loc, get_thread_id());
    DBGTRC_NOPREFIX(debug, DDCA_TRC_API,
           "library_initialized=%s, ddc_displays_already_detected() = %ld",
           sbool(library_initialized), ddc_displays_already_detected());
-   free_thread_error_detail();
    TRACED_ASSERT(library_initialized);
    TRACED_ASSERT(ddc_displays_already_detected());
 
-   API_PRECOND(dh_loc);
-
+   API_PRECOND_W_EPILOG(dh_loc);
+   Display_Ref * dref = NULL;
    *dh_loc = NULL;        // in case of error
    DDCA_Status rc = 0;
-   Display_Ref * dref = validated_ddca_display_ref(ddca_dref);
-   if (!dref) {
-      rc = DDCRC_ARG;
-   }
-   else {
-      // for testing failure:
-      // TRACED_ASSERT(1==2);
-      // assert(1==2);
-
+   Error_Info * err = NULL;
+   rc  = validate_ddca_display_ref(ddca_dref, /* basic_only*/ false, /* require_not_asleep */ true, &dref);
+   if (!rc) {
      Display_Handle* dh = NULL;
      Call_Options callopts = CALLOPT_NONE;
      if (options & DDCA_OPENOPT_WAIT)
         callopts |= CALLOPT_WAIT;
      if (options & DDCA_OPENOPT_FORCE_SLAVE_ADDR)
         callopts |= CALLOPT_FORCE_SLAVE_ADDR;
-     rc = ddc_open_display(dref,  callopts, &dh);
-     if (rc == 0)
+     err = ddc_open_display(dref,  callopts, &dh);
+     if (!err)
         *dh_loc = dh;
+     else {
+        rc = err->status_code;
+        DDCA_Error_Detail * public_error_detail = error_info_to_ddca_detail(err);
+        errinfo_free_with_report(err, debug, __func__);
+        save_thread_error_detail(public_error_detail);
+     }
    }
 
-   DBGTRC_RET_DDCRC(debug, DDCA_TRC_API, rc, "*dh_loc=%p -> %s",
-                                             *dh_loc, dh_repr(*dh_loc));
+   API_EPILOG_WO_RETURN(debug, rc, "*dh_loc=%p -> %s", *dh_loc, dh_repr(*dh_loc));
    TRACED_ASSERT_IFF(rc==0, *dh_loc);
    return rc;
 }
@@ -474,20 +574,28 @@ DDCA_Status
 ddca_close_display(DDCA_Display_Handle ddca_dh) {
    bool debug = false;
    free_thread_error_detail();
-   assert(library_initialized);
    DDCA_Status rc = 0;
+   Error_Info * err = NULL;
    Display_Handle * dh = (Display_Handle *) ddca_dh;
-   DBGTRC_STARTING(debug, DDCA_TRC_API, "dh = %s", dh_repr(dh));
+   API_PROLOGX(debug, "dh = %s", dh_repr(dh));
    if (dh) {
       if (memcmp(dh->marker, DISPLAY_HANDLE_MARKER, 4) != 0 )  {
-         rc = DDCRC_ARG;
+         err = errinfo_new(DDCRC_ARG, __func__, "Invalid display handle");
       }
       else {
          // TODO: ddc_close_display() needs an action if failure parm,
-         rc = ddc_close_display(dh);
+         err = ddc_close_display(dh);
       }
    }
-   DBGTRC_DONE(debug, DDCA_TRC_API, "Returning %s(%d)", psc_name(rc), rc);
+
+   if (err) {
+      rc = err->status_code;
+      DDCA_Error_Detail * public_error_detail = error_info_to_ddca_detail(err);
+      errinfo_free_with_report(err, debug, __func__);
+      save_thread_error_detail(public_error_detail);
+   }
+
+   API_EPILOG_WO_RETURN(debug, rc, "");
    return rc;
 }
 
@@ -543,6 +651,7 @@ ddca_get_mccs_version_by_dh(
 }
 
 
+#ifdef NOT_PUBLISHED
 // not published
 DDCA_Status
 ddca_get_mccs_version_with_default(
@@ -555,6 +664,7 @@ ddca_get_mccs_version_with_default(
       *p_spec = default_spec;
    return rc;
 }
+#endif
 
 
 #ifdef MCCS_VERSION_ID
@@ -614,15 +724,17 @@ ddca_mccs_version_id_desc(DDCA_MCCS_Version_Id version_id) {
 // Monitor Model Identifier
 //
 
-const DDCA_Monitor_Model_Key DDCA_UNDEFINED_MONITOR_MODEL_KEY = {{0}};
+#ifdef REMOVED
 
-DDCA_Monitor_Model_Key
+const Monitor_Model_Key UNDEFINED_MONITOR_MODEL_KEY = {{0}};
+
+Monitor_Model_Key
 ddca_mmk(
       const char * mfg_id,
       const char * model_name,
       uint16_t     product_code)
 {
-   DDCA_Monitor_Model_Key result = DDCA_UNDEFINED_MONITOR_MODEL_KEY;
+   Monitor_Model_Key result = UNDEFINED_MONITOR_MODEL_KEY;
    if (mfg_id     && strlen(mfg_id)     < DDCA_EDID_MFG_ID_FIELD_SIZE &&
        model_name && strlen(model_name) < DDCA_EDID_MODEL_NAME_FIELD_SIZE)
    {
@@ -633,8 +745,8 @@ ddca_mmk(
 
 bool
 ddca_mmk_eq(
-      DDCA_Monitor_Model_Key mmk1,
-      DDCA_Monitor_Model_Key mmk2)
+      Monitor_Model_Key mmk1,
+      Monitor_Model_Key mmk2)
 {
    return monitor_model_key_eq(mmk1, mmk2);
 }
@@ -642,17 +754,17 @@ ddca_mmk_eq(
 
 bool
 ddca_mmk_is_defined(
-      DDCA_Monitor_Model_Key mmk)
+      Monitor_Model_Key mmk)
 {
    return mmk.defined;
 }
 
 
-DDCA_Monitor_Model_Key
+Monitor_Model_Key
 ddca_mmk_from_dref(
       DDCA_Display_Ref   ddca_dref)
 {
-   DDCA_Monitor_Model_Key result = DDCA_UNDEFINED_MONITOR_MODEL_KEY;
+   Monitor_Model_Key result = UNDEFINED_MONITOR_MODEL_KEY;
    Display_Ref * dref = (Display_Ref *) ddca_dref;
    if (valid_display_ref(dref) && dref->mmid)
       result = *dref->mmid;
@@ -660,22 +772,31 @@ ddca_mmk_from_dref(
 }
 
 
-DDCA_Monitor_Model_Key
+Monitor_Model_Key
 ddca_mmk_from_dh(
       DDCA_Display_Handle   ddca_dh)
 {
-   DDCA_Monitor_Model_Key result = DDCA_UNDEFINED_MONITOR_MODEL_KEY;
+   Monitor_Model_Key result = UNDEFINED_MONITOR_MODEL_KEY;
    Display_Handle * dh = (Display_Handle *) ddca_dh;
    if (valid_display_handle(dh) && dh->dref->mmid)
       result = *dh->dref->mmid;
    return result;
 }
-
+#endif
 
 //
 // Display Info
 //
 
+#ifdef DEPRECATED
+/** @deprecated use #ddca_get_display_info_list2()
+ * Gets a list of the detected displays.
+ *
+ *  Displays that do not support DDC are not included.
+ *
+ *  @return list of display summaries
+ */
+__attribute__ ((deprecated ("use ddca_get_display_info_list2()")))
 DDCA_Display_Info_List *
 ddca_get_display_info_list(void)
 {
@@ -683,9 +804,10 @@ ddca_get_display_info_list(void)
    ddca_get_display_info_list2(false, &result);
    return result;
 }
+#endif
 
 
-static void init_display_info(Display_Ref * dref, DDCA_Display_Info * curinfo) {
+STATIC void init_display_info(Display_Ref * dref, DDCA_Display_Info * curinfo) {
    bool debug = false;
    DBGMSF(debug, "dref=%p, curinfo=%p", dref,curinfo);
    memcpy(curinfo->marker, DDCA_DISPLAY_INFO_MARKER, 4);
@@ -741,27 +863,32 @@ ddca_get_display_info(
       DDCA_Display_Info ** dinfo_loc)
 {
    bool debug = false;
-   DBGTRC_STARTING(debug, DDCA_TRC_API,  "ddca_dref=%p", ddca_dref);
+   // causes return DDCRC_UNITIALIZED: called after explicit ddca_init()/init2() call failed
+   API_PROLOGX(debug, "ddca_dref=%p", ddca_dref);
+   // causes return DDCRC_ARG if dinfo_loc == NULL
+   API_PRECOND_W_EPILOG(dinfo_loc);
    DDCA_Status ddcrc = 0;
 
-   WITH_VALIDATED_DR3(
+   // if ddc_validate_display_ref() fails, returns its status code
+   WITH_BASIC_VALIDATED_DR3(
          ddca_dref, ddcrc,
          {
-               API_PRECOND(dinfo_loc);
-               DDCA_Display_Info * info = calloc(1, sizeof(DDCA_Display_Info));
-               init_display_info(dref, info);
-               *dinfo_loc = info;
+            DDCA_Display_Info * info = calloc(1, sizeof(DDCA_Display_Info));
+            init_display_info(dref, info);
+            *dinfo_loc = info;
          }
    )
 
-   DBGTRC_RET_DDCRC(debug, DDCA_TRC_API, ddcrc, "");
+   API_EPILOG_WO_RETURN(debug, ddcrc, "");
    return ddcrc;
 }
 
 
-void set_ddca_error_detail_from_open_errors() {
+STATIC DDCA_Status
+set_ddca_error_detail_from_open_errors() {
    bool debug = false;
    GPtrArray * errs = ddc_get_bus_open_errors();
+   DDCA_Status master_rc = 0;
    if (errs && errs->len > 0) {
       Error_Info * master_error = errinfo_new(DDCRC_OTHER, __func__, "Error(s) opening ddc devices");
       for (int ndx = 0; ndx < errs->len; ndx++) {
@@ -771,14 +898,16 @@ void set_ddca_error_detail_from_open_errors() {
             errinfo = errinfo_new(cur->error, __func__, "Error %s opening /dev/i2c-%d",
                                              psc_desc(cur->error), cur->devno);
          else
-            errinfo = errinfo_new(cur->error, __func__, "Error %s opening /dev/usb/hiddev%d",
-                                             psc_desc(cur->error), cur->devno);
+            errinfo = errinfo_new(cur->error, __func__, "Error %s opening /dev/usb/hiddev%d %s",
+                                             psc_desc(cur->error), cur->devno, (cur->detail) ? cur->detail : "");
          errinfo_add_cause(master_error, errinfo);
       }
+      master_rc = master_error->status_code;
       DDCA_Error_Detail * public_error_detail = error_info_to_ddca_detail(master_error);
       errinfo_free_with_report(master_error, debug, __func__);
       save_thread_error_detail(public_error_detail);
    }
+   return master_rc;
 }
 
 
@@ -789,14 +918,14 @@ ddca_get_display_refs(
       DDCA_Display_Ref**  drefs_loc)
 {
    bool debug = false;
-   DBGTRC_STARTING(debug, DDCA_TRC_API|DDCA_TRC_DDC,
-                 "include_invalid_displays=%s", SBOOL(include_invalid_displays));
    free_thread_error_detail();
-   API_PRECOND(drefs_loc);
+   API_PROLOGX(debug, "include_invalid_displays=%s", SBOOL(include_invalid_displays));
 
+   API_PRECOND_W_EPILOG(drefs_loc);
+   int dref_ct = 0;
+   DDCA_Status ddcrc = 0;
    ddc_ensure_displays_detected();
-   GPtrArray * filtered_displays = ddc_get_filtered_displays(include_invalid_displays);  // array of Display_Ref
-
+   GPtrArray * filtered_displays = ddc_get_filtered_display_refs(include_invalid_displays);  // array of Display_Ref
    DDCA_Display_Ref * result_list = calloc(filtered_displays->len + 1,sizeof(DDCA_Display_Ref));
    DDCA_Display_Ref * cur_ddca_dref = result_list;
    for (int ndx = 0; ndx < filtered_displays->len; ndx++) {
@@ -807,7 +936,7 @@ ddca_get_display_refs(
    *cur_ddca_dref = NULL; // terminating NULL ptr, redundant since calloc()
    g_ptr_array_free(filtered_displays, true);
 
-   int dref_ct = 0;
+   dref_ct = 0;
    if (IS_DBGTRC(debug, DDCA_TRC_API|DDCA_TRC_DDC )) {
       DBGMSG("          *drefs_loc=%p");
       DDCA_Display_Ref * cur_ddca_dref = result_list;
@@ -822,10 +951,9 @@ ddca_get_display_refs(
    *drefs_loc = result_list;
    assert(*drefs_loc);
 
-   set_ddca_error_detail_from_open_errors();
+   ddcrc = set_ddca_error_detail_from_open_errors();
 
-   DBGTRC_RET_DDCRC(debug, DDCA_TRC_API, 0, "Returned list has %d displays", dref_ct);
-   return 0;
+   API_EPILOG(debug, ddcrc, "Returned list has %d displays", dref_ct);
 }
 
 
@@ -835,14 +963,17 @@ ddca_get_display_info_list2(
       DDCA_Display_Info_List**  dlist_loc)
 {
    bool debug = false;
-   DBGTRC_STARTING(debug, DDCA_TRC_API|DDCA_TRC_DDC, "");
    free_thread_error_detail();
-   API_PRECOND(dlist_loc);
+   API_PROLOGX(debug, "");
 
+   int filtered_ct = 0;
+   API_PRECOND_W_EPILOG(dlist_loc);
+
+   DDCA_Status ddcrc = 0;
    ddc_ensure_displays_detected();
-   GPtrArray * filtered_displays = ddc_get_filtered_displays(include_invalid_displays);  // array of Display_Ref
+   GPtrArray * filtered_displays = ddc_get_filtered_display_refs(include_invalid_displays);  // array of Display_Ref
 
-   int filtered_ct = filtered_displays->len;
+   filtered_ct = filtered_displays->len;
 
    int reqd_size =   offsetof(DDCA_Display_Info_List,info) + filtered_ct * sizeof(DDCA_Display_Info);
    DBGMSF(debug, "reqd_size=%d", reqd_size);
@@ -870,19 +1001,18 @@ ddca_get_display_info_list2(
       dbgrpt_display_info_list(result_list, 2);
    }
 
-   set_ddca_error_detail_from_open_errors();
+   ddcrc = set_ddca_error_detail_from_open_errors();
    *dlist_loc = result_list;
    assert(*dlist_loc);
-   DBGTRC_RET_DDCRC(debug, DDCA_TRC_API|DDCA_TRC_DDC, 0,
-                           "Returned list has %d displays", result_list->ct);
-   return 0;
+
+   API_EPILOG(debug, ddcrc, "Returned list has %d displays", filtered_ct);
 }
 
 
 void
 ddca_free_display_info(DDCA_Display_Info * info_rec) {
    bool debug = false;
-   DBGTRC_STARTING(debug, DDCA_TRC_API, "info_rec->%p", info_rec);
+   API_PROLOG(debug, "info_rec->%p", info_rec);
    // DDCA_Display_Info contains no pointers, can simply be free'd
    // data structures.  Nothing to free.
    if (info_rec && memcmp(info_rec->marker, DDCA_DISPLAY_INFO_MARKER, 4) == 0) {
@@ -890,13 +1020,14 @@ ddca_free_display_info(DDCA_Display_Info * info_rec) {
       free(info_rec);
    }
    DBGTRC_DONE(debug, DDCA_TRC_API,"");
+   DISABLE_API_CALL_TRACING();
 }
 
 
 void
 ddca_free_display_info_list(DDCA_Display_Info_List * dlist) {
    bool debug = false;
-   DBGTRC_STARTING(debug, DDCA_TRC_API, "dlist=%p", dlist);
+   API_PROLOG(debug, "dlist=%p", dlist);
    if (dlist) {
       // n. DDCA_Display_Info contains no pointers,
       // DDCA_Display_Info_List can simply be free'd.
@@ -908,108 +1039,107 @@ ddca_free_display_info_list(DDCA_Display_Info_List * dlist) {
       free(dlist);
    }
    DBGTRC_DONE(debug, DDCA_TRC_API, "");
+   DISABLE_API_CALL_TRACING();
 }
 
 
-void
+DDCA_Status
 ddca_report_display_info(
       DDCA_Display_Info * dinfo,
       int                 depth)
 {
-   API_PRECOND_NORC(dinfo);
-   API_PRECOND_NORC(memcmp(dinfo->marker, DDCA_DISPLAY_INFO_MARKER, 4) == 0);
    bool debug = false;
-   DBGMSF(debug, "Starting. dinfo=%p, dinfo->dispno=%d, depth=%d", dinfo, dinfo->dispno, depth);
+   API_PROLOGX(debug, "Starting. dinfo=%p, dinfo->dispno=%d, depth=%d", dinfo, dinfo->dispno, depth);
+   DDCA_Status rc = 0;
+   API_PRECOND_W_EPILOG(dinfo);
+   API_PRECOND_W_EPILOG(memcmp(dinfo->marker, DDCA_DISPLAY_INFO_MARKER, 4) == 0);
+   if (rc == 0) {
+      int d0 = depth;
+      int d1 = depth+1;
+      int d2 = depth+2;
+      if (dinfo->dispno > 0)
+         rpt_vstring(d0, "Display number:  %d", dinfo->dispno);
+      else if (dinfo->dispno == DISPNO_BUSY)
+         rpt_vstring(d0, "Busy display - Cannot communicate DDC");
+      else
+         rpt_label(  d0, "Invalid display - Does not support DDC");
+      // rpt_vstring(      d1, "Display ref:         %p -> %s", dinfo->dref, dref_repr_t(dinfo->dref) );
+      // rpt_vstring(d1, "IO mode:             %s", io_mode_name(dinfo->path.io_mode));
+      switch(dinfo->path.io_mode) {
+      case (DDCA_IO_I2C):
+            rpt_vstring(d1, "I2C bus:              /dev/i2c-%d", dinfo->path.path.i2c_busno);
+            break;
+      case (DDCA_IO_USB):
+            rpt_vstring(d1, "USB bus.device:       %d.%d",
+                            dinfo->usb_bus, dinfo->usb_device);
+            rpt_vstring(d1, "USB hiddev device:   /dev/usb/hiddev%d", dinfo->path.path.hiddev_devno);
+            break;
+      }
 
-   int d0 = depth;
-   int d1 = depth+1;
-   int d2 = depth+2;
-   if (dinfo->dispno > 0)
-      rpt_vstring(d0, "Display number:  %d", dinfo->dispno);
-   else if (dinfo->dispno == DISPNO_BUSY)
-      rpt_vstring(d0, "Busy display - Cannot communicate DDC");
-   else
-      rpt_label(  d0, "Invalid display - Does not support DDC");
-   // rpt_vstring(      d1, "Display ref:         %p -> %s", dinfo->dref, dref_repr_t(dinfo->dref) );
-   // rpt_vstring(d1, "IO mode:             %s", io_mode_name(dinfo->path.io_mode));
-   switch(dinfo->path.io_mode) {
-   case (DDCA_IO_I2C):
-         rpt_vstring(d1, "I2C bus:              /dev/i2c-%d", dinfo->path.path.i2c_busno);
-         break;
-   case (DDCA_IO_ADL):
-         rpt_vstring(d1, "ADL adapter.display:  %d.%d",
-                         dinfo->path.path.adlno.iAdapterIndex, dinfo->path.path.adlno.iDisplayIndex);
-         break;
-   case (DDCA_IO_USB):
-         rpt_vstring(d1, "USB bus.device:       %d.%d",
-                         dinfo->usb_bus, dinfo->usb_device);
-         rpt_vstring(d1, "USB hiddev device:   /dev/usb/hiddev%d", dinfo->path.path.hiddev_devno);
-         break;
-   }
+      rpt_vstring(d1, "Mfg Id:               %s", dinfo->mfg_id);
+      rpt_vstring(d1, "Model:                %s", dinfo->model_name);
+      rpt_vstring(d1, "Product code:         %u", dinfo->product_code);
+      rpt_vstring(d1, "Serial number:        %s", dinfo->sn);
 
-   rpt_vstring(d1, "Mfg Id:               %s", dinfo->mfg_id);
-   rpt_vstring(d1, "Model:                %s", dinfo->model_name);
-   rpt_vstring(d1, "Product code:         %u", dinfo->product_code);
-   rpt_vstring(d1, "Serial number:        %s", dinfo->sn);
-
-   // binary SN is not part of DDCA_Display_Info
-   Parsed_Edid * edid = create_parsed_edid(dinfo->edid_bytes);
-   if (edid) {     // should never fail, but being ultra-cautious
-      // Binary serial number is typically 0x00000000 or 0x01010101, but occasionally
-      // useful for differentiating displays that share a generic ASCII "serial number"
-      rpt_vstring(d1,"Binary serial number: %"PRIu32" (0x%08x)", edid->serial_binary, edid->serial_binary);
-      free_parsed_edid(edid);
-   }
+      // binary SN is not part of DDCA_Display_Info
+      Parsed_Edid * edid = create_parsed_edid(dinfo->edid_bytes);
+      if (edid) {     // should never fail, but being ultra-cautious
+         // Binary serial number is typically 0x00000000 or 0x01010101, but occasionally
+         // useful for differentiating displays that share a generic ASCII "serial number"
+         rpt_vstring(d1,"Binary serial number: %"PRIu32" (0x%08x)", edid->serial_binary, edid->serial_binary);
+         free_parsed_edid(edid);
+      }
 
 #ifdef NOT_WORKING
-   if (dinfo->path.io_mode == DDCA_IO_I2C) {
-      I2C_Sys_Info * info = get_i2c_sys_info(dinfo->path.path.i2c_busno, -1);
-      rpt_vstring(d1, "DRM Connector:        %s", (info->connector) ? info->connector : "");
-   }
+      if (dinfo->path.io_mode == DDCA_IO_I2C) {
+         I2C_Sys_Info * info = get_i2c_sys_info(dinfo->path.path.i2c_busno, -1);
+         rpt_vstring(d1, "DRM Connector:        %s", (info->connector) ? info->connector : "");
+      }
 #endif
 
-   // rpt_label(  d1, "Monitor Model Id:");
-   // rpt_vstring(d2, "Mfg Id:           %s", dinfo->mmid.mfg_id);
-   // rpt_vstring(d2, "Model name:       %s", dinfo->mmid.model_name);
-   // rpt_vstring(d2, "Product code:     %d", dinfo->mmid.product_code);
-   rpt_vstring(d1, "EDID:");
-   rpt_hex_dump(dinfo->edid_bytes, 128, d2);
-   // rpt_vstring(d1, "dref:                %p", dinfo->dref);
-   rpt_vstring(d1, "VCP Version:          %s", format_vspec(dinfo->vcp_version));
-// rpt_vstring(d1, "VCP Version Id:      %s", format_vcp_version_id(dinfo->vcp_version_id) );
+      // rpt_label(  d1, "Monitor Model Id:");
+      // rpt_vstring(d2, "Mfg Id:           %s", dinfo->mmid.mfg_id);
+      // rpt_vstring(d2, "Model name:       %s", dinfo->mmid.model_name);
+      // rpt_vstring(d2, "Product code:     %d", dinfo->mmid.product_code);
+      rpt_vstring(d1, "EDID:");
+      rpt_hex_dump(dinfo->edid_bytes, 128, d2);
+      // rpt_vstring(d1, "dref:                %p", dinfo->dref);
+      rpt_vstring(d1, "VCP Version:          %s", format_vspec(dinfo->vcp_version));
+   // rpt_vstring(d1, "VCP Version Id:      %s", format_vcp_version_id(dinfo->vcp_version_id) );
 
 
-   if (dinfo->dispno == DISPNO_BUSY) {
-#ifdef OLD
-      rpt_nl();
-      char fn[20];
-      int busno =  dinfo->path.path.i2c_busno;
-      g_snprintf(fn, 20, "/dev/bus/ddcci/%d", busno);
-      struct stat statrec;
-      if (stat(fn, &statrec) == 0 )
-         rpt_vstring(d1, "Driver ddcci is hogging I2C slave address x37 (DDC) on /dev/i2c-%d", busno);
-#endif
-      Display_Ref * dref = (Display_Ref *) dinfo->dref;
-      int busno = dref->io_path.path.i2c_busno;
-      GPtrArray * conflicts = collect_conflicting_drivers(busno, -1);
-      if (conflicts && conflicts->len > 0) {
-         rpt_vstring(d1, "I2C bus is busy. Likely conflicting driver(s): %s",
-                         conflicting_driver_names_string_t(conflicts));
-         free_conflicting_drivers(conflicts);
+      if (dinfo->dispno == DISPNO_BUSY) {
+   #ifdef OLD
+         rpt_nl();
+         char fn[20];
+         int busno =  dinfo->path.path.i2c_busno;
+         g_snprintf(fn, 20, "/dev/bus/ddcci/%d", busno);
+         struct stat statrec;
+         if (stat(fn, &statrec) == 0 )
+            rpt_vstring(d1, "Driver ddcci is hogging I2C slave address x37 (DDC) on /dev/i2c-%d", busno);
+   #endif
+         Display_Ref * dref = (Display_Ref *) dinfo->dref;
+         int busno = dref->io_path.path.i2c_busno;
+         GPtrArray * conflicts = collect_conflicting_drivers(busno, -1);
+         if (conflicts && conflicts->len > 0) {
+            rpt_vstring(d1, "I2C bus is busy. Likely conflicting driver(s): %s",
+                            conflicting_driver_names_string_t(conflicts));
+            free_conflicting_drivers(conflicts);
+         }
+         else {
+            struct stat stat_buf;
+            char buf[20];
+            g_snprintf(buf, 20, "/dev/bus/ddcci/%d", busno);
+            // DBGMSG("buf: %s", buf);
+            int rc = stat(buf, &stat_buf);
+            // DBGMSG("stat returned %d", rc);
+            if (rc == 0)
+               rpt_label(d1, "I2C bus is busy. Likely conflict with driver ddcci.");
+         }
+         rpt_vstring(d1, "Consider using option --force-slave-address.");
       }
-      else {
-         struct stat stat_buf;
-         char buf[20];
-         g_snprintf(buf, 20, "/dev/bus/ddcci/%d", busno);
-         // DBGMSG("buf: %s", buf);
-         int rc = stat(buf, &stat_buf);
-         // DBGMSG("stat returned %d", rc);
-         if (rc == 0)
-            rpt_label(d1, "I2C bus is busy. Likely conflict with driver ddcci.");
-      }
-      rpt_vstring(d1, "Consider using option --force-slave-address.");
    }
-   DBGMSF(debug, "Done.");
+   API_EPILOG(debug, rc, "");
 }
 
 
@@ -1067,6 +1197,16 @@ dbgrpt_display_info_list(
 // Miscellaneous
 //
 
+#ifdef DEPRECATED
+
+// /** \deprecated */
+__attribute__ ((deprecated))
+DDCA_Status
+ddca_get_edid_by_dref(
+      DDCA_Display_Ref ddca_dref,
+      uint8_t **       pbytes_loc);   // pointer into ddcutil data structures, do not free
+
+
 // deprecated
 DDCA_Status
 ddca_get_edid_by_dref(
@@ -1094,6 +1234,7 @@ ddca_get_edid_by_dref(
 bye:
    return rc;
 }
+#endif
 
 
 #ifdef UNIMPLEMENTED
@@ -1109,16 +1250,203 @@ ddca_get_edid(DDCA_Display_Handle * dh, uint8_t* edid_buffer);
 // Reports
 //
 
+#ifdef DEPRECATED
+/** \deprecated use #ddca_report_displays()
+ * Reports on all active displays.
+ *  This function hooks into the code used by command "ddcutil detect"
+ *
+ *  @param[in] depth  logical indentation depth
+ *  @return    number of MCCS capable displays
+ */
+__attribute__ ((deprecated ("use ddca_report_displays()")))
+int
+ddca_report_active_displays(
+      int depth);
+
+
 // deprecated, use ddca_report_displays()
 int
 ddca_report_active_displays(int depth) {
-   return ddc_report_displays(false, depth);
+   bool debug = false;
+   API_PROLOG(debug, "");
+   int display_ct = ddc_report_displays(false, depth);
+   DBGTRC_DONE(debug, DDCA_TRC_API, "Returning %d", display_ct);
+   DISABLE_API_CALL_TRACING();
+   return display_ct;
 }
+#endif
+
 
 int
 ddca_report_displays(bool include_invalid_displays, int depth) {
-   return ddc_report_displays(include_invalid_displays, depth);
+   bool debug = false;
+   API_PROLOG(debug, "");
+   int display_ct = 0;
+   if (!library_initialization_failed) {
+      display_ct = ddc_report_displays(include_invalid_displays, depth);
+   }
+   DBGTRC_DONE(debug, DDCA_TRC_API, "Returning: %d", display_ct);
+   DISABLE_API_CALL_TRACING();
+   return display_ct;
+}
+
+#ifdef DETAILED_DISPLAY_CHANGE_HANDLING
+
+typedef enum {
+   DDCA_DISPLAY_ADDED   = 0,
+   DDCA_DISPLAY_REMOVED = 1,
+} DDCA_Display_Detection_Op;
+
+typedef struct {
+   DDCA_Display_Ref dref;
+   DDCA_Display_Detection_Op operation;
+} DDCA_Display_Detection_Report;
+
+
+typedef void (*DDCA_Display_Status_Callback_Func)(DDCA_Display_Detection_Report);
+
+DDCA_Status
+ddca_register_display_status_callback(DDCA_Display_Status_Callback_Func func);
+#endif
+
+
+
+//
+// Display Status Change Communication
+//
+
+DDCA_Status
+ddca_register_display_status_callback(DDCA_Display_Status_Callback_Func func) {
+   bool debug = false;
+   free_thread_error_detail();
+   API_PROLOGX(debug, "func=%p", func);
+
+   DDCA_Status result = DDCRC_INVALID_OPERATION;
+ #ifdef ENABLE_UDEV
+    result = (all_sysfs_i2c_info_drm(/*rescan=*/false))
+                       ? ddc_register_display_status_callback(func)
+                       : DDCRC_INVALID_OPERATION;
+ #endif
+
+
+   API_EPILOG(debug, result, "");
+   return result;
 }
 
 
+DDCA_Status
+ddca_unregister_display_status_callback(DDCA_Display_Status_Callback_Func func) {
+   bool debug = false;
+   free_thread_error_detail();
+   API_PROLOGX(debug, "func=%p", func);
+
+   DDCA_Status result = ddc_unregister_display_status_callback(func);
+
+   API_EPILOG(debug, result, "");
+   return result;
+}
+
+
+const char *
+   ddca_display_event_type_name(DDCA_Display_Event_Type event_type) {
+      return ddc_display_event_type_name(event_type);
+}
+
+//
+// Sleep Multiplier Control
+//
+
+DDCA_Status
+ddca_set_display_sleep_multiplier(
+      DDCA_Display_Ref      ddca_dref,
+      DDCA_Sleep_Multiplier multiplier)
+{
+   bool debug = false;
+   free_thread_error_detail();
+    API_PROLOGX(debug, "ddca_dref=%p", ddca_dref);
+
+    assert(library_initialized);
+    Display_Ref * dref = NULL;
+    DDCA_Status rc = validate_ddca_display_ref(ddca_dref, /* basic_only*/ true, /*require_not_asleep*/false, &dref);
+    if (rc == 0)  {
+       Per_Display_Data * pdd = dref->pdd;
+       if (multiplier >= 0.0 && multiplier <= 10.0) {
+          pdd_reset_multiplier(pdd, multiplier);
+       }
+       else
+          rc = DDCRC_ARG;
+    }
+    API_EPILOG_WO_RETURN(debug, rc, "");
+    return rc;
+}
+
+
+DDCA_Status
+ddca_get_current_display_sleep_multiplier(
+      DDCA_Display_Ref        ddca_dref,
+      DDCA_Sleep_Multiplier*  multiplier_loc)
+{
+   bool debug = false;
+   free_thread_error_detail();
+    API_PROLOGX(debug, "ddca_dref=%p", ddca_dref);
+
+    assert(library_initialized);
+    Display_Ref * dref = NULL;
+    DDCA_Status rc = validate_ddca_display_ref(ddca_dref, true, false, &dref);
+    if (rc == 0) {
+       Per_Display_Data * pdd = dref->pdd;
+       *multiplier_loc        = pdd->final_successful_adjusted_sleep_multiplier;
+    }
+    API_EPILOG_WO_RETURN(debug, rc, "");
+    return rc;
+}
+
+
+bool
+ddca_enable_dynamic_sleep(bool onoff)
+{
+   bool debug = false;
+   API_PROLOG(debug, "");
+   free_thread_error_detail();
+
+   bool old = pdd_is_dynamic_sleep_enabled();
+   pdd_enable_dynamic_sleep_all(onoff);
+
+   API_EPILOG_NO_RETURN(debug, "Returning %s", sbool(old));
+   return old;
+}
+
+
+bool ddca_is_dynamic_sleep_enabled()
+{
+   bool debug = false;
+   API_PROLOG(debug, "");
+   free_thread_error_detail();
+
+   bool result = pdd_is_dynamic_sleep_enabled();
+
+   API_EPILOG_NO_RETURN(debug, "Returning %s", sbool(result));
+   return result;
+}
+
+
+//
+// Module initialization
+//
+
+void init_api_displays() {
+   RTTI_ADD_FUNC(ddca_close_display);
+   RTTI_ADD_FUNC(ddca_get_display_info_list2);
+   RTTI_ADD_FUNC(ddca_get_display_info);
+   RTTI_ADD_FUNC(ddca_get_display_ref);
+   RTTI_ADD_FUNC(ddca_get_display_refs);
+   RTTI_ADD_FUNC(ddca_open_display2);
+   RTTI_ADD_FUNC(ddca_open_display3);
+   RTTI_ADD_FUNC(ddca_redetect_displays);
+   RTTI_ADD_FUNC(ddca_report_display_by_dref);
+   RTTI_ADD_FUNC(ddca_register_display_status_callback);
+   RTTI_ADD_FUNC(ddca_unregister_display_status_callback);
+   RTTI_ADD_FUNC(validate_ddca_display_ref);
+   RTTI_ADD_FUNC(ddca_validate_display_ref);
+}
 
