@@ -3,7 +3,7 @@
  *  Functions to get EDID for USB connected monitors
  */
 
-// Copyright (C) 2014-2022 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2023 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 #include <config.h>
@@ -29,6 +29,8 @@
 #include "util/x11_util.h"         // for EDID fallback
 #endif
 
+#include "public/ddcutil_types.h"
+
 #include "usb_util/hiddev_reports.h"
 #include "usb_util/hiddev_util.h"
 
@@ -36,6 +38,7 @@
 #include "base/ddc_errno.h"
 #include "base/execution_stats.h"
 #include "base/linux_errno.h"
+#include "base/rtti.h"
 
 #include "i2c/i2c_bus_core.h"     // for EDID fallback
 #include "i2c/i2c_bus_selector.h" // for EDID fallback
@@ -46,19 +49,11 @@
 
 
 // Trace class for this file
-
-// Doesn't work
-// Avoid unused variable warning if all debug code turned off
-// #pragma GCC diagnostic push
-// #pragma GCC diagnostic ignored "-Wunused-variable"
-// static Trace_Group TRACE_GROUP = TRC_USB;
-// #pragma GCC diagnostic pop
-
 static DDCA_Trace_Group TRACE_GROUP = DDCA_TRC_USB;
 
-void usb_edid_unused_function_to_avoid_unused_variable_warning() {
-   printf("0x%02x\n",TRACE_GROUP);
-}
+// void usb_edid_unused_function_to_avoid_unused_variable_warning() {
+//    printf("0x%02x\n",TRACE_GROUP);
+// }
 
 // struct model_sn_pair
 
@@ -95,8 +90,10 @@ void report_model_sn_pair(struct model_sn_pair * p, int depth) {
  * Returns:    pointer to a newly allocated struct hid_field_locator
  */
 
-struct hid_field_locator * find_eizo_model_sn_report(int fd) {
+struct hid_field_locator *
+find_eizo_model_sn_report(int fd) {
    bool debug = false;
+   DBGTRC_STARTING(debug, TRACE_GROUP, "");
    struct hid_field_locator * loc = NULL;
    struct hiddev_devinfo dev_info;
 
@@ -109,11 +106,10 @@ struct hid_field_locator * find_eizo_model_sn_report(int fd) {
    }
 
 bye:
-   if (debug) {
-      DBGMSG("Returning: %p", (void*)loc);
-      // if (loc)
+   DBGTRC_DONE(debug, TRACE_GROUP, "Returning: %p", (void*)loc);
+      // if (loc ) {
       //    report_hid_field_locator(loc,2);
-   }
+      // }
    return loc;
 }
 
@@ -138,7 +134,7 @@ bye:
 #endif
 
 
-/* Gets the module and serial number of an Eizo monitor using an Eizo specific report.
+/* Gets the model and serial number of an Eizo monitor using an Eizo specific report.
  *
  * Finds the specific report, then reads it.
  *
@@ -151,9 +147,10 @@ bye:
  *
  * Returns:  model and serial number strings
  */
-struct model_sn_pair *  get_eizo_model_sn_by_report(int fd) {
+struct model_sn_pair *
+get_eizo_model_sn_by_report(int fd) {
    bool debug = false;
-   DBGMSF(debug, "Starting");
+   DBGTRC_STARTING(debug, TRACE_GROUP, "");
    struct model_sn_pair* result = NULL;
    Buffer * modelsn  = NULL;
    Buffer * modelsn2 = NULL;
@@ -193,14 +190,14 @@ struct model_sn_pair *  get_eizo_model_sn_by_report(int fd) {
    if (loc)
       free_hid_field_locator(loc);
 
-   if (debug) {
-      if (result) {
-         DBGMSG("Returning: %p -> mode=|%s|, sn=|%s|",
-                (void*) result, result->model, result->sn);
-         // report_model_sn_pair(result, 1);
-      }
-      else
-         DBGMSG("Returning: %p", (void*) result);
+    if (result) {
+       DBGTRC_DONE(debug, TRACE_GROUP, "Returning: %p -> model=|%s|, sn=|%s|",
+                  (void*) result, result->model, result->sn);
+       // if (debug || IS_TRACING())
+       //    report_model_sn_pair(result, 1);
+    }
+      else {
+         DBGTRC_DONE(debug, TRACE_GROUP, "Returning: %p", (void*) result);
    }
    return result;
 }
@@ -219,21 +216,22 @@ struct model_sn_pair *  get_eizo_model_sn_by_report(int fd) {
  *
  * Returns:   parsed EDID if found
  */
-Parsed_Edid * get_x11_edid_by_model_sn(char * model_name, char * sn_ascii) {
+Parsed_Edid *
+get_x11_edid_by_model_sn(char * model_name, char * sn_ascii) {
    bool debug = false;
-   DBGMSF(debug, "Starting.  model_name=|%s|, sn_ascii=|%s|", model_name, sn_ascii);
+   DBGTRC_STARTING(debug, TRACE_GROUP, "model_name=|%s|, sn_ascii=|%s|", model_name, sn_ascii);
    Parsed_Edid * parsed_edid = NULL;
 
    GPtrArray* edid_recs = get_x11_edids();
    // puts("");
    // printf("EDIDs reported by X11 for connected xrandr outputs:\n");
-   // DBGMSG("Got %d X11_Edid_Recs\n", edid_recs->len);
+   DBGTRC_NOPREFIX(debug, TRACE_GROUP, "Got %d X11_Edid_Recs", edid_recs->len);
 
    for (int ndx=0; ndx < edid_recs->len; ndx++) {
       X11_Edid_Rec * prec = g_ptr_array_index(edid_recs, ndx);
       // printf(" Output name: %s -> %p\n", prec->output_name, prec->edid);
       // hex_dump(prec->edid, 128);
-      DBGMSF(debug, "Comparing EDID for xrandr output: %s", prec->output_name);
+      DBGTRC_NOPREFIX(debug, TRACE_GROUP, "Comparing EDID for xrandr output: %s", prec->output_name);
       parsed_edid = create_parsed_edid2(prec->edidbytes, "X11");
       if (parsed_edid) {
          if (debug) {
@@ -243,13 +241,13 @@ Parsed_Edid * get_x11_edid_by_model_sn(char * model_name, char * sn_ascii) {
          if (streq(parsed_edid->model_name, model_name) &&
                streq(parsed_edid->serial_ascii, sn_ascii) )
          {
-            DBGMSF(debug, "Found matching EDID from X11");
+            DBGTRC_NOPREFIX(debug, TRACE_GROUP, "Found matching EDID from X11");
             break;
          }
          free_parsed_edid(parsed_edid);
       }
       else {
-         if (debug || get_output_level() >= DDCA_OL_VERBOSE) {
+         if (debug || IS_TRACING() || get_output_level() >= DDCA_OL_VERBOSE) {
             DBGMSG("Unparsable EDID for output name: %s -> %p",
                    prec->output_name, prec->edidbytes);
             rpt_hex_dump(prec->edidbytes, 128, /*depth=*/ 1);
@@ -257,7 +255,6 @@ Parsed_Edid * get_x11_edid_by_model_sn(char * model_name, char * sn_ascii) {
       }
    }
 
-
 #ifdef MOCK_DATA_FOR_DEVELOPMENT
    if (!parsed_edid && edid_recs->len > 0) {
       printf("(%s) HACK FOR TESTING: Using last X11 EDID\n", __func__);
@@ -267,23 +264,25 @@ Parsed_Edid * get_x11_edid_by_model_sn(char * model_name, char * sn_ascii) {
 #endif
 
    g_ptr_array_free(edid_recs, true);
-   DBGMSF(debug, "returning %p", parsed_edid);
+   DBGTRC_DONE(debug, TRACE_GROUP, "returning %p", parsed_edid);
    return parsed_edid;
 }
 #endif
 
 
-Parsed_Edid * get_fallback_hiddev_edid(int fd, struct hiddev_devinfo * dev_info) {
+Parsed_Edid *
+get_fallback_hiddev_edid(int fd, struct hiddev_devinfo * dev_info) {
    bool debug = false;
-   DBGMSF(debug, "Starting");
+   DBGTRC_STARTING(debug, TRACE_GROUP, "busnum=%d, devnum=%d, vendor=-x%08x, product=0x%08x",
+                          dev_info->busnum, dev_info->devnum, dev_info->vendor, dev_info->product);
 
    Parsed_Edid * parsed_edid = NULL;
    char * edid_source;
-
    struct model_sn_pair * model_sn = NULL;
 
    // Special handling for Eizo monitors
-   if (dev_info->vendor == 0x056d && dev_info->product == 0x0002) {   // if is EIZO monitor?
+   // if (dev_info->vendor == 0x056d && dev_info->product == 0x0002) {   // if is EIZO monitor?
+   if (dev_info->vendor == 0x056d) {   // if is EIZO monitor?
       DBGMSG("*** Special fixup for Eizo monitor ***");
 
       model_sn  = get_eizo_model_sn_by_report(fd);
@@ -337,7 +336,7 @@ Parsed_Edid * get_fallback_hiddev_edid(int fd, struct hiddev_devinfo * dev_info)
       free_model_sn_pair(model_sn);
    if (parsed_edid)
       STRLCPY(parsed_edid->edid_source, edid_source, EDID_SOURCE_FIELD_SIZE);
-   DBGMSF(debug, "Returning: %p", parsed_edid);
+   DBGTRC_DONE(debug, TRACE_GROUP, "Returning: %p", parsed_edid);
    return parsed_edid;
 }
 
@@ -353,18 +352,21 @@ Parsed_Edid * get_fallback_hiddev_edid(int fd, struct hiddev_devinfo * dev_info)
  *
  * It is the responsibility of the caller to free the returned buffer.
  */
-Parsed_Edid * get_hiddev_edid_with_fallback(int fd, struct hiddev_devinfo * dev_info)  {
+Parsed_Edid *
+get_hiddev_edid_with_fallback(int fd, struct hiddev_devinfo * dev_info)  {
    bool debug = false;
-   if (debug) {
-      DBGMSG("Starting");
+   DBGTRC_STARTING(debug, TRACE_GROUP, "busnum=%d, devnum=%d, vendor=-x%08x, product=0x%08x",
+                           dev_info->busnum, dev_info->devnum, dev_info->vendor, dev_info->product);
+   if (debug || IS_TRACING())
       dbgrpt_hiddev_devinfo(dev_info, true, 1);
-   }
 
    Parsed_Edid * parsed_edid = NULL;
 
    Buffer * edid_buffer = hiddev_get_edid(fd);    // in hiddev_util.c
+   DBGTRC_NOPREFIX(debug, TRACE_GROUP, "hiddev_get_edid() returned %p", edid_buffer);
    // try alternative - both work, pick one
    Buffer * edid_buf2   = hiddev_get_multibyte_value_by_ucode(fd, 0x00800002, 128);
+   DBGTRC_NOPREFIX(debug, TRACE_GROUP, "hiddev_get_multibyte_value_by_ucode() returned %p", edid_buf2);
    if (edid_buffer && edid_buffer->len > 128)
       buffer_set_length(edid_buffer,128);
    // printf("edid_buffer:\n");
@@ -374,12 +376,17 @@ Parsed_Edid * get_hiddev_edid_with_fallback(int fd, struct hiddev_devinfo * dev_
    assert(buffer_eq(edid_buffer, edid_buf2));
    if (edid_buf2)
       buffer_free(edid_buf2, __func__);
+   if (edid_buffer) {
+      if (debug || IS_TRACING())
+         rpt_hex_dump(edid_buffer->bytes, edid_buffer->len, 2);
+   }
 
    if (edid_buffer) {
        parsed_edid = create_parsed_edid2(edid_buffer->bytes, "USB");  // copies bytes
        if (!parsed_edid) {
-          DBGMSF(debug, "get_hiddev_edid() returned invalid EDID");
+          DBGTRC_NOPREFIX(debug, TRACE_GROUP, "get_hiddev_edid() returned invalid EDID");
           // if debug or verbose, dump the bad edid  ??
+          // rpt_hex_dump(edid_buffer->bytes, edid_buffer->len, 2);
        }
        buffer_free(edid_buffer, __func__);
        edid_buffer = NULL;
@@ -388,7 +395,21 @@ Parsed_Edid * get_hiddev_edid_with_fallback(int fd, struct hiddev_devinfo * dev_
    if (!parsed_edid)
       parsed_edid = get_fallback_hiddev_edid(fd, dev_info);
 
-   DBGMSF(debug, "Returning: %p", parsed_edid);
+   DBGTRC_DONE(debug, TRACE_GROUP, "Returning: %p", parsed_edid);
    return parsed_edid;
 }
 
+
+void init_usb_edid() {
+   RTTI_ADD_FUNC(find_eizo_model_sn_report);
+   RTTI_ADD_FUNC(get_eizo_model_sn_by_report);
+#ifdef USE_X11
+   RTTI_ADD_FUNC(get_x11_edid_by_model_sn);
+#endif
+   RTTI_ADD_FUNC(get_fallback_hiddev_edid);
+   RTTI_ADD_FUNC(get_hiddev_edid_with_fallback);
+}
+
+
+
+
