@@ -1,8 +1,6 @@
-/** @file data_structures.c
- *  General purpose data structures
- */
+/** @file data_structures.c  General purpose data structures */
 
-// Copyright (C) 2014-2022 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2023 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 /** \cond */
@@ -16,6 +14,7 @@
 #include <sys/param.h>     // for MIN, MAX
 /** \endcond */
 
+#include "debug_util.h"
 #include "report_util.h"
 #include "string_util.h"
 
@@ -205,8 +204,8 @@ void buffer_set_byte(Buffer * buf, int offset, Byte byte) {
  */
 void buffer_set_bytes(Buffer * buf, int offset, Byte * bytes, int bytect) {
    if (trace_buffer)
-      printf("(%s) Storing %d bytes at offset %d, buffer_size=%d\n",
-             __func__, bytect, offset, buf->buffer_size);
+      printf("(%s) Storing %d bytes at offset %d (buf->bytes+offset=%p), buffer_size=%d, bytes=%p, bytes+bytect=%p\n",
+             __func__, bytect, offset, buf->bytes+offset, buf->buffer_size, bytes, bytes+bytect);
    assert(offset >= 0 && (offset + bytect) <= buf->buffer_size);
 
    memcpy(buf->bytes+offset, bytes, bytect);
@@ -362,7 +361,7 @@ void buffer_rpt(Buffer * buffer, int depth) {
 /** Allocates a new #Circular_String_Buffer
  *
  *  @param  size  buffer size (number of entries)
- *  @return newly allocated #Circular_String_Buffer
+ *  @return pointer to newly allocated #Circular_String_Buffer
  */
 Circular_String_Buffer *
 csb_new(int size) {
@@ -376,7 +375,7 @@ csb_new(int size) {
 
 /** Appends a string to a #Circular_String_Buffer.
  *
- *  @param   csb   #Circular_String_Buffer
+ *  @param   csb   pointer to #Circular_String_Buffer
  *  @param   line  string to append
  *  @param   copy  if true, a copy of the string is appended to the buffer
  *                 if false, the string itself is appended
@@ -385,21 +384,45 @@ void
 csb_add(Circular_String_Buffer * csb, char * line, bool copy) {
     int nextpos = csb->ct % csb->size;
     // printf("(%s) Adding at ct %d, pos %d, line |%s|\n", __func__, csb->ct, nextpos, line);
-    if (csb->lines[nextpos])
+    if (csb->lines[nextpos] && copy)
        free(csb->lines[nextpos]);
-    if (copy)
-       csb->lines[nextpos] = g_strdup(line);
-    else
-       csb->lines[nextpos] = line;
+    csb->lines[nextpos] = (copy) ? g_strdup(line) : line;
     csb->ct++;
 }
 
 
+/** Frees a #Circular_String_Buffer
+ *
+ *  @param  csb pointer to #Circular_String_Buffer
+ *  @param  free_strings  free the strings pointed to by the #Cirular_String_Buffer
+ */
+
+void
+csb_free(Circular_String_Buffer * csb, bool free_strings) {
+   if (free_strings) {
+      int first = 0;
+      if (csb->ct > csb->size)
+         first = csb->ct % csb->size;
+      // printf("(%s) first=%d\n", __func__, first);
+
+      for (int ndx = 0; ndx < csb->ct; ndx++) {
+         int pos = (first + ndx) % csb->size;
+         char * s = csb->lines[pos];
+         // printf("(%s) line %d, |%s|\n", __func__, ndx, s);
+         free(s);
+      }
+      csb->ct = 0;
+   }
+   free(csb->lines);
+   free(csb);
+}
+
+
 /** All the strings in a #Circular_String_Buffer are moved to  a newly
  *  allocated GPtrArray. The count of lines in the now empty #Circular_String_Buffer
  *  is set to 0.
  *
- *   @param csb #Circular_String_Buffer to convert
+ *   @param csb pointer to #Circular_String_Buffer to convert
  *   @return    newly allocated #GPtrArray
  */
 GPtrArray *
@@ -424,6 +447,65 @@ csb_to_g_ptr_array(Circular_String_Buffer * csb) {
 }
 
 
+#ifdef OLD
+//
+// Circular Integer Buffer
+//
+
+typedef struct {
+   int *    values;
+   int      size;
+   int      ct;
+} Circular_Integer_Buffer;
+
+
+/** Allocates a new #Circular_Integer_Buffer
+ *
+ *  @param  size  buffer size (number of entries)
+ *  @return newly allocated #Circular_Integer_Buffer
+ */
+Circular_Integer_Buffer *
+cib_new(int size) {
+   Circular_Integer_Buffer * cib = calloc(1, sizeof(Circular_Integer_Buffer));
+   cib->values = calloc(size, sizeof(int));
+   cib->size = size;
+   cib->ct = 0;
+   return cib;
+}
+
+
+void cib_free(Circular_Integer_Buffer * cib) {
+   free(cib->values);
+   free(cib);
+}
+
+
+/** Appends an integer to a #Circular_Integer_Buffer.
+ *
+ *  @param   cib   #Circular_Integer_Buffer
+ *  @param   value value to append
+ */
+void
+cib_add(Circular_Integer_Buffer * cib, int value) {
+    int nextpos = cib->ct % cib->size;
+    // printf("(%s) Adding at ct %d, pos %d, value %d\n", __func__, cib->ct, nextpos, value);
+       cib->values[nextpos] = value;
+    cib->ct++;
+}
+
+
+void cib_get_latest(Circular_Integer_Buffer * cib, int ct, int latest_values[]) {
+   assert(ct <= cib->ct);
+   int ctr = 0;
+
+   while(ctr < ct) {int_min
+      int ndx = (ctr > 0) ? (ctr-1) % cib->size : cib->size - 1;
+      latest_values[ctr] = cib->values[ ndx ];
+   }
+}
+
+#endif
+
 
 //
 // Identifier id to name and description lookup
@@ -487,15 +569,15 @@ char * vnt_title(Value_Name_Title* table, uint32_t val) {
  *
  *  @result value id
  */
-uint32_t vnt_find_id(
+int32_t vnt_find_id(
            Value_Name_Title_Table table,
            const char * s,
            bool use_title,       // if false, search by symbolic name, if true, search by title
            bool ignore_case,
-           uint32_t default_id)
+           int32_t default_id)
 {
    assert(s);
-   uint32_t result = default_id;
+   int32_t result = default_id;
    Value_Name_Title * cur = table;
    for (; cur->name; cur++) {
       char * comparand = (use_title) ? cur->title : cur->name;
@@ -515,11 +597,11 @@ uint32_t vnt_find_id(
 
 /** Interprets an integer whose bits represent named flags.
  *
- * @param flags_val      value to interpret
- * @param bitname_table  pointer to Value_Name table
- * @param use_title      if **true**, use the **title** field of the table,\n
- *                       if **false**, use the **name** field of the table
- * @param sepstr         if non-NULL, separator string to insert between values
+ * @param  flags_val      value to interpret
+ * @param  bitname_table  pointer to Value_Name table
+ * @param  use_title      if **true**, use the **title** field of the table,\n
+ *                        if **false**, use the **name** field of the table
+ * @param  sepstr         if non-NULL, separator string to insert between values
  *
  * @return newly allocated character string
  *
@@ -569,13 +651,40 @@ char * vnt_interpret_flags(
         }
         cur_entry++;
      }
-     char * result = strdup(sbuf->str);
+     char * result = g_strdup(sbuf->str);
      g_string_free(sbuf, true);
 
      if (debug)
         printf("(%s) Done. Returning: |%s|\n", __func__, result);
      return result;
+}
 
+
+/** Interprets a flags value as a printable string. The returned string is
+ *  valid until the next call of this function in the current thread.
+ *
+ *  @param flags_val      value to interpret
+ *  @param bitname_table  pointer to Value_Name_Title table
+ *  @param use_title      if **true**, use the **title** field of the table,\n
+ *                        if **false**, use the **name** field of the table
+ *  @param sepstr         if non-NULL, separator string to insert between values
+ *
+ *  @return interpreted value
+ */
+char * vnt_interpret_flags_t(
+      uint32_t                flags_val,
+      Value_Name_Title_Table  bitname_table,
+      bool                    use_title,
+      char *                  sepstr)
+{
+   static GPrivate  x_key = G_PRIVATE_INIT(g_free);
+   static GPrivate  x_len_key = G_PRIVATE_INIT(g_free);
+
+   char * buftemp = vnt_interpret_flags(flags_val, bitname_table, use_title, sepstr);
+   char * buf = get_thread_dynamic_buffer(&x_key, &x_len_key, strlen(buftemp)+1);
+   strcpy(buf, buftemp);
+   free(buftemp);
+   return buf;
 }
 
 
@@ -868,9 +977,6 @@ void test_value_array() {
 #endif
 
 
-
-
-
 //
 // Bit_Set_256 - A data structure containing 256 flags
 //
@@ -900,8 +1006,8 @@ Bit_Set_256 bs256_insert(
     result.bytes[bytendx] |= flagbit;
 
     if (debug) {
-       char * bs1 = strdup(bs256_to_string(bitset,  "",""));
-       char * bs2 = strdup(bs256_to_string(result, "",""));
+       char * bs1 = g_strdup(bs256_to_string_t(bitset,  "",""));
+       char * bs2 = g_strdup(bs256_to_string_t(result, "",""));
        printf("(%s) old bitstring=%s, value %d, returning: %s\n",
               __func__, bs1, bitno, bs2);
        free( bs1);
@@ -1096,7 +1202,7 @@ int bs256_count(
  *  @return string representation, caller should not free
  */
 static char *
-bs256_to_string_general(
+bs256_to_string_general_t(
       Bit_Set_256  bitset,
       bool         decimal_format,
       const char * value_prefix,
@@ -1149,7 +1255,7 @@ bs256_to_string_general(
    // DBGMSG("Returned string length: %d", strlen(buf));
    // DBGMSG("Returning %p - %s", buf, buf);
    if (debug)
-   printf("(%s) Returning: len=%d, %s\n", __func__, (int) strlen(buf), buf);
+      printf("(%s) Returning: string of length %d: |%s|\n", __func__, (int) strlen(buf), buf);
 
    return buf;
 }
@@ -1166,12 +1272,12 @@ bs256_to_string_general(
  *  @return string representation, caller should not free
  */
 char *
-bs256_to_string(
+bs256_to_string_t(
       Bit_Set_256  bitset,
       const char * value_prefix,
       const char * sepstr)
 {
-   return bs256_to_string_general(bitset, false, value_prefix, sepstr);
+   return bs256_to_string_general_t(bitset, false, value_prefix, sepstr);
 }
 
 /** Returns a string representation of a #Bit_Set_256 as a list of decimal numbers.
@@ -1185,24 +1291,21 @@ bs256_to_string(
  *  @return string representation, caller should not free
  */
 char *
-bs256_to_string_decimal(
+bs256_to_string_decimal_t(
       Bit_Set_256  bitset,
       const char * value_prefix,
       const char * sepstr)
 {
-   return bs256_to_string_general(bitset, true, value_prefix, sepstr);
+   return bs256_to_string_general_t(bitset, true, value_prefix, sepstr);
 }
 
 
-
 int bs256_to_bytes(Bit_Set_256 flags, Byte * buffer, int buflen) {
    // printf("(%s) Starting\n", __func__);
-
 #ifndef NDEBUG
    int bit_set_ct = bs256_count(flags);
    assert(buflen >= bit_set_ct);
 #endif
-
    unsigned int bufpos = 0;
    unsigned int flagno = 0;
    // printf("(%s) bs256lags->byte=0x%s\n", __func__, hexstring(flags->byte,32));
@@ -1322,7 +1425,6 @@ bs256_iter_next(
 }
 
 
-
 // TODO:
 // Extracted from feature_list.cpp in ddcui. parse_custom_feature_list()
 // should be rewritten to call this function.
@@ -1344,7 +1446,7 @@ Bit_Set_256 bs256_from_string(
 {
     assert(unparsed_string);
     assert(error_msgs_loc);
-    bool debug = true;
+    bool debug = false;
     if (debug)
        printf("(bs256_from_string) unparsed_string = |%s|\n", unparsed_string );
 
@@ -1390,7 +1492,7 @@ Bit_Set_256 bs256_from_string(
            }
        }
        assert(ndx == ntsal);
-       ntsa_free(pieces, /* free_strings */ true);
+
 
        ASSERT_IFF(ok, errors->len == 0);
 
@@ -1403,10 +1505,11 @@ Bit_Set_256 bs256_from_string(
           *error_msgs_loc = g_ptr_array_to_ntsa(errors, false);
           g_ptr_array_free(errors, false);
        }
-     }
+    }
+    ntsa_free(pieces, /* free_strings */ true);
 
     if (debug) {
-       const char * s = bs256_to_string(result, /*prefix*/ "x", /*sepstr*/ ",");
+       const char * s = bs256_to_string_t(result, /*prefix*/ "x", /*sepstr*/ ",");
        printf("Returning bit set: %s\n", s);
        if (*error_msgs_loc) {
           printf("(bs256_from_string) Returning error messages:\n");
@@ -1417,12 +1520,200 @@ Bit_Set_256 bs256_from_string(
 }
 
 
+Bit_Set_256 bs256_from_bva(Byte_Value_Array bva) {
+   Bit_Set_256 result = EMPTY_BIT_SET_256;
+   int ct = bva_length(bva);
+   for (int ndx = 0; ndx < ct; ndx++) {
+      Byte bitno = bva_get(bva, ndx);
+      result = bs256_insert(result, bitno);
+   }
+   return result;
+}
+
+
+//
+// Bit_Set_32 - A set of 32 flags
+//
+
+const Bit_Set_32 EMPTY_BIT_SET_32 = 0;
+const int BIT_SET_32_MAX = 32;
+
+
+#define BB32_REPR_BUF_SZ 10
+/** Represents a #Bit_Set_32 value as a sequence of 4 hex values.
+ *
+ *  @param buf   buffer in which to return value
+ *  @param bufsz buffer size, must be at least #BB256_REPR_BUF_SZ
+ *  @param bbset value to represent
+ */
+void bb32_repr(char * buf, int bufsz, Bit_Set_32 bbset) {
+   assert(bufsz >= BB256_REPR_BUF_SZ);
+   g_snprintf(buf, bufsz, "0x%08x", bbset);
+}
+
+
+/** Returns the number of bits set in a #Bit_Set_32 instance.
+ *
+ *  @param  bbset  value to examine
+ *  @return number of bits set
+ */
+int bs32_count(
+   Bit_Set_32 bbset)
+{
+   bool debug = false;
+
+   int result = 0;
+   for (int bitndx = 0; bitndx < 32; bitndx++) {
+      unsigned char flagbit = (0x80 >> bitndx);
+      if (bbset & flagbit)
+         result += 1;
+   }
+   if (debug) {
+      char buf[BB32_REPR_BUF_SZ];
+      bb32_repr(buf, sizeof(buf), bbset);
+      printf("(%s) Returning %d. bbset: %s\n", __func__, result, buf);
+   }
+   return result;
+}
+
+
+bool bs32_contains(Bit_Set_32 flags, uint8_t val) {
+   assert(val < BIT_SET_32_MAX);
+   bool result = flags & (1 << val);
+   return result;
+}
+
+
+Bit_Set_32 bs32_insert(Bit_Set_32 flags, uint8_t val) {
+   Bit_Set_32 result = flags | (1 << val);
+   return result;
+}
+
+
+/** Returns a string representation of a #Bit_Set_32 as a list of hex numbers.
+ *
+ *  The value returned is valid until the next call to this function in the
+ *  current thread.
+ *
+ *  @param  bitset value to represent
+ *  @param  decimal_format  if true,  represent values as decimal numbers
+ *                          if false, represent values as hex numbers
+ *  @param  value_prefix  prefix for each hex number, typically "0x" or ""
+ *  @param  sepstr        string to insert between each value, typically "", ",", or " "
+ *  @return string representation, caller should not free
+ */
+static char *
+bs32_to_string_general(
+      Bit_Set_32   bitset,
+      bool         decimal_format,
+      const char * value_prefix,
+      const char * sepstr)
+{
+   bool debug = false;
+   if (debug) {
+      printf("(%s) value_prefix=|%s|, sepstr=|%s| bitset: 0x%08x",
+             __func__, value_prefix, sepstr, bitset);
+      // rpt_hex_dump((Byte*)feature_list, 32, 2);
+   }
+
+   static GPrivate  key =     G_PRIVATE_INIT(g_free);
+   static GPrivate  len_key = G_PRIVATE_INIT(g_free);
+
+   if (!value_prefix)
+      value_prefix = "";
+   if (!sepstr)
+      sepstr = "";
+   int vsize = strlen(value_prefix) + 2 + strlen(sepstr);   // for hex
+   if (decimal_format)
+       vsize = strlen(value_prefix) + 3 + strlen(sepstr);   // for decimal
+   int bit_ct = bs32_count(bitset);
+   int reqd_size = (bit_ct*vsize)+1;   // +1 for trailing null
+
+   char * buf = get_thread_dynamic_buffer(&key, &len_key, reqd_size);
+   // char * buf = calloc(1, reqd_size);
+
+   buf[0] = '\0';
+   // printf("(%s) feature_ct=%d, vsize=%d, buf size = %d",
+   //          __func__, feature_ct, vsize, vsize*feature_ct);
+
+   for (int ndx = 0; ndx < 32; ndx++) {
+      if ( bs32_contains(bitset, ndx) ) {
+         if (decimal_format)
+            sprintf(buf + strlen(buf), "%s%d%s", value_prefix, ndx, sepstr);
+         else
+            sprintf(buf + strlen(buf), "%s%02x%s", value_prefix, ndx, sepstr);
+      }
+   }
+
+   if (bit_ct > 0)
+      buf[ strlen(buf)-strlen(sepstr)] = '\0';
+
+   // printf("(%s) wolf 4\n", __func__);
+   // DBGMSG("Returned string length: %d", strlen(buf));
+   // DBGMSG("Returning %p - %s", buf, buf);
+   if (debug)
+      printf("(%s) Returning: len=%d, %s\n", __func__, (int) strlen(buf), buf);
+
+   return buf;
+}
+
+
+char* bs32_to_bitstring(Bit_Set_32 val, char * buf, int bufsz) {
+   assert(bufsz >= BIT_SET_32_MAX+1);
+
+   // char result[BIT_SET_32_MAX+1];
+   for (int ndx = 0; ndx < BIT_SET_32_MAX; ndx++) {
+      buf[(BIT_SET_32_MAX-1)-ndx] = (val & 0x01) ? '1' : '0';
+      val = val >> 1;
+   }
+   buf[BIT_SET_32_MAX] = '\0';
+   return buf;
+}
+
+
+/** Returns a string representation of a #Bit_Set_32 as a list of hex numbers.
+ *
+ *  The value returned is valid until the next call to this function or to
+ *  #bs32_to_string_decimal() in the current thread.
+ *
+ *  @param  bitset value to represent
+ *  @param  value_prefix  prefix for each hex number, typically "0x" or ""
+ *  @param  sepstr        string to insert between each value, typically "", ",", or " "
+ *  @return string representation, caller should not free
+ */
+char *
+bs32_to_string(
+      Bit_Set_32   bitset,
+      const char * value_prefix,
+      const char * sepstr)
+{
+   return bs32_to_string_general(bitset, false, value_prefix, sepstr);
+}
+
+/** Returns a string representation of a #Bit_Set_32 as a list of decimal numbers.
+ *
+ *  The value returned is valid until the next call to this function or to
+ *  #bs32_to_string() in the current thread.
+ *
+ *  @param  bitset value to represent
+ *  @param  value_prefix  prefix for each decimal number
+ *  @param  sepstr        string to insert between each value, typically "", ",", or " "
+ *  @return string representation, caller should not free
+ */
+char *
+bs32_to_string_decimal(
+      Bit_Set_32   bitset,
+      const char * value_prefix,
+      const char * sepstr)
+{
+   return bs32_to_string_general(bitset, true, value_prefix, sepstr);
+}
+
 
 //
 // Cross data structure functions bba <-> bs256
 //
 
-
 /** Tests if the bit number of every byte in a #Byte_Value_Array is set
  *  in a #Bit_Set_256, and conversely that for every bit set in the
  *  #Bit_Set_256 there is a corresponding byte in the #Byte_Value_Array.
@@ -1450,6 +1741,12 @@ bool bva_bs256_same_values( Byte_Value_Array bva , Bit_Set_256 bbflags) {
 }
 
 
+
+
+
+
+
+
 /** Convert a #Byte_Value_Array to a #Bit_Set_256
  *
  *  @param  bva  Byte_Value_Array
@@ -1469,14 +1766,21 @@ Bit_Set_256 bva_to_bs256(Byte_Value_Array bva) {
 /** Function matching signature #Byte_Appender that adds a byte
  * to a #Byte_Value_Array.
  *
- * @param data_struct pointer to #Byte_Value_Array
- * @param val  byte to append
+ * @param data_struct  pointer to #Byte_Value_Array to modify
+ * @param val          byte to append
  */
 void bva_appender(void * data_struct, Byte val) {
    Byte_Value_Array bva = (Byte_Value_Array) data_struct;
    bva_append(bva, val);
 }
 
+
+/** Function matching signature #Byte_Appender that sets a bit
+ *  in a #Bit_Set_256.
+ *
+ * @param data_struct  pointer to #Bit_Set_256 to modify
+ * @param val          number of bit to set
+ */
 void bs256_appender(void * data_struct, Byte val) {
    assert(data_struct);
    Bit_Set_256 * bitset = (Bit_Set_256*) data_struct;
@@ -1564,3 +1868,63 @@ bool bs256_store_bytehex_list(Bit_Set_256 * pbitset, char * start, int len) {
 }
 
 
+//
+// Generic code to register and deregister callback functions.
+//
+
+/** Adds function to a set of registered callbacks
+ *
+ * @param  array of registered callbacks
+ * @param  function to add
+ * @retval true  success
+ *
+ * @remark
+ * It is not an error if the function is already registered.
+ */
+bool generic_register_callback(GPtrArray** registered_callbacks_loc, void * func) {
+   bool debug = false;
+   DBGF(debug, "Starting. registered_callbacks=%p, func=%p", *registered_callbacks_loc, func);
+
+   if (!*registered_callbacks_loc) {
+      *registered_callbacks_loc = g_ptr_array_new();
+   }
+
+   bool new_registration = true;
+   for (int ndx = 0; ndx < (*registered_callbacks_loc)->len; ndx++) {
+      if (func == g_ptr_array_index(*registered_callbacks_loc, ndx)) {
+         new_registration = false;
+         break;
+      }
+   }
+   if (new_registration) {
+      g_ptr_array_add(*registered_callbacks_loc, func);
+   }
+
+   bool result = true;
+   DBGF(debug, "Done.     Returning %s", SBOOL(result));
+   return result;
+}
+
+
+/** Unregisters a callback function
+ *
+ *  @param func function to remove
+ *  @retval true  function deregistered
+ *  @retval false function not found
+ *   */
+bool generic_unregister_callback(GPtrArray* registered_callbacks, void *func) {
+     bool debug = false;
+     DBGF(debug, "Starting. registered_callbacks=%p, func=%p", registered_callbacks, func);
+     bool found = false;
+     if (registered_callbacks) {
+        for (int ndx = 0; ndx < registered_callbacks->len; ndx++) {
+           if ( func == g_ptr_array_index(registered_callbacks, ndx)) {
+              g_ptr_array_remove_index(registered_callbacks,ndx);
+              found = true;
+           }
+        }
+     }
+     DBGF(debug, "Done.     Returning: %s", SBOOL(found));
+     return found;
+}
+
