@@ -15,14 +15,16 @@
 
 /** \cond */
 #include <assert.h>
-#include <inttypes.h>
-#include <stdio.h>
+#include <inttypes.h>  // printf() format macros for stdint.h
 #include <string.h>
+#include <syslog.h>
 /** \endcond */
 
+#include "debug_util.h"
 #include "pnp_ids.h"
 #include "report_util.h"
 #include "string_util.h"
+#include "debug_util.h"
 
 #include "edid.h"
 
@@ -38,7 +40,7 @@
  * Note that the checksum byte (offset 127) is itself
  * included in the checksum calculation.
  */
-Byte edid_checksum(Byte * edid) {
+Byte edid_checksum(const Byte * edid) {
    Byte checksum = 0;
    for (int ndx = 0; ndx < 128; ndx++) {
       checksum += edid[ndx];
@@ -47,12 +49,12 @@ Byte edid_checksum(Byte * edid) {
 }
 
 
-bool is_valid_edid_checksum(Byte * edidbytes) {
+bool is_valid_edid_checksum(const Byte * edidbytes) {
    return (edid_checksum(edidbytes) == 0);
 }
 
 
-bool is_valid_edid_header(Byte * edidbytes) {
+bool is_valid_edid_header(const Byte * edidbytes) {
    bool debug = false;
    bool result = true;
 
@@ -69,12 +71,12 @@ bool is_valid_edid_header(Byte * edidbytes) {
 }
 
 
-bool is_valid_raw_edid(Byte * edid, int len) {
+bool is_valid_raw_edid(const Byte * edid, int len) {
    return (len >= 128) && is_valid_edid_header(edid) && is_valid_edid_checksum(edid);
 }
 
 
-bool is_valid_raw_cea861_extension_block(Byte * edid, int len) {
+bool is_valid_raw_cea861_extension_block(const Byte * edid, int len) {
    return (len >= 128) && edid[0] == 0x02 && is_valid_edid_checksum(edid);
 }
 
@@ -90,7 +92,7 @@ bool is_valid_raw_cea861_extension_block(Byte * edid, int len) {
  * Since the unpacked value is 4 bytes in length (3 characters plus a trailing '\0')
  * it could easily be returned on the stack.  Consider.
  */
-void parse_mfg_id_in_buffer(Byte * mfg_id_bytes, char * result, int bufsize) {
+void parse_mfg_id_in_buffer(const Byte * mfg_id_bytes, char * result, int bufsize) {
       assert(bufsize >= 4);
       result[0] = (mfg_id_bytes[0] >> 2) & 0x1f;
       result[1] = ((mfg_id_bytes[0] & 0x03) << 3) | ((mfg_id_bytes[1] >> 5) & 0x07);
@@ -112,7 +114,7 @@ void parse_mfg_id_in_buffer(Byte * mfg_id_bytes, char * result, int bufsize) {
  *  @param  result       buffer  in which to return manufacturer ID
  *  @param  bufsize      buffer size (must be >= 4)
  */
-void get_edid_mfg_id_in_buffer(Byte* edidbytes, char * result, int bufsize) {
+void get_edid_mfg_id_in_buffer(const Byte* edidbytes, char * result, int bufsize) {
    parse_mfg_id_in_buffer(edidbytes+8, result, bufsize);
 }
 
@@ -144,7 +146,7 @@ void get_edid_mfg_id_in_buffer(Byte* edidbytes, char * result, int bufsize) {
  * - Buffer for edidbytes, and just return pointers to newly allocated memory for found strings
  */
 static void get_edid_descriptor_strings(
-        Byte* edidbytes,
+        const Byte* edidbytes,
         char* namebuf,
         int   namebuf_len,
         char* snbuf,
@@ -164,7 +166,7 @@ static void get_edid_descriptor_strings(
    // In each block, bytes 0-3 indicates the contents.
    int descriptor_ndx = 0;
    for (descriptor_ndx = 0; descriptor_ndx < EDID_DESCRIPTOR_BLOCK_CT; descriptor_ndx++) {
-      Byte * descriptor = edidbytes +
+      const Byte * descriptor = edidbytes +
                           EDID_DESCRIPTORS_BLOCKS_START +
                           descriptor_ndx * EDID_DESCRIPTOR_BLOCK_SIZE;
       if (debug)
@@ -186,7 +188,7 @@ static void get_edid_descriptor_strings(
          }
 
          if (nameslot) {
-            Byte * textstart = descriptor+5;
+            const Byte * textstart = descriptor+5;
             // DBGMSF(debug, "String in descriptor: %s", hexstring(textstart, 14));
             int    offset = 0;
             while (textstart[offset] != 0x0a && offset < 13) {
@@ -212,8 +214,13 @@ static void get_edid_descriptor_strings(
  * @return pointer to newly allocated Parsed_Edid struct,
  *         or NULL if the bytes could not be parsed.
  *         It is the responsibility of the caller to free this memory.
+ *
+ * @remark
+ * The bytes pointed to by **edidbytes** are copied into the newly
+ * allocated Parsed_Edid.  If they were previously malloc'd they
+ * need to free'd.
  */
-Parsed_Edid * create_parsed_edid(Byte* edidbytes) {
+Parsed_Edid * create_parsed_edid(const Byte* edidbytes) {
    assert(edidbytes);
    // bool debug = false;
    Parsed_Edid* parsed_edid = NULL;
@@ -283,8 +290,11 @@ bye:
  * @return pointer to newly allocated Parsed_Edid struct,
  *         or NULL if the bytes could not be parsed.
  *         It is the responsibility of the caller to free this memory.
+ *
+ * **source** is copied to the Parsed_Edid and should be freed is it
+ * was allocated.
  */
-Parsed_Edid * create_parsed_edid2(Byte* edidbytes, char * source) {
+Parsed_Edid * create_parsed_edid2(const Byte* edidbytes, const char * source) {
    Parsed_Edid * edid = create_parsed_edid(edidbytes);
    if (edid) {
       assert(source && strlen(source) < EDID_SOURCE_FIELD_SIZE);
@@ -294,16 +304,105 @@ Parsed_Edid * create_parsed_edid2(Byte* edidbytes, char * source) {
 }
 
 
+Parsed_Edid * copy_parsed_edid(Parsed_Edid * original) {
+   bool debug = false;
+   DBGF(debug, "Starting. original=%p", original);
+   Parsed_Edid * copy =  NULL;
+   if (original) {
+      copy = create_parsed_edid(original->bytes);
+      memcpy(&copy->edid_source, original->edid_source, sizeof(original->edid_source));
+      // report_parsed_edid(copy, true, 2);
+   }
+   DBGF(debug, "Done. returning %p -> ", copy);
+   return copy;
+}
+
+
 /** Frees a Parsed_Edid struct.
  *
  * @param  parsed_edid  pointer to Parsed_Edid struct to free
  */
 void free_parsed_edid(Parsed_Edid * parsed_edid) {
+   bool debug = false;
    assert( parsed_edid );
-   assert( memcmp(parsed_edid->marker, EDID_MARKER_NAME, 4)==0 );
-   parsed_edid->marker[3] = 'x';
-   // n. Parsed_Edid contains no pointers
-   free(parsed_edid);
+   // show_backtrace(1);
+   DBGF(debug, "(free_parsed_edid) parsed_edid=%p", parsed_edid);
+   // ASSERT_WITH_BACKTRACE(memcmp(parsed_edid->marker, EDID_MARKER_NAME, 4)==0);
+   if ( memcmp(parsed_edid->marker, EDID_MARKER_NAME, 4)==0 ) {
+      parsed_edid->marker[3] = 'x';
+      // n. Parsed_Edid contains no pointers
+      free(parsed_edid);
+   }
+   else {
+      char * s = g_strdup_printf("Invalid free of Parsed_Edid@%p, marker=%s",
+            parsed_edid, hexstring_t((unsigned char *) parsed_edid->marker, 4));
+      DBGF(true, "%s", s);
+      syslog(LOG_USER|LOG_ERR, "(%s) %s", __func__, s);
+      free(s);
+   }
+}
+
+
+// TODO: generalize to base_asciify(char* s, char* prefix, char* suffix)
+//       move to string_util.h
+
+/** Replaces every character in a string whose value is > 127 with
+ *  the string "<xHH>", where HH is the hex value of the character.
+ *
+ *  @param   s  string to convert
+ *  @return  newly allocated modified character string
+ *
+ *  The caller is responsible for freeing the returned string
+ */
+char * base_asciify(char * s) {
+   int badct = 0;
+   int ndx = 0;
+   while (s[ndx]) {
+      if (s[ndx] & 0x80 || s[ndx] < 32)
+         badct++;
+      ndx++;
+   }
+   int reqd = ndx + 1 + 4*badct;
+   char* result = malloc(reqd);
+   int respos = 0;
+   ndx = 0;
+   while(s[ndx]) {
+      // printf("s[ndx] = %u\n", (unsigned char)s[ndx]);
+      bool is_printable = s[ndx] >=32 && s[ndx] < 128 ;
+      if ( is_printable) {
+         result[respos++] = s[ndx];
+      }
+      else {
+         sprintf(result+respos, "<x%02x>", (unsigned char) s[ndx]);
+         respos += 5;
+      }
+      ndx++;
+   }
+   result[respos] = '\0';
+   // printf("respos=%d, reqd=%d\n", respos, reqd);
+   assert(respos == (reqd-1));
+   return result;
+}
+
+
+/** Replaces every character in a string whose value is > 127 with
+ *  the string "<xHH>", where HH is the hex value of the character.
+ *
+ *  @param   s  string to convert
+ *  @return  converted string
+ *
+ *  The returned value is valid until the next call to this function
+ *  in the current thread.
+ */
+char * base_asciify_t(char * s) {
+   static GPrivate  x_key = G_PRIVATE_INIT(g_free);
+   static GPrivate  x_len_key = G_PRIVATE_INIT(g_free);
+
+   char * buftemp = base_asciify(s);
+   char * buf = get_thread_dynamic_buffer(&x_key, &x_len_key, strlen(s)+1);
+   strcpy(buf, buftemp);
+   free(buftemp);
+   return buf;
 }
 
 
@@ -335,10 +434,10 @@ void report_parsed_edid_base(
    if (edid) {
       rpt_vstring(depth,"EDID synopsis:");
       rpt_vstring(d1,"Mfg id:               %s - %s",     edid->mfg_id, pnp_name(edid->mfg_id));
-      rpt_vstring(d1,"Model:                %s",          edid->model_name);
+      rpt_vstring(d1,"Model:                %s",          base_asciify_t(edid->model_name));
       rpt_vstring(d1,"Product code:         %u  (0x%04x)", edid->product_code, edid->product_code);
    // rpt_vstring(d1,"Product code:         %u",          edid->product_code);
-      rpt_vstring(d1,"Serial number:        %s",          edid->serial_ascii);
+      rpt_vstring(d1,"Serial number:        %s",          base_asciify_t(edid->serial_ascii));
       // Binary serial number is typically 0x00000000 or 0x01010101, but occasionally
       // useful for differentiating displays that share a generic ASCII "serial number"
       rpt_vstring(d1,"Binary serial number: %"PRIu32" (0x%08x)", edid->serial_binary, edid->serial_binary);
@@ -348,7 +447,11 @@ void report_parsed_edid_base(
       rpt_vstring(d1,"Manufacture year:     %d,  Week: %d", edid->year, edid->manufacture_week);
       if (verbose_synopsis) {
          rpt_vstring(d1,"EDID version:         %d.%d",       edid->edid_version_major, edid->edid_version_minor);
-         rpt_vstring(d1,"Extra descriptor:        %s",          edid->extra_descriptor_string);
+#ifdef TEST_ASCIIFY
+         char bad[] = {0x81, 0x32, 0x83, 0x34, 0x85, 0x00};
+         strcpy(edid->extra_descriptor_string, bad);
+#endif
+         rpt_vstring(d1,"Extra descriptor:        %s",          base_asciify_t(edid->extra_descriptor_string));
          char explbuf[100];
          explbuf[0] = '\0';
          if (edid->video_input_definition & 0x80) {
@@ -487,6 +590,17 @@ void report_parsed_edid_base(
 }
 
 
+/** Reports whether this is an analog or digital display.
+ *
+ *  @param edid    pointer to parse edid struct
+ *  @retval false  analog display
+ *  @retval true   digital display
+ */
+bool is_input_digital(Parsed_Edid * edid) {
+   return edid->video_input_definition & 0x80;
+}
+
+
 /** Writes a summary of an EDID to the current report output destination.
  * (normally stdout, but may be changed by rpt_push_output_dest())
  *
@@ -507,11 +621,10 @@ void report_parsed_edid(Parsed_Edid * edid, bool verbose, int depth) {
 /** Heuristic test for a laptop display.  Observed laptop displays
  *  never have the model name and serial numbers set.
  */
-bool is_embedded_parsed_edid(Parsed_Edid * parsed_edid) {
+bool is_laptop_parsed_edid(Parsed_Edid * parsed_edid) {
    assert(parsed_edid);
    bool result = streq(parsed_edid->model_name,  "") &&
                  streq(parsed_edid->serial_ascii,"");
    return result;
 }
 
-
