@@ -2,15 +2,19 @@
  *  File utility functions
  */
 
-// Copyright (C) 2014-2022 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2024 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
+#define _GNU_SOURCE
+
 /** \cond */
 #include <assert.h>
 #include <dirent.h>
 #include <errno.h>
 #include <glib-2.0/glib.h>
+#include <linux/limits.h>
 #include <stdbool.h>
+#include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -19,6 +23,7 @@
 /** \endcond */
 
 #include "data_structures.h"
+#include "debug_util.h"
 #include "report_util.h"
 #include "string_util.h"
 #include "subprocess_util.h"
@@ -26,7 +31,6 @@
 #include "file_util.h"
 
 
-
 /** Reads the lines of a text file into a GPtrArray, returning a #Error_Info struct
  *  if an error occurs.
  *
@@ -104,6 +108,7 @@ file_get_last_lines(
          csb_add(csb, line, /*copy=*/ true);
 
          // printf("(%s) Retrieved line of length %zu: %s\n", __func__, read, line);
+         free(line);
          line = NULL;  // reset for next getline() call
          len  = 0;
       }
@@ -112,7 +117,6 @@ file_get_last_lines(
          if (verbose)
             fprintf(stderr, "Error reading file %s: %s\n", fn, strerror(-rc));
       }
-      free(line);
       rc = linectr;
       if (debug)
          printf("(%s) Read %d lines\n", __func__, linectr);
@@ -120,7 +124,7 @@ file_get_last_lines(
          rc = maxlines;
 
       *line_array_loc = csb_to_g_ptr_array(csb);
-      free(csb);
+      csb_free(csb,false);
 //      if (debug) {
 //         GPtrArray * la = *line_array_loc;
 //         printf("(%s) (*line_array_loc)->len=%d\n", __func__, la->len);
@@ -172,13 +176,15 @@ file_get_first_line(
            printf("Nothing to read from %s\n", fn);
       }
       else {
-         if (strlen(single_line) > 0)
+         if (strlen(single_line) > 0) {
+            // single_line has trailing \n, replace it with '\0'
             single_line[strlen(single_line)-1] = '\0';
-         // printf("\n%s", single_line);     // single_line has trailing \n
+            // printf("\n%s", single_line);     // single_line has trailing \n
+         }
       }
       fclose(fp);
    }
-   // printf("(%s) fn=|%s|, returning: |%s|\n", __func__, fn, single_line);
+   // printf("(%s) fn=|%s|, returning: %p -> |%s|\n", __func__, fn, single_line, single_line);
    return single_line;
 }
 
@@ -243,6 +249,49 @@ bye:
 }
 
 
+/** Reads an entire file as a single string.
+ *  As a precaution, a trailing \0 is appended to the returned value.
+ *
+ *  The caller is responsible for freeing the returned buffer.
+ *
+ *  @param  filename   file name
+ *  @param  verbose    white error message to stderr if unable to read
+ *  @return pointer to newly allocated buffer, NULL if file not read
+ */
+char * read_file_single_string(const char * filename, bool verbose) {
+   char * buffer = NULL;
+   long length;
+   FILE * fp = fopen (filename, "rb");
+   if (!fp) {
+      if (verbose)
+         fprintf(stderr, "Error opening \"%s\", %s\n", filename, strerror(errno));
+      goto bye;
+   }
+
+   if (fp) {
+     fseek (fp, 0, SEEK_END);
+     length = ftell (fp);
+     if (length < 0) {       // make coverity happy
+        if (verbose) {
+           fprintf(stderr, "ftell() error on file \"%s\", %s\n", filename, strerror(errno));
+        }
+        fclose(fp);
+        goto bye;
+     }
+     fseek(fp, 0, SEEK_SET);
+     buffer = malloc (length+1);
+     assert(buffer);
+     size_t len1 = fread(buffer, 1, length, fp);   // len1 assignment to avoid unused result error
+     assert(len1 == length);
+     fclose (fp);
+     buffer[len1] = '\0';    // ensure there's a trailing null
+   }
+
+bye:
+   return buffer;
+}
+
+
 /** Checks if a regular file exists.
  *
  * @param fqfn fully qualified file name
@@ -311,7 +360,7 @@ get_filenames_by_filter(
       }
       for (int j = 0; j < count; j++) {
          snprintf(path, PATH_MAX, "%s%s", dirnames[i], filelist[j]->d_name);
-         g_ptr_array_add(devnames, strdup(path));
+         g_ptr_array_add(devnames, g_strdup(path));
          free(filelist[j]);
       }
       free(filelist);
@@ -474,7 +523,7 @@ dir_ordered_foreach(
          // DBGMSG("%s", dent->d_name);
          if (!streq(dent->d_name, ".") && !streq(dent->d_name, "..") ) {
             if (!fn_filter || fn_filter(dent->d_name)) {
-               g_ptr_array_add(simple_filenames, strdup(dent->d_name));
+               g_ptr_array_add(simple_filenames, g_strdup(dent->d_name));
             }
          }
       }
@@ -524,7 +573,7 @@ dir_ordered_foreach_with_arg(
             if (!fn_filter || fn_filter(dent->d_name, fn_filter_argument)) {
                if (debug)
                   printf("(%s) Adding simple filename |%s|\n", __func__, dent->d_name);
-               g_ptr_array_add(simple_filenames, strdup(dent->d_name));
+               g_ptr_array_add(simple_filenames, g_strdup(dent->d_name));
             }
          }
       }
@@ -582,7 +631,7 @@ dir_filtered_ordered_foreach(
          // DBGMSG("%s", dent->d_name);
          if (!streq(dent->d_name, ".") && !streq(dent->d_name, "..") ) {
             if (!dir_filter || dir_filter(dirname, dent->d_name)) {
-               g_ptr_array_add(simple_filenames, strdup(dent->d_name));
+               g_ptr_array_add(simple_filenames, g_strdup(dent->d_name));
             }
          }
       }
@@ -631,7 +680,8 @@ int read_file_with_filter(
       const char * fn,
       char **      filter_terms,
       bool         ignore_case,
-      int          limit)
+      int          limit,
+      bool         free_strings)
 {
    bool debug = false;
    if (debug) {
@@ -656,7 +706,8 @@ int read_file_with_filter(
          line_array,
          filter_terms,
          ignore_case,
-         limit);
+         limit,
+         free_strings);
    }
    else { // rc == 0
       if (debug)
@@ -680,15 +731,18 @@ int read_file_with_filter(
  *  \param limit  if 0,   return all lines that pass filter terms
  *                if > 0, return at most the first #limit lines that satisfy the filter terms
  *                if < 0, return at most the last  #limit lines that satisfy the filter terms
- *
+ *  \param free_strings free strings removed from the array
  *  \remark
  *  Consider allowing filter_terms to be regular expressions.
+ *
+ *  1/2024: rare segfault seen
  */
 void filter_and_limit_g_ptr_array(
       GPtrArray * line_array,
       char **     filter_terms,
       bool        ignore_case,
-      int         limit)
+      int         limit,
+      bool        free_strings)
 {
 //   bool debug = false;
 //   if (debug) {
@@ -711,12 +765,14 @@ void filter_and_limit_g_ptr_array(
    for (int ndx = (line_array->len)-1 ; ndx >= 0; ndx--) {
       char * s = g_ptr_array_index(line_array, ndx);
       assert(s);
-      // DBGMSF(debug, "s=|%s|", s);
+      // printf("s=|%s|\n", s);
       bool keep = true;
       if (filter_terms)
          keep = apply_filter_terms(s, filter_terms, ignore_case);
       if (!keep) {
          g_ptr_array_remove_index(line_array, ndx);
+         if (free_strings)
+            free(s);
       }
    }
    gaux_ptr_array_truncate(line_array, limit);
@@ -725,6 +781,51 @@ void filter_and_limit_g_ptr_array(
 }
 
 
+   void filter_and_limit_g_ptr_array2(
+         GPtrArray * line_array,
+         char **     filter_terms,
+         bool        ignore_case,
+         int         limit)
+   {
+   bool debug = false;
+   if (debug) {
+     DBG("line_array=%p, line_array->len=%d, ct(filter_terms)=%d, ignore_case=%s, limit=%d",
+              line_array, line_array->len, ntsa_length(filter_terms), sbool(ignore_case), limit);
+     // (const char **) cast to conform to strjoin() signature
+     char * s = strjoin( (const char **) filter_terms, -1, ", ");
+     DBG("Filter terms: %s", s);
+     free(s);
+   }
+
+   int max_size = line_array->len;
+   if (limit > 0)
+      max_size = limit;
+   else if (limit < 0)
+      max_size = -limit;
+   GPtrArray * new_array = g_ptr_array_sized_new(max_size);
+   g_ptr_array_set_free_func(new_array, g_free);
+   for (int ndx = 0; ndx < line_array->len; ndx++) {
+      char * s = g_ptr_array_index(line_array, ndx);
+      assert(s);
+      DBGF(debug, "s=|%s|", s);
+      bool keep = true;
+      if (filter_terms)
+         keep = apply_filter_terms(s, filter_terms, ignore_case);
+      if (keep)
+            g_ptr_array_add(new_array, g_strdup(s));
+   }
+   // g_ptr_array_remove_range(line_array, 0, line_array->len);
+   g_ptr_array_set_size(line_array, 0);
+   for (int ndx = 0; ndx < new_array->len; ndx++) {
+      g_ptr_array_add(line_array, g_ptr_array_index(new_array, ndx));
+   }
+   g_ptr_array_free(new_array, false);
+
+   DBGF(debug, "Done. line_array->len=%d", line_array->len);
+}
+
+
+
 /** Given a directory, if the directory does not already exist,
  *  creates the directory along with any required parent directories.
  *
@@ -757,7 +858,8 @@ int rek_mkdir(
             printf("(%s) Creating path %s\n", __func__, path);
          if ( mkdir(path, 0777) < 0) {
             result = -errno;
-            f0printf(ferr, "Unable to create '%s', %s\n", path, strerror(errno));
+            if (ferr)
+               f0printf(ferr, "Unable to create '%s', %s\n", path, strerror(errno));
          }
       }
    }
@@ -790,7 +892,7 @@ int fopen_mkdir(
    *fp_loc = NULL;
    char *sep = strrchr(path, '/');
    if (sep) {
-      char *path0 = strdup(path);
+      char *path0 = g_strdup(path);
       path0[ sep - path ] = 0;
       rc = rek_mkdir(path0, ferr);
       free(path0);
@@ -799,7 +901,8 @@ int fopen_mkdir(
       *fp_loc = fopen(path,mode);
       if (!*fp_loc) {
          rc = -errno;
-         f0printf(ferr, "Unable to open %s with mode %s: %s\n", path, mode, strerror(errno));
+         if (ferr)
+            f0printf(ferr, "Unable to open %s with mode %s: %s\n", path, mode, strerror(errno));
       }
    }
    assert( (rc == 0 && *fp_loc) || (rc != 0 && !*fp_loc ) );
@@ -809,3 +912,26 @@ int fopen_mkdir(
    return rc;
 }
 
+
+long get_inode_by_fn(const char * fqfn) {
+   long result = -1;
+   if (fqfn) {
+      struct stat stat_buf;
+      int rc = stat(fqfn, &stat_buf);
+      if (rc == 0) {
+         result = stat_buf.st_ino;
+      }
+   }
+   return result;
+}
+
+
+long get_inode_by_fd(int fd) {
+   long result = -1;
+   struct stat stat_buf;
+   int rc = fstat(fd, &stat_buf);
+   if (rc == 0) {
+      result = stat_buf.st_ino;
+   }
+   return result;
+}
