@@ -2,7 +2,7 @@
  *  File utility functions
  */
 
-// Copyright (C) 2014-2022 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2024 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 #ifndef FILE_UTIL_H_
@@ -37,6 +37,10 @@ GByteArray * read_binary_file(
       int           est_size,
       bool          verbose);
 
+char * read_file_single_string(
+      const char * filename,
+      bool         verbose);
+
 bool regular_file_exists(
       const char *  fqfn);
 
@@ -111,6 +115,13 @@ void dir_filtered_ordered_foreach(
       int                   depth);
 
 void filter_and_limit_g_ptr_array(
+      GPtrArray * line_array,
+      char **     filter_terms,
+      bool        ignore_case,
+      int         limit,
+      bool        free_strings);
+
+void filter_and_limit_g_ptr_array2(
       GPtrArray * line_array,
       char **     filter_terms,
       bool        ignore_case,
@@ -121,7 +132,8 @@ int read_file_with_filter(
       const char * fn,
       char **      filter_terms,
       bool         ignore_case,
-      int          limit);
+      int          limit,
+      bool         free_strings);
 
 int rek_mkdir(
       const char * path,
@@ -133,4 +145,9 @@ int fopen_mkdir(
       FILE *       ferr,
       FILE **      fp_loc);
 
+long get_inode_by_fn(const char * fqfn);
+
+long get_inode_by_fd(int fd);
+
+
 #endif /* FILE_UTIL_H_ */
