@@ -8,6 +8,7 @@
 /** \cond */
 #include <assert.h>
 #include <glib-2.0/glib.h>
+#include <fcntl.h>    // for all_displays_drm2()
 #include <inttypes.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -19,6 +20,12 @@
 #include "report_util.h"
 #include "string_util.h"
 
+// for all_displays_drm2():
+#include <unistd.h>  // for close()
+#include "debug_util.h"
+#include "file_util.h"
+
+
 #include "libdrm_util.h"
 
 
@@ -659,3 +666,4 @@ void summarize_drm_modeProperty(drmModePropertyRes * p, int depth) {
               );
 }
 
+
