@@ -11,7 +11,7 @@
  * - destination stack
  */
 
-// Copyright (C) 2014-2022 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2023 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 /** \cond */
@@ -25,8 +25,8 @@
 #include <unistd.h>
 /** \endcond */
 
-#include "coredefs.h"
-#include "file_util.h"
+#include "coredefs_base.h"
+#include "file_util_base.h"
 #include "string_util.h"
 
 #include "report_util.h"
@@ -255,6 +255,34 @@ void rpt_nl() {
 }
 
 
+/** Writes a constant string to the current output destination, or adds
+ *  the string to a specified GPtrArray.
+ *
+ * A newline is appended to the string specified.
+ *
+ * The output is indented per the specified indentation depth.
+ *
+ * @param title     string to write
+ * @param collector if non-NULL, add string to this GPtrArray instead of
+ *                  writing it to the current output destination
+ * @param depth     logical indentation depth.
+ *
+ * @remark This is the core function through which all output is funneled.
+ */
+void rpt_title_collect(const char * title, GPtrArray * collector, int depth) {
+   bool debug = false;
+   if (debug)
+      printf("(%s) Writing to %p\n", __func__, (void*)rpt_cur_output_dest());
+
+   if (collector) {
+      g_ptr_array_add(collector, g_strdup_printf("%*s%s\n", rpt_get_indent(depth), "", title));
+   }
+   else {
+      f0printf(rpt_cur_output_dest(), "%*s%s\n", rpt_get_indent(depth), "", title);
+   }
+}
+
+
 /** Writes a constant string to the current output destination.
  *
  * A newline is appended to the string specified.
@@ -263,14 +291,26 @@ void rpt_nl() {
  *
  * @param title string to write
  * @param depth logical indentation depth.
- *
- * @remark This is the core function through which all output is funneled.
  */
 void rpt_title(const char * title, int depth) {
-   bool debug = false;
-   if (debug)
-      printf("(%s) Writing to %p\n", __func__, (void*)rpt_cur_output_dest());
-   f0printf(rpt_cur_output_dest(), "%*s%s\n", rpt_get_indent(depth), "", title);
+   rpt_title_collect(title, NULL, depth);
+}
+
+
+/** Writes a constant string to the current output destination, or
+ *  adds it to a GPtrArray if one is given.
+ *
+ * A newline is appended to the string specified.
+ *
+ * The output is indented per the specified indentation depth.
+ *
+ * @param depth     logical indentation depth.
+ * @param collector if non-NULL, append string to this GPtrArray
+ *                  instead of writing it to the current output device
+ * @param title     string to write
+ */
+void rpt_label_collect(int depth, GPtrArray * collector, const char * text) {
+   rpt_title_collect(text, collector, depth);
 }
 
 
@@ -294,6 +334,7 @@ void rpt_label(int depth, const char * text) {
    rpt_title(text, depth);
 }
 
+
 /** Writes a formatted string to the current output destination.
  *
  * A newline is appended to the string specified
@@ -327,6 +368,47 @@ void rpt_vstring(int depth, char * format, ...) {
 }
 
 
+/** Writes a formatted string to the current output destination, or
+ *  adds the string to a collector array.
+ *
+ * A newline is appended to the string specified
+ *
+ * @param  depth     logical indentation depth
+ * @param  collector if non-NULL, add string to GPtrArray instead of
+ *                   writing to current output destination
+ * @param  format    format string (normal printf argument)
+ * @param  ap        va_list array of arguments
+ *
+ * @remark Note that the depth parm is first on this function because of variable args
+ */
+void vrpt_vstring_collect(int depth, GPtrArray * collector, char * format, va_list ap) {
+   char * s = g_strdup_vprintf(format, ap);
+   rpt_title_collect(s, collector, depth);
+   free(s);
+}
+
+
+/** Writes a formatted string to the current output destination.
+ *
+ * A newline is appended to the string specified
+ *
+ * @param  depth     logical indentation depth
+ * @param  collector if non-NULL, add string to GPtrArray instead of
+ *                   writing to current output destination
+ * @param  format    format string (normal printf argument)
+ * @param  ...       arguments
+ *
+ * @remark Note that the depth parm is first on this function because of variable args
+ */
+void rpt_vstring_collect(int depth, GPtrArray* collector, char * format, ...) {
+   va_list(args);
+   va_start(args, format);
+   vrpt_vstring_collect(depth, collector, format, args);
+   va_end(args);
+}
+
+
+
 /** Convenience function that writes multiple constant strings.
  *
  *  @param depth    logical indentation depth
