@@ -1,14 +1,14 @@
 /** @file string_util.c
+ *
  *  String utility functions
  */
 
-// Copyright (C) 2014-2022 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2023 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 
 /** \cond */
-// for strcasestr()
-// #define _GNU_SOURCE
+#define _GNU_SOURCE
 
 #include <assert.h>
 #include <ctype.h>
@@ -404,6 +404,23 @@ char * strjoin( const char ** pieces, const int ct0, const char * sepstr) {
    return result;
 }
 
+
+// TO DO: generalize
+char * int_array_to_string(uint16_t * start, int ct) {
+   int bufsz = ct*10;
+   char * buf = calloc(1, bufsz);
+   int next = 0;
+   for (int ctr =0; ctr < ct && next < bufsz; ctr++) {
+      g_snprintf(buf+next, bufsz-next,"%s%d", (next > 0) ? ", " : "", *(start+ctr) );
+      next = strlen(buf);
+   }
+   // DBGMSG("start=%p, ct=%d, returning %s", start, ct, buf);
+   return buf;
+}
+
+
+
+
 #ifdef FUTURE
 // YAGNI: String_Array
 
@@ -443,7 +460,7 @@ Null_Terminated_String_Array strsplit(const char * str_to_split, const char * de
    char** workstruct = calloc(sizeof(char *), max_pieces+1);
    int piecect = 0;
 
-   char * str_to_split_dup = strdup(str_to_split);
+   char * str_to_split_dup = g_strdup(str_to_split);
    char * rest = str_to_split_dup;
    char * token;
    // originally token assignment was in while() clause, but valgrind
@@ -453,7 +470,7 @@ Null_Terminated_String_Array strsplit(const char * str_to_split, const char * de
       if (debug)
          printf("(%s) token: |%s|\n", __func__, token);
       if (strlen(token) > 0)
-         workstruct[piecect++] = strdup(token);
+         workstruct[piecect++] = g_strdup(token);
       token = strsep(&rest, delims);
    }
    if (debug)
@@ -504,7 +521,7 @@ strsplit_maxlength(
              __func__, max_piece_length, delims, str_to_split);
 
    GPtrArray * pieces = g_ptr_array_sized_new(20);
-   char * str_to_split2 = strdup(str_to_split);   // work around constness
+   char * str_to_split2 = g_strdup(str_to_split);   // work around constness
    char * start = str_to_split2;
    char * str_to_split2_end = str_to_split2 + strlen(str_to_split);
    if (debug)
@@ -622,7 +639,7 @@ Null_Terminated_String_Array ntsa_join(
    char ** from = a1;
    while (*from) {
       if (dup)
-         *to = strdup(*from);
+         *to = g_strdup(*from);
       else
          *to = *from;
       to++;
@@ -631,7 +648,7 @@ Null_Terminated_String_Array ntsa_join(
    from = a2;
    while (*from) {
       if (dup)
-         *to = strdup(*from);
+         *to = g_strdup(*from);
       else
          *to = *from;
       to++;
@@ -660,7 +677,7 @@ Null_Terminated_String_Array ntsa_copy(Null_Terminated_String_Array a1, bool dup
    char ** from = a1;
    while (*from) {
       if (dup)
-         *to = strdup(*from);
+         *to = g_strdup(*from);
       else
          *to = *from;
       to++;
@@ -679,10 +696,10 @@ Null_Terminated_String_Array  ntsa_prepend(
    int new_ct = old_ct + 1;
    Null_Terminated_String_Array new_array = calloc((new_ct+1), sizeof(char *));
    char ** to = new_array;
-   *to++ = (dup) ? strdup(value) : value;
+   *to++ = (dup) ? g_strdup(value) : value;
    char ** from = old_array;
    while (*from) {
-      *to++ = (dup) ? strdup(*from) : *from;
+      *to++ = (dup) ? g_strdup(*from) : *from;
       from++;
    }
    *to = NULL;
@@ -710,7 +727,7 @@ Null_Terminated_String_Array ntsa_create_empty_array() {
  */
 int ntsa_findx(
       Null_Terminated_String_Array string_array,
-      char *                       value,
+      const char *                 value,
       String_Comp_Func             func)
 {
    assert(string_array);
@@ -737,7 +754,7 @@ int ntsa_findx(
  *  @param  value         value to look for
  *  @return index of first matching entry, -1 if not found
  */
-int  ntsa_find(  Null_Terminated_String_Array string_array, char * value) {
+int  ntsa_find(  Null_Terminated_String_Array string_array, const char * value) {
    return ntsa_findx(string_array, value, streq);
 }
 
@@ -794,7 +811,7 @@ g_ptr_array_to_ntsa(
    Null_Terminated_String_Array ntsa = calloc(gparray->len+1, sizeof(char *));
    for (guint ndx=0; ndx < gparray->len; ndx++) {
       if (duplicate)
-         ntsa[ndx] = strdup(g_ptr_array_index(gparray,ndx));
+         ntsa[ndx] = g_strdup(g_ptr_array_index(gparray,ndx));
       else
          ntsa[ndx] = g_ptr_array_index(gparray,ndx);
    }
@@ -840,7 +857,7 @@ void strlower(char * s) {
 char * strdup_uc(const char* s) {
    if (!s)
       return NULL;
-   char * us = strdup( s );
+   char * us = g_strdup( s );
    char * p = us;
    while (*p) {*p=toupper(*p); p++; }
    return us;
@@ -967,10 +984,10 @@ bool sbuf_append(char * buf, int bufsz, char * sepstr, char * nextval)
 // Numeric conversion
 //
 
-/** Converts a decimal or hexadecimal string to an integer value.
+/** Converts a decimal or hexadecimal string to a long integer value.
  *
  * @param sval   string representing an integer
- * @param p_ival address at which to store integer value
+ * @param p_ival address at which to store long integer value
  * @param base   10, 16, or 0 (see below)
  * @return true if conversion succeeded, false if it failed
  *
@@ -986,7 +1003,7 @@ bool sbuf_append(char * buf, int bufsz, char * sepstr, char * nextval)
  * @remark
  * This function wraps system function strtol(), hiding the ugly details.
  */
-bool str_to_int(const char * sval, int * p_ival, int base)
+bool str_to_long(const char * sval, long * p_ival, int base)
 {
    assert (base == 0 || base == 10 || base == 16);
    bool debug = false;
@@ -997,19 +1014,36 @@ bool str_to_int(const char * sval, int * p_ival, int base)
    bool ok = false;
    if (sval) {
       if ( *sval != '\0') {
-         long result = strtol(sval, &endptr, base); // allow hex
-         // printf("(%s) sval=%p, endptr=%p, *endptr=|%c| (0x%02x), result=%ld\n",
-         //        __func__, sval, endptr, *endptr, *endptr, result);
-         if (*endptr == '\0') {
-            *p_ival = result;
-            ok = true;
+         char * work = NULL;
+         bool has_digits = false;
+         if (sval[0] == 'x' || sval[0] == 'X') {
+            work = g_strdup_printf("0%s", sval);
+            has_digits = strlen(work) > 2;
          }
+         else {
+            work = strdup(sval);
+            has_digits = strlen(work) > 0;
+         }
+
+         if (debug)
+            printf("(%s) work = %s\n", __func__, work);
+
+         if (has_digits) {
+            long result = strtol(work, &endptr, base); // allow hex
+            // printf("(%s) sval=%p, endptr=%p, *endptr=|%c| (0x%02x), result=%ld\n",
+            //        __func__, sval, endptr, *endptr, *endptr, result);
+            if (*endptr == '\0') {
+               *p_ival = result;
+               ok = true;
+            }
+         }
+         free(work);
       }
    }
 
    if (debug) {
       if (ok)
-        printf("(%s) sval=%s, Returning: %s, *ival = %d\n", __func__, sval, sbool(ok), *p_ival);
+        printf("(%s) sval=%s, Returning: %s, *ival = %ld\n", __func__, sval, sbool(ok), *p_ival);
       else
         printf("(%s) sval=%s, Returning: %s\n", __func__, sval, sbool(ok));
    }
@@ -1017,6 +1051,31 @@ bool str_to_int(const char * sval, int * p_ival, int base)
 }
 
 
+/** Converts a decimal or hexadecimal string to an integer value.
+ *
+ * @param sval   string representing an integer
+ * @param p_ival address at which to store integer value
+ * @param base   10, 16, or 0 (see below)
+ * @return true if conversion succeeded, false if it failed
+ *
+ * @remark
+ * This function is implemented using #str_to_long().  See the documentation
+ * of that function for details.
+ * This function returns false if the value returned by #str_to_long() does
+ * fit in an int.
+ */
+bool str_to_int(const char * sval, int * p_ival, int base)
+{
+   long lval;
+   bool result = str_to_long(sval, &lval, base);
+   if (result) {
+      *p_ival = lval;
+      if (*p_ival!= lval)     // ensure that lval fits in an int
+         result = false;
+   }
+   return result;
+}
+
 /** Converts a string to a float value.
  *
  * @param sval   string representing an integer
@@ -1094,7 +1153,7 @@ bool hhs_to_byte_in_buf(const char * s, Byte * result)
       *result = (Byte) longtemp;
    }
 
-   // printf("(%s) Returning ok=%d\n", __func__, ok);
+   // printf("(%s) Returning ok=%s, *result=0x%02x\n", __func__, sbool(ok), *result);
    return ok;
 }
 
@@ -1134,7 +1193,7 @@ bool any_one_byte_hex_string_to_byte_in_buf(const char * s, Byte * result)
  *  The characters need not be null terminated.
  *
  *  @param  p_hh      pointer to hex characters.
- *  @param  converted pointer go byte in which converted value will be returned
+ *  @param  converted pointer to byte in which converted value will be returned
  *  @retval **true**  successful conversion
  *  @retval **false** **s** does not point to hex characters
  */
@@ -1153,15 +1212,18 @@ bool hhc_to_byte_in_buf(const char * p_hh, Byte * converted)
 /** Converts a string of hex characters (null terminated) to an array of bytes.
  *
  *  @param   hhs     string of hex characters
- *  @param   pBa     address at which to return pointer to byte array
+ *  @param   ba_loc  address at which to return pointer to byte array
  *  @retval  >= 0 number of bytes in array,
  *  @retval  -1   string could not be converted
  *
- * If successful, the byte array whose address is returned in pBa has
+ * If successful, the byte array whose address is returned in ba_loc has
  * been malloc'd.  It is the responsibility of the caller to free it.
  */
-int hhs_to_byte_array(const char * hhs, Byte** pBa)
+int hhs_to_byte_array(const char * hhs, Byte** ba_loc)
 {
+   bool debug = false;
+   if (debug)
+      printf("(%s) strlen(hhs) = %ld, ba_loc=%p\n", __func__, strlen(hhs), ba_loc);
    if ( strlen(hhs) % 2)     // if odd number of characters
       return -1;
    char xlate[] = "0123456789ABCDEF";
@@ -1189,12 +1251,69 @@ int hhs_to_byte_array(const char * hhs, Byte** pBa)
       bytect = -1;
    }
    else {
-      *pBa = ba;
+      *ba_loc = ba;
+   }
+   if (debug) {
+      printf("(%s) Returning: %d.  *ba_loc = %p\n", __func__, bytect, *ba_loc);
+      if (bytect > 0) {
+         printf("   0x");
+         for (int ndx = 0; ndx< bytect; ndx++) {
+            printf("%02x", ba[ndx]);
+         }
+         printf("\n");
+      }
    }
    return bytect;
 }
 
 
+bool hhs4_to_uint16(char * hhs4, uint16_t* result_loc) {
+   // printf("(%s) Starting. hhs4 = |%s|\n", __func__, hhs4);
+   bool ok = false;
+   *result_loc = 0;
+   if (strlen(hhs4) == 4) {
+      uint8_t hi_byte;
+      uint8_t lo_byte;
+      if (hhc_to_byte_in_buf(&hhs4[0], &hi_byte) && hhc_to_byte_in_buf(&hhs4[2], &lo_byte) )
+         *result_loc = hi_byte << 8 | lo_byte;
+      ok = true;
+   }
+   // printf("(%s) Returning %s, *result_loc = 0x%04x\n", __func__, sbool(ok), *result_loc);
+   return ok;
+}
+
+
+
+/** Converts a single byte to a 2 byte hex string.
+ *
+ *  @param ch        byte to convert
+ *  @param out       buffer to write to, must be at least 3 bytes
+ *  @param uppercase if true, use uppercase letters
+ */
+static void byte_to_hs(const unsigned char ch, char * out, bool uppercase) {
+   bool debug = false;
+   if (debug)
+      printf("(%s) out=%p\n", __func__, out);
+   assert(out);
+   unsigned int hi = ch >> 4;
+   unsigned int lo = ch & 0x0f;
+   assert (hi < 16);
+   assert (lo < 16);
+   char uptable[] = {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A','B','C','D', 'E', 'F'};
+   char lotable[] = {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'a','b','c','d', 'e', 'f'};
+
+   if (uppercase) {
+      out[0] = uptable[hi];
+      out[1] = uptable[lo];
+   }
+   else {
+      out[0] = lotable[hi];
+      out[1] = lotable[lo];
+   }
+   out[2] = '\0';
+}
+
+
 /** Converts a sequence of bytes to its representation as a string of hex characters.
  *
  *  @param  bytes     pointer to bytes
@@ -1346,13 +1465,18 @@ char * hexstring3_t(
    char * buf = get_thread_dynamic_buffer(&hexstring3_key, &hexstring3_len_key, required_size);
    // char * buf = get_thread_private_buffer(&hexstring3_key, NULL, required_size);
 
-   char * pattern = (uppercase) ? "%02X" : "%02x";
+   // char * pattern = (uppercase) ? "%02X" : "%02x";
+   // if (debug)
+   //    printf("(%s) pattern=%s, buf=%p\n", __func__,  pattern, buf);
 
    // int incr1 = 2 + sepsize;
    *buf = '\0';
    for (int i=0; i < len; i++) {
-      // printf("(%s) i=%d, strlen(buf)=%ld\n", __func__, i, strlen(buf));
-      sprintf(buf+strlen(buf), pattern, bytes[i]);
+      if (debug)
+         printf("(%s) i=%d, buf=%p, strlen(buf)=%ld\n", __func__, i, buf, strlen(buf));
+      // sprintf(buf+strlen(buf), pattern, bytes[i]);
+      byte_to_hs(bytes[i], buf+strlen(buf), uppercase);
+
       bool insert_sepstr = (hunk_size == 0)
                                ? (i < (len-1) && sepstr)
                                : (i < (len-1) && sepstr && (i+1)%hunk_size == 0);
