@@ -3,11 +3,14 @@
  * Functions to execute shell commands
  */
 
-// Copyright (C) 2014-2021 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2023 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
+#define _GNU_SOURCE
+
 /** \cond */
 #include <assert.h>
+#include <glib-2.0/glib.h>
 #include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -45,13 +48,11 @@ bool execute_shell_cmd_rpt(const char * shell_cmd, int depth) {
    snprintf(cmdbuf, bufsz, "(%s) 2>&1", shell_cmd);
    // printf("(%s) cmdbuf=|%s|\n", __func__, cmdbuf);
    fp = popen(cmdbuf, "r");
-   // printf("(%s) open. errno=%d\n", __func__, errno);
-    if (!fp) {
-       // int errsv = errno;
-       printf("Unable to execute command \"%s\": %s\n", shell_cmd, strerror(errno));
-       ok = false;
-    }
-    else {
+   if (!fp) {
+      printf("Unable to execute command \"%s\": %s\n", shell_cmd, strerror(errno));
+      ok = false;
+   }
+   else {
        char * a_line = NULL;
        size_t len = 0;
        bool first_line = true;
@@ -95,7 +96,7 @@ bool execute_shell_cmd_rpt(const char * shell_cmd, int depth) {
           a_line = NULL;
           len = 0;
        }
-       // per getline() doc, buffer is allocated even if getline(),
+       // per getline() doc, buffer is allocated even if getline() fails
        free(a_line);
        int pclose_rc = pclose(fp);
        int errsv = errno;
@@ -131,7 +132,7 @@ GPtrArray * execute_shell_cmd_collect(const char * shell_cmd) {
    GPtrArray * result = g_ptr_array_new();
    g_ptr_array_set_free_func(result, g_free);
    if (debug)
-      printf("(%s) Starting. shell_cmd = |%s|", __func__, shell_cmd);
+      printf("(%s) Starting. shell_cmd = |%s|\n", __func__, shell_cmd);
    bool ok = true;
    FILE * fp;
    int bufsz = strlen(shell_cmd) + 50;
@@ -150,8 +151,11 @@ GPtrArray * execute_shell_cmd_collect(const char * shell_cmd) {
        size_t len = 0;
        bool first_line = true;
        while ( getline(&a_line, &len, fp) >= 0) {
+
           if (strlen(a_line) > 0)
              a_line[strlen(a_line)-1] = '\0';
+          if (debug)
+             printf("(%s) a_line = |%s|\n", __func__, a_line);
           if (first_line) {
              if (str_ends_with(a_line, "not found")) {
                 // printf("(%s) found \"not found\"\n", __func__);
@@ -160,7 +164,7 @@ GPtrArray * execute_shell_cmd_collect(const char * shell_cmd) {
              }
              first_line = false;
           }
-          g_ptr_array_add(result, strdup(a_line));
+          g_ptr_array_add(result, g_strdup(a_line));
           free(a_line);
           a_line = NULL;
           len = 0;
@@ -186,7 +190,7 @@ GPtrArray * execute_shell_cmd_collect(const char * shell_cmd) {
  *
  *  \param  cmd        command to execute
  *  \param  fn         file name
- *  \param  filter_terms  #Null_Terminated_String_Away of filter terms
+ *  \param  filter_terms  #Null_Terminated_String_Array of filter terms
  *  \param  ignore_case   ignore case when testing filter terms
  *  \param  limit if 0, return all lines that pass filter terms
  *                if > 0, return at most the first #limit lines that satisfy the filter terms
@@ -219,7 +223,8 @@ int execute_cmd_collect_with_filter(
             line_array,
             filter_terms,
             ignore_case,
-            limit);
+            limit,
+            false);     // line_array has free function set
       }
    }
    *result_loc = line_array;
@@ -245,7 +250,7 @@ char * execute_shell_cmd_one_line_result(const char * shell_cmd) {
    char * result = NULL;
    GPtrArray * response = execute_shell_cmd_collect(shell_cmd);
    if (response) {
-      result = strdup(g_ptr_array_index(response, 0));
+      result = g_strdup(g_ptr_array_index(response, 0));
       g_ptr_array_free(response, true);
    }
    return result;
