@@ -3,9 +3,11 @@
  *  Timestamp management
  */
 
-// Copyright (C) 2014-2021 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2023 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
+#define _GNU_SOURCE
+
 /** \cond */
 #include <assert.h>
 #include <glib-2.0/glib.h>
@@ -33,8 +35,6 @@ static int   timestamp_ct = 0;
 static uint64_t * timestamp_history = NULL;
 
 
-
-
 /** Returns the current value of the realtime clock in nanoseconds.
  *
  * @return timestamp, in nanoseconds
@@ -143,15 +143,14 @@ uint64_t elapsed_time_nanosec() {
 }
 
 
-static uint64_t ipow(const uint64_t base, int n)
+static uint64_t ipow(const uint64_t base, guint n)
 {
-   assert(n >= 0);
-    int p = base;
-    if (n == 0)
-       return 1;
-    for (int i = 1; i < n; ++i)
-        p *= base;
-    return p;
+   int p = base;
+   if (n == 0)
+      return 1;
+   for (int i = 1; i < n; ++i)
+      p *= base;
+   return p;
 }
 
 
@@ -164,7 +163,7 @@ static uint64_t ipow(const uint64_t base, int n)
  *  @param  precision  number of digits after the decimal point
  *  @return formatted elapsed time
  */
-char * formatted_elapsed_time(uint precision) {
+char * formatted_elapsed_time_t(guint precision) {
    static GPrivate  formatted_elapsed_time_key = G_PRIVATE_INIT(g_free);
    char * elapsed_buf = get_thread_fixed_buffer(&formatted_elapsed_time_key, 40);
 
@@ -192,7 +191,7 @@ char * formatted_elapsed_time(uint precision) {
  *  @param  end     end time, in nanoseconds
  *  @return formatted time difference
  */
-char *   formatted_time(uint64_t nanos) {
+char *   formatted_time_t(uint64_t nanos) {
    static GPrivate  formatted_time_key = G_PRIVATE_INIT(g_free);
    char * elapsed_buf = get_thread_fixed_buffer(&formatted_time_key, 40);
 
@@ -202,3 +201,23 @@ char *   formatted_time(uint64_t nanos) {
    return elapsed_buf;
 }
 
+
+/** Thread safe function that returns a string representation of an epoch
+ *  time value. The returned value is valid until the next call to this
+ *  function on the current thread.
+ *
+ *  \param  epoch_time
+ *  \return string representation
+ */
+char * formatted_epoch_time_t(long epoch_seconds) {
+   static GPrivate  formatted_epoch_time_key = G_PRIVATE_INIT(g_free);
+   char * buf = get_thread_fixed_buffer(&formatted_epoch_time_key, 40);
+
+   struct tm broken_down_time;
+   localtime_r(&epoch_seconds, &broken_down_time);
+   strftime(buf, 40, "%b %d %T", &broken_down_time);
+
+   return buf;
+}
+
+
